Package: ShrinkageTrees
Type: Package
Title: Bayesian Tree Ensembles for Survival Analysis and Causal
        Inference
Version: 2.0.2
Date: 2026-04-20
Authors@R: 
    c(person("Tijn", "Jacobs",
             role = c("aut", "cre"),
             email = "t.jacobs@vu.nl",
             comment = c(ORCID = "0009-0003-6188-9296")))
Maintainer: Tijn Jacobs <t.jacobs@vu.nl>
Description: Bayesian regression tree ensembles for survival analysis
    and causal inference. Implements BART, DART, Bayesian Causal
    Forests (BCF), and Horseshoe Forest models. Supports
    right-censored and interval-censored survival outcomes via
    accelerated failure time (AFT) formulations. Designed for
    high-dimensional prediction and heterogeneous treatment effect
    estimation.
URL: https://github.com/tijn-jacobs/ShrinkageTrees
BugReports: https://github.com/tijn-jacobs/ShrinkageTrees/issues
License: MIT + file LICENSE
Depends: R (>= 4.1.0)
Imports: parallel, Rcpp
LinkingTo: Rcpp (>= 1.0.11)
Suggests: coda, ggplot2, knitr, rmarkdown, survival, testthat (>=
        3.0.0)
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
LazyData: true
LazyDataCompression: xz
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-04-21 19:25:57 UTC; tijnjacobs
Author: Tijn Jacobs [aut, cre] (ORCID: <https://orcid.org/0009-0003-6188-9296>)
Repository: CRAN
Date/Publication: 2026-04-21 19:53:08 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-22 00:06:42 UTC; unix
Archs: ShrinkageTrees.so.dSYM

Package: ShrinkageTrees
Type: Package
Title: Bayesian Tree Ensembles for Survival Analysis and Causal
        Inference
Version: 2.0.2
Date: 2026-04-20
Authors@R: 
    c(person("Tijn", "Jacobs",
             role = c("aut", "cre"),
             email = "t.jacobs@vu.nl",
             comment = c(ORCID = "0009-0003-6188-9296")))
Maintainer: Tijn Jacobs <t.jacobs@vu.nl>
Description: Bayesian regression tree ensembles for survival analysis
    and causal inference. Implements BART, DART, Bayesian Causal
    Forests (BCF), and Horseshoe Forest models. Supports
    right-censored and interval-censored survival outcomes via
    accelerated failure time (AFT) formulations. Designed for
    high-dimensional prediction and heterogeneous treatment effect
    estimation.
URL: https://github.com/tijn-jacobs/ShrinkageTrees
BugReports: https://github.com/tijn-jacobs/ShrinkageTrees/issues
License: MIT + file LICENSE
Depends: R (>= 4.1.0)
Imports: parallel, Rcpp
LinkingTo: Rcpp (>= 1.0.11)
Suggests: coda, ggplot2, knitr, rmarkdown, survival, testthat (>=
        3.0.0)
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
LazyData: true
LazyDataCompression: xz
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-04-21 19:25:57 UTC; tijnjacobs
Author: Tijn Jacobs [aut, cre] (ORCID: <https://orcid.org/0009-0003-6188-9296>)
Repository: CRAN
Date/Publication: 2026-04-21 19:53:08 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 22:20:02 UTC; unix
Archs: ShrinkageTrees.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(coda::as.mcmc.list,ShrinkageTrees)
S3method(plot,CausalShrinkageForest)
S3method(plot,ShrinkageTrees)
S3method(plot,ShrinkageTreesPrediction)
S3method(predict,CausalShrinkageForest)
S3method(predict,ShrinkageTrees)
S3method(print,CausalShrinkageForest)
S3method(print,CausalShrinkageForestPrediction)
S3method(print,ShrinkageTrees)
S3method(print,ShrinkageTreesPrediction)
S3method(print,summary.CausalShrinkageForest)
S3method(print,summary.CausalShrinkageForestPrediction)
S3method(print,summary.ShrinkageTrees)
S3method(print,summary.ShrinkageTreesPrediction)
S3method(summary,CausalShrinkageForest)
S3method(summary,CausalShrinkageForestPrediction)
S3method(summary,ShrinkageTrees)
S3method(summary,ShrinkageTreesPrediction)
export(CausalHorseForest)
export(CausalShrinkageForest)
export(HorseTrees)
export(ShrinkageTrees)
export(SurvivalBART)
export(SurvivalBCF)
export(SurvivalDART)
export(SurvivalShrinkageBCF)
export(bayesian_bootstrap_ate)
importFrom(Rcpp,evalCpp)
importFrom(parallel,detectCores)
importFrom(parallel,mclapply)
importFrom(stats,dnorm)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rexp)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(utils,head)
useDynLib(ShrinkageTrees, .registration = TRUE)

# ShrinkageTrees <img src="https://img.shields.io/badge/R%3E%3D-4.1-blue" alt="R >= 4.1"> ![License: MIT](https://img.shields.io/badge/license-MIT-green) ![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen) [![](https://cranlogs.r-pkg.org/badges/grand-total/ShrinkageTrees)](https://cran.r-project.org/package=ShrinkageTrees)

 <img src="sticker/ShrinkageTrees_hex.png" align="right" width="150"/>

## Overview

**ShrinkageTrees** provides a unified framework for survival analysis using Bayesian regression tree ensembles, with a particular focus on **causal inference and high-dimensional data**.

The package implements Horseshoe Trees, Causal Horseshoe Forests, and their more general counterparts — Shrinkage Trees and Causal Shrinkage Forests — alongside well-known Bayesian tree models such as BART, DART, and BCF. All models are adapted to right-censored and interval-censored data via accelerated failure time (AFT) formulations.

Its central methodological innovation is the **Horseshoe regularisation mechanism applied directly to tree step heights**, enabling adaptive global–local shrinkage in high-dimensional settings. In addition to classical BART priors, the package supports:

- Horseshoe priors
- Forest-wide Horseshoe shrinkage
- Empirical Bayes Horseshoe calibration
- Half-Cauchy priors
- Dirichlet splitting priors (DART)

These models enable flexible non-linear modelling for:

1. High-dimensional prediction
2. High-dimensional causal inference
3. Estimation of heterogeneous (conditional average) treatment effects (CATE)

Supported outcome types:

- Continuous outcomes
- Binary outcomes
- **Right-censored and interval-censored survival times (AFT framework)**

All models are implemented with an efficient C++ backend via Rcpp, allowing scalable MCMC sampling in high-dimensional settings.

## ⭐ Core Contribution: Horseshoe Forests

Traditional BART and DART primarily regularise model complexity through the **tree structure** (e.g., depth constraints or splitting probabilities).

ShrinkageTrees instead introduces global–local shrinkage directly on the leaf (step height) parameters via the Horseshoe prior.

A global parameter controls overall shrinkage, while local parameters allow strong signals to escape shrinkage. Small effects are aggressively shrunk toward zero, whereas large effects are preserved due to the heavy-tailed prior.

This strategy retains all covariates, reduces noise in high-dimensional settings, and improves robustness in causal models.

This methodology is introduced in:

> _Horseshoe Forests for High-Dimensional Causal Survival Analysis_  
> T. Jacobs, W.N. van Wieringen, S.L. van der Pas  
> https://arxiv.org/abs/2507.22004

## 🧠 Implemented Models

ShrinkageTrees implements a modular family of Bayesian tree models:

### Single-Forest Models

- **HorseTrees** — BART with Horseshoe shrinkage on leaf parameters
- **DART** — Dirichlet prior on splitting probabilities
- **Standard BART** — Classical Gaussian leaf prior

### Causal Models (BCF-style decomposition)

- **CausalHorseForest** — Prognostic + treatment forests with Horseshoe shrinkage
- **CausalShrinkageForest** — General shrinkage framework
- **BCF** — Bayesian Causal Forest for right-censored and interval-censored AFT models
- **Shrinkage BCF** — Combined structural and magnitude shrinkage

All causal models support right-censored and interval-censored survival data and heterogeneous treatment effect (CATE) estimation.

## 🌲 Tree Regularisation Strategies

ShrinkageTrees supports multiple regularisation mechanisms:

- **Standard BART prior**  
  Structural regularisation through tree depth and splitting probabilities.

- **Dirichlet splitting prior (DART)**  
  Structural sparsity induced via a Dirichlet prior on splitting proportions.

- **Horseshoe shrinkage on step heights**  
  Global–local shrinkage applied directly to leaf parameters.

- **Alternative shrinkage priors on step heights**  
  Including global Cauchy and forest-wide shrinkage formulations for flexible control of overall regularisation strength.

## 📦 Installation

The released version of ShrinkageTrees can be installed from [CRAN](https://cran.r-project.org/package=ShrinkageTrees):

```r
install.packages("ShrinkageTrees")
```

You can install the development version from [GitHub](https://github.com/tijn-jacobs/ShrinkageTrees):

```r
# Install devtools if not already installed
install.packages("devtools")
devtools::install_github("tijn-jacobs/ShrinkageTrees")
```

## 🚀 Example

```r
library(ShrinkageTrees)

set.seed(42)
n <- 100
p <- 1000

# Generate covariates
X <- matrix(runif(n * p), ncol = p)
X_treat <- X_control <- X
treatment <- rbinom(n, 1, X[, 1])
tau <- 1 + X[, 2]/2 - X[, 3]/3 + X[, 4]/4

# Generate survival times (on log-scale)
true_time <- X[, 1] + treatment * tau + rnorm(n)
censor_time <- log(rexp(n, rate = 0.05))
follow_up <- pmin(true_time, censor_time)
status <- as.integer(true_time <= censor_time)

# Fit a standard Causal Horseshoe Forest (without propensity score adjustment)
fit_horseshoe <- CausalHorseForest(
  y = follow_up,
  status = status,
  X_train_control = X_control,
  X_train_treat = X_treat,
  treatment_indicator_train = treatment,
  outcome_type = "right-censored",
  timescale = "log",
  number_of_trees = 200,
  k = 0.1,
  N_post = 5000,
  N_burn = 5000,
  store_posterior_sample = TRUE
)

# Posterior mean CATEs
CATE_horseshoe <- colMeans(fit_horseshoe$train_predictions_sample_treat)

# Posteriors of the ATE
post_ATE_horseshoe <- rowMeans(fit_horseshoe$train_predictions_sample_treat)

# Posterior mean ATE
ATE_horseshoe <- mean(post_ATE_horseshoe)

# Plot the posterior of the ATE
```

![Posterior ATE plot](man/figures/posterior_ate_plot.png)

## 📊 Included Datasets

The package ships with two TCGA datasets for high-dimensional survival analysis and causal inference:

| Dataset   | Cancer            | n   | Covariates              | Treatment                |
| --------- | ----------------- | --- | ----------------------- | ------------------------ |
| `pdac`    | Pancreatic (PAAD) | 178 | ~3,000 genes + clinical | Radiation vs control     |
| `ovarian` | Ovarian (OV)      | 357 | 2,000 genes + clinical  | Carboplatin vs cisplatin |

```r
data("pdac")     # data frame with time, status, treatment, gene expression, ...
data("ovarian")  # data frame with OS_time, OS_event, treatment, clinical vars, and gene expression columns
```

## 🩺 Pancreatic Cancer Analysis Demo

The package includes a **demo analysis** based on the TCGA PAAD (pancreatic cancer) dataset to showcase how ShrinkageTrees can be used in practice. This demo replicates the main case study from the preprint _"Horseshoe Forests for High-Dimensional Causal Survival Analysis"_ ([arXiv:2507.22004](https://arxiv.org/abs/2507.22004)).

The demo:

- Estimates propensity scores for treatment assignment
- Fits a Causal Horseshoe Forest to the survival times with right-censoring or interval-censoring
- Computes the posterior mean ATE and individual CATEs with 95% credible intervals
- Produces diagnostic plots (propensity score overlap, posterior ATE, CATE estimates, sigma trace)

You can run it directly from R after installing the package:

```r
demo("pdac_analysis", package = "ShrinkageTrees")
```

## 🔬 Why ShrinkageTrees Is Unique

ShrinkageTrees provides:

- Bayesian Causal Forests for right-censored and interval-censored survival data
- Dirichlet splitting priors for structural sparsity
- Horseshoe shrinkage applied directly to tree step heights
- A unified AFT-based framework for heterogeneous treatment effects

To our knowledge, no other R package combines:

- BCF
- DART
- Horseshoe leaf shrinkage
- Survival data support

within a single coherent Bayesian tree framework.

## 📄 Documentation

- In R: `?ShrinkageTrees`, `?HorseTrees`, `?CausalHorseForest`, and `?CausalShrinkageForest` for detailed help of the Horseshoe Forests.
- For survival-specific Bayesian tree functions, see: `?SurvivalBART`, `?SurvivalDART`, `?SurvivalBCF`, and `?SurvivalShrinkageBCF`.
- Examples and parameter descriptions can be found in each function’s documentation.

## 🤝 Contributing

Contributions are welcome! Feel free to open an [issue](https://github.com/tijn-jacobs/ShrinkageTrees/issues) or submit a pull request.
The software is designed to be flexible and modular, allowing for a wide variety of global-local shrinkage priors to be easily implemented and extended in future versions.

## 📜 License

This package is licensed under the [MIT License](https://cran.r-project.org/web/licenses/MIT).

## 🇪🇺 Acknowledgments

This project has received funding from the European Research Council (ERC) under the European Union’s Horizon Europe program under Grant agreement No. 101074082. Views and opinions expressed are however those of the author(s) only and do not necessarily reflect those of the European Union or the European Research Council Executive Agency. Neither the European Union nor the granting authority can be held responsible for them

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  message   = FALSE,
  warning   = FALSE,
  fig.width = 6,
  fig.height = 3.5,
  out.width = "100%"
)
library(ShrinkageTrees)
set.seed(42)

## ----tc-data------------------------------------------------------------------
set.seed(50)
n_tc <- 60;  p_tc <- 5
X_tc <- matrix(rnorm(n_tc * p_tc), n_tc, p_tc)
W_tc <- rbinom(n_tc, 1, 0.5)
tau_tc <- 1.5 * (X_tc[, 1] > 0)
y_tc <- X_tc[, 1] + W_tc * tau_tc + rnorm(n_tc, sd = 0.5)

## ----tc-centered--------------------------------------------------------------
# Centered (default)
fit_tc_cen <- CausalHorseForest(
  y = y_tc,
  X_train_control = X_tc, X_train_treat = X_tc,
  treatment_indicator_train = W_tc,
  treatment_coding = "centered",
  number_of_trees = 5, N_post = 50, N_burn = 25,
  store_posterior_sample = TRUE, verbose = FALSE
)
cat("Centered — ATE:",
    round(mean(fit_tc_cen$train_predictions_treat), 3), "\n")

## ----tc-binary----------------------------------------------------------------
# Binary
fit_tc_bin <- CausalHorseForest(
  y = y_tc,
  X_train_control = X_tc, X_train_treat = X_tc,
  treatment_indicator_train = W_tc,
  treatment_coding = "binary",
  number_of_trees = 5, N_post = 50, N_burn = 25,
  store_posterior_sample = TRUE, verbose = FALSE
)
cat("Binary — ATE:",
    round(mean(fit_tc_bin$train_predictions_treat), 3), "\n")

## ----tc-adaptive--------------------------------------------------------------
# Adaptive (requires propensity scores)
ps_tc <- pnorm(0.3 * X_tc[, 1])   # simple propensity model for illustration

fit_tc_ada <- CausalHorseForest(
  y = y_tc,
  X_train_control = X_tc, X_train_treat = X_tc,
  treatment_indicator_train = W_tc,
  treatment_coding = "adaptive",
  propensity = ps_tc,
  number_of_trees = 5, N_post = 50, N_burn = 25,
  store_posterior_sample = TRUE, verbose = FALSE
)
cat("Adaptive — ATE:",
    round(mean(fit_tc_ada$train_predictions_treat), 3), "\n")

## ----tc-invariant-------------------------------------------------------------
# Invariant (parameter-expanded)
fit_tc_inv <- CausalHorseForest(
  y = y_tc,
  X_train_control = X_tc, X_train_treat = X_tc,
  treatment_indicator_train = W_tc,
  treatment_coding = "invariant",
  number_of_trees = 5, N_post = 50, N_burn = 25,
  store_posterior_sample = TRUE, verbose = FALSE
)
cat("Invariant — ATE:",
    round(mean(fit_tc_inv$train_predictions_treat), 3), "\n")

# Posterior draws of b0 and b1 are stored in the fitted object
cat("b0 posterior mean:", round(mean(fit_tc_inv$b0), 3), "\n")
cat("b1 posterior mean:", round(mean(fit_tc_inv$b1), 3), "\n")

## ----load-data----------------------------------------------------------------
library(ShrinkageTrees)
data("pdac")

# Dimensions and column overview
cat("Patients:", nrow(pdac), "\n")
cat("Columns :", ncol(pdac), "\n")
cat("Clinical columns:", paste(names(pdac)[1:13], collapse = ", "), "\n")
cat("Survival: time (months), censoring rate =",
    round(1 - mean(pdac$status), 2), "\n")
cat("Treatment: radiation =", sum(pdac$treatment),
    "/ control =", sum(1 - pdac$treatment), "\n")

## ----prepare-data-------------------------------------------------------------
time      <- pdac$time
status    <- pdac$status
treatment <- pdac$treatment
X         <- as.matrix(pdac[, !(names(pdac) %in% c("time", "status", "treatment"))])

## ----horsetrees-binary, eval=FALSE--------------------------------------------
# ps_fit <- HorseTrees(
#   y            = treatment,
#   X_train      = X,
#   outcome_type = "binary",
#   k            = 0.1,
#   N_post       = 5000,
#   N_burn       = 5000,
#   verbose      = FALSE
# )
# 
# propensity <- pnorm(ps_fit$train_predictions)

## ----horsetrees-binary-small--------------------------------------------------
set.seed(1)
n <- 80;  p <- 10
X_syn  <- matrix(rnorm(n * p), n, p)
W_syn  <- rbinom(n, 1, pnorm(0.8 * X_syn[, 1]))

ps_fit <- HorseTrees(
  y            = W_syn,
  X_train      = X_syn,
  outcome_type = "binary",
  number_of_trees = 5,
  k            = 0.5,
  N_post       = 50,
  N_burn       = 25,
  verbose      = FALSE
)

propensity_syn <- pnorm(ps_fit$train_predictions)
cat("Propensity scores — range: [",
    round(range(propensity_syn), 3), "]\n")

## ----horsetrees-survival------------------------------------------------------
set.seed(2)
log_T <- X_syn[, 1] + rnorm(n)
C     <- rexp(n, 0.5)
y_syn   <- pmin(exp(log_T), C)
d_syn   <- as.integer(exp(log_T) <= C)

ht_surv <- HorseTrees(
  y               = y_syn,
  status          = d_syn,
  X_train         = X_syn,
  outcome_type    = "right-censored",
  timescale       = "time",
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  store_posterior_sample = TRUE,
  verbose         = FALSE
)

cat("Posterior mean log-time (first 5 obs):",
    round(ht_surv$train_predictions[1:5], 3), "\n")
cat("Posterior sigma — mean:",
    round(mean(ht_surv$sigma), 3), "\n")

## ----horsetrees-ic------------------------------------------------------------
set.seed(20)

# Generate true event times
true_T <- rexp(n, rate = exp(-0.5 * X_syn[, 1]))

# Create interval-censored observations
left_syn  <- true_T * runif(n, 0.5, 1.0)
right_syn <- true_T * runif(n, 1.0, 1.5)

# Mark some as exact observations and some as right-censored
exact_idx <- sample(n, 25)
left_syn[exact_idx]  <- true_T[exact_idx]
right_syn[exact_idx] <- true_T[exact_idx]

rc_idx <- sample(setdiff(seq_len(n), exact_idx), 15)
right_syn[rc_idx] <- Inf

cat("Exact events:", sum(left_syn == right_syn), "\n")
cat("Interval-censored:", sum(left_syn < right_syn & is.finite(right_syn)), "\n")
cat("Right-censored:", sum(!is.finite(right_syn)), "\n")

ht_ic <- HorseTrees(
  left_time       = left_syn,
  right_time      = right_syn,
  X_train         = X_syn,
  outcome_type    = "interval-censored",
  timescale       = "time",
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  store_posterior_sample = TRUE,
  verbose         = FALSE
)

cat("Posterior mean log-time (first 5 obs):",
    round(ht_ic$train_predictions[1:5], 3), "\n")
cat("Posterior sigma — mean:",
    round(mean(ht_ic$sigma), 3), "\n")

## ----sbart-ic-----------------------------------------------------------------
set.seed(21)
fit_sbart_ic <- SurvivalBART(
  left_time       = left_syn,
  right_time      = right_syn,
  X_train         = X_syn,
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

cat("SurvivalBART (IC) class:", class(fit_sbart_ic), "\n")

## ----shrinkage-continuous-----------------------------------------------------
set.seed(3)
y_cont <- X_syn[, 1] + 0.5 * X_syn[, 2] + rnorm(n)

# Horseshoe prior (default for HorseTrees)
fit_hs <- ShrinkageTrees(
  y               = y_cont,
  X_train         = X_syn,
  outcome_type    = "continuous",
  prior_type      = "horseshoe",
  local_hp        = 0.1 / sqrt(5),
  global_hp       = 0.1 / sqrt(5),
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

# Forest-wide horseshoe (horseshoe_fw)
fit_fw <- ShrinkageTrees(
  y               = y_cont,
  X_train         = X_syn,
  outcome_type    = "continuous",
  prior_type      = "horseshoe_fw",
  local_hp        = 0.1 / sqrt(5),
  global_hp       = 0.1 / sqrt(5),
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

cat("Horseshoe   — train RMSE:",
    round(sqrt(mean((fit_hs$train_predictions - y_cont)^2)), 3), "\n")
cat("Horseshoe FW— train RMSE:",
    round(sqrt(mean((fit_fw$train_predictions - y_cont)^2)), 3), "\n")

## ----survival-bart-dart-------------------------------------------------------
set.seed(4)

# SurvivalBART: classical BART prior, AFT likelihood
fit_sbart <- SurvivalBART(
  time            = y_syn,
  status          = d_syn,
  X_train         = X_syn,
  number_of_trees = 5,
  k               = 2.0,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

# SurvivalDART: Dirichlet (DART) splitting prior
fit_sdart <- SurvivalDART(
  time            = y_syn,
  status          = d_syn,
  X_train         = X_syn,
  number_of_trees = 5,
  k               = 2.0,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

cat("SurvivalBART  class:", class(fit_sbart), "\n")
cat("SurvivalDART  class:", class(fit_sdart), "\n")

## ----hd-data------------------------------------------------------------------
set.seed(20)
n_hd <- 60;  p_hd <- 200
X_hd <- matrix(rnorm(n_hd * p_hd), n_hd, p_hd)

# True log-survival depends only on predictors 1, 2, and 3
log_T_hd <- 1.5 * X_hd[, 1] - 1.0 * X_hd[, 2] + 0.5 * X_hd[, 3] + rnorm(n_hd)
C_hd     <- rexp(n_hd, rate = 0.5)
y_hd     <- pmin(exp(log_T_hd), C_hd)
d_hd     <- as.integer(exp(log_T_hd) <= C_hd)

cat("n =", n_hd, "| p =", p_hd,
    "| active predictors = 3",
    "| censoring rate =", round(1 - mean(d_hd), 2), "\n")

## ----hd-horseshoe-------------------------------------------------------------
set.seed(21)
fit_hd_hs <- ShrinkageTrees(
  y               = y_hd,
  status          = d_hd,
  X_train         = X_hd,
  outcome_type    = "right-censored",
  prior_type      = "horseshoe",
  local_hp        = 0.1 / sqrt(10),
  global_hp       = 0.1 / sqrt(10),
  number_of_trees = 10,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

## ----hd-dart------------------------------------------------------------------
set.seed(22)
fit_hd_dart <- SurvivalDART(
  time            = y_hd,
  status          = d_hd,
  X_train         = X_hd,
  number_of_trees = 10,
  rho_dirichlet   = 3,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

## ----hd-compare---------------------------------------------------------------
rmse_hs   <- sqrt(mean((fit_hd_hs$train_predictions  - log_T_hd)^2))
rmse_dart <- sqrt(mean((fit_hd_dart$train_predictions - log_T_hd)^2))

cat(sprintf("%-18s  train RMSE (log-time): %.3f\n", "Horseshoe", rmse_hs))
cat(sprintf("%-18s  train RMSE (log-time): %.3f\n", "DART",      rmse_dart))

## ----hd-vi, eval=requireNamespace("ggplot2", quietly=TRUE)--------------------
plot(fit_hd_dart, type = "vi", n_vi = 10)

## ----causal-data--------------------------------------------------------------
set.seed(5)
tau_true <- 1.5 * (X_syn[, 1] > 0)    # heterogeneous treatment effect
y_causal <- X_syn[, 1] + W_syn * tau_true + rnorm(n, sd = 0.5)

## ----survbcf, eval=FALSE------------------------------------------------------
# # Full analysis (eval=FALSE — use larger MCMC settings in practice)
# fit_sbcf <- SurvivalBCF(
#   time       = time,
#   status     = status,
#   X_train    = X,
#   treatment  = treatment,
#   propensity = propensity,   # from HorseTrees above
#   N_post     = 5000,
#   N_burn     = 5000,
#   verbose    = FALSE
# )

## ----survbcf-small------------------------------------------------------------
set.seed(6)
fit_sbcf <- SurvivalBCF(
  time            = y_syn,
  status          = d_syn,
  X_train         = X_syn,
  treatment       = W_syn,
  number_of_trees_control = 5,
  number_of_trees_treat   = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)
cat("SurvivalBCF class:", class(fit_sbcf), "\n")
cat("ATE (posterior mean):",
    round(mean(fit_sbcf$train_predictions_treat), 3), "\n")

## ----survsbcf-small-----------------------------------------------------------
set.seed(7)
fit_ssbcf <- SurvivalShrinkageBCF(
  time            = y_syn,
  status          = d_syn,
  X_train         = X_syn,
  treatment       = W_syn,
  number_of_trees_control = 5,
  number_of_trees_treat   = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)
cat("SurvivalShrinkageBCF class:", class(fit_ssbcf), "\n")

## ----causal-horse-------------------------------------------------------------
set.seed(8)
fit_chf <- CausalHorseForest(
  y                         = y_causal,
  X_train_control           = X_syn,
  X_train_treat             = X_syn,
  treatment_indicator_train = W_syn,
  outcome_type              = "continuous",
  number_of_trees           = 5,
  N_post                    = 50,
  N_burn                    = 25,
  store_posterior_sample    = TRUE,
  verbose                   = FALSE
)

cat("CausalHorseForest class:", class(fit_chf), "\n")

# Posterior mean CATE
cate_mean <- fit_chf$train_predictions_treat
cat("CATE — posterior mean (first 5):",
    round(cate_mean[1:5], 3), "\n")

# Posterior ATE
ate_samples <- rowMeans(fit_chf$train_predictions_sample_treat)
cat("ATE posterior mean:",
    round(mean(ate_samples), 3),
    "  95% CI: [",
    round(quantile(ate_samples, 0.025), 3), ",",
    round(quantile(ate_samples, 0.975), 3), "]\n")

## ----causal-horse-test--------------------------------------------------------
set.seed(9)
X_test <- matrix(rnorm(20 * p), 20, p)
W_test <- rbinom(20, 1, 0.5)

fit_chf_test <- CausalHorseForest(
  y                         = y_causal,
  X_train_control           = X_syn,
  X_train_treat             = X_syn,
  treatment_indicator_train = W_syn,
  X_test_control            = X_test,
  X_test_treat              = X_test,
  treatment_indicator_test  = W_test,
  outcome_type              = "continuous",
  number_of_trees           = 5,
  N_post                    = 50,
  N_burn                    = 25,
  store_posterior_sample    = TRUE,
  verbose                   = FALSE
)

cat("Test CATE (first 5):",
    round(fit_chf_test$test_predictions_treat[1:5], 3), "\n")

## ----causal-shrinkage---------------------------------------------------------
set.seed(10)
lh <- 0.1 / sqrt(5)

fit_csf <- CausalShrinkageForest(
  y                         = y_causal,
  X_train_control           = X_syn,
  X_train_treat             = X_syn,
  treatment_indicator_train = W_syn,
  outcome_type              = "continuous",
  prior_type_control        = "horseshoe",
  prior_type_treat          = "horseshoe",
  local_hp_control          = lh,
  global_hp_control         = lh,
  local_hp_treat            = lh,
  global_hp_treat           = lh,
  number_of_trees_control   = 5,
  number_of_trees_treat     = 5,
  N_post                    = 50,
  N_burn                    = 25,
  store_posterior_sample    = TRUE,
  verbose                   = FALSE
)

cat("CausalShrinkageForest class:", class(fit_csf), "\n")
cat("Acceptance ratio (control):",
    round(fit_csf$acceptance_ratio_control, 3), "\n")
cat("Acceptance ratio (treat)  :",
    round(fit_csf$acceptance_ratio_treat, 3), "\n")

## ----causal-shrinkage-fw------------------------------------------------------
set.seed(11)
fit_fw2 <- CausalShrinkageForest(
  y                         = y_causal,
  X_train_control           = X_syn,
  X_train_treat             = X_syn,
  treatment_indicator_train = W_syn,
  outcome_type              = "continuous",
  prior_type_control        = "horseshoe_fw",
  prior_type_treat          = "horseshoe_fw",
  local_hp_control          = lh,
  global_hp_control         = lh,
  local_hp_treat            = lh,
  global_hp_treat           = lh,
  number_of_trees_control   = 5,
  number_of_trees_treat     = 5,
  N_post                    = 50,
  N_burn                    = 25,
  verbose                   = FALSE
)

cat("Forest-wide shrinkage (control, first 5 draws):\n")
print(round(fit_fw2$forestwide_shrinkage_control[1:5], 4))

## ----print--------------------------------------------------------------------
print(fit_chf)

## ----print-csf----------------------------------------------------------------
print(fit_csf)

## ----summary-shrinkage--------------------------------------------------------
smry <- summary(fit_hs)
print(smry)

## ----summary-causal-----------------------------------------------------------
smry_c <- summary(fit_chf)
print(smry_c)

# Access the ATE directly
cat("ATE mean  :", round(smry_c$treatment_effect$ate, 3), "\n")
cat("ATE 95% CI: [",
    round(smry_c$treatment_effect$ate_lower, 3), ",",
    round(smry_c$treatment_effect$ate_upper, 3), "]\n")

## ----summary-pate-mate--------------------------------------------------------
smry_pate <- summary(fit_chf, bayesian_bootstrap = TRUE)   # default
smry_mate <- summary(fit_chf, bayesian_bootstrap = FALSE)

## ----bb-ate-------------------------------------------------------------------
bb <- bayesian_bootstrap_ate(fit_chf)
cat("PATE:", round(bb$pate_mean, 3),
    " 95% CI: [", round(bb$pate_ci$lower, 3), ",",
                  round(bb$pate_ci$upper, 3), "]\n")
cat("MATE:", round(bb$mate_mean, 3),
    " 95% CI: [", round(bb$mate_ci$lower, 3), ",",
                  round(bb$mate_ci$upper, 3), "]\n")

## ----predict------------------------------------------------------------------
X_new  <- matrix(rnorm(10 * p), 10, p)

pred <- predict(fit_hs, newdata = X_new)
print(pred)

## ----predict-ci---------------------------------------------------------------
# Point estimates and 95% credible intervals
head(data.frame(
  mean  = round(pred$mean,  3),
  lower = round(pred$lower, 3),
  upper = round(pred$upper, 3)
))

## ----predict-causal-----------------------------------------------------------
X_new_ctrl  <- matrix(rnorm(10 * p), 10, p)
X_new_treat <- matrix(rnorm(10 * p), 10, p)

pred_c <- predict(fit_chf, newdata_control = X_new_ctrl,
                  newdata_treat = X_new_treat)
print(pred_c)

## ----predict-causal-detail----------------------------------------------------
# Extract individual components
head(data.frame(
  prognostic = round(pred_c$prognostic$mean, 3),
  cate       = round(pred_c$cate$mean, 3),
  total      = round(pred_c$total$mean, 3)
))

## ----plot-trace, eval=requireNamespace("ggplot2", quietly=TRUE)---------------
plot(fit_hs, type = "trace")

## ----plot-ate, eval=requireNamespace("ggplot2", quietly=TRUE)-----------------
plot(fit_chf, type = "ate")                         # PATE (default)
plot(fit_chf, type = "ate", bayesian_bootstrap = FALSE)  # MATE

## ----plot-cate, eval=requireNamespace("ggplot2", quietly=TRUE)----------------
plot(fit_chf, type = "cate")

## ----vi-fit-------------------------------------------------------------------
set.seed(12)
fit_dart <- ShrinkageTrees(
  y               = y_cont,
  X_train         = X_syn,
  outcome_type    = "continuous",
  prior_type      = "dirichlet",
  local_hp        = 0.1 / sqrt(5),
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

## ----plot-vi, eval=requireNamespace("ggplot2", quietly=TRUE)------------------
plot(fit_dart, type = "vi", n_vi = 10)

## ----surv-curve-pop, eval=requireNamespace("ggplot2", quietly=TRUE)-----------
# Population-averaged survival curve with 95% credible band
plot(ht_surv, type = "survival")

## ----surv-curve-km, eval=requireNamespace("ggplot2", quietly=TRUE) && requireNamespace("survival", quietly=TRUE)----
# Same curve with the Kaplan-Meier estimate overlaid for comparison
plot(ht_surv, type = "survival", km = TRUE)

## ----surv-curve-ind, eval=requireNamespace("ggplot2", quietly=TRUE)-----------
# Individual survival curves for observations 1, 20, 40, 60, and 80
plot(ht_surv, type = "survival", obs = c(1, 20, 40, 60, 80))

## ----surv-curve-single, eval=requireNamespace("ggplot2", quietly=TRUE)--------
# Single individual with a narrower 90% credible band
plot(ht_surv, type = "survival", obs = 1, level = 0.90)

## ----surv-pred-setup----------------------------------------------------------
# New observations for prediction
set.seed(99)
X_new <- matrix(rnorm(20 * p), ncol = p)
pred_surv <- predict(ht_surv, newdata = X_new)

## ----surv-pred-pop, eval=requireNamespace("ggplot2", quietly=TRUE)------------
# Population-averaged posterior predictive survival curve
plot(pred_surv, type = "survival")

## ----surv-pred-ind, eval=requireNamespace("ggplot2", quietly=TRUE)------------
# Individual posterior predictive curves for selected new observations
plot(pred_surv, type = "survival", obs = c(1, 5, 10))

## ----multi-chain--------------------------------------------------------------
set.seed(13)
fit_2chain <- ShrinkageTrees(
  y               = y_cont,
  X_train         = X_syn,
  outcome_type    = "continuous",
  prior_type      = "horseshoe",
  local_hp        = 0.1 / sqrt(5),
  global_hp       = 0.1 / sqrt(5),
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  n_chains        = 2,
  verbose         = FALSE
)

cat("n_chains stored  :", fit_2chain$mcmc$n_chains, "\n")
cat("Total sigma draws:", length(fit_2chain$sigma),
    " (2 chains x 50 draws)\n")
cat("Per-chain acceptance ratios:\n")
print(round(fit_2chain$chains$acceptance_ratios, 3))

## ----multi-chain-causal-------------------------------------------------------
set.seed(14)
fit_causal_2chain <- CausalShrinkageForest(
  y                         = y_causal,
  X_train_control           = X_syn,
  X_train_treat             = X_syn,
  treatment_indicator_train = W_syn,
  outcome_type              = "continuous",
  prior_type_control        = "horseshoe",
  prior_type_treat          = "horseshoe",
  local_hp_control          = lh,
  global_hp_control         = lh,
  local_hp_treat            = lh,
  global_hp_treat           = lh,
  number_of_trees_control   = 5,
  number_of_trees_treat     = 5,
  N_post                    = 50,
  N_burn                    = 25,
  n_chains                  = 2,
  verbose                   = FALSE
)

cat("Pooled sigma draws:", length(fit_causal_2chain$sigma), "\n")
cat("Per-chain acceptance ratios (control):\n")
print(round(fit_causal_2chain$chains$acceptance_ratios_control, 3))

## ----plot-trace-2chain, eval=requireNamespace("ggplot2", quietly=TRUE)--------
plot(fit_2chain, type = "trace")

## ----plot-density-2chain, eval=requireNamespace("ggplot2", quietly=TRUE)------
plot(fit_2chain, type = "density")

## ----coda-summary, eval=requireNamespace("coda", quietly=TRUE)----------------
# summary() includes convergence diagnostics when coda is available
summary(fit_2chain)

## ----coda-diagnostics, eval=requireNamespace("coda", quietly=TRUE)------------
library(coda)
mcmc_obj <- as.mcmc.list(fit_2chain)

# Gelman-Rubin R-hat (values near 1 indicate convergence)
coda::gelman.diag(mcmc_obj)

# Effective sample size
coda::effectiveSize(mcmc_obj)

# Geweke diagnostic (per chain)
coda::geweke.diag(mcmc_obj[[1]])

## ----pdac-ps, eval=FALSE------------------------------------------------------
# data("pdac")
# 
# time      <- pdac$time
# status    <- pdac$status
# treatment <- pdac$treatment
# X         <- as.matrix(pdac[, !(names(pdac) %in% c("time","status","treatment"))])
# 
# set.seed(2025)
# ps_fit <- HorseTrees(
#   y            = treatment,
#   X_train      = X,
#   outcome_type = "binary",
#   k            = 0.1,
#   N_post       = 5000,
#   N_burn       = 5000,
#   verbose      = FALSE
# )
# 
# propensity <- pnorm(ps_fit$train_predictions)

## ----pdac-ps-overlap, eval=FALSE----------------------------------------------
# # Overlap plot
# p0 <- propensity[treatment == 0]
# p1 <- propensity[treatment == 1]
# 
# hist(p0, breaks = 15, col = rgb(1, 0.5, 0, 0.5), xlim = range(propensity),
#      xlab = "Propensity score", main = "Propensity score overlap")
# hist(p1, breaks = 15, col = rgb(0, 0.5, 0, 0.5), add = TRUE)
# legend("topright", legend = c("Control", "Treated"),
#        fill = c(rgb(1,0.5,0,0.5), rgb(0,0.5,0,0.5)))

## ----pdac-causal, eval=FALSE--------------------------------------------------
# # Augment control matrix with propensity scores (BCF-style)
# X_control <- cbind(propensity, X)
# 
# # Log-transform and centre survival times
# log_time <- log(time) - mean(log(time))
# 
# set.seed(2025)
# fit_pdac <- CausalHorseForest(
#   y                         = log_time,
#   status                    = status,
#   X_train_control           = X_control,
#   X_train_treat             = X,
#   treatment_indicator_train = treatment,
#   outcome_type              = "right-censored",
#   timescale                 = "log",
#   number_of_trees           = 200,
#   N_post                    = 5000,
#   N_burn                    = 5000,
#   store_posterior_sample    = TRUE,
#   verbose                   = FALSE
# )

## ----pdac-ate, eval=FALSE-----------------------------------------------------
# # Print model summary
# print(fit_pdac)
# smry_pdac <- summary(fit_pdac)
# print(smry_pdac)
# 
# # ATE
# cat("ATE posterior mean:",
#     round(smry_pdac$treatment_effect$ate, 3), "\n")
# cat("95% CI: [",
#     round(smry_pdac$treatment_effect$ate_lower, 3), ",",
#     round(smry_pdac$treatment_effect$ate_upper, 3), "]\n")

## ----pdac-diag, eval=FALSE----------------------------------------------------
# # Sigma convergence
# plot(fit_pdac, type = "trace")
# 
# # Posterior ATE density
# plot(fit_pdac, type = "ate")
# 
# # CATE caterpillar plot
# plot(fit_pdac, type = "cate")

## Demo: Analysis of the TCGA PAAD dataset (pdac)
## ShrinkageTrees package

# Load package
library(ShrinkageTrees)

# Load the data
data("pdac")

# Retrieve the data
time <- pdac$time
status <- pdac$status
treatment <- pdac$treatment
covariates <- pdac[, !(colnames(pdac) %in% c("time", "status", "treatment"))]

# Estimate the propensity scores
propensity_fit <- ShrinkageTrees::HorseTrees(
  y = treatment,
  X_train = covariates,
  outcome_type = "binary",
  k = 0.1,
  N_post = 5000,
  N_burn = 5000
)

# Retrieve estimated propensity scores
propensity <- pnorm(propensity_fit$train_predictions)

# Define p0 and p1
p0 <- propensity[treatment == 0]
p1 <- propensity[treatment == 1]

# Define colors
col_treated <- rgb(0, 0.5, 0, 0.5)  # green
col_control <- rgb(1, 0.5, 0, 0.5)  # orange

# Plot histograms
hist(p0,
     breaks = 10,
     xlim = c(0.33, 0.63),
     col = col_control,
     xlab = "Propensity score",
     ylab = "Frequency",
     main = "Propensity score overlap",
)
hist(p1,
     breaks = 10,
     xlim = range(propensity),
     col = col_treated,
     add = TRUE
)
legend("topright",
       legend = c("Control", "Treated"),
       fill = c(col_control, col_treated)
)

# Placeholder for your analysis
cat("\nAnalysis section TBD\n")

# Adjust prognostic covariates
extended_covariates <- cbind(propensity, covariates)

# Define X_test
X_test <- covariates

# Transform the outcome
time <- log(time)
time <- time - mean(time)

# Fit a Causal Horseshoe Forest
fit <- CausalShrinkageForest(
  y = time,
  status = status,
  X_train_control = extended_covariates,
  X_train_treat = covariates,
  treatment_indicator_train = treatment,
  outcome_type = "right-censored",
  prior_type_control = "horseshoe",
  prior_type_treat = "horseshoe",
  local_hp_control = 0.05 / sqrt(200),
  local_hp_treat = 0.05 / sqrt(200),
  global_hp_control = 0.05 / sqrt(200),
  global_hp_treat = 0.05 / sqrt(200),
  store_posterior_sample = TRUE,
  timescale = "log",
  number_of_trees_treat = 200,
  N_post = 5000,
  N_burn = 5000
)

# Evaluate C-index if survival package is available
if (requireNamespace("survival", quietly = TRUE)) {
  predicted_survtime <- fit$train_predictions
  cindex_result <- survival::concordance(Surv(time = time, event = status) ~ predicted_survtime)
  c_index <- cindex_result$concordance
  cat("C-index:", c_index, "\n")
} else {
  cat("Package 'survival' not available. Skipping C-index computation.\n")
}

# Compute posterior ATE samples
ate_samples <- rowMeans(fit$train_predictions_sample_treat)
mean_ate <- mean(ate_samples)
ci_95 <- quantile(ate_samples, probs = c(0.025, 0.975))

cat("Posterior mean ATE:", round(mean_ate, 3), "\n")
cat("95% credible interval: [", round(ci_95[1], 3), ", ", round(ci_95[2], 3), "]\n", sep = "")

# Plot histogram of ATE
hist(ate_samples,
     breaks = 30,
     col = col_treated,
     freq = FALSE,
     border = "white",
     xlab = "Average Treatment Effect",
     ylab = NULL,
     main = "Posterior distribution of ATE"
)
abline(v = mean_ate, col = "darkorchid4", lwd = 2)
abline(v = ci_95, col = "darkorchid4", lty = 2, lwd = 2)
legend("topright",
       legend = c("Mean", "95% CI"),
       col = c("darkorchid4", "darkorchid4"),
       lty = c(1, 2),
       lwd = 2)

# Prepare CATE summary
posterior_matrix <- fit$train_predictions_sample_treat
posterior_mean <- colMeans(posterior_matrix)
posterior_ci <- apply(posterior_matrix, 2, quantile, probs = c(0.025, 0.975))

df_cate <- data.frame(
  mean = posterior_mean,
  lower = posterior_ci[1, ],
  upper = posterior_ci[2, ]
)

# Sort by mean CATE
df_cate_sorted <- df_cate[order(df_cate$mean), ]
n <- nrow(df_cate_sorted)

# Plot CATE estimates
plot(
  x = df_cate_sorted$mean,
  y = 1:n,
  type = "n",
  xlab = "CATE per patient (95% credible interval)",
  ylab = "Patient index (sorted)",
  main = "Posterior CATE estimates",
  xlim = range(df_cate_sorted$lower, df_cate_sorted$upper)
)
segments(
  x0 = df_cate_sorted$lower,
  x1 = df_cate_sorted$upper,
  y0 = 1:n,
  y1 = 1:n,
  col = col_treated
)
lines(df_cate_sorted$mean, 1:n, col = "darkorchid4", lwd = 2)
abline(v = 0, col = "black", lwd = 2)

# Plot sigma trace
plot(fit$sigma,
     type = "l",
     xlab = "Iteration",
     ylab = expression(sigma),
     main = "Traceplot of Sigma",
     col = rgb(0, 0.5, 0, 0.5)
)

#' Horseshoe Regression Trees (HorseTrees)
#'
#' Fits a Bayesian Horseshoe Trees model with a single learner.
#' Implements regularization on the step heights using a global-local Horseshoe
#' prior, controlled via the parameter \code{k}. Supports continuous, binary,
#' right-censored, and interval-censored (survival) outcomes.
#'
#' For continuous outcomes, the model centers and optionally standardizes the
#' outcome using a prior guess of the standard deviation.
#' For binary outcomes, the function uses a probit link formulation.
#' For right-censored outcomes (survival data), the function can handle
#' follow-up times either on the original time scale or log-transformed.
#' For interval-censored outcomes, provide \code{left_time} and
#' \code{right_time} instead of \code{y} and \code{status}; the event
#' indicators are derived internally following the
#' \code{survival::Surv(type = "interval2")} convention.
#' Generalized implementation with multiple prior possibilities is given by
#' \code{\link{ShrinkageTrees}}.
#' 
#' @examples
#' # Minimal example: continuous outcome
#' n <- 25
#' p <- 5
#' X <- matrix(rnorm(n * p), ncol = p)
#' y <- X[, 1] + rnorm(n)
#' fit1 <- HorseTrees(y = y, X_train = X, outcome_type = "continuous", 
#'                    number_of_trees = 5, N_post = 75, N_burn = 25, 
#'                    verbose = FALSE)
#'
#' # Minimal example: binary outcome
#' X <- matrix(rnorm(n * p), ncol = p)
#' y <- ifelse(X[, 1] + rnorm(n) > 0, 1, 0)
#' fit2 <- HorseTrees(y = y, X_train = X, outcome_type = "binary", 
#'                    number_of_trees = 5, N_post = 75, N_burn = 25, 
#'                    verbose = FALSE)
#'
#' # Minimal example: right-censored outcome
#' X <- matrix(rnorm(n * p), ncol = p)
#' time <- rexp(n, rate = 0.1)
#' status <- rbinom(n, 1, 0.7)
#' fit3 <- HorseTrees(y = time, status = status, X_train = X,
#'                    outcome_type = "right-censored", number_of_trees = 5,
#'                    N_post = 75, N_burn = 25, verbose = FALSE)
#'
#' # Minimal example: interval-censored outcome
#' X <- matrix(rnorm(n * p), ncol = p)
#' true_t <- rexp(n, rate = 0.1)
#' left_t  <- true_t * runif(n, 0.5, 1)
#' right_t <- true_t * runif(n, 1, 1.5)
#' # Mark some as exact, some as right-censored
#' exact <- sample(n, 8); left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
#' rc <- sample(setdiff(seq_len(n), exact), 5); right_t[rc] <- Inf
#' fit4 <- HorseTrees(left_time = left_t, right_time = right_t, X_train = X,
#'                    outcome_type = "interval-censored", number_of_trees = 5,
#'                    N_post = 75, N_burn = 25, verbose = FALSE)
#'
#' # Larger continuous example (not run automatically)
#' \donttest{
#' n <- 100
#' p <- 100
#' X <- matrix(rnorm(100 * p), ncol = p)
#' X_test <- matrix(rnorm(50 * p), ncol = p)
#' y <- X[, 1] + X[, 2] - X[, 3] + rnorm(100, sd = 0.5)
#'
#' fit5 <- HorseTrees(y = y,
#'                    X_train = X,
#'                    X_test = X_test,
#'                    outcome_type = "continuous",
#'                    number_of_trees = 200,
#'                    N_post = 2500,
#'                    N_burn = 2500,
#'                    store_posterior_sample = TRUE,
#'                    verbose = TRUE)
#'
#' plot(fit4$sigma, type = "l", ylab = expression(sigma),
#'      xlab = "Iteration", main = "Sigma traceplot")
#'
#' hist(fit4$train_predictions_sample[, 1],
#'      main = "Posterior distribution of prediction outcome individual 1",
#'      xlab = "Prediction", breaks = 20)
#' }
#'                        
#' @param y Outcome vector. Numeric. Can represent continuous outcomes, binary
#' outcomes (0/1), or follow-up times for survival data. Set to \code{NULL}
#' (default) when using \code{outcome_type = "interval-censored"}, as values
#' are derived from \code{left_time} and \code{right_time}.
#' @param status Optional censoring indicator vector (1 = event occurred,
#' 0 = censored). Required if \code{outcome_type = "right-censored"}.
#' For interval-censored outcomes, this is derived automatically from
#' \code{left_time} and \code{right_time}.
#' @param X_train Covariate matrix for training. Each row corresponds to an
#' observation, and each column to a covariate.
#' @param X_test Optional covariate matrix for test data. If NULL, defaults to
#' the mean of the training covariates.
#' @param left_time Optional numeric vector of left (lower) time boundaries.
#' Required when \code{outcome_type = "interval-censored"}. Exact events
#' have \code{left_time == right_time}; right-censored observations have
#' \code{right_time = Inf}; interval-censored observations have finite
#' \code{left_time < right_time}.
#' @param right_time Optional numeric vector of right (upper) time boundaries.
#' Required when \code{outcome_type = "interval-censored"}. Use \code{Inf}
#' for right-censored observations.
#' @param outcome_type Type of outcome. One of \code{"continuous"},
#' \code{"binary"}, \code{"right-censored"}, or \code{"interval-censored"}.
#' @param timescale Indicates the scale of follow-up times. Options are 
#' `"time"` (nonnegative follow-up times, will be log-transformed internally) 
#'   or `"log"` (already log-transformed). Only used when 
#'   `outcome_type = "right-censored"` or `"interval-censored"`.
#' @param number_of_trees Number of trees in the ensemble. Default is 200.
#' @param k Horseshoe scale hyperparameter (default 0.1). This parameter 
#' controls the overall level of shrinkage by setting the scale for both 
#' global and local shrinkage components. The local and global hyperparameters 
#' are parameterized as 
#' \eqn{\alpha = \frac{k}{\sqrt{\mathrm{number\_of\_trees}}}} 
#' to ensure adaptive regularization across trees.
#' @param power Power parameter for tree structure prior. Default is 2.0.
#' @param base Base parameter for tree structure prior. Default is 0.95.
#' @param p_grow Probability of proposing a grow move. Default is 0.4.
#' @param p_prune Probability of proposing a prune move. Default is 0.4.
#' @param nu Degrees of freedom for the error distribution prior. Default is 3.
#' @param q Quantile hyperparameter for the error variance prior. 
#' Default is 0.90.
#' @param sigma Optional known value for error standard deviation. If NULL, 
#' estimated from data.
#' @param N_post Number of posterior samples to store. Default is 1000.
#' @param N_burn Number of burn-in iterations. Default is 1000.
#' @param delayed_proposal Number of delayed iterations before proposal. Only 
#' for reversible updates. Default is 5.
#' @param store_posterior_sample Logical; whether to store posterior samples for
#'  each iteration. Default is TRUE.
#' @param n_chains Number of independent MCMC chains to run. Default is
#'   \code{1} (standard single-chain behaviour). When \code{n_chains > 1} the
#'   chains are run in parallel via \code{parallel::mclapply} and their
#'   posterior samples are pooled into a single \code{ShrinkageTrees} object,
#'   so all existing \code{print}, \code{summary}, and \code{predict} methods
#'   work without modification. On Windows, \code{mclapply} falls back to
#'   sequential execution.
#' @param verbose Logical; whether to print verbose output. Default is TRUE.
#'
#' @return An S3 object of class \code{"ShrinkageTrees"} with the following elements:
#' \describe{
#'   \item{train_predictions}{Vector of posterior mean predictions on the 
#'   training data.}
#'   \item{test_predictions}{Vector of posterior mean predictions on the test 
#'   data (or on mean covariate vector if `X_test` not provided).}
#'   \item{sigma}{Vector of posterior samples of the error variance.}
#'   \item{acceptance_ratio}{Average acceptance ratio across trees during 
#'   sampling.}
#'   \item{train_predictions_sample}{Matrix of posterior samples of training 
#'   predictions (iterations in rows, observations in columns). Present only 
#'   if `store_posterior_sample = TRUE`.}
#'   \item{test_predictions_sample}{Matrix of posterior samples of test 
#'   predictions. Present only if `store_posterior_sample = TRUE`.}
#'   \item{train_probabilities}{Vector of posterior mean probabilities on the 
#'   training data (only for `outcome_type = "binary"`).}
#'   \item{test_probabilities}{Vector of posterior mean probabilities on the 
#'   test data (only for `outcome_type = "binary"`).}
#'   \item{train_probabilities_sample}{Matrix of posterior samples of training 
#'   probabilities (only for `outcome_type = "binary"` and if 
#'   `store_posterior_sample = TRUE`).}
#'   \item{test_probabilities_sample}{Matrix of posterior samples of test 
#'   probabilities (only for `outcome_type = "binary"` and if 
#'   `store_posterior_sample = TRUE`).}
#' }
#' 
#' @seealso
#' Model family: \code{\link{ShrinkageTrees}} (flexible prior choice),
#' \code{\link{CausalHorseForest}} (causal inference),
#' \code{\link{CausalShrinkageForest}} (causal, flexible prior).
#'
#' Survival wrappers: \code{\link{SurvivalBART}}, \code{\link{SurvivalDART}}.
#'
#' S3 methods: \code{\link{print.ShrinkageTrees}},
#' \code{\link{summary.ShrinkageTrees}},
#' \code{\link{predict.ShrinkageTrees}},
#' \code{\link{plot.ShrinkageTrees}}.
#' 
#' @importFrom Rcpp evalCpp
#' @useDynLib ShrinkageTrees, .registration = TRUE
#' @importFrom stats sd qchisq qnorm pnorm
#' @importFrom parallel mclapply detectCores
#' @export

HorseTrees <- function(y = NULL,
                       status = NULL,
                       X_train,
                       X_test = NULL,
                       left_time = NULL,
                       right_time = NULL,
                       outcome_type = "continuous",
                       timescale = "time",
                       number_of_trees = 200,
                       k = 0.1,
                       power = 2.0,
                       base = 0.95,
                       p_grow = 0.4,
                       p_prune = 0.4,
                       nu = 3,
                       q = 0.90,
                       sigma = NULL,
                       N_post = 1000,
                       N_burn = 1000,
                       delayed_proposal = 5,
                       store_posterior_sample = TRUE,
                       n_chains = 1,
                       verbose = TRUE) {

  # ── Multi-chain dispatch ──────────────────────────────────────────────────
  if (n_chains > 1) {
    mc <- match.call()

    chain_args <- list(
      y                      = y,
      status                 = status,
      X_train                = X_train,
      X_test                 = X_test,
      left_time              = left_time,
      right_time             = right_time,
      outcome_type           = outcome_type,
      timescale              = timescale,
      number_of_trees        = number_of_trees,
      k                      = k,
      power                  = power,
      base                   = base,
      p_grow                 = p_grow,
      p_prune                = p_prune,
      nu                     = nu,
      q                      = q,
      sigma                  = sigma,
      N_post                 = N_post,
      N_burn                 = N_burn,
      delayed_proposal       = delayed_proposal,
      store_posterior_sample = store_posterior_sample,
      n_chains               = 1,
      verbose                = FALSE
    )

    n_cores <- .resolve_cores(n_chains)
    if (verbose)
      message("Running ", n_chains, " chains (", n_cores, " cores) ...")

    chains <- parallel::mclapply(
      seq_len(n_chains),
      function(i) do.call(HorseTrees, chain_args),
      mc.cores = n_cores
    )

    combined      <- .combine_chains(chains)
    combined$call <- mc
    return(combined)
  }

  # Check outcome_type value
  allowed_types <- c("continuous", "binary", "right-censored", "interval-censored")
  if (!outcome_type %in% allowed_types) {
    stop("Invalid outcome_type. Please choose 'continuous', 'binary',
         'right-censored', or 'interval-censored'.")
  }

  # Check that y is provided for non-interval-censored types
  if (is.null(y) && outcome_type != "interval-censored") {
    stop("'y' is required when outcome_type is not 'interval-censored'.")
  }

  # Check interval-censored arguments
  if (outcome_type == "interval-censored") {
    if (is.null(left_time) || is.null(right_time))
      stop("outcome_type = 'interval-censored' requires 'left_time' and 'right_time'.")
    if (length(left_time) != length(right_time))
      stop("'left_time' and 'right_time' must have the same length.")
    if (any(left_time > right_time))
      stop("All 'left_time' values must be <= corresponding 'right_time' values.")
    if (timescale == "time" && any(left_time <= 0))
      stop("left_time contains non-positive values, but timescale = 'time' requires strictly positive times.")
  }

  # Check consistency with status argument
  if (outcome_type == "right-censored" && is.null(status)) {
    stop("You specified outcome_type = 'right-censored', but did not provide a
         'status' vector.")
  }
  if (!is.null(status) && !is.null(y) && length(status) != length(y)) {
    stop("The length of 'status' must match the length of 'y'.")
  }

  if (!outcome_type %in% c("right-censored", "interval-censored") && !is.null(status)) {
    warning("You provided a 'status' vector, but outcome_type is not
            'right-censored' or 'interval-censored'. The 'status' vector will be ignored.")
  }

  # Check binary data consistency
  if (outcome_type == "binary" && !all(y %in% c(0, 1))) {
    stop("For outcome_type = 'binary', y must contain only 0 and 1.")
  }
  if (outcome_type != "binary" && !is.null(y) && all(y %in% c(0, 1))) {
    warning("The outcome y contains only 0 and 1, but outcome_type is not set to
            'binary'. Consider setting outcome_type = 'binary'.")
  }

  # Check survival data and timescale
  if (outcome_type == "right-censored" && timescale == "time" && any(y < 0)) {
    stop("Outcome contains negative values, but timescale = 'time' for survival
         data requires non-negative times.")
  }

  # Retrieve dimensions of training data
  n_train <- nrow(X_train)
  p_features <- ncol(X_train)

  # Check if dimensions covariates match with outcome
  if (outcome_type == "interval-censored") {
    if (length(left_time) != n_train)
      stop("The length of 'left_time' must match the number of rows in X_train.")
  } else {
    if (length(y) != n_train)
      stop("The length of outcome vector y must match the number of rows in X_train.")
  }
  
  # Check if dimensions match with test data
  if (!is.null(X_test)) {
    n_test <- nrow(X_test)
    if (ncol(X_test) != p_features) {
      stop("The number of covariates in X_test must match X_train.")
    }
    X_test <- as.numeric(t(X_test))
  } else {
    n_test <- 1
    X_test <- as.numeric(colMeans(X_train))
  }
    
  # Force data types to be numeric and plain arrays
  N_post <- as.integer(N_post)[1]
  N_burn <- as.integer(N_burn)[1]
  power <- as.numeric(power)[1]
  base <- as.numeric(base)[1]
  p_grow <- as.numeric(p_grow)[1]
  p_prune <- as.numeric(p_prune)[1]
  X_train_mat <- X_train                 # preserve matrix for predict() / vi plots
  X_train <- as.numeric(t(X_train))
  

  if (outcome_type == "right-censored") {

    # Convert y to numeric for C++ compatibility
    y <- as.numeric(y)

    # Log-transform survival times if timescale = "time"
    if (timescale == "time") {
      y <- log(y)
    }

    # Save y before centering/standardizing (for predict())
    y_train_raw <- y

    # Obtain estimated mean and standard deviation using censored_info()
    cens_inf <- censored_info(y, status)

    # Center the data using estimated mean
    y_mean <- cens_inf$mu
    y <- y - y_mean

    # Determine sigma (timescale parameter) and whether it is known
    if (is.null(sigma)) {
      sigma_hat <- cens_inf$sd
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma
      sigma_known <- TRUE
    }

    # Standardize the centered data
    y <- y / sigma_hat

    # Set survival flag
    survival <- TRUE

    # Compute lambda parameter for error distribution prior
    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu

    # Fit a HorseTrees model
    fit <- HorseTrees_cpp(
      nSEXP = n_train,
      pSEXP = p_features,
      n_testSEXP = n_test,
      X_trainSEXP = X_train,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = numeric(n_train),
      observed_right_timeSEXP = y + 0,
      interval_censoring_indicatorSEXP = numeric(n_train),
      X_testSEXP = X_test,
      number_of_treesSEXP = number_of_trees,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      powerSEXP = power,
      baseSEXP = base,
      p_growSEXP = p_grow,
      p_pruneSEXP = p_prune,
      nuSEXP = nu,
      lambdaSEXP = lambda,
      dirichlet_boolSEXP = FALSE,
      a_dirichletSEXP = 1,
      b_dirichletSEXP = 1,
      rho_dirichletSEXP = 1,
      sigmaSEXP = sigma_hat,
      sigma_knownSEXP = sigma_known,
      omegaSEXP = 1,
      param1SEXP = k / sqrt(number_of_trees),
      param2SEXP = k / sqrt(number_of_trees),
      prior_typeSEXP = "horseshoe",
      reversibleSEXP = TRUE,
      store_parametersSEXP = FALSE,
      store_posterior_sampleSEXP = store_posterior_sample,
      verboseSEXP = verbose
    )
    
    if (timescale == "time") {
      fit$train_predictions <- exp(fit$train_predictions * sigma_hat + y_mean)
      fit$test_predictions <- exp(fit$test_predictions * sigma_hat + y_mean)
      if (store_posterior_sample) {
        fit$train_predictions_sample <- exp(fit$train_predictions_sample * sigma_hat + y_mean)
        fit$test_predictions_sample <- exp(fit$test_predictions_sample * sigma_hat + y_mean)
      }
    } else { # If timescale is "log"
      fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
      fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
      if (store_posterior_sample) { 
        fit$train_predictions_sample <- fit$train_predictions_sample * sigma_hat + y_mean
        fit$test_predictions_sample <- fit$test_predictions_sample * sigma_hat + y_mean
      }
    }

  # If interval-censored
  } else if (outcome_type == "interval-censored") {

    left_time <- as.numeric(left_time)
    right_time <- as.numeric(right_time)

    # Derive status: 1 if exact (left == right), 0 otherwise
    status <- as.integer(left_time == right_time)

    # Derive interval-censoring indicator: 1 if left < right and right is finite
    ic_indicator <- as.integer(left_time < right_time & is.finite(right_time))

    # Derive initial y values for MCMC
    y <- ifelse(status == 1, left_time,
                ifelse(ic_indicator == 1, (left_time + right_time) / 2,
                       left_time))

    # For right-censored obs (right_time == Inf): C++ expects finite boundary
    right_time[!is.finite(right_time)] <- left_time[!is.finite(right_time)]

    # Log-transform if timescale = "time"
    if (timescale == "time") {
      y <- log(y)
      left_time <- log(left_time)
      right_time <- log(right_time)
    }

    # Save before centering/standardizing (for predict())
    y_train_raw <- y
    left_time_raw <- left_time
    right_time_raw <- right_time

    # Estimate mu and sd using censored_info with interval-censored extension
    cens_inf <- censored_info(y, status, left_time = left_time,
                              right_time = right_time, ic_indicator = ic_indicator)

    # Center using estimated mean
    y_mean <- cens_inf$mu
    y <- y - y_mean
    left_time <- left_time - y_mean
    right_time <- right_time - y_mean

    # Determine sigma
    if (is.null(sigma)) {
      sigma_hat <- cens_inf$sd
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma
      sigma_known <- TRUE
    }

    # Standardize
    y <- y / sigma_hat
    left_time <- left_time / sigma_hat
    right_time <- right_time / sigma_hat

    survival <- TRUE

    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu

    fit <- HorseTrees_cpp(
      nSEXP = n_train,
      pSEXP = p_features,
      n_testSEXP = n_test,
      X_trainSEXP = X_train,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = left_time,
      observed_right_timeSEXP = right_time,
      interval_censoring_indicatorSEXP = ic_indicator,
      X_testSEXP = X_test,
      number_of_treesSEXP = number_of_trees,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      powerSEXP = power,
      baseSEXP = base,
      p_growSEXP = p_grow,
      p_pruneSEXP = p_prune,
      nuSEXP = nu,
      lambdaSEXP = lambda,
      dirichlet_boolSEXP = FALSE,
      a_dirichletSEXP = 1,
      b_dirichletSEXP = 1,
      rho_dirichletSEXP = 1,
      sigmaSEXP = sigma_hat,
      sigma_knownSEXP = sigma_known,
      omegaSEXP = 1,
      param1SEXP = k / sqrt(number_of_trees),
      param2SEXP = k / sqrt(number_of_trees),
      prior_typeSEXP = "horseshoe",
      reversibleSEXP = TRUE,
      store_parametersSEXP = FALSE,
      store_posterior_sampleSEXP = store_posterior_sample,
      verboseSEXP = verbose
    )

    # Back-transform predictions (identical to right-censored)
    if (timescale == "time") {
      fit$train_predictions <- exp(fit$train_predictions * sigma_hat + y_mean)
      fit$test_predictions <- exp(fit$test_predictions * sigma_hat + y_mean)
      if (store_posterior_sample) {
        fit$train_predictions_sample <- exp(fit$train_predictions_sample * sigma_hat + y_mean)
        fit$test_predictions_sample <- exp(fit$test_predictions_sample * sigma_hat + y_mean)
      }
    } else {
      fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
      fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
      if (store_posterior_sample) {
        fit$train_predictions_sample <- fit$train_predictions_sample * sigma_hat + y_mean
        fit$test_predictions_sample <- fit$test_predictions_sample * sigma_hat + y_mean
      }
    }

  # If binary
  } else if (outcome_type == "binary") {
    y <- as.numeric(y)
    y_train_raw      <- y
    latent_threshold <- qnorm(mean(y))
    sigma_hat        <- 1
    sigma_known      <- TRUE
    y_mean           <- 0
    lambda           <- 0   # unused for probit
    fit <- probitHorseTrees_cpp(
      nSEXP = n_train,
      pSEXP = p_features,
      n_testSEXP = n_test,
      X_trainSEXP = X_train,
      ySEXP = y,
      X_testSEXP = X_test,
      number_of_treesSEXP = number_of_trees,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      powerSEXP = power,
      baseSEXP = base,
      p_growSEXP = p_grow,
      p_pruneSEXP = p_prune,
      omegaSEXP = 1,
      latent_thresholdSEXP = latent_threshold,
      param1SEXP = k / sqrt(number_of_trees),
      param2SEXP = k / sqrt(number_of_trees),
      prior_typeSEXP = "horseshoe",
      reversibleSEXP = TRUE,
      store_posterior_sampleSEXP = store_posterior_sample,
      verboseSEXP = verbose
    )

    # Add the pnorm transform
    fit$train_probabilities <- pnorm(fit$train_predictions)
    fit$test_probabilities<- pnorm(fit$test_predictions)
    if (store_posterior_sample) {
      fit$train_probabilities_sample <- pnorm(fit$train_predictions_sample)
      fit$test_probabilities_sample <- pnorm(fit$test_predictions_sample)
    }

  # Otherwise, continuous
  } else {

    # Force outcome to plain numeric vector
    y <- as.numeric(y)

    # Save y before centering/standardizing (for predict())
    y_train_raw <- y

    # Create dummy status vector (not used for continuous)
    status <- rep(1, n_train)
    survival <- FALSE

    # Determine prior guess of sigma 
    if (is.null(sigma)) {
      sigma_hat <- sd(y)      # Estimate sigma from data
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma      # Use provided sigma
      sigma_known <- TRUE
    }

    # Compute hyperparameters of error variance prior
    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu

    # Compute mean and standardize  the outcome
    y_mean <- mean(y)
    y <- y - y_mean
    y <- y / sigma_hat  # Standardize y

    fit <- HorseTrees_cpp(
      nSEXP = n_train,
      pSEXP = p_features,
      n_testSEXP = n_test,
      X_trainSEXP = X_train,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = numeric(n_train),
      observed_right_timeSEXP = y + 0,
      interval_censoring_indicatorSEXP = numeric(n_train),
      X_testSEXP = X_test,
      number_of_treesSEXP = number_of_trees,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      powerSEXP = power,
      baseSEXP = base,
      p_growSEXP = p_grow,
      p_pruneSEXP = p_prune,
      nuSEXP = nu,
      lambdaSEXP = lambda,
      dirichlet_boolSEXP = FALSE,
      a_dirichletSEXP = 1,
      b_dirichletSEXP = 1,
      rho_dirichletSEXP = 1,
      sigmaSEXP = sigma_hat,
      sigma_knownSEXP = sigma_known,
      omegaSEXP = 1,
      param1SEXP = k / sqrt(number_of_trees),
      param2SEXP = k / sqrt(number_of_trees),
      prior_typeSEXP = "horseshoe",
      reversibleSEXP = TRUE,
      store_parametersSEXP = FALSE,
      store_posterior_sampleSEXP = store_posterior_sample,
      verboseSEXP = verbose
    )
    
    fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
    fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
    if (store_posterior_sample) {
      fit$train_predictions_sample <- fit$train_predictions_sample * sigma_hat + y_mean
      fit$test_predictions_sample <- fit$test_predictions_sample * sigma_hat + y_mean
    }
  }
  
  # remove burn-in of sigma
  if (!sigma_known) {
      fit$sigma <- fit$sigma[-seq_len(N_burn)]
    } 

    obj <- NewShrinkageTrees(
    fit              = fit,
    call             = match.call(),
    outcome_type     = outcome_type,
    timescale        = timescale,
    prior_type_user  = "horseshoe",
    prior_type_cpp   = "horseshoe",
    n_train          = n_train,
    p_features       = p_features,
    n_test           = n_test,
    test_provided    = !is.null(X_test),
    number_of_trees  = number_of_trees,
    N_post           = N_post,
    N_burn           = N_burn,
    store_posterior_sample = store_posterior_sample,
    sigma_hat        = sigma_hat,
    sigma_known      = sigma_known,
    y_mean           = y_mean,
    y_train          = y_train_raw,
    X_train          = X_train_mat,
    status_train     = if (outcome_type %in% c("right-censored", "interval-censored")) status else NULL,
    left_time_train  = if (outcome_type == "interval-censored") left_time_raw else NULL,
    right_time_train = if (outcome_type == "interval-censored") right_time_raw else NULL,
    ic_indicator_train = if (outcome_type == "interval-censored") ic_indicator else NULL,
    power            = power,
    base             = base,
    p_grow           = p_grow,
    p_prune          = p_prune,
    delayed_proposal = delayed_proposal,
    reversible       = TRUE,
    param1           = k / sqrt(number_of_trees),
    param2           = k / sqrt(number_of_trees),
    omega            = 1,
    a_dirichlet      = 1,
    b_dirichlet      = 1,
    rho_dirichlet    = 1,
    nu               = nu,
    lambda           = lambda,
    latent_threshold = if (outcome_type == "binary") latent_threshold else NULL
  )

  return(obj)
}

# Tests for plot.ShrinkageTrees and plot.CausalShrinkageForest.
# All tests are skipped when ggplot2 is not installed.

# -- Helper -------------------------------------------------------------------

.skip_plots <- function() {
  skip_if_not_installed("ggplot2")
}

# Helper to build a tiny CausalShrinkageForest fit (horseshoe priors).
.causal_fit <- function(n_chains = 1L, store = TRUE) {
  n <- 30L; p <- 3L
  set.seed(1)
  X         <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y         <- X[, 1] + (0.5 - treatment) * 2 + rnorm(n)
  lh        <- 0.1 / sqrt(5)
  CausalShrinkageForest(
    y = y, X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat   = lh, global_hp_treat   = lh,
    N_post = 10, N_burn = 5,
    store_posterior_sample = store,
    n_chains = n_chains,
    verbose = FALSE
  )
}


# -- plot.ShrinkageTrees: trace -----------------------------------------------

test_that("plot.ShrinkageTrees type='trace' returns a ggplot", {
  .skip_plots()
  n <- 30L; p <- 3L
  set.seed(1)
  X <- matrix(runif(n * p), ncol = p)
  y <- X[, 1] + rnorm(n)
  fit <- HorseTrees(y, X_train = X, outcome_type = "continuous",
                    number_of_trees = 5, N_post = 10, N_burn = 5,
                    verbose = FALSE)
  out <- expect_no_error(plot(fit, type = "trace"))
  expect_s3_class(out, "gg")
})


# -- plot.ShrinkageTrees: density ---------------------------------------------

test_that("plot.ShrinkageTrees type='density' works with n_chains > 1", {
  .skip_plots()
  n <- 30L; p <- 3L
  set.seed(1)
  X <- matrix(runif(n * p), ncol = p)
  y <- X[, 1] + rnorm(n)
  fit <- HorseTrees(y, X_train = X, outcome_type = "continuous",
                    number_of_trees = 5, N_post = 10, N_burn = 5,
                    n_chains = 2, verbose = FALSE)
  out <- expect_no_error(plot(fit, type = "density"))
  expect_s3_class(out, "gg")
})

test_that("plot.ShrinkageTrees type='density' works with n_chains == 1", {
  .skip_plots()
  n <- 30L; p <- 3L
  set.seed(1)
  X <- matrix(runif(n * p), ncol = p)
  y <- X[, 1] + rnorm(n)
  fit <- HorseTrees(y, X_train = X, outcome_type = "continuous",
                    number_of_trees = 5, N_post = 10, N_burn = 5,
                    verbose = FALSE)
  out <- expect_no_error(plot(fit, type = "density"))
  expect_s3_class(out, "gg")
})


# -- plot.ShrinkageTrees: vi --------------------------------------------------

test_that("plot.ShrinkageTrees type='vi' works for Dirichlet model", {
  .skip_plots()
  n <- 30L; p <- 3L
  set.seed(1)
  X <- matrix(runif(n * p), ncol = p)
  y <- X[, 1] + rnorm(n)
  fit <- ShrinkageTrees(y, X_train = X, outcome_type = "continuous",
                        number_of_trees = 5, prior_type = "dirichlet",
                        local_hp = 1 / sqrt(5),
                        N_post = 10, N_burn = 5, verbose = FALSE)
  out <- expect_no_error(plot(fit, type = "vi", n_vi = 3))
  expect_s3_class(out, "gg")
})

test_that("plot.ShrinkageTrees type='vi' errors for non-Dirichlet model", {
  .skip_plots()
  n <- 30L; p <- 3L
  set.seed(1)
  X <- matrix(runif(n * p), ncol = p)
  y <- X[, 1] + rnorm(n)
  fit <- HorseTrees(y, X_train = X, outcome_type = "continuous",
                    number_of_trees = 5, N_post = 10, N_burn = 5,
                    verbose = FALSE)
  expect_error(plot(fit, type = "vi"), "split_probs is NULL")
})


# -- plot.CausalShrinkageForest: trace ----------------------------------------

test_that("plot.CausalShrinkageForest type='trace' returns a ggplot", {
  .skip_plots()
  fit <- .causal_fit()
  out <- expect_no_error(plot(fit, type = "trace"))
  expect_s3_class(out, "gg")
})


# -- plot.CausalShrinkageForest: density --------------------------------------

test_that("plot.CausalShrinkageForest type='density' works with n_chains > 1", {
  .skip_plots()
  fit <- .causal_fit(n_chains = 2L)
  out <- expect_no_error(plot(fit, type = "density"))
  expect_s3_class(out, "gg")
})

test_that("plot.CausalShrinkageForest type='density' works with n_chains == 1", {
  .skip_plots()
  fit <- .causal_fit()
  out <- expect_no_error(plot(fit, type = "density"))
  expect_s3_class(out, "gg")
})


# -- plot.CausalShrinkageForest: ate ------------------------------------------

test_that("plot.CausalShrinkageForest type='ate' returns a ggplot", {
  .skip_plots()
  fit <- .causal_fit(store = TRUE)
  out <- expect_no_error(plot(fit, type = "ate"))
  expect_s3_class(out, "gg")
})

test_that("plot.CausalShrinkageForest type='ate' accepts bayesian_bootstrap toggle", {
  .skip_plots()
  fit <- .causal_fit(store = TRUE)
  out_pate <- expect_no_error(plot(fit, type = "ate", bayesian_bootstrap = TRUE))
  out_mate <- expect_no_error(plot(fit, type = "ate", bayesian_bootstrap = FALSE))
  expect_s3_class(out_pate, "gg")
  expect_s3_class(out_mate, "gg")
  expect_equal(out_pate$labels$x, "PATE")
  expect_equal(out_mate$labels$x, "MATE")
})

test_that("plot.CausalShrinkageForest type='ate' errors without posterior samples", {
  .skip_plots()
  fit <- .causal_fit(store = FALSE)
  expect_error(plot(fit, type = "ate"), "store_posterior_sample")
})


# -- plot.CausalShrinkageForest: cate -----------------------------------------

test_that("plot.CausalShrinkageForest type='cate' returns a ggplot", {
  .skip_plots()
  fit <- .causal_fit(store = TRUE)
  out <- expect_no_error(plot(fit, type = "cate"))
  expect_s3_class(out, "gg")
})

test_that("plot.CausalShrinkageForest type='cate' errors without posterior samples", {
  .skip_plots()
  fit <- .causal_fit(store = FALSE)
  expect_error(plot(fit, type = "cate"), "store_posterior_sample")
})


# -- plot.CausalShrinkageForest: vi -------------------------------------------

test_that("plot.CausalShrinkageForest type='vi' works for Dirichlet model", {
  .skip_plots()
  n <- 30L; p <- 3L
  set.seed(1)
  X         <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y         <- X[, 1] + (0.5 - treatment) * 2 + rnorm(n)
  lh        <- 1 / sqrt(5)
  fit <- CausalShrinkageForest(
    y = y, X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "dirichlet", prior_type_treat = "dirichlet",
    local_hp_control = lh, local_hp_treat = lh,
    N_post = 10, N_burn = 5, verbose = FALSE
  )

  # forest = "both" returns a named list of two ggplots
  out_both <- expect_no_error(plot(fit, type = "vi", forest = "both", n_vi = 3))
  expect_type(out_both, "list")
  expect_s3_class(out_both$control, "gg")
  expect_s3_class(out_both$treat,   "gg")

  # forest = "control" returns a single ggplot
  out_c <- expect_no_error(plot(fit, type = "vi", forest = "control", n_vi = 3))
  expect_s3_class(out_c, "gg")

  # forest = "treat" returns a single ggplot
  out_t <- expect_no_error(plot(fit, type = "vi", forest = "treat", n_vi = 3))
  expect_s3_class(out_t, "gg")
})

test_that("plot.CausalShrinkageForest type='vi' errors for non-Dirichlet model", {
  .skip_plots()
  fit <- .causal_fit()
  expect_error(plot(fit, type = "vi"), "not available")
})


# -- plot.ShrinkageTreesPrediction: survival -----------------------------------

test_that("plot.ShrinkageTreesPrediction type='survival' returns a ggplot", {
  .skip_plots()
  set.seed(1)
  n <- 30L; p <- 3L
  X      <- matrix(runif(n * p), ncol = p)
  X_test <- matrix(runif(10 * p), ncol = p)
  time   <- rexp(n, rate = exp(0.5 * X[, 1]))
  status <- rbinom(n, 1, 0.7)

  fit <- HorseTrees(
    y = time, status = status, X_train = X, X_test = X_test,
    outcome_type = "right-censored",
    number_of_trees = 5, N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE, verbose = FALSE
  )
  pred <- predict(fit, newdata = X_test)

  # Population-averaged
  out <- expect_no_error(plot(pred, type = "survival"))
  expect_s3_class(out, "gg")

  # Individual curves
  out2 <- expect_no_error(plot(pred, type = "survival", obs = c(1, 3)))
  expect_s3_class(out2, "gg")
})

# High-dimensional regression tests (p >> n).
#
# These tests use n = 50, p = 200 with only 3 active predictors.
# They guard against C++ crashes, numerical blow-up, and wrong dimension
# handling in the p > n regime — scenarios that the standard p = 3 tests
# cannot detect.
#
# MCMC settings are kept minimal (N_post = 10, N_burn = 5, trees = 10)
# to keep runtime acceptable on CI.

# ---------------------------------------------------------------------------
# Shared sparse-AFT data (n = 50, p = 200, 3 active predictors)
# ---------------------------------------------------------------------------
.hd_data <- local({
  set.seed(42)
  n <- 50L; p <- 200L
  X      <- matrix(rnorm(n * p), nrow = n)
  log_T  <- 1.5 * X[, 1] - 1.0 * X[, 2] + 0.5 * X[, 3] + rnorm(n)
  C      <- rexp(n, rate = 0.4)
  time   <- pmin(exp(log_T), C)
  status <- as.integer(exp(log_T) <= C)
  X_test <- matrix(rnorm(20L * p), nrow = 20L)
  list(X = X, log_T = log_T, time = time, status = status,
       X_test = X_test, n = n, p = p)
})


# ---------------------------------------------------------------------------
# HorseTrees — survival, p >> n
# ---------------------------------------------------------------------------

test_that("HorseTrees handles p >> n survival without error", {
  d <- .hd_data
  set.seed(1)
  fit <- HorseTrees(
    y               = d$time,
    status          = d$status,
    X_train         = d$X,
    X_test          = d$X_test,
    outcome_type    = "right-censored",
    number_of_trees = 10,
    N_post          = 10,
    N_burn          = 5,
    store_posterior_sample = TRUE,
    verbose         = FALSE
  )

  # Dimensions
  expect_length(fit$train_predictions, d$n)
  expect_length(fit$test_predictions,  nrow(d$X_test))
  expect_length(fit$sigma, 10)
  expect_equal(dim(fit$train_predictions_sample), c(10L, d$n))
  expect_equal(dim(fit$test_predictions_sample),  c(10L, nrow(d$X_test)))

  # Numerical sanity
  expect_true(all(is.finite(fit$train_predictions)))
  expect_true(all(is.finite(fit$test_predictions)))
  expect_true(all(is.finite(fit$train_predictions_sample)))
  expect_true(all(fit$sigma > 0))

  # Predictions are not all identical (shrinkage didn't collapse to zero)
  expect_gt(sd(fit$train_predictions), 0)
  expect_gt(sd(fit$test_predictions),  0)
})


# ---------------------------------------------------------------------------
# ShrinkageTrees (horseshoe) — survival, p >> n
# ---------------------------------------------------------------------------

test_that("ShrinkageTrees horseshoe handles p >> n survival without error", {
  d  <- .hd_data
  lh <- 0.1 / sqrt(10)
  set.seed(2)
  fit <- ShrinkageTrees(
    y               = d$time,
    status          = d$status,
    X_train         = d$X,
    X_test          = d$X_test,
    outcome_type    = "right-censored",
    prior_type      = "horseshoe",
    local_hp        = lh,
    global_hp       = lh,
    number_of_trees = 10,
    N_post          = 10,
    N_burn          = 5,
    store_posterior_sample = TRUE,
    verbose         = FALSE
  )

  expect_length(fit$train_predictions, d$n)
  expect_length(fit$test_predictions,  nrow(d$X_test))
  expect_length(fit$sigma, 10)
  expect_equal(dim(fit$train_predictions_sample), c(10L, d$n))
  expect_equal(dim(fit$test_predictions_sample),  c(10L, nrow(d$X_test)))

  expect_true(all(is.finite(fit$train_predictions)))
  expect_true(all(is.finite(fit$test_predictions)))
  expect_true(all(is.finite(fit$train_predictions_sample)))
  expect_true(all(fit$sigma > 0))
  expect_gt(sd(fit$train_predictions), 0)
  expect_gt(sd(fit$test_predictions),  0)
})


# ---------------------------------------------------------------------------
# SurvivalDART — survival, p >> n
# ---------------------------------------------------------------------------

test_that("SurvivalDART handles p >> n survival without error", {
  d <- .hd_data
  set.seed(3)
  fit <- SurvivalDART(
    time            = d$time,
    status          = d$status,
    X_train         = d$X,
    X_test          = d$X_test,
    number_of_trees = 10,
    rho_dirichlet   = 3,      # prior: ~3 active predictors
    N_post          = 10,
    N_burn          = 5,
    verbose         = FALSE
  )

  expect_length(fit$train_predictions, d$n)
  expect_length(fit$test_predictions,  nrow(d$X_test))
  expect_length(fit$sigma, 10)
  expect_equal(dim(fit$train_predictions_sample), c(10L, d$n))
  expect_equal(dim(fit$test_predictions_sample),  c(10L, nrow(d$X_test)))

  expect_true(all(is.finite(fit$train_predictions)))
  expect_true(all(is.finite(fit$test_predictions)))
  expect_true(all(is.finite(fit$train_predictions_sample)))
  expect_true(all(fit$sigma > 0))
  expect_gt(sd(fit$train_predictions), 0)
  expect_gt(sd(fit$test_predictions),  0)

  # DART should record split probabilities for variable importance
  expect_false(is.null(fit$split_probs))
  expect_equal(ncol(fit$split_probs), d$p)
})


# ---------------------------------------------------------------------------
# predict() after high-dimensional fit
# ---------------------------------------------------------------------------

test_that("predict() works after high-dimensional ShrinkageTrees fit", {
  d  <- .hd_data
  lh <- 0.1 / sqrt(10)
  set.seed(4)
  fit <- ShrinkageTrees(
    y               = d$time,
    status          = d$status,
    X_train         = d$X,
    outcome_type    = "right-censored",
    prior_type      = "horseshoe",
    local_hp        = lh,
    global_hp       = lh,
    number_of_trees = 10,
    N_post          = 10,
    N_burn          = 5,
    store_posterior_sample = TRUE,
    verbose         = FALSE
  )

  pred <- predict(fit, newdata = d$X_test)
  expect_s3_class(pred, "ShrinkageTreesPrediction")
  expect_length(pred$mean,  nrow(d$X_test))
  expect_length(pred$lower, nrow(d$X_test))
  expect_length(pred$upper, nrow(d$X_test))
  expect_true(all(is.finite(pred$mean)))
  expect_true(all(pred$lower <= pred$mean))
  expect_true(all(pred$mean  <= pred$upper))
})


# ---------------------------------------------------------------------------
# ShrinkageTrees (horseshoe_fw) — survival, p >> n
# ---------------------------------------------------------------------------

test_that("ShrinkageTrees horseshoe_fw handles p >> n survival without error", {
  d  <- .hd_data
  lh <- 0.1 / sqrt(10)
  set.seed(5)
  fit <- ShrinkageTrees(
    y               = d$time,
    status          = d$status,
    X_train         = d$X,
    outcome_type    = "right-censored",
    prior_type      = "horseshoe_fw",
    local_hp        = lh,
    global_hp       = lh,
    number_of_trees = 10,
    N_post          = 10,
    N_burn          = 5,
    verbose         = FALSE
  )

  expect_length(fit$train_predictions, d$n)
  expect_true(all(is.finite(fit$train_predictions)))
  expect_true(all(fit$sigma > 0))
  expect_true(is.numeric(fit$forestwide_shrinkage))
  expect_true(all(fit$forestwide_shrinkage >= 0))
})

# ── predict.ShrinkageTrees ────────────────────────────────────────────────────

test_that("predict.ShrinkageTrees works for continuous outcome", {
  n <- 50; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  y <- X[, 1] + rnorm(n)

  set.seed(1)
  fit <- HorseTrees(
    y = y, X_train = X,
    outcome_type = "continuous",
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  X_new <- matrix(runif(20 * p), ncol = p)
  pred <- predict(fit, newdata = X_new)

  expect_s3_class(pred, "ShrinkageTreesPrediction")
  expect_length(pred$mean, 20)
  expect_length(pred$lower, 20)
  expect_length(pred$upper, 20)
  expect_true(all(pred$lower <= pred$mean))
  expect_true(all(pred$mean <= pred$upper))
  expect_false(any(is.na(pred$mean)))
  expect_true(all(is.finite(pred$mean)))

  # print and summary should work
  expect_no_error(capture.output(print(pred)))
  expect_no_error(capture.output(summary(pred)))
})


test_that("predict.ShrinkageTrees works for binary outcome", {
  n <- 50; p <- 3
  X <- matrix(rnorm(n * p), ncol = p)
  y <- ifelse(X[, 1] + rnorm(n) > 0, 1, 0)

  set.seed(1)
  fit <- HorseTrees(
    y = y, X_train = X,
    outcome_type = "binary",
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  X_new <- matrix(rnorm(15 * p), ncol = p)
  pred <- predict(fit, newdata = X_new)

  expect_s3_class(pred, "ShrinkageTreesPrediction")
  expect_length(pred$mean, 15)
  # Probabilities should be in [0, 1]
  expect_true(all(pred$mean >= 0 & pred$mean <= 1))
  expect_true(all(pred$lower >= 0 & pred$lower <= 1))
  expect_true(all(pred$upper >= 0 & pred$upper <= 1))
  expect_true(all(pred$lower <= pred$upper))
})


test_that("predict.ShrinkageTrees works for right-censored survival", {
  n <- 50; p <- 3
  X <- matrix(rnorm(n * p), ncol = p)

  true_time <- rexp(n, rate = exp(0.3 * X[, 1]))
  censor_time <- rexp(n, rate = 0.5)
  time_obs <- pmin(true_time, censor_time)
  status <- as.numeric(true_time <= censor_time)

  set.seed(1)
  fit <- HorseTrees(
    y = time_obs, X_train = X,
    status = status,
    outcome_type = "right-censored",
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  X_new <- matrix(rnorm(10 * p), ncol = p)
  pred <- predict(fit, newdata = X_new)

  expect_s3_class(pred, "ShrinkageTreesPrediction")
  expect_length(pred$mean, 10)
  expect_true(all(pred$mean > 0))
  expect_true(all(is.finite(pred$mean)))
  expect_true(all(pred$lower <= pred$upper))

  # Survival predictions should include posterior samples and sigma
  expect_true(!is.null(pred$predictions_sample))
  expect_equal(nrow(pred$predictions_sample), 10)  # N_post
  expect_equal(ncol(pred$predictions_sample), 10)   # n_new
  expect_true(!is.null(pred$sigma))
  expect_length(pred$sigma, 10)
  expect_true(all(pred$sigma > 0))
})


test_that("predict.ShrinkageTrees errors on dimension mismatch", {
  n <- 50; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  y <- rnorm(n)

  set.seed(1)
  fit <- HorseTrees(
    y = y, X_train = X,
    outcome_type = "continuous",
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  X_wrong <- matrix(runif(20 * 5), ncol = 5)  # wrong number of columns
  expect_error(predict(fit, newdata = X_wrong), "columns")
})


# ── predict.CausalShrinkageForest ────────────────────────────────────────────

test_that("predict.CausalShrinkageForest works for continuous outcome", {
  n <- 50; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)
  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat = lh, global_hp_treat = lh,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  X_new <- matrix(runif(15 * p), ncol = p)
  pred <- predict(fit, newdata_control = X_new, newdata_treat = X_new)

  expect_s3_class(pred, "CausalShrinkageForestPrediction")
  expect_equal(pred$n, 15)

  # Prognostic component
  expect_length(pred$prognostic$mean, 15)
  expect_length(pred$prognostic$lower, 15)
  expect_length(pred$prognostic$upper, 15)
  expect_true(all(pred$prognostic$lower <= pred$prognostic$upper))

  # CATE component
  expect_length(pred$cate$mean, 15)
  expect_true(all(pred$cate$lower <= pred$cate$upper))

  # Total component
  expect_length(pred$total$mean, 15)
  expect_true(all(pred$total$lower <= pred$total$upper))

  # No NAs
  expect_false(any(is.na(pred$prognostic$mean)))
  expect_false(any(is.na(pred$cate$mean)))
  expect_false(any(is.na(pred$total$mean)))

  # ATE summary and stored CATE samples
  expect_true(isTRUE(pred$bayesian_bootstrap))
  expect_equal(dim(pred$cate_samples), c(10, 15))
  expect_true(is.numeric(pred$ate$mean))
  expect_true(pred$ate$lower <= pred$ate$mean)
  expect_true(pred$ate$mean <= pred$ate$upper)

  # MATE path should match rowMeans of stored CATE samples.
  pred_mate <- predict(fit, newdata_control = X_new, newdata_treat = X_new,
                       bayesian_bootstrap = FALSE)
  expect_false(isTRUE(pred_mate$bayesian_bootstrap))
  ate_rowmean <- mean(rowMeans(pred_mate$cate_samples))
  expect_equal(pred_mate$ate$mean, ate_rowmean)

  # Standalone helper on a prediction object.
  bb <- bayesian_bootstrap_ate(pred)
  expect_equal(bb$n, 15)
  expect_equal(bb$mate_samples, rowMeans(pred$cate_samples))

  # print and summary
  expect_no_error(capture.output(print(pred)))
  expect_no_error(capture.output(summary(pred)))
})


test_that("predict.CausalShrinkageForest works for right-censored survival", {
  n <- 50; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)

  log_T0 <- X[, 1] + rnorm(n)
  log_T <- log_T0 + treatment * (-0.5)
  true_time <- exp(log_T)
  censor_time <- rexp(n, rate = 0.1)
  time_obs <- pmin(true_time, censor_time)
  status <- as.integer(true_time <= censor_time)

  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = time_obs, status = status,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "right-censored", timescale = "log",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat = lh, global_hp_treat = lh,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  X_new <- matrix(runif(10 * p), ncol = p)
  pred <- predict(fit, newdata_control = X_new, newdata_treat = X_new)

  expect_s3_class(pred, "CausalShrinkageForestPrediction")
  expect_equal(pred$n, 10)
  expect_length(pred$prognostic$mean, 10)
  expect_length(pred$cate$mean, 10)
  expect_length(pred$total$mean, 10)
  expect_false(any(is.na(pred$prognostic$mean)))
  expect_true(all(is.finite(pred$cate$mean)))
})


test_that("predict.CausalShrinkageForest errors on dimension mismatch", {
  n <- 30; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)
  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat = lh, global_hp_treat = lh,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  X_wrong <- matrix(runif(10 * 5), ncol = 5)
  expect_error(predict(fit, newdata_control = X_wrong, newdata_treat = X_wrong),
               "columns")

  # Row mismatch
  X_a <- matrix(runif(10 * p), ncol = p)
  X_b <- matrix(runif(5 * p), ncol = p)
  expect_error(predict(fit, newdata_control = X_a, newdata_treat = X_b),
               "same number of rows")
})

test_that("HorseTrees for continuous outcome", {
  
  # Generate data
  X <- matrix(runif(50 * 3), ncol = 3)
  X_test <- matrix(runif(50 * 3), ncol = 3)
  y <- X[, 1] + rnorm(50)
  
  # Fit the model
  set.seed(1)
  fit <- HorseTrees(y = y, 
                    X_train = X,
                    X_test = X_test,
                    outcome_type = "continuous",
                    number_of_trees = 5,
                    N_post = 10,
                    N_burn = 5,
                    store_posterior_sample = TRUE,
                    verbose = FALSE)
  
  # Basic checks
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_true("test_predictions" %in% names(fit))
  expect_length(fit$train_predictions, 50)
  expect_length(fit$sigma, 10)
  
  # Numerical checks
  expect_false(any(is.na(fit$train_predictions)))
  expect_false(any(is.nan(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))
  
  # Check posterior samples
  expect_true("train_predictions_sample" %in% names(fit))
  expect_true("test_predictions_sample" %in% names(fit))
  expect_equal(dim(fit$train_predictions_sample), c(10, 50))
  expect_equal(dim(fit$test_predictions_sample), c(10, 50))
  
  # Check sigma positivity
  expect_true(all(fit$sigma > 0))
  
  # Check predictions are not all zero
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_sample) > 0)
  expect_true(sd(fit$test_predictions) > 0)
  expect_true(sd(fit$test_predictions_sample) > 0)
  
  # Check reproducibility
  set.seed(1)
  fit2 <- HorseTrees(y = y, X_train = X, outcome_type = "continuous", number_of_trees = 5, 
                     N_post = 10, N_burn = 5, store_posterior_sample = TRUE, 
                     verbose = FALSE)
  expect_equal(fit$train_predictions, fit2$train_predictions)
})


test_that("HorseTrees for binary outcome", {
  
  # Generate data
  X <- matrix(rnorm(50 * 3), ncol = 3)
  X_test <- matrix(runif(50 * 3), ncol = 3)
  y <- ifelse(X[, 1] + rnorm(50) > 0, 1, 0)
  
  # Fit the model
  set.seed(1)
  fit <- HorseTrees(y = y, 
                    X_train = X,
                    X_test = X_test,
                    outcome_type = "binary",
                    number_of_trees = 5,
                    N_post = 10,
                    N_burn = 5,
                    store_posterior_sample = TRUE,
                    verbose = FALSE)
  
  # Basic checks
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_length(fit$train_predictions, 50)

  # Numerical checks
  expect_false(any(is.na(fit$train_predictions)))
  expect_false(any(is.nan(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))
  
  # Check posterior samples
  expect_true("train_predictions_sample" %in% names(fit))
  expect_true("test_predictions_sample" %in% names(fit))
  expect_true("train_probabilities_sample" %in% names(fit))
  expect_true("test_probabilities_sample" %in% names(fit))
  expect_equal(dim(fit$train_predictions_sample), c(10, 50))
  expect_equal(dim(fit$test_predictions_sample), c(10, 50))
  expect_equal(dim(fit$train_probabilities_sample), c(10, 50))
  expect_equal(dim(fit$test_probabilities_sample), c(10, 50))
  
  # Check estimated probabilities are within [0,1]
  expect_true(all(fit$train_probabilities >= 0 & fit$train_probabilities <= 1))
  expect_true(all(fit$test_probabilities >= 0 & fit$test_probabilities <= 1))
  
  # Check predictions are not all constant
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_sample) > 0)
  expect_true(sd(fit$test_predictions) > 0)
  expect_true(sd(fit$test_predictions_sample) > 0)
  
  # Check reproducibility
  set.seed(1)
  fit2 <- HorseTrees(y = y, X_train = X, outcome_type = "binary", number_of_trees = 5, 
                     N_post = 10, N_burn = 5, store_posterior_sample = TRUE, 
                     verbose = FALSE)
  expect_equal(fit$train_predictions, fit2$train_predictions)
})

test_that("HorseTrees for right-censored survival outcome", {
  
  # Generate covariates
  X <- matrix(rnorm(50 * 3), ncol = 3)
  X_test <- matrix(runif(50 * 3), ncol = 3)
  
  # Generate survival times (exponential baseline hazard depending on X[,1])
  linpred <- X[, 1]
  true_time <- rexp(50, rate = exp(linpred))
  
  # Generate censoring times
  censor_time <- rexp(50, rate = 0.5)
  
  # Observed time and status indicator
  time_obs <- pmin(true_time, censor_time)
  status <- as.numeric(true_time <= censor_time)
  
  # Fit the model
  set.seed(1)
  fit <- HorseTrees(y = time_obs, 
                    X_train = X,
                    X_test = X_test,
                    outcome_type = "right-censored",
                    status = status,
                    number_of_trees = 5,
                    N_post = 10,
                    N_burn = 5,
                    store_posterior_sample = TRUE,
                    verbose = FALSE)
  
  # Basic checks
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_true("test_predictions" %in% names(fit))
  expect_true("test_predictions" %in% names(fit))
  expect_length(fit$train_predictions, 50)
  expect_length(fit$sigma, 10)
  
  # Numerical checks
  expect_false(any(is.na(fit$train_predictions)))
  expect_false(any(is.nan(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))
  
  # Check posterior samples
  expect_true("train_predictions_sample" %in% names(fit))
  expect_true("test_predictions_sample" %in% names(fit))
  expect_equal(dim(fit$train_predictions_sample), c(10, 50))
  expect_equal(dim(fit$test_predictions_sample), c(10, 50))
  
  # Check sigma positivity
  expect_true(all(fit$sigma > 0))
  
  # Check predictions are not all constant
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_sample) > 0)
  expect_true(sd(fit$test_predictions) > 0)
  expect_true(sd(fit$test_predictions_sample) > 0)
  
  # Check reproducibility
  set.seed(1)
  fit2 <- HorseTrees(y = time_obs, X_train = X, outcome_type = "right-censored",
                     status = status, number_of_trees = 5, N_post = 10,
                     N_burn = 5, store_posterior_sample = TRUE, verbose = FALSE)
  expect_equal(fit$train_predictions, fit2$train_predictions)
})


# ── S3 class and methods ──────────────────────────────────────────────────────

test_that("HorseTrees returns ShrinkageTrees S3 object with working methods", {
  X <- matrix(runif(50 * 3), ncol = 3)
  y <- X[, 1] + rnorm(50)

  set.seed(1)
  fit <- HorseTrees(
    y = y, X_train = X,
    outcome_type = "continuous",
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "ShrinkageTrees")
  expect_no_error(capture.output(print(fit)))
  expect_no_error(smry <- summary(fit))
  expect_type(smry, "list")
  expect_true(!is.null(smry$sigma))
})


# ── Multi-chain (n_chains) ────────────────────────────────────────────────────

test_that("HorseTrees n_chains > 1 pools chains correctly", {
  X <- matrix(runif(50 * 3), ncol = 3)
  y <- X[, 1] + rnorm(50)

  set.seed(1)
  fit <- HorseTrees(
    y = y, X_train = X,
    outcome_type = "continuous",
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    n_chains = 2,
    verbose = FALSE
  )

  expect_s3_class(fit, "ShrinkageTrees")
  expect_length(fit$sigma, 20)
  expect_equal(fit$mcmc$n_chains, 2)
  expect_length(fit$chains$acceptance_ratios, 2)
})


# ── Interval-censored survival ───────────────────────────────────────────────

test_that("HorseTrees for interval-censored survival outcome", {

  n <- 50; p <- 3
  X <- matrix(rnorm(n * p), ncol = p)
  X_test <- matrix(rnorm(n * p), ncol = p)

  # Generate true event times
  true_time <- rexp(n, rate = exp(0.3 * X[, 1]))

  # Create mixed censoring:
  #   ~40% exact events, ~30% interval-censored, ~30% right-censored
  set.seed(42)
  obs_type <- sample(c("exact", "interval", "right"),
                     n, replace = TRUE, prob = c(0.4, 0.3, 0.3))

  left_time  <- numeric(n)
  right_time <- numeric(n)

  for (i in seq_len(n)) {
    if (obs_type[i] == "exact") {
      left_time[i]  <- true_time[i]
      right_time[i] <- true_time[i]
    } else if (obs_type[i] == "interval") {
      width <- runif(1, 0.1, 0.5) * true_time[i]
      left_time[i]  <- true_time[i] - width / 2
      right_time[i] <- true_time[i] + width / 2
    } else {
      left_time[i]  <- true_time[i] * runif(1, 0.3, 0.9)
      right_time[i] <- Inf
    }
  }

  # Fit the model
  set.seed(1)
  fit <- HorseTrees(left_time = left_time,
                    right_time = right_time,
                    X_train = X,
                    X_test = X_test,
                    outcome_type = "interval-censored",
                    number_of_trees = 5,
                    N_post = 10,
                    N_burn = 5,
                    store_posterior_sample = TRUE,
                    verbose = FALSE)

  # Basic checks
  expect_type(fit, "list")
  expect_s3_class(fit, "ShrinkageTrees")
  expect_true("train_predictions" %in% names(fit))
  expect_true("test_predictions" %in% names(fit))
  expect_length(fit$train_predictions, n)
  expect_length(fit$sigma, 10)

  # Predictions should be positive (back-transformed survival times)
  expect_true(all(fit$train_predictions > 0))
  expect_true(all(fit$test_predictions > 0))

  # Numerical checks
  expect_false(any(is.na(fit$train_predictions)))
  expect_false(any(is.nan(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))

  # Check posterior samples
  expect_true("train_predictions_sample" %in% names(fit))
  expect_true("test_predictions_sample" %in% names(fit))
  expect_equal(dim(fit$train_predictions_sample), c(10, n))
  expect_equal(dim(fit$test_predictions_sample), c(10, n))

  # Check sigma positivity
  expect_true(all(fit$sigma > 0))

  # Check predictions are not all constant
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$test_predictions) > 0)

  # Check stored data fields
  expect_true(!is.null(fit$data$left_time_train))
  expect_true(!is.null(fit$data$right_time_train))
  expect_true(!is.null(fit$data$ic_indicator_train))
  expect_length(fit$data$left_time_train, n)
  expect_length(fit$data$right_time_train, n)
  expect_length(fit$data$ic_indicator_train, n)

  # Check print/summary work
  expect_no_error(capture.output(print(fit)))
  expect_no_error(smry <- summary(fit))
  expect_type(smry, "list")

  # Check reproducibility
  set.seed(1)
  fit2 <- HorseTrees(left_time = left_time,
                     right_time = right_time,
                     X_train = X,
                     X_test = X_test,
                     outcome_type = "interval-censored",
                     number_of_trees = 5,
                     N_post = 10,
                     N_burn = 5,
                     store_posterior_sample = TRUE,
                     verbose = FALSE)
  expect_equal(fit$train_predictions, fit2$train_predictions)
})


test_that("HorseTrees interval-censored predict on new data works", {

  n <- 50; p <- 3
  X <- matrix(rnorm(n * p), ncol = p)

  true_time <- rexp(n, rate = exp(0.3 * X[, 1]))
  set.seed(42)
  obs_type <- sample(c("exact", "interval", "right"),
                     n, replace = TRUE, prob = c(0.4, 0.3, 0.3))

  left_time  <- numeric(n)
  right_time <- numeric(n)
  for (i in seq_len(n)) {
    if (obs_type[i] == "exact") {
      left_time[i] <- true_time[i]; right_time[i] <- true_time[i]
    } else if (obs_type[i] == "interval") {
      w <- runif(1, 0.1, 0.5) * true_time[i]
      left_time[i] <- true_time[i] - w / 2
      right_time[i] <- true_time[i] + w / 2
    } else {
      left_time[i] <- true_time[i] * runif(1, 0.3, 0.9)
      right_time[i] <- Inf
    }
  }

  set.seed(1)
  fit <- HorseTrees(left_time = left_time,
                    right_time = right_time,
                    X_train = X,
                    outcome_type = "interval-censored",
                    number_of_trees = 5,
                    N_post = 10, N_burn = 5,
                    store_posterior_sample = TRUE,
                    verbose = FALSE)

  # Predict on new data
  X_new <- matrix(rnorm(20 * p), ncol = p)
  pred <- predict(fit, newdata = X_new)

  expect_true(all(pred$mean > 0))
  expect_true(all(is.finite(pred$mean)))
  expect_length(pred$mean, 20)
  expect_no_error(capture.output(print(pred)))
})


test_that("HorseTrees interval-censored validation errors", {

  n <- 50; p <- 3
  X <- matrix(rnorm(n * p), ncol = p)
  lt <- rexp(n); rt <- lt + runif(n, 0.1, 1)

  # Missing right_time
  expect_error(
    HorseTrees(left_time = lt, X_train = X,
               outcome_type = "interval-censored",
               number_of_trees = 5, N_post = 5, N_burn = 5, verbose = FALSE),
    "left_time.*right_time"
  )

  # Missing left_time
  expect_error(
    HorseTrees(right_time = rt, X_train = X,
               outcome_type = "interval-censored",
               number_of_trees = 5, N_post = 5, N_burn = 5, verbose = FALSE),
    "left_time.*right_time"
  )

  # left > right
  bad_lt <- rt + 1
  expect_error(
    HorseTrees(left_time = bad_lt, right_time = rt, X_train = X,
               outcome_type = "interval-censored",
               number_of_trees = 5, N_post = 5, N_burn = 5, verbose = FALSE),
    "left_time.*right_time"
  )

  # Negative values with timescale = "time"
  neg_lt <- lt; neg_lt[1] <- -1
  expect_error(
    HorseTrees(left_time = neg_lt, right_time = rt, X_train = X,
               outcome_type = "interval-censored",
               number_of_trees = 5, N_post = 5, N_burn = 5, verbose = FALSE),
    "positive"
  )
})

# -- Internal helpers ----------------------------------------------------------

.check_ggplot2 <- function() {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is needed for plotting. ",
      "Install it with: install.packages('ggplot2')",
      call. = FALSE
    )
  }
}

# Reshape a concatenated posterior vector into a long data.frame
# with columns: iteration, chain, value.
.to_draws_df <- function(vec, par_name, N_post, n_chains) {
  mat <- matrix(vec, nrow = N_post, ncol = n_chains)
  data.frame(
    iteration = rep(seq_len(N_post), n_chains),
    chain     = factor(rep(seq_len(n_chains), each = N_post)),
    value     = as.vector(mat)
  )
}

# Extract feature names from a matrix, falling back to X1, X2, ...
.feature_names <- function(mat) {
  nms <- colnames(mat)
  if (is.null(nms)) nms <- paste0("X", seq_len(ncol(mat)))
  nms
}

# Build a top-n variable importance plot from a split_probs matrix.
.vi_intervals <- function(sp, X_mat, n_vi, title, ...) {
  colnames(sp) <- .feature_names(X_mat)
  means <- colMeans(sp)
  top <- order(means, decreasing = TRUE)[seq_len(min(n_vi, ncol(sp)))]
  sp_top <- sp[, top, drop = FALSE]

  # Compute summary statistics for each variable
  vi_df <- data.frame(
    variable = factor(colnames(sp_top), levels = rev(colnames(sp_top))),
    mean     = colMeans(sp_top),
    lo50     = apply(sp_top, 2, quantile, 0.25),
    hi50     = apply(sp_top, 2, quantile, 0.75),
    lo95     = apply(sp_top, 2, quantile, 0.025),
    hi95     = apply(sp_top, 2, quantile, 0.975)
  )

  ggplot2::ggplot(vi_df, ggplot2::aes(x = .data$mean, y = .data$variable)) +
    ggplot2::geom_linerange(
      ggplot2::aes(xmin = .data$lo95, xmax = .data$hi95),
      colour = "grey70", linewidth = 0.5
    ) +
    ggplot2::geom_linerange(
      ggplot2::aes(xmin = .data$lo50, xmax = .data$hi50),
      colour = "steelblue", linewidth = 1.5
    ) +
    ggplot2::geom_point(size = 2, colour = "steelblue") +
    ggplot2::labs(x = "Inclusion probability", y = NULL, title = title) +
    ggplot2::theme_bw()
}

# Traceplot: line plot of MCMC draws over iterations, coloured by chain.
.trace_plot <- function(vec, par_name, N_post, n_chains, ...) {
  df <- .to_draws_df(vec, par_name, N_post, n_chains)
  ggplot2::ggplot(df, ggplot2::aes(
    x = .data$iteration, y = .data$value, colour = .data$chain
  )) +
    ggplot2::geom_line(alpha = 0.45, linewidth = 0.3) +
    ggplot2::labs(x = "Iteration", y = par_name, colour = "Chain") +
    ggplot2::theme_bw()
}

# Overlaid density plot of MCMC draws, one curve per chain.
.density_plot <- function(vec, par_name, N_post, n_chains, ...) {
  df <- .to_draws_df(vec, par_name, N_post, n_chains)
  ggplot2::ggplot(df, ggplot2::aes(
    x = .data$value, colour = .data$chain, fill = .data$chain
  )) +
    ggplot2::geom_density(alpha = 0.15) +
    ggplot2::labs(x = par_name, y = "Density", colour = "Chain",
                  fill = "Chain") +
    ggplot2::theme_bw()
}

# Posterior density with shaded credible region.
.area_plot <- function(draws, par_name, prob = 0.95, ...) {
  ci <- quantile(draws, probs = c((1 - prob) / 2, 1 - (1 - prob) / 2))
  df <- data.frame(value = draws)
  ci_pct <- paste0(round(prob * 100), "%")

  ggplot2::ggplot(df, ggplot2::aes(x = .data$value)) +
    ggplot2::geom_density(fill = "steelblue", alpha = 0.3, colour = "steelblue") +
    ggplot2::geom_vline(xintercept = median(draws), colour = "steelblue",
                        linewidth = 0.8) +
    ggplot2::geom_vline(xintercept = ci, colour = "steelblue",
                        linetype = "dashed", linewidth = 0.5) +
    ggplot2::labs(x = par_name, y = "Density",
                  subtitle = paste0(ci_pct, " credible interval")) +
    ggplot2::theme_bw()
}


# -- Survival curve helper -----------------------------------------------------

# Compute posterior survival curves for specified observations or population avg.
# Returns a data.frame with columns: time, surv_mean, surv_lower, surv_upper, obs_id
.survival_curves <- function(x, obs, t_grid, level) {

  if (!x$outcome_type %in% c("right-censored", "interval-censored"))
    stop("Survival curve plot requires a survival outcome type.", call. = FALSE)

  alpha     <- (1 - level) / 2
  sigma_hat <- x$preprocess$sigma_hat
  sigma_log <- x$sigma * sigma_hat   # posterior sigma on log-time scale
  N_post    <- length(sigma_log)
  has_samples <- isTRUE(x$mcmc$store_posterior_sample) &&
                 !is.null(x$train_predictions_sample)

  # Reconstruct log-scale mu: y_train stores log-times for both timescale values
  if (has_samples) {
    mu_matrix <- if (x$timescale == "time")
      log(x$train_predictions_sample)  # N_post x n_train
    else
      x$train_predictions_sample
  }
  mu_mean <- if (x$timescale == "time")
    log(x$train_predictions)           # n_train vector
  else
    x$train_predictions

  # Auto-generate time grid on original time scale
  if (is.null(t_grid)) {
    orig_times <- exp(x$data$y_train)  # original-scale times
    t_lo  <- max(min(orig_times) * 0.5, 1e-6)
    t_hi  <- max(orig_times) * 1.2
    t_grid <- seq(t_lo, t_hi, length.out = 200)
  }
  log_t <- log(t_grid)  # length n_t
  n_t   <- length(t_grid)

  # -- Population average (obs = NULL) -----------------------------------------
  if (is.null(obs)) {
    n_train <- length(mu_mean)
    S_sum   <- matrix(0, N_post, n_t)

    if (has_samples) {
      for (i in seq_len(n_train)) {
        z <- (rep(1, N_post) %o% log_t - mu_matrix[, i]) / sigma_log
        S_sum <- S_sum + (1 - pnorm(z))
      }
    } else {
      for (i in seq_len(n_train)) {
        z <- (rep(1, N_post) %o% log_t - mu_mean[i]) / sigma_log
        S_sum <- S_sum + (1 - pnorm(z))
      }
    }
    S_avg <- S_sum / n_train

    df <- data.frame(
      time       = t_grid,
      surv_mean  = colMeans(S_avg),
      surv_lower = cummin(apply(S_avg, 2, quantile, alpha)),
      surv_upper = cummin(apply(S_avg, 2, quantile, 1 - alpha)),
      obs_id     = factor("Average")
    )
    return(df)
  }

  # -- Individual observations -------------------------------------------------
  if (any(obs < 1) || any(obs > length(mu_mean)))
    stop("obs indices must be between 1 and ", length(mu_mean), ".", call. = FALSE)

  dfs <- vector("list", length(obs))
  for (k in seq_along(obs)) {
    i <- obs[k]
    if (has_samples) {
      mu_i <- mu_matrix[, i]  # N_post vector
    } else {
      mu_i <- rep(mu_mean[i], N_post)
    }
    z <- (rep(1, N_post) %o% log_t - mu_i) / sigma_log  # N_post x n_t
    S <- 1 - pnorm(z)

    dfs[[k]] <- data.frame(
      time       = t_grid,
      surv_mean  = colMeans(S),
      surv_lower = cummin(apply(S, 2, quantile, alpha)),
      surv_upper = cummin(apply(S, 2, quantile, 1 - alpha)),
      obs_id     = factor(paste0("Obs ", i))
    )
  }
  do.call(rbind, dfs)
}


# Compute posterior predictive survival curves for a ShrinkageTreesPrediction.
# predictions_sample: N_post x n matrix, sigma: N_post vector, both on log-time
# scale if timescale == "time".
.survival_curves_pred <- function(x, obs, t_grid, level) {

  if (!x$outcome_type %in% c("right-censored", "interval-censored"))
    stop("Survival curve plot requires a survival outcome type.", call. = FALSE)
  if (is.null(x$predictions_sample) || is.null(x$sigma))
    stop("Survival curve plot requires posterior samples. ",
         "These are stored automatically when predict() is called on a ",
         "survival model.", call. = FALSE)

  alpha     <- (1 - level) / 2
  sigma_log <- x$sigma          # already on log-time scale
  N_post    <- length(sigma_log)

  # Log-scale mu matrix (N_post x n)
  mu_matrix <- if (x$timescale == "time")
    log(x$predictions_sample)
  else
    x$predictions_sample

  n_obs <- ncol(mu_matrix)

  # Auto-generate time grid on original time scale
  if (is.null(t_grid)) {
    orig_preds <- if (x$timescale == "time") x$mean else exp(x$mean)
    t_lo  <- max(min(orig_preds) * 0.2, 1e-6)
    t_hi  <- max(orig_preds) * 3
    t_grid <- seq(t_lo, t_hi, length.out = 200)
  }
  log_t <- log(t_grid)
  n_t   <- length(t_grid)

  # -- Population average (obs = NULL) -----------------------------------------
  if (is.null(obs)) {
    S_sum <- matrix(0, N_post, n_t)
    for (i in seq_len(n_obs)) {
      z <- (rep(1, N_post) %o% log_t - mu_matrix[, i]) / sigma_log
      S_sum <- S_sum + (1 - pnorm(z))
    }
    S_avg <- S_sum / n_obs

    df <- data.frame(
      time       = t_grid,
      surv_mean  = colMeans(S_avg),
      surv_lower = cummin(apply(S_avg, 2, quantile, alpha)),
      surv_upper = cummin(apply(S_avg, 2, quantile, 1 - alpha)),
      obs_id     = factor("Average")
    )
    return(df)
  }

  # -- Individual observations -------------------------------------------------
  if (any(obs < 1) || any(obs > n_obs))
    stop("obs indices must be between 1 and ", n_obs, ".", call. = FALSE)

  dfs <- vector("list", length(obs))
  for (k in seq_along(obs)) {
    i <- obs[k]
    mu_i <- mu_matrix[, i]
    z <- (rep(1, N_post) %o% log_t - mu_i) / sigma_log
    S <- 1 - pnorm(z)

    dfs[[k]] <- data.frame(
      time       = t_grid,
      surv_mean  = colMeans(S),
      surv_lower = cummin(apply(S, 2, quantile, alpha)),
      surv_upper = cummin(apply(S, 2, quantile, 1 - alpha)),
      obs_id     = factor(paste0("Obs ", i))
    )
  }
  do.call(rbind, dfs)
}


# Shared ggplot2 builder for survival curve data.frames.
# df: data.frame from .survival_curves() or .survival_curves_pred()
# km_data: optional data.frame(time, surv) for KM overlay
.plot_survival_curves <- function(df, obs, level, km_data = NULL,
                                  title_prefix = "Posterior") {
  ci_pct   <- paste0(round(level * 100), "%")
  n_curves <- nlevels(df$obs_id)

  if (n_curves == 1L) {
    p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$time, y = .data$surv_mean)) +
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = .data$surv_lower, ymax = .data$surv_upper),
        fill = "steelblue", alpha = 0.3
      ) +
      ggplot2::geom_line(colour = "steelblue", linewidth = 0.8)
  } else {
    p <- ggplot2::ggplot(df, ggplot2::aes(
      x = .data$time, y = .data$surv_mean, colour = .data$obs_id,
      fill = .data$obs_id
    )) +
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = .data$surv_lower, ymax = .data$surv_upper),
        alpha = 0.15, colour = NA
      ) +
      ggplot2::geom_line(linewidth = 0.7)
  }

  title <- if (is.null(obs)) {
    paste0(title_prefix, " population-averaged survival curve")
  } else {
    paste0(title_prefix, " survival curve", if (n_curves > 1) "s" else "")
  }

  if (!is.null(km_data)) {
    p <- p +
      ggplot2::geom_step(
        data = km_data,
        ggplot2::aes(x = .data$time, y = .data$surv),
        colour = "black", linetype = "dashed", linewidth = 0.5,
        inherit.aes = FALSE
      )
  }

  p +
    ggplot2::labs(
      x        = "Time",
      y        = "Survival probability",
      title    = title,
      subtitle = paste0(ci_pct, " credible band"),
      colour   = NULL,
      fill     = NULL
    ) +
    ggplot2::ylim(0, 1) +
    ggplot2::theme_minimal()
}


# -- plot.ShrinkageTrees -------------------------------------------------------

#' Plot diagnostics for a ShrinkageTrees model
#'
#' Visualises posterior draws using \pkg{ggplot2}.
#' Requires the suggested package \pkg{ggplot2}.
#'
#' @param x A \code{ShrinkageTrees} object.
#' @param type Character; one of:
#'   \describe{
#'     \item{\code{"trace"}}{Sigma traceplot across MCMC iterations (one line
#'       per chain). Useful for assessing chain mixing.}
#'     \item{\code{"density"}}{Overlaid posterior density of sigma, one curve
#'       per chain.}
#'     \item{\code{"vi"}}{Posterior credible intervals for variable inclusion
#'       probabilities (top \code{n_vi} predictors). Only available for
#'       Dirichlet prior models.}
#'     \item{\code{"survival"}}{Posterior survival curves
#'     \eqn{S(t | x_i) = 1 - \Phi((\log t - \mu_i) / \sigma)} with pointwise
#'     credible bands, derived from the AFT log-normal model. Only available
#'     for survival outcome types (\code{"right-censored"} or
#'     \code{"interval-censored"}).
#'     \strong{Population-averaged curve} (default, \code{obs = NULL}):
#'     computes \eqn{\bar{S}(t) = n^{-1} \sum_i S(t | x_i)} at each MCMC
#'     iteration. The credible band reflects posterior uncertainty in both
#'     \eqn{\mu_i} and \eqn{\sigma} when \code{store_posterior_sample = TRUE},
#'     or sigma-only uncertainty otherwise.
#'     \strong{Individual curves} (\code{obs = c(1, 5, ...)}):
#'     one curve per selected training observation with its own credible band.
#'     Set \code{km = TRUE} to overlay the Kaplan--Meier estimate as a
#'     non-parametric reference (population-averaged plot only).}
#'   }
#' @param n_vi Integer; number of top variables to display when
#'   \code{type = "vi"}. Default \code{10}.
#' @param obs Integer vector of training-set observation indices for
#'   individual survival curves, or \code{NULL} (default) for the
#'   population-averaged curve. Indices must be between 1 and the number
#'   of training observations. Used only when \code{type = "survival"}.
#' @param t_grid Optional numeric vector of time points (on the original
#'   time scale) at which to evaluate the survival function. If \code{NULL}
#'   (default), a grid of 200 equally spaced points is generated from the
#'   range of observed training times. Used only when
#'   \code{type = "survival"}.
#' @param level Width of the pointwise credible band for
#'   \code{type = "survival"}. Default \code{0.95} (a 95 percent credible
#'   interval at each time point).
#' @param km Logical; if \code{TRUE} and \code{type = "survival"}
#'   with \code{obs = NULL}, overlay the Kaplan--Meier curve (dashed black
#'   step function) as a non-parametric reference. Default \code{FALSE}.
#'   Requires the \pkg{survival} package. Ignored with a message when
#'   \code{obs} is not \code{NULL}.
#' @param ... Additional arguments (currently unused).
#' @return A \pkg{ggplot2} object.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   set.seed(1)
#'   n <- 50; p <- 5
#'   X <- matrix(rnorm(n * p), ncol = p)
#'   y <- X[, 1] + rnorm(n)
#'
#'   # Fit a small continuous model
#'   fit <- ShrinkageTrees(
#'     y = y, X_train = X,
#'     prior_type = "horseshoe",
#'     local_hp = 0.1, global_hp = 0.1,
#'     number_of_trees = 5,
#'     N_post = 50, N_burn = 25,
#'     verbose = FALSE
#'   )
#'
#'   # Sigma traceplot -- check chain mixing
#'   plot(fit, type = "trace")
#'
#'   # Overlaid posterior densities of sigma per chain
#'   plot(fit, type = "density")
#' }
#' }
#' @export
plot.ShrinkageTrees <- function(x,
                                type = c("trace", "density", "vi", "survival"),
                                n_vi = 10,
                                obs = NULL,
                                t_grid = NULL,
                                level = 0.95,
                                km = FALSE,
                                ...) {
  type <- match.arg(type)
  .check_ggplot2()

  if (type == "survival") {
    df <- .survival_curves(x, obs = obs, t_grid = t_grid, level = level)

    # Build KM data if requested
    km_data <- NULL
    if (isTRUE(km) && !is.null(obs)) {
      message("km = TRUE is ignored for individual survival curves ",
              "(only available when obs = NULL).")
    }
    if (isTRUE(km) && is.null(obs)) {
      if (!requireNamespace("survival", quietly = TRUE)) {
        warning("Package 'survival' needed for KM overlay. Skipping.",
                call. = FALSE)
      } else {
        orig_times <- exp(x$data$y_train)
        km_fit <- survival::survfit(
          survival::Surv(orig_times, x$data$status_train) ~ 1
        )
        km_data <- data.frame(time = km_fit$time, surv = km_fit$surv)
      }
    }

    return(.plot_survival_curves(df, obs = obs, level = level,
                                 km_data = km_data))
  }

  n_ch   <- if (!is.null(x$mcmc$n_chains)) x$mcmc$n_chains else 1L
  N_post <- x$mcmc$N_post

  if (type == "trace") {
    .trace_plot(x$sigma, "sigma", N_post, n_ch, ...)

  } else if (type == "density") {
    .density_plot(x$sigma, "sigma", N_post, n_ch, ...)

  } else {
    sp <- x$split_probs
    if (is.null(sp))
      stop("Variable importance not available (split_probs is NULL).",
           call. = FALSE)
    .vi_intervals(sp, x$data$X_train, n_vi,
                  title = "Variable importance", ...)
  }
}


# -- plot.CausalShrinkageForest ------------------------------------------------

#' Plot diagnostics for a CausalShrinkageForest model
#'
#' Visualises posterior draws using \pkg{ggplot2}.
#' Requires the suggested package \pkg{ggplot2}.
#'
#' @param x A \code{CausalShrinkageForest} object.
#' @param type Character; one of:
#'   \describe{
#'     \item{\code{"trace"}}{Sigma traceplot (chain mixing).}
#'     \item{\code{"density"}}{Overlaid posterior density of sigma per chain.}
#'     \item{\code{"ate"}}{Posterior density of the average treatment effect
#'       (ATE) with 95 percent credible region. Requires
#'       \code{store_posterior_sample = TRUE}.}
#'     \item{\code{"cate"}}{Point estimates and 95 percent credible intervals
#'       for the CATE of each training observation, sorted by posterior mean.
#'       Requires \code{store_posterior_sample = TRUE}.}
#'     \item{\code{"vi"}}{Posterior credible intervals for variable inclusion
#'       probabilities (Dirichlet prior only). Controlled by \code{forest}.}
#'   }
#' @param forest For \code{type = "vi"}: which forest to display.
#'   One of \code{"both"} (default), \code{"control"}, or \code{"treat"}.
#'   When \code{"both"}, a named list of two \pkg{ggplot2} objects is returned.
#' @param n_vi Integer; number of top variables for \code{type = "vi"}.
#'   Default \code{10}.
#' @param bayesian_bootstrap Logical; only used when \code{type = "ate"}. If
#'   \code{TRUE} (default), the ATE posterior is computed by reweighting each
#'   iteration's CATE vector with Dirichlet(1, ..., 1) weights, giving the
#'   population ATE (PATE). If \code{FALSE}, equal \eqn{1/n} weights are used,
#'   giving the mixed ATE (MATE).
#' @param ... Additional arguments (currently unused).
#' @return A \pkg{ggplot2} object, or (for \code{type = "vi"} with
#'   \code{forest = "both"}) a named list with elements \code{control} and
#'   \code{treat}.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   set.seed(1)
#'   n <- 60; p <- 5
#'   X <- matrix(rnorm(n * p), ncol = p)
#'   w <- rbinom(n, 1, 0.5)
#'   y <- X[, 1] + w * 1.5 * (X[, 2] > 0) + rnorm(n, sd = 0.5)
#'
#'   fit <- CausalShrinkageForest(
#'     y = y,
#'     X_train_control = X, X_train_treat = X,
#'     treatment_indicator_train = w,
#'     prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
#'     local_hp_control = 0.1, global_hp_control = 0.1,
#'     local_hp_treat  = 0.1, global_hp_treat  = 0.1,
#'     number_of_trees_control = 5, number_of_trees_treat = 5,
#'     N_post = 50, N_burn = 25,
#'     store_posterior_sample = TRUE,
#'     verbose = FALSE
#'   )
#'
#'   plot(fit, type = "trace")
#'   plot(fit, type = "ate")
#'   plot(fit, type = "cate")
#' }
#' }
#' @export
plot.CausalShrinkageForest <- function(x,
                                       type   = c("trace", "density",
                                                  "ate", "cate", "vi"),
                                       forest = c("both", "control", "treat"),
                                       n_vi   = 10,
                                       bayesian_bootstrap = TRUE,
                                       ...) {
  type   <- match.arg(type)
  forest <- match.arg(forest)
  .check_ggplot2()

  n_ch   <- if (!is.null(x$mcmc$n_chains)) x$mcmc$n_chains else 1L
  N_post <- x$mcmc$N_post

  # -- Sigma trace / density --------------------------------------------------
  if (type == "trace") {
    return(.trace_plot(x$sigma, "sigma", N_post, n_ch, ...))
  }

  if (type == "density") {
    return(.density_plot(x$sigma, "sigma", N_post, n_ch, ...))
  }

  # -- ATE posterior ----------------------------------------------------------
  if (type == "ate") {
    st <- x$train_predictions_sample_treat
    if (is.null(st))
      stop("ATE plot requires store_posterior_sample = TRUE.", call. = FALSE)

    ate_draws <- .ate_samples(st, bayesian_bootstrap = bayesian_bootstrap)
    label <- if (bayesian_bootstrap) "PATE" else "MATE"
    return(.area_plot(ate_draws, label, prob = 0.95, ...))
  }

  # -- CATE distribution ------------------------------------------------------
  if (type == "cate") {
    st <- x$train_predictions_sample_treat
    if (is.null(st))
      stop("CATE plot requires store_posterior_sample = TRUE.", call. = FALSE)

    cate_draws <- st
    cate_mean  <- colMeans(cate_draws)
    cate_lo    <- apply(cate_draws, 2, quantile, 0.025)
    cate_hi    <- apply(cate_draws, 2, quantile, 0.975)

    ord <- order(cate_mean)
    df  <- data.frame(
      obs  = seq_along(cate_mean),
      cate = cate_mean[ord],
      lo   = cate_lo[ord],
      hi   = cate_hi[ord]
    )

    return(
      ggplot2::ggplot(df, ggplot2::aes(x = .data$cate, y = .data$obs)) +
        ggplot2::geom_vline(xintercept = 0, linetype = "dashed",
                            colour = "grey50") +
        ggplot2::geom_linerange(
          ggplot2::aes(xmin = .data$lo, xmax = .data$hi),
          colour = "grey70", alpha = 0.35
        ) +
        ggplot2::geom_point(size = 0.8, alpha = 0.6) +
        ggplot2::labs(
          x     = "CATE",
          y     = "Observation (sorted by posterior mean CATE)",
          title = "Conditional Average Treatment Effects"
        ) +
        ggplot2::theme_bw()
    )
  }

  # -- Variable importance ----------------------------------------------------
  if (forest %in% c("control", "both")) {
    sp_c <- x$split_probs_control
    if (is.null(sp_c))
      stop("Control variable importance not available.", call. = FALSE)
    p_control <- .vi_intervals(sp_c, x$data$X_train_control, n_vi,
                               title = "Variable importance - control forest",
                               ...)
  }
  if (forest %in% c("treat", "both")) {
    sp_t <- x$split_probs_treat
    if (is.null(sp_t))
      stop("Treatment variable importance not available.", call. = FALSE)
    p_treat <- .vi_intervals(sp_t, x$data$X_train_treat, n_vi,
                             title = "Variable importance - treatment forest",
                             ...)
  }

  if (forest == "control") return(p_control)
  if (forest == "treat")   return(p_treat)
  list(control = p_control, treat = p_treat)
}


# -- plot.ShrinkageTreesPrediction ---------------------------------------------

#' Plot posterior predictive survival curves
#'
#' Plots posterior predictive survival curves for new observations from
#' a \code{ShrinkageTreesPrediction} object. Only available for survival
#' outcome types (\code{"right-censored"} or \code{"interval-censored"}).
#'
#' @param x A \code{ShrinkageTreesPrediction} object returned by
#'   \code{\link{predict.ShrinkageTrees}} for a survival model.
#' @param type Character; currently only \code{"survival"} is supported.
#' @param obs Integer vector of predicted-observation indices for individual
#'   survival curves, or \code{NULL} (default) for the population-averaged
#'   curve across all predicted observations.
#' @param t_grid Optional numeric vector of time points (on the original
#'   time scale) at which to evaluate the survival function. If \code{NULL}
#'   (default), a grid of 200 equally spaced points is generated
#'   automatically.
#' @param level Width of the pointwise credible band. Default \code{0.95}.
#' @param ... Additional arguments (currently unused).
#' @return A \pkg{ggplot2} object.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   set.seed(1)
#'   n <- 40; p <- 3
#'   X <- matrix(rnorm(n * p), ncol = p)
#'   X_test <- matrix(rnorm(10 * p), ncol = p)
#'   time <- rexp(n, rate = exp(0.5 * X[, 1]))
#'   status <- rbinom(n, 1, 0.7)
#'
#'   fit_surv <- SurvivalBART(
#'     time = time, status = status, X_train = X,
#'     number_of_trees = 5, N_post = 50, N_burn = 25,
#'     store_posterior_sample = TRUE, verbose = FALSE
#'   )
#'
#'   pred <- predict(fit_surv, newdata = X_test)
#'   plot(pred, type = "survival")
#' }
#' }
#' @seealso \code{\link{predict.ShrinkageTrees}},
#'   \code{\link{plot.ShrinkageTrees}}
#' @export
plot.ShrinkageTreesPrediction <- function(x,
                                          type = "survival",
                                          obs = NULL,
                                          t_grid = NULL,
                                          level = 0.95,
                                          ...) {
  type <- match.arg(type, choices = "survival")
  .check_ggplot2()

  df <- .survival_curves_pred(x, obs = obs, t_grid = t_grid, level = level)
  .plot_survival_curves(df, obs = obs, level = level,
                        title_prefix = "Posterior predictive")
}

#' @keywords internal
#' @importFrom stats median quantile pnorm sd
#' @importFrom utils head
"_PACKAGE"

# Suppress R CMD check NOTE for .data used inside ggplot2 aes() calls.
utils::globalVariables(".data")

#' @noRd
NewShrinkageTrees <- function(
  fit,
  call,
  outcome_type,
  timescale,
  prior_type_user,
  prior_type_cpp,
  n_train,
  p_features,
  n_test,
  test_provided,
  number_of_trees,
  N_post,
  N_burn,
  store_posterior_sample,
  sigma_hat,
  sigma_known,
  y_mean,
  # Data stored for predict()
  y_train,
  X_train,
  status_train,
  left_time_train = NULL,
  right_time_train = NULL,
  ic_indicator_train = NULL,
  # Hyperparameters stored for predict()
  power,
  base,
  p_grow,
  p_prune,
  delayed_proposal,
  reversible,
  param1,
  param2,
  omega,
  a_dirichlet,
  b_dirichlet,
  rho_dirichlet,
  nu,
  lambda,
  latent_threshold
) {

  meta <- list(
    call = call,
    outcome_type = outcome_type,
    timescale = timescale,
    prior = list(
      prior_type_user = prior_type_user,
      prior_type_cpp = prior_type_cpp
    ),
    data_info = list(
      n_train = n_train,
      p_features = p_features,
      n_test = n_test,
      test_provided = test_provided
    ),
    mcmc = list(
      number_of_trees = number_of_trees,
      N_post = N_post,
      N_burn = N_burn,
      store_posterior_sample = store_posterior_sample
    ),
    preprocess = list(
      sigma_hat = sigma_hat,
      sigma_known = sigma_known,
      y_mean = y_mean,
      latent_threshold = latent_threshold
    ),
    data = list(
      y_train            = y_train,
      X_train            = X_train,
      status_train       = status_train,
      left_time_train    = left_time_train,
      right_time_train   = right_time_train,
      ic_indicator_train = ic_indicator_train
    ),
    args = list(
      power            = power,
      base             = base,
      p_grow           = p_grow,
      p_prune          = p_prune,
      delayed_proposal = delayed_proposal,
      reversible       = reversible,
      param1           = param1,
      param2           = param2,
      omega            = omega,
      a_dirichlet      = a_dirichlet,
      b_dirichlet      = b_dirichlet,
      rho_dirichlet    = rho_dirichlet,
      nu               = nu,
      lambda           = lambda
    )
  )

  obj <- c(fit, meta)
  class(obj) <- "ShrinkageTrees"
  obj
}

#' @noRd
NewCausalShrinkageForest <- function(
  fit,
  call,
  outcome_type,
  timescale,
  n_train,
  p_control,
  p_treat,
  n_test,
  test_provided,
  number_of_trees_control,
  number_of_trees_treat,
  N_post,
  N_burn,
  store_posterior_sample,
  sigma_hat,
  sigma_known,
  y_mean,
  prior_type_control_user,
  prior_type_control_cpp,
  prior_type_treat_user,
  prior_type_treat_cpp,
  dirichlet_bool_control = FALSE,
  dirichlet_bool_treat = FALSE,
  # Data stored for predict()
  y_train,
  X_train_control,
  X_train_treat,
  treatment_indicator_train,
  status_train,
  left_time_train = NULL,
  right_time_train = NULL,
  ic_indicator_train = NULL,
  # Hyperparameters stored for predict()
  p_grow,
  p_prune,
  delayed_proposal,
  nu,
  lambda,
  power_control,
  base_control,
  param1_control,
  param2_control,
  omega_control,
  reversible_control,
  a_dirichlet_control,
  b_dirichlet_control,
  rho_dirichlet_control,
  power_treat,
  base_treat,
  param1_treat,
  param2_treat,
  omega_treat,
  reversible_treat,
  a_dirichlet_treat,
  b_dirichlet_treat,
  rho_dirichlet_treat,
  treatment_coding = "centered",
  propensity_train = NULL
) {

  meta <- list(
    call = call,
    outcome_type = outcome_type,
    timescale = timescale,

    data_info = list(
      n_train = n_train,
      n_test = n_test,
      test_provided = test_provided,
      p_control = p_control,
      p_treat = p_treat
    ),

    mcmc = list(
      number_of_trees_control = number_of_trees_control,
      number_of_trees_treat = number_of_trees_treat,
      N_post = N_post,
      N_burn = N_burn,
      store_posterior_sample = store_posterior_sample
    ),

    preprocess = list(
      sigma_hat = sigma_hat,
      sigma_known = sigma_known,
      y_mean = y_mean
    ),

    prior = list(
      control = list(
        prior_type_user = prior_type_control_user,
        prior_type_cpp = prior_type_control_cpp,
        dirichlet = dirichlet_bool_control
      ),
      treat = list(
        prior_type_user = prior_type_treat_user,
        prior_type_cpp = prior_type_treat_cpp,
        dirichlet = dirichlet_bool_treat
      )
    ),

    data = list(
      y_train                  = y_train,
      X_train_control          = X_train_control,
      X_train_treat            = X_train_treat,
      treatment_indicator_train = treatment_indicator_train,
      status_train             = status_train,
      left_time_train          = left_time_train,
      right_time_train         = right_time_train,
      ic_indicator_train       = ic_indicator_train,
      propensity_train         = propensity_train
    ),

    treatment_coding = treatment_coding,

    args = list(
      p_grow           = p_grow,
      p_prune          = p_prune,
      delayed_proposal = delayed_proposal,
      nu               = nu,
      lambda           = lambda,
      control = list(
        power       = power_control,
        base        = base_control,
        param1      = param1_control,
        param2      = param2_control,
        omega       = omega_control,
        reversible  = reversible_control,
        a_dirichlet = a_dirichlet_control,
        b_dirichlet = b_dirichlet_control,
        rho_dirichlet = rho_dirichlet_control
      ),
      treat = list(
        power       = power_treat,
        base        = base_treat,
        param1      = param1_treat,
        param2      = param2_treat,
        omega       = omega_treat,
        reversible  = reversible_treat,
        a_dirichlet = a_dirichlet_treat,
        b_dirichlet = b_dirichlet_treat,
        rho_dirichlet = rho_dirichlet_treat
      )
    )
  )

  obj <- c(fit, meta)
  class(obj) <- "CausalShrinkageForest"
  obj
}

#' Posterior predictive inference for a ShrinkageTrees model
#'
#' Re-runs the MCMC sampler on new covariate data using the stored training
#' data and hyperparameters, returning posterior mean predictions and credible
#' interval bounds.
#'
#' @param object A fitted \code{ShrinkageTrees} model object.
#' @param newdata A matrix (or object coercible to one) of new covariates with
#'   the same number of columns as the training data.
#' @param level Credible interval width. Default \code{0.95}.
#' @param ... Currently unused.
#' @return A \code{ShrinkageTreesPrediction} object with elements:
#'   \describe{
#'     \item{mean}{Posterior mean predictions (length \code{nrow(newdata)}).}
#'     \item{lower}{Lower credible interval bound.}
#'     \item{upper}{Upper credible interval bound.}
#'     \item{n}{Number of test observations.}
#'     \item{level}{Credible level used.}
#'     \item{outcome_type}{Outcome type inherited from the fitted model.}
#'     \item{timescale}{Timescale inherited from the fitted model (survival only).}
#'     \item{predictions_sample}{(Survival only) \code{N_post} x \code{n}
#'       matrix of posterior predictive draws on the original scale.}
#'     \item{sigma}{(Survival only) Posterior draws of sigma on the log-time
#'       scale (length \code{N_post}).}
#'   }
#' @seealso \code{\link{HorseTrees}}, \code{\link{ShrinkageTrees}},
#'   \code{\link{print.ShrinkageTreesPrediction}},
#'   \code{\link{summary.ShrinkageTreesPrediction}},
#'   \code{\link{plot.ShrinkageTreesPrediction}}
#' @export
predict.ShrinkageTrees <- function(object, newdata, level = 0.95, ...) {

  if (!is.matrix(newdata)) newdata <- as.matrix(newdata)
  if (ncol(newdata) != object$data_info$p_features)
    stop("newdata has ", ncol(newdata), " columns but the model expects ",
         object$data_info$p_features, ".")

  n_new   <- nrow(newdata)
  X_new   <- as.numeric(t(newdata))
  n_train <- object$data_info$n_train
  p       <- object$data_info$p_features
  pre     <- object$preprocess
  args    <- object$args
  alpha   <- (1 - level) / 2

  if (object$outcome_type == "binary") {

    fit <- probitHorseTrees_cpp(
      nSEXP                    = n_train,
      pSEXP                    = p,
      n_testSEXP               = n_new,
      X_trainSEXP              = object$data$X_train,
      ySEXP                    = object$data$y_train,
      X_testSEXP               = X_new,
      number_of_treesSEXP      = object$mcmc$number_of_trees,
      N_postSEXP               = object$mcmc$N_post,
      N_burnSEXP               = object$mcmc$N_burn,
      delayed_proposalSEXP     = args$delayed_proposal,
      powerSEXP                = args$power,
      baseSEXP                 = args$base,
      p_growSEXP               = args$p_grow,
      p_pruneSEXP              = args$p_prune,
      omegaSEXP                = args$omega,
      latent_thresholdSEXP     = pre$latent_threshold,
      param1SEXP               = args$param1,
      param2SEXP               = args$param2,
      prior_typeSEXP           = object$prior$prior_type_cpp,
      reversibleSEXP           = args$reversible,
      store_posterior_sampleSEXP = TRUE,
      verboseSEXP              = FALSE
    )

    out <- list(
      mean         = pnorm(fit$test_predictions),
      lower        = pnorm(apply(fit$test_predictions_sample, 2, quantile, alpha)),
      upper        = pnorm(apply(fit$test_predictions_sample, 2, quantile, 1 - alpha)),
      n            = n_new,
      level        = level,
      outcome_type = object$outcome_type,
      timescale    = object$timescale
    )
    class(out) <- "ShrinkageTreesPrediction"
    return(out)

  } else {

    y_std    <- (object$data$y_train - pre$y_mean) / pre$sigma_hat
    survival <- object$outcome_type %in% c("right-censored", "interval-censored")
    status   <- if (survival) object$data$status_train else rep(1L, n_train)

    # Reconstruct interval-censored boundaries if applicable
    if (object$outcome_type == "interval-censored") {
      left_std  <- (object$data$left_time_train  - pre$y_mean) / pre$sigma_hat
      right_std <- (object$data$right_time_train - pre$y_mean) / pre$sigma_hat
      ic_ind    <- object$data$ic_indicator_train
    } else {
      left_std  <- numeric(n_train)
      right_std <- y_std + 0
      ic_ind    <- numeric(n_train)
    }

    fit <- HorseTrees_cpp(
      nSEXP                    = n_train,
      pSEXP                    = p,
      n_testSEXP               = n_new,
      X_trainSEXP              = object$data$X_train,
      ySEXP                    = y_std,
      status_indicatorSEXP     = status,
      is_survivalSEXP          = survival,
      observed_left_timeSEXP   = left_std,
      observed_right_timeSEXP  = right_std,
      interval_censoring_indicatorSEXP = ic_ind,
      X_testSEXP               = X_new,
      number_of_treesSEXP      = object$mcmc$number_of_trees,
      N_postSEXP               = object$mcmc$N_post,
      N_burnSEXP               = object$mcmc$N_burn,
      delayed_proposalSEXP     = args$delayed_proposal,
      powerSEXP                = args$power,
      baseSEXP                 = args$base,
      p_growSEXP               = args$p_grow,
      p_pruneSEXP              = args$p_prune,
      nuSEXP                   = args$nu,
      lambdaSEXP               = args$lambda,
      dirichlet_boolSEXP       = isTRUE(object$prior$dirichlet),
      a_dirichletSEXP          = args$a_dirichlet,
      b_dirichletSEXP          = args$b_dirichlet,
      rho_dirichletSEXP        = args$rho_dirichlet,
      sigmaSEXP                = pre$sigma_hat,
      sigma_knownSEXP          = object$preprocess$sigma_known,
      omegaSEXP                = args$omega,
      param1SEXP               = args$param1,
      param2SEXP               = args$param2,
      prior_typeSEXP           = object$prior$prior_type_cpp,
      reversibleSEXP           = args$reversible,
      store_parametersSEXP     = FALSE,
      store_posterior_sampleSEXP = TRUE,
      verboseSEXP              = FALSE
    )

    # De-standardize
    if (survival && object$timescale == "time") {
      mean_pred   <- exp(fit$test_predictions       * pre$sigma_hat + pre$y_mean)
      sample_pred <- exp(fit$test_predictions_sample * pre$sigma_hat + pre$y_mean)
    } else {
      mean_pred   <- fit$test_predictions       * pre$sigma_hat + pre$y_mean
      sample_pred <- fit$test_predictions_sample * pre$sigma_hat + pre$y_mean
    }

    out <- list(
      mean         = mean_pred,
      lower        = apply(sample_pred, 2, quantile, alpha),
      upper        = apply(sample_pred, 2, quantile, 1 - alpha),
      n            = n_new,
      level        = level,
      outcome_type = object$outcome_type,
      timescale    = object$timescale
    )

    # Store posterior samples for survival curve plotting
    if (survival) {
      out$predictions_sample <- sample_pred   # N_post x n_new
      # sigma is jointly sampled with the trees (sigma_known = FALSE for
      # survival); remove burn-in to match predictions_sample rows.
      out$sigma <- fit$sigma[-seq_len(object$mcmc$N_burn)] * pre$sigma_hat
    }

    class(out) <- "ShrinkageTreesPrediction"
    out
  }
}

#' Print a ShrinkageTreesPrediction object
#'
#' Displays a formatted table of posterior mean predictions and credible
#' interval bounds for the first \code{n_head} observations.
#'
#' @param x A \code{ShrinkageTreesPrediction} object.
#' @param n_head Number of observations to display. Default \code{6}.
#' @param digits Number of decimal places. Default \code{3}.
#' @param ... Currently unused.
#' @return Invisibly returns \code{x}.
#' @seealso \code{\link{predict.ShrinkageTrees}},
#'   \code{\link{summary.ShrinkageTreesPrediction}}
#' @export
print.ShrinkageTreesPrediction <- function(x, n_head = 6, digits = 3, ...) {

  ci_pct <- paste0(round(x$level * 100), "%")
  scale_label <- switch(
    x$outcome_type,
    "binary"            = "probability",
    "right-censored"    = if (x$timescale == "time") "survival time" else "log survival time",
    "interval-censored" = if (x$timescale == "time") "survival time" else "log survival time",
    "fitted value"
  )

  cat("\n")
  cat("ShrinkageTrees predictions\n")
  cat("--------------------------\n")
  lbl <- function(s) sprintf("%-22s", s)
  cat(lbl("Observations:"),      x$n,         "\n", sep = "")
  cat(lbl("Credible interval:"), ci_pct,      "\n", sep = "")
  cat(lbl("Scale:"),             scale_label, "\n", sep = "")

  n_show <- min(n_head, x$n)
  cat("\n")
  cat(sprintf("  %5s  %8s  %8s  %8s\n", "", "mean", "lower", "upper"))
  cat(sprintf("  %5s  %8s  %8s  %8s\n", "", strrep("-", 8), strrep("-", 8), strrep("-", 8)))
  for (i in seq_len(n_show)) {
    cat(sprintf("  [%3d]  %8.*f  %8.*f  %8.*f\n",
                i, digits, x$mean[i], digits, x$lower[i], digits, x$upper[i]))
  }
  if (x$n > n_head)
    cat("  ... (", x$n - n_head, " more)\n", sep = "")

  cat("\n")
  invisible(x)
}

#' Summarise a ShrinkageTreesPrediction object
#'
#' Returns distributional summaries (min, Q1, median, max) of the posterior
#' mean predictions and credible interval bounds across all observations.
#'
#' @param object A \code{ShrinkageTreesPrediction} object.
#' @param ... Currently unused.
#' @return A \code{summary.ShrinkageTreesPrediction} object.
#' @seealso \code{\link{predict.ShrinkageTrees}},
#'   \code{\link{print.summary.ShrinkageTreesPrediction}}
#' @export
summary.ShrinkageTreesPrediction <- function(object, ...) {
  out <- list(
    n            = object$n,
    level        = object$level,
    outcome_type = object$outcome_type,
    timescale    = object$timescale,
    mean         = quantile(object$mean,  c(0, 0.25, 0.5, 1)),
    lower        = quantile(object$lower, c(0, 0.25, 0.5, 1)),
    upper        = quantile(object$upper, c(0, 0.25, 0.5, 1))
  )
  class(out) <- "summary.ShrinkageTreesPrediction"
  out
}

#' Print a ShrinkageTreesPrediction summary
#'
#' Displays distributional summaries (min, Q1, median, max) of the posterior
#' mean predictions and credible interval bounds.
#'
#' @param x A \code{summary.ShrinkageTreesPrediction} object.
#' @param digits Number of decimal places. Default \code{3}.
#' @param ... Currently unused.
#' @return Invisibly returns \code{x}.
#' @seealso \code{\link{summary.ShrinkageTreesPrediction}}
#' @export
print.summary.ShrinkageTreesPrediction <- function(x, digits = 3, ...) {

  ci_pct <- paste0(round(x$level * 100), "%")

  cat("\n")
  cat("ShrinkageTrees prediction summary\n")
  cat("----------------------------------\n")
  lbl <- function(s) sprintf("%-22s", s)
  cat(lbl("Observations:"),      x$n,    "\n", sep = "")
  cat(lbl("Credible interval:"), ci_pct, "\n", sep = "")

  cat("\n")
  cat(sprintf("  %-8s  %8s  %8s  %8s\n", "", "mean", "lower", "upper"))
  cat(sprintf("  %-8s  %8s  %8s  %8s\n", "", strrep("-", 8), strrep("-", 8), strrep("-", 8)))
  nms <- c("Min.", "Q1", "Median", "Max.")
  for (i in seq_along(nms)) {
    cat(sprintf("  %-8s  %8.*f  %8.*f  %8.*f\n",
                nms[i], digits, x$mean[i], digits, x$lower[i], digits, x$upper[i]))
  }

  cat("\n")
  invisible(x)
}

#' Print a ShrinkageTrees model
#'
#' Displays a concise summary of a fitted \code{ShrinkageTrees} model,
#' including outcome type, prior, MCMC settings, acceptance ratio, and
#' posterior mean sigma.
#'
#' @param x A fitted \code{ShrinkageTrees} model object.
#' @param ... Currently unused.
#' @return Invisibly returns \code{x}.
#' @seealso \code{\link{summary.ShrinkageTrees}}, \code{\link{HorseTrees}},
#'   \code{\link{ShrinkageTrees}}
#' @export
print.ShrinkageTrees <- function(x, ...) {
  cat("\n")
  cat("ShrinkageTrees model\n")
  cat("---------------------\n")

  # Outcome type
  outcome_label <- switch(
    x$outcome_type,
    "continuous" = "Continuous",
    "binary" = "Binary (probit)",
    "right-censored" = paste0("Right-censored (AFT, timescale = ",
                              x$timescale, ")"),
    "interval-censored" = paste0("Interval-censored (AFT, timescale = ",
                                  x$timescale, ")"),
    x$outcome_type
  )

  lbl <- function(s) sprintf("%-22s", s)

  cat(lbl("Outcome type:"), outcome_label, "\n", sep = "")
  cat(lbl("Prior:"), x$prior$prior_type_user, "\n", sep = "")
  cat(lbl("Number of trees:"), x$mcmc$number_of_trees, "\n", sep = "")
  cat(lbl("Training size (n):"), x$data_info$n_train, "\n", sep = "")
  cat(lbl("Number of features:"), x$data_info$p_features, "\n", sep = "")
  n_ch <- x$mcmc$n_chains
  if (!is.null(n_ch) && n_ch > 1) {
    cat(lbl("Chains:"),          n_ch,         "\n", sep = "")
    cat(lbl("Draws per chain:"), x$mcmc$N_post,
        " (burn-in ", x$mcmc$N_burn, ")\n", sep = "")
  } else {
    cat(lbl("Posterior draws:"), x$mcmc$N_post,
        " (burn-in ", x$mcmc$N_burn, ")\n", sep = "")
  }

  if (!is.null(x$chains)) {
    cat(lbl("Acceptance ratio:"),
        paste(round(x$chains$acceptance_ratios, 3), collapse = ", "),
        " (per chain)\n", sep = "")
  } else if (!is.null(x$acceptance_ratio)) {
    cat(lbl("Acceptance ratio:"), round(mean(x$acceptance_ratio), 3), "\n",
        sep = "")
  }

  if (!x$preprocess$sigma_known && !is.null(x$sigma))
    cat(lbl("Posterior mean sigma:"), round(mean(x$sigma), 3), "\n", sep = "")

  cat("\n")
  invisible(x)
}

# -- summary ------------------------------------------------------------------

.outcome_label <- function(outcome_type, timescale) {
  switch(
    outcome_type,
    "continuous"        = "Continuous",
    "binary"            = "Binary (probit)",
    "right-censored"    = paste0("Right-censored (AFT, timescale = ",
                                  timescale, ")"),
    "interval-censored" = paste0("Interval-censored (AFT, timescale = ",
                                  timescale, ")"),
    outcome_type
  )
}

.sigma_summary <- function(sigma) {
  c(
    mean  = mean(sigma),
    sd    = sd(sigma),
    lower = unname(quantile(sigma, 0.025)),
    upper = unname(quantile(sigma, 0.975))
  )
}

.pred_summary <- function(preds) {
  c(mean = mean(preds), sd = sd(preds),
    min  = min(preds),  max = max(preds))
}

.vi_from_counts <- function(counts) {
  vi <- colMeans(counts / rowSums(counts))
  names(vi) <- if (!is.null(colnames(counts))) colnames(counts)
               else paste0("X", seq_along(vi))
  sort(vi, decreasing = TRUE)
}

.vi_from_probs <- function(probs) {
  vi <- colMeans(probs)
  names(vi) <- if (!is.null(colnames(probs))) colnames(probs)
               else paste0("X", seq_along(vi))
  sort(vi, decreasing = TRUE)
}

#' Summarise a ShrinkageTrees model
#'
#' Returns an inspectable list with posterior sigma summaries, prediction
#' summaries, variable importance (posterior inclusion probabilities), and
#' MCMC diagnostics.
#'
#' @param object A fitted \code{ShrinkageTrees} model object.
#' @param ... Currently unused.
#' @return A \code{summary.ShrinkageTrees} object with elements:
#'   \describe{
#'     \item{call}{The original model call.}
#'     \item{outcome_type}{Outcome type (\code{"continuous"}, \code{"binary"},
#'       \code{"right-censored"}, or \code{"interval-censored"}).}
#'     \item{timescale}{Timescale for survival outcomes (\code{"time"} or
#'       \code{"log"}).}
#'     \item{prior}{Prior specification.}
#'     \item{mcmc}{MCMC settings (trees, draws, burn-in).}
#'     \item{data_info}{Training and test data dimensions.}
#'     \item{sigma}{Named vector with posterior mean, SD, and 95 percent
#'       credible interval of sigma (continuous and survival outcomes only).}
#'     \item{predictions}{List with \code{train} (and optionally \code{test})
#'       prediction summaries (mean, SD, range).}
#'     \item{variable_importance}{Named vector of posterior inclusion
#'       probabilities, sorted decreasingly (if available).}
#'     \item{acceptance_ratio}{MCMC acceptance ratio vector.}
#'     \item{diagnostics}{(When \pkg{coda} is installed) A list with
#'       \code{ess} (effective sample size) and, for multi-chain fits,
#'       \code{rhat} (Gelman--Rubin \eqn{\hat{R}}).}
#'   }
#' @seealso \code{\link{print.summary.ShrinkageTrees}},
#'   \code{\link{as.mcmc.list.ShrinkageTrees}},
#'   \code{\link{HorseTrees}}, \code{\link{ShrinkageTrees}}
#' @export
summary.ShrinkageTrees <- function(object, ...) {

  out <- list(
    call         = object$call,
    outcome_type = object$outcome_type,
    timescale    = object$timescale,
    prior        = object$prior,
    mcmc         = object$mcmc,
    data_info    = object$data_info
  )

  if (!object$preprocess$sigma_known && !is.null(object$sigma))
    out$sigma <- .sigma_summary(object$sigma)

  out$predictions <- list(
    train = .pred_summary(object$train_predictions),
    test  = if (isTRUE(object$data_info$test_provided))
              .pred_summary(object$test_predictions)
  )

  if (isTRUE(object$prior$dirichlet) && !is.null(object$split_probs))
    out$variable_importance <- .vi_from_probs(object$split_probs)
  else if (!is.null(object$store_split_counts))
    out$variable_importance <- .vi_from_counts(object$store_split_counts)

  out$acceptance_ratio <- object$acceptance_ratio
  if (!is.null(object$chains))
    out$chains <- object$chains

  out$diagnostics <- .mcmc_diagnostics(object)

  class(out) <- "summary.ShrinkageTrees"
  out
}

#' Print a ShrinkageTrees model summary
#'
#' Displays a detailed summary of a \code{ShrinkageTrees} model, including
#' model specification, posterior sigma, prediction summaries, variable
#' importance, and MCMC diagnostics.
#'
#' @param x A \code{summary.ShrinkageTrees} object.
#' @param n_vi Maximum number of variables to display in the variable
#'   importance table. Default \code{10}.
#' @param ... Currently unused.
#' @return Invisibly returns \code{x}.
#' @seealso \code{\link{summary.ShrinkageTrees}}
#' @export
print.summary.ShrinkageTrees <- function(x, n_vi = 10, ...) {

  cat("\n")
  cat("ShrinkageTrees model summary\n")
  cat("============================\n")
  cat("Call: "); print(x$call)
  cat("\n")
  cat("Outcome: ", .outcome_label(x$outcome_type, x$timescale),
      " | Prior: ", x$prior$prior_type_user,
      " | Trees: ", x$mcmc$number_of_trees, "\n", sep = "")
  n_ch <- x$mcmc$n_chains
  draws_str <- if (!is.null(n_ch) && n_ch > 1)
    paste0(x$mcmc$N_post, " x ", n_ch, " chains (burn-in ", x$mcmc$N_burn, ")")
  else
    paste0(x$mcmc$N_post, " (burn-in ", x$mcmc$N_burn, ")")
  cat("Data:    n = ", x$data_info$n_train,
      ", p = ", x$data_info$p_features,
      " | Draws: ", draws_str, "\n", sep = "")

  if (!is.null(x$sigma)) {
    cat("\nPosterior sigma:\n")
    cat("  Mean: ", round(x$sigma["mean"],  3),
        "  SD: ",   round(x$sigma["sd"],    3),
        "  95% CI: [", round(x$sigma["lower"], 3),
        ", ",          round(x$sigma["upper"], 3), "]\n", sep = "")
  }

  cat("\nPredictions (posterior mean):\n")
  fmt_pred <- function(p) {
    paste0("mean = ", round(p["mean"], 3),
           ", sd = ",  round(p["sd"],   3),
           ", range = [", round(p["min"], 3), ", ", round(p["max"], 3), "]")
  }
  cat("  Train: ", fmt_pred(x$predictions$train), "\n", sep = "")
  if (!is.null(x$predictions$test))
    cat("  Test:  ", fmt_pred(x$predictions$test), "\n", sep = "")

  if (!is.null(x$variable_importance)) {
    p_vi <- length(x$variable_importance)
    if (p_vi <= n_vi) {
      cat("\nVariable importance (posterior inclusion probability):\n")
      vi <- head(x$variable_importance, n_vi)
      cat(" ", paste(names(vi), round(vi, 3), sep = ": ", collapse = "   "), "\n")
    }
  }

  if (!is.null(x$chains)) {
    cat("\nMCMC acceptance ratio (per chain): ",
        paste(round(x$chains$acceptance_ratios, 3), collapse = ", "), "\n",
        sep = "")
  } else if (!is.null(x$acceptance_ratio)) {
    cat("\nMCMC acceptance ratio: ", round(mean(x$acceptance_ratio), 3), "\n",
        sep = "")
  }

  if (!is.null(x$diagnostics)) {
    cat("\nConvergence diagnostics (coda):\n")
    if (!is.null(x$diagnostics$rhat))
      cat("  Gelman-Rubin R-hat: ",
          paste(names(x$diagnostics$rhat), round(x$diagnostics$rhat, 3),
                sep = " = ", collapse = ", "), "\n", sep = "")
    cat("  Effective sample size: ",
        paste(names(x$diagnostics$ess), round(x$diagnostics$ess, 0),
              sep = " = ", collapse = ", "), "\n", sep = "")
  }

  cat("\n")
  invisible(x)
}

#' Summarise a CausalShrinkageForest model
#'
#' Returns an inspectable list with treatment effect estimates, prognostic
#' function summaries, posterior sigma, variable importance for each forest,
#' and MCMC diagnostics.
#'
#' @param object A fitted \code{CausalShrinkageForest} model object.
#' @param bayesian_bootstrap Logical; if \code{TRUE} (default), the ATE
#'   posterior is computed by reweighting the per-iteration CATE vector with
#'   Dirichlet(1, ..., 1) weights (Bayesian bootstrap). This gives a draw from
#'   the posterior of the \emph{population} ATE (PATE), with a credible
#'   interval that accounts for uncertainty in the covariate distribution. If
#'   \code{FALSE}, equal \eqn{1/n} weights are used, giving the mixed ATE
#'   (MATE), which conditions on the observed covariates. Ignored when
#'   posterior samples are not stored.
#' @param ... Currently unused.
#' @return A \code{summary.CausalShrinkageForest} object with elements:
#'   \describe{
#'     \item{call}{The original model call.}
#'     \item{outcome_type}{Outcome type.}
#'     \item{timescale}{Timescale for survival outcomes.}
#'     \item{prior}{Prior specification for control and treatment forests.}
#'     \item{mcmc}{MCMC settings.}
#'     \item{data_info}{Training and test data dimensions.}
#'     \item{treatment_effect}{List with \code{ate} (posterior mean ATE),
#'       \code{cate_sd} (SD of individual CATEs), and optionally
#'       \code{ate_lower}, \code{ate_upper} (95 percent credible interval;
#'       requires \code{store_posterior_sample = TRUE}) and
#'       \code{bayesian_bootstrap} (the flag used to produce the CI).}
#'     \item{prognostic}{Summary of the prognostic function (mean, SD, range).}
#'     \item{sigma}{Named vector with posterior mean, SD, and 95 percent
#'       credible interval of sigma (if estimated).}
#'     \item{variable_importance_control}{Variable importance for the control
#'       forest (if available).}
#'     \item{variable_importance_treat}{Variable importance for the treatment
#'       forest (if available).}
#'     \item{acceptance_ratios}{List with acceptance ratios for each forest.}
#'   }
#' @seealso \code{\link{print.summary.CausalShrinkageForest}},
#'   \code{\link{CausalHorseForest}}, \code{\link{CausalShrinkageForest}}
#' @export
summary.CausalShrinkageForest <- function(object, bayesian_bootstrap = TRUE, ...) {


  out <- list(
    call         = object$call,
    outcome_type = object$outcome_type,
    timescale    = object$timescale,
    prior        = object$prior,
    mcmc         = object$mcmc,
    data_info    = object$data_info
  )

  tau <- object$train_predictions_treat
  te  <- list(ate = mean(tau), cate_sd = sd(tau))

  if (!is.null(object$train_predictions_sample_treat)) {
    ate_samples  <- .ate_samples(object$train_predictions_sample_treat,
                                 bayesian_bootstrap = bayesian_bootstrap)
    te$ate       <- mean(ate_samples)
    te$ate_lower <- unname(quantile(ate_samples, 0.025))
    te$ate_upper <- unname(quantile(ate_samples, 0.975))
    te$bayesian_bootstrap <- bayesian_bootstrap
  }
  out$treatment_effect <- te

  out$prognostic <- .pred_summary(object$train_predictions_control)

  if (!object$preprocess$sigma_known && !is.null(object$sigma))
    out$sigma <- .sigma_summary(object$sigma)

  if (isTRUE(object$prior$control$dirichlet) && !is.null(object$split_probs_control))
    out$variable_importance_control <- .vi_from_probs(object$split_probs_control)
  else if (!is.null(object$split_counts_control))
    out$variable_importance_control <- .vi_from_counts(object$split_counts_control)

  if (isTRUE(object$prior$treat$dirichlet) && !is.null(object$split_probs_treat))
    out$variable_importance_treat <- .vi_from_probs(object$split_probs_treat)
  else if (!is.null(object$split_counts_treat))
    out$variable_importance_treat <- .vi_from_counts(object$split_counts_treat)

  out$acceptance_ratios <- list(
    control = object$acceptance_ratio_control,
    treat   = object$acceptance_ratio_treat
  )
  if (!is.null(object$chains))
    out$acceptance_ratios$per_chain <- object$chains

  class(out) <- "summary.CausalShrinkageForest"
  out
}

#' Print a CausalShrinkageForest model summary
#'
#' Displays a detailed summary of a \code{CausalShrinkageForest} model,
#' including model specification, treatment effect estimates, prognostic
#' function, posterior sigma, variable importance for each forest, and MCMC
#' diagnostics.
#'
#' @param x A \code{summary.CausalShrinkageForest} object.
#' @param n_vi Maximum number of variables to display per variable importance
#'   table. Default \code{10}.
#' @param ... Currently unused.
#' @return Invisibly returns \code{x}.
#' @seealso \code{\link{summary.CausalShrinkageForest}}
#' @export
print.summary.CausalShrinkageForest <- function(x, n_vi = 10, ...) {
  
  cat("\n")
  cat("CausalShrinkageForest model summary\n")
  cat("=====================================\n")
  cat("Call: "); print(x$call)
  cat("\n")
  cat("Outcome: ", .outcome_label(x$outcome_type, x$timescale), "\n", sep = "")
  cat("Prior:   control = ", x$prior$control$prior_type_user,
      ", treatment = ", x$prior$treat$prior_type_user, "\n", sep = "")
  cat("Trees:   control = ", x$mcmc$number_of_trees_control,
      ", treatment = ", x$mcmc$number_of_trees_treat, "\n", sep = "")
  n_ch <- x$mcmc$n_chains
  draws_str <- if (!is.null(n_ch) && n_ch > 1)
    paste0(x$mcmc$N_post, " x ", n_ch, " chains (burn-in ", x$mcmc$N_burn, ")")
  else
    paste0(x$mcmc$N_post, " (burn-in ", x$mcmc$N_burn, ")")
  cat("Data:    n = ", x$data_info$n_train,
      ", p_control = ", x$data_info$p_control,
      ", p_treat = ", x$data_info$p_treat,
      " | Draws: ", draws_str, "\n", sep = "")

  cat("\nTreatment effect:\n")
  te <- x$treatment_effect
  if (!is.null(te$ate_lower)) {
    ate_label <- if (isTRUE(te$bayesian_bootstrap)) "PATE" else "MATE"
    ci_label  <- if (isTRUE(te$bayesian_bootstrap))
      "95% CI (Bayesian bootstrap)"
    else
      "95% CI"
    cat("  ", ate_label, ":    ", round(te$ate, 4),
        "  ", ci_label, ": [", round(te$ate_lower, 4), ", ",
                               round(te$ate_upper, 4), "]\n", sep = "")
  } else {
    cat("  ATE:     ", round(te$ate, 4),
        "  (no CI -- refit with store_posterior_sample = TRUE)\n", sep = "")
  }
  cat("  CATE SD: ", round(te$cate_sd, 4), "\n", sep = "")

  cat("\nPrognostic function (mu):\n")
  p <- x$prognostic
  cat("  Mean: ", round(p["mean"], 3),
      "  SD: ",   round(p["sd"],   3),
      "  Range: [", round(p["min"], 3), ", ", round(p["max"], 3), "]\n",
      sep = "")

  if (!is.null(x$sigma)) {
    cat("\nPosterior sigma:\n")
    cat("  Mean: ", round(x$sigma["mean"],  3),
        "  SD: ",   round(x$sigma["sd"],    3),
        "  95% CI: [", round(x$sigma["lower"], 3),
        ", ",          round(x$sigma["upper"], 3), "]\n", sep = "")
  }

  fmt_vi <- function(vi) {
    vi <- head(vi, n_vi)
    paste(names(vi), round(vi, 3), sep = ": ", collapse = "   ")
  }
  show_vi <- function(vi, forest_label, access_field) {
    if (is.null(vi)) return(invisible(NULL))
    if (length(vi) <= n_vi) {
      cat("\nVariable importance - ", forest_label,
          " (posterior inclusion probability):\n", sep = "")
      cat(" ", fmt_vi(vi), "\n")
    }
  }
  show_vi(x$variable_importance_control, "control forest",
          "variable_importance_control")
  show_vi(x$variable_importance_treat,   "treatment forest",
          "variable_importance_treat")

  if (!is.null(x$acceptance_ratios$per_chain)) {
    pc <- x$acceptance_ratios$per_chain
    cat("\nMCMC acceptance ratios (per chain):\n")
    cat("  control:   ",
        paste(round(pc$acceptance_ratios_control, 3), collapse = ", "), "\n",
        sep = "")
    cat("  treatment: ",
        paste(round(pc$acceptance_ratios_treat,   3), collapse = ", "), "\n",
        sep = "")
  } else {
    cat("\nMCMC acceptance ratios:",
        " control = ", round(mean(x$acceptance_ratios$control), 3),
        ", treatment = ", round(mean(x$acceptance_ratios$treat), 3), "\n",
        sep = "")
  }

  cat("\n")
  invisible(x)
}

#' Print a CausalShrinkageForest model
#'
#' Displays a concise summary of a fitted \code{CausalShrinkageForest} model
#' with per-forest columns for priors, tree counts, feature counts, and MCMC
#' acceptance ratios.
#'
#' @param x A fitted \code{CausalShrinkageForest} model object.
#' @param ... Currently unused.
#' @return Invisibly returns \code{x}.
#' @seealso \code{\link{summary.CausalShrinkageForest}},
#'   \code{\link{CausalHorseForest}}, \code{\link{CausalShrinkageForest}}
#' @export
print.CausalShrinkageForest <- function(x, ...) {

  cat("\n")
  cat("CausalShrinkageForest model\n")
  cat("---------------------------\n")

  lbl <- function(s) sprintf("%-22s", s)

  # -- General info ----------------------------------------------------------
  cat(lbl("Outcome type:"), .outcome_label(x$outcome_type, x$timescale),
      "\n", sep = "")
  cat(lbl("Training size (n):"), x$data_info$n_train, "\n", sep = "")
  n_ch <- x$mcmc$n_chains
  if (!is.null(n_ch) && n_ch > 1) {
    cat(lbl("Chains:"),          n_ch,          "\n", sep = "")
    cat(lbl("Draws per chain:"), x$mcmc$N_post,
        " (burn-in ", x$mcmc$N_burn, ")\n", sep = "")
  } else {
    cat(lbl("Posterior draws:"), x$mcmc$N_post,
        " (burn-in ", x$mcmc$N_burn, ")\n", sep = "")
  }

  if (!x$preprocess$sigma_known && !is.null(x$sigma))
    cat(lbl("Posterior mean sigma:"), round(mean(x$sigma), 3), "\n", sep = "")

  # -- Per-forest info (side by side) ----------------------------------------
  col  <- function(s) sprintf("%-20s", s)
  hdr  <- function(s) sprintf("%-20s", s)

  cat("\n")
  cat(sprintf("%-22s", ""), hdr("Control"), hdr("Treatment"), "\n", sep = "")
  cat(sprintf("%-22s", ""), strrep("-", 19), " ", strrep("-", 19), "\n",
      sep = "")

  cat(lbl("Prior:"),
      col(x$prior$control$prior_type_user),
      col(x$prior$treat$prior_type_user), "\n", sep = "")
  cat(lbl("Number of trees:"),
      col(x$mcmc$number_of_trees_control),
      col(x$mcmc$number_of_trees_treat), "\n", sep = "")
  cat(lbl("Number of features:"),
      col(x$data_info$p_control),
      col(x$data_info$p_treat), "\n", sep = "")

  if (!is.null(x$chains)) {
    acc_c <- paste(round(x$chains$acceptance_ratios_control, 3), collapse = ", ")
    acc_t <- paste(round(x$chains$acceptance_ratios_treat,   3), collapse = ", ")
    cat(lbl("Acceptance ratio:"), col(acc_c), col(acc_t), "(per chain)\n",
        sep = "")
  } else if (!is.null(x$acceptance_ratio_control) ||
             !is.null(x$acceptance_ratio_treat)) {
    acc_c <- if (!is.null(x$acceptance_ratio_control))
               round(mean(x$acceptance_ratio_control), 3) else "-"
    acc_t <- if (!is.null(x$acceptance_ratio_treat))
               round(mean(x$acceptance_ratio_treat), 3) else "-"
    cat(lbl("Acceptance ratio:"), col(acc_c), col(acc_t), "\n", sep = "")
  }

  cat("\n")
  invisible(x)
}

# ── as.mcmc.list.ShrinkageTrees ───────────────────────────────────────────────

#' Convert MCMC output to a coda mcmc.list
#'
#' Converts the posterior draws stored in a \code{ShrinkageTrees} object into
#' a \code{\link[coda]{mcmc.list}} for use with the \pkg{coda} package's
#' convergence diagnostics (Gelman--Rubin \eqn{\hat{R}}, effective sample
#' size, Geweke test, etc.).
#'
#' @param x A fitted \code{ShrinkageTrees} object.
#' @param ... Currently unused.
#' @return A \code{\link[coda]{mcmc.list}} object.
#'   Each chain is an \code{\link[coda]{mcmc}} object whose columns include:
#'   \describe{
#'     \item{sigma}{Posterior draws of the residual standard deviation
#'       (continuous and survival outcomes only).}
#'   }
#' @details
#' Requires the suggested package \pkg{coda}.
#' For single-chain fits the returned object contains one chain.
#' @seealso \code{\link{summary.ShrinkageTrees}} which reports R-hat and ESS
#'   automatically when coda is available.
#' @examples
#' \donttest{
#' fit <- HorseTrees(y = rnorm(50), X_train = matrix(rnorm(250), 50, 5),
#'                   N_post = 200, N_burn = 100, n_chains = 2)
#' if (requireNamespace("coda", quietly = TRUE)) {
#'   mcmc_obj <- coda::as.mcmc.list(fit)
#'   coda::gelman.diag(mcmc_obj)
#'   coda::effectiveSize(mcmc_obj)
#' }
#' }
#' @exportS3Method coda::as.mcmc.list
as.mcmc.list.ShrinkageTrees <- function(x, ...) {

  if (!requireNamespace("coda", quietly = TRUE))
    stop("Package 'coda' is needed for as.mcmc.list(). ",
         "Install it with: install.packages('coda')", call. = FALSE)

  N_post   <- x$mcmc$N_post
  n_chains <- if (!is.null(x$mcmc$n_chains)) x$mcmc$n_chains else 1L

  # Build a matrix of parameters for each chain
  chain_list <- vector("list", n_chains)
  for (ch in seq_len(n_chains)) {
    idx <- ((ch - 1L) * N_post + 1L):(ch * N_post)
    params <- list()

    if (!is.null(x$sigma))
      params$sigma <- x$sigma[idx]

    mat <- do.call(cbind, params)
    if (!is.matrix(mat)) mat <- matrix(mat, ncol = 1, dimnames = list(NULL, names(params)))
    chain_list[[ch]] <- coda::mcmc(mat, start = 1, end = N_post, thin = 1)
  }

  coda::mcmc.list(chain_list)
}

# ── MCMC diagnostics helpers ─────────────────────────────────────────────────

# Compute Gelman-Rubin R-hat and bulk/tail ESS using coda, if available.
# Returns NULL when coda is not installed or only one chain was run.
.mcmc_diagnostics <- function(object) {
  if (!requireNamespace("coda", quietly = TRUE)) return(NULL)
  n_chains <- if (!is.null(object$mcmc$n_chains)) object$mcmc$n_chains else 1L
  if (is.null(object$sigma)) return(NULL)

  mcmc_obj <- as.mcmc.list.ShrinkageTrees(object)
  ess <- coda::effectiveSize(mcmc_obj)

  rhat <- if (n_chains >= 2L) {
    gr <- coda::gelman.diag(mcmc_obj, autoburnin = FALSE, multivariate = FALSE)
    gr$psrf[, "Point est."]
  }

  list(ess = ess, rhat = rhat)
}

# ── predict.CausalShrinkageForest ─────────────────────────────────────────────

#' Posterior predictive inference for a CausalShrinkageForest model
#'
#' Re-runs the MCMC sampler on new covariate data using the stored training
#' data and hyperparameters, returning posterior mean predictions and credible
#' interval bounds for three quantities: the \strong{prognostic function}
#' (control-forest prediction \eqn{\mu(X)}), the \strong{Conditional Average
#' Treatment Effect} (CATE, \eqn{\tau(X)}), and the \strong{total predicted
#' outcome} (\eqn{\mu(X) + \tau(X)}).
#'
#' @details
#' The causal forest decomposes the expected outcome as
#' \deqn{E[Y \mid X] = \mu(X) + \tau(X) \cdot W,}
#' where \eqn{\mu(X)} is the prognostic function (control forest),
#' \eqn{\tau(X)} is the CATE (treatment forest), and \eqn{W} is the
#' treatment indicator.
#'
#' For \strong{continuous} outcomes and survival with
#' \code{timescale = "log"}, all three components are on the response scale:
#' \code{prognostic} and \code{total} include the intercept shift
#' (\eqn{+ \bar{y}}), while \code{cate} is the pure additive treatment
#' effect with no intercept.
#'
#' For \strong{survival with \code{timescale = "time"}}, predictions are
#' back-transformed to the original time scale:
#' \itemize{
#'   \item \code{prognostic}: posterior expected baseline survival time
#'     \eqn{E[\exp(\mu(X))]}.
#'   \item \code{cate}: multiplicative effect on survival time
#'     \eqn{\exp(\tau(X))}; a value greater than 1 means treatment prolongs
#'     survival.
#'   \item \code{total}: posterior expected survival time under the observed
#'     treatment \eqn{E[\exp(\mu(X) + \tau(X))]}.
#' }
#'
#' @param object A fitted \code{CausalShrinkageForest} model object.
#' @param newdata_control A matrix of new covariates for the control forest,
#'   with the same number of columns as \code{X_train_control} at fit time.
#' @param newdata_treat A matrix of new covariates for the treatment forest,
#'   with the same number of columns as \code{X_train_treat} at fit time.
#'   Must have the same number of rows as \code{newdata_control}.
#' @param level Credible interval width. Default \code{0.95}.
#' @param bayesian_bootstrap Logical; if \code{TRUE} (default), the ATE over
#'   \code{newdata} is computed by reweighting each iteration's CATE vector
#'   with Dirichlet(1, ..., 1) weights (population ATE, PATE). If \code{FALSE},
#'   equal \eqn{1/n} weights are used (mixed ATE, MATE).
#' @param ... Currently unused.
#' @return A \code{CausalShrinkageForestPrediction} object with elements:
#'   \describe{
#'     \item{prognostic}{List with \code{mean}, \code{lower}, \code{upper}:
#'       posterior summaries of the prognostic function
#'       \eqn{\mu(X_{\text{new}})}.}
#'     \item{cate}{List with \code{mean}, \code{lower}, \code{upper}:
#'       posterior summaries of the CATE \eqn{\tau(X_{\text{new}})}.}
#'     \item{total}{List with \code{mean}, \code{lower}, \code{upper}:
#'       posterior summaries of the total outcome
#'       \eqn{\mu(X_{\text{new}}) + \tau(X_{\text{new}})}.}
#'     \item{ate}{List with \code{mean}, \code{lower}, \code{upper}: posterior
#'       summary of the average treatment effect over \code{newdata}. For
#'       survival with \code{timescale = "time"}, reported as a multiplicative
#'       time ratio on the original scale.}
#'     \item{cate_samples}{\eqn{S \times n_{\text{new}}} matrix of posterior
#'       CATE draws on the scale reported in \code{cate}.}
#'     \item{bayesian_bootstrap}{Flag indicating whether the reported ATE CI
#'       used Dirichlet reweighting.}
#'     \item{n}{Number of test observations.}
#'     \item{level}{Credible level used.}
#'     \item{outcome_type}{Outcome type inherited from the fitted model.}
#'     \item{timescale}{Timescale inherited from the fitted model.}
#'   }
#' @seealso \code{\link{CausalHorseForest}}, \code{\link{CausalShrinkageForest}},
#'   \code{\link{print.CausalShrinkageForestPrediction}},
#'   \code{\link{summary.CausalShrinkageForestPrediction}}
#' @export
predict.CausalShrinkageForest <- function(object, newdata_control, newdata_treat,
                                          level = 0.95,
                                          bayesian_bootstrap = TRUE, ...) {

  if (!is.matrix(newdata_control)) newdata_control <- as.matrix(newdata_control)
  if (!is.matrix(newdata_treat))   newdata_treat   <- as.matrix(newdata_treat)

  p_control <- object$data_info$p_control
  p_treat   <- object$data_info$p_treat

  if (ncol(newdata_control) != p_control)
    stop("newdata_control has ", ncol(newdata_control),
         " columns but the model expects ", p_control, ".")
  if (ncol(newdata_treat) != p_treat)
    stop("newdata_treat has ", ncol(newdata_treat),
         " columns but the model expects ", p_treat, ".")

  n_new <- nrow(newdata_control)
  if (nrow(newdata_treat) != n_new)
    stop("newdata_control and newdata_treat must have the same number of rows.")

  n_train  <- object$data_info$n_train
  pre      <- object$preprocess
  args     <- object$args
  survival <- object$outcome_type %in% c("right-censored", "interval-censored")

  # Reconstruct standardised training response (inverse of fit-time transforms)
  y      <- as.numeric(object$data$y_train)
  status <- object$data$status_train
  if (survival && object$timescale == "time") y <- log(y)
  y <- (y - pre$y_mean) / pre$sigma_hat
  if (!survival) status <- rep(1L, n_train)

  # Format matrices for C++ (row-major: each observation is a contiguous block)
  X_train_control <- as.numeric(t(object$data$X_train_control))
  X_train_treat   <- as.numeric(t(object$data$X_train_treat))
  X_test_control  <- as.numeric(t(newdata_control))
  X_test_treat    <- as.numeric(t(newdata_treat))
  trt_train       <- as.integer(object$data$treatment_indicator_train)
  trt_test        <- as.integer(rep(1L, n_new))  # placeholder; not used in predictions

  # Treatment coding and propensity for C++
  tc <- if (!is.null(object$treatment_coding)) object$treatment_coding else "centered"
  prop_train_cpp <- if (!is.null(object$data$propensity_train)) {
    as.numeric(object$data$propensity_train)
  } else {
    numeric(n_train)
  }
  prop_test_cpp <- rep(0.5, n_new)

  alpha <- (1 - level) / 2

  # Reconstruct interval-censored boundaries if applicable
  if (object$outcome_type == "interval-censored") {
    # For causal functions, stored raw values are on original scale
    lt_raw <- object$data$left_time_train
    rt_raw <- object$data$right_time_train
    if (object$timescale == "time") {
      lt_raw <- log(lt_raw)
      rt_raw <- log(rt_raw)
    }
    left_std_c  <- (lt_raw - pre$y_mean) / pre$sigma_hat
    right_std_c <- (rt_raw - pre$y_mean) / pre$sigma_hat
    ic_ind_c    <- object$data$ic_indicator_train
  } else {
    left_std_c  <- numeric(n_train)
    right_std_c <- y + 0
    ic_ind_c    <- numeric(n_train)
  }

  fit <- CausalHorseForest_cpp(
    nSEXP                              = n_train,
    p_treatSEXP                        = p_treat,
    p_controlSEXP                      = p_control,
    X_train_treatSEXP                  = X_train_treat,
    X_train_controlSEXP                = X_train_control,
    ySEXP                              = y,
    status_indicatorSEXP               = status,
    is_survivalSEXP                    = survival,
    observed_left_timeSEXP             = left_std_c,
    observed_right_timeSEXP            = right_std_c,
    interval_censoring_indicatorSEXP   = ic_ind_c,
    treatment_indicatorSEXP            = trt_train,
    n_testSEXP                         = n_new,
    X_test_controlSEXP                 = X_test_control,
    X_test_treatSEXP                   = X_test_treat,
    treatment_indicator_testSEXP       = trt_test,
    no_trees_treatSEXP                 = object$mcmc$number_of_trees_treat,
    power_treatSEXP                    = args$treat$power,
    base_treatSEXP                     = args$treat$base,
    p_grow_treatSEXP                   = args$p_grow,
    p_prune_treatSEXP                  = args$p_prune,
    omega_treatSEXP                    = args$treat$omega,
    prior_type_treatSEXP               = object$prior$treat$prior_type_cpp,
    param1_treatSEXP                   = args$treat$param1,
    param2_treatSEXP                   = args$treat$param2,
    reversible_treatSEXP               = args$treat$reversible,
    dirichlet_bool_treatSEXP           = isTRUE(object$prior$treat$dirichlet),
    a_dirichlet_treatSEXP              = args$treat$a_dirichlet,
    b_dirichlet_treatSEXP              = args$treat$b_dirichlet,
    rho_dirichlet_treatSEXP            = args$treat$rho_dirichlet,
    no_trees_controlSEXP               = object$mcmc$number_of_trees_control,
    power_controlSEXP                  = args$control$power,
    base_controlSEXP                   = args$control$base,
    p_grow_controlSEXP                 = args$p_grow,
    p_prune_controlSEXP                = args$p_prune,
    omega_controlSEXP                  = args$control$omega,
    prior_type_controlSEXP             = object$prior$control$prior_type_cpp,
    param1_controlSEXP                 = args$control$param1,
    param2_controlSEXP                 = args$control$param2,
    reversible_controlSEXP             = args$control$reversible,
    dirichlet_bool_controlSEXP         = isTRUE(object$prior$control$dirichlet),
    a_dirichlet_controlSEXP            = args$control$a_dirichlet,
    b_dirichlet_controlSEXP            = args$control$b_dirichlet,
    rho_dirichlet_controlSEXP          = args$control$rho_dirichlet,
    sigma_knownSEXP                    = TRUE,
    sigmaSEXP                          = pre$sigma_hat,
    lambdaSEXP                         = args$lambda,
    nuSEXP                             = args$nu,
    N_postSEXP                         = object$mcmc$N_post,
    N_burnSEXP                         = object$mcmc$N_burn,
    delayed_proposalSEXP               = args$delayed_proposal,
    store_posterior_sample_controlSEXP = TRUE,
    store_posterior_sample_treatSEXP   = TRUE,
    verboseSEXP                        = FALSE,
    treatment_codingSEXP               = tc,
    propensity_trainSEXP               = prop_train_cpp,
    propensity_testSEXP                = prop_test_cpp
  )

  # C++ does not return a combined total sample; reconstruct from components.
  control_std <- fit$test_predictions_sample_control  # N_post x n_new
  treat_std   <- fit$test_predictions_sample_treat    # N_post x n_new
  total_std   <- control_std + treat_std

  sigma_hat <- pre$sigma_hat
  y_mean    <- pre$y_mean

  if (survival && object$timescale == "time") {
    # Back-transform to original time scale
    prognostic_sample <- exp(control_std * sigma_hat + y_mean)
    cate_sample       <- exp(treat_std   * sigma_hat)          # multiplicative ratio
    total_sample      <- exp(total_std   * sigma_hat + y_mean)
  } else {
    # Continuous or log-scale survival: additive de-standardisation
    prognostic_sample <- control_std * sigma_hat + y_mean
    cate_sample       <- treat_std   * sigma_hat               # no intercept shift
    total_sample      <- total_std   * sigma_hat + y_mean
  }

  ci <- function(mat) list(
    mean  = colMeans(mat),
    lower = apply(mat, 2, quantile, alpha),
    upper = apply(mat, 2, quantile, 1 - alpha)
  )

  # Posterior ATE draws over newdata. For survival-with-time-timescale, reduce
  # on the log scale and exponentiate the summaries; otherwise the per-iteration
  # CATE is already additive on the response scale.
  if (survival && object$timescale == "time") {
    ate_draws_raw <- .ate_samples(treat_std * sigma_hat,
                                  bayesian_bootstrap = bayesian_bootstrap)
    ate_summary <- list(
      mean  = exp(mean(ate_draws_raw)),
      lower = unname(exp(quantile(ate_draws_raw, alpha))),
      upper = unname(exp(quantile(ate_draws_raw, 1 - alpha)))
    )
  } else {
    ate_draws_raw <- .ate_samples(cate_sample,
                                  bayesian_bootstrap = bayesian_bootstrap)
    ate_summary <- list(
      mean  = mean(ate_draws_raw),
      lower = unname(quantile(ate_draws_raw, alpha)),
      upper = unname(quantile(ate_draws_raw, 1 - alpha))
    )
  }

  out <- list(
    prognostic         = ci(prognostic_sample),
    cate               = ci(cate_sample),
    total              = ci(total_sample),
    ate                = ate_summary,
    cate_samples       = cate_sample,
    bayesian_bootstrap = bayesian_bootstrap,
    n                  = n_new,
    level              = level,
    outcome_type       = object$outcome_type,
    timescale          = object$timescale
  )
  class(out) <- "CausalShrinkageForestPrediction"
  out
}

# ── print / summary for CausalShrinkageForestPrediction ───────────────────────

#' Print a CausalShrinkageForestPrediction object
#'
#' Displays a formatted table of posterior mean predictions and credible
#' interval bounds for the first \code{n_head} observations, with separate
#' sections for the prognostic function \eqn{\mu(X)}, the CATE \eqn{\tau(X)},
#' and the total outcome \eqn{\mu(X) + \tau(X)}.
#'
#' @param x A \code{CausalShrinkageForestPrediction} object.
#' @param n_head Number of observations to display per section. Default \code{6}.
#' @param digits Number of decimal places. Default \code{3}.
#' @param ... Currently unused.
#' @return Invisibly returns \code{x}.
#' @seealso \code{\link{predict.CausalShrinkageForest}},
#'   \code{\link{summary.CausalShrinkageForestPrediction}}
#' @export
print.CausalShrinkageForestPrediction <- function(x, n_head = 6, digits = 3, ...) {

  ci_pct     <- paste0(round(x$level * 100), "%")
  time_scale <- x$outcome_type %in% c("right-censored", "interval-censored") && x$timescale == "time"

  cat("\n")
  cat("CausalShrinkageForest predictions\n")
  cat("----------------------------------\n")
  lbl <- function(s) sprintf("%-22s", s)
  cat(lbl("Observations:"),      x$n,            "\n", sep = "")
  cat(lbl("Credible interval:"), ci_pct,         "\n", sep = "")
  cat(lbl("Outcome type:"),      x$outcome_type, "\n", sep = "")

  if (!is.null(x$ate)) {
    ate_label <- if (isTRUE(x$bayesian_bootstrap)) "PATE" else "MATE"
    ci_suffix <- if (isTRUE(x$bayesian_bootstrap))
      " (Bayesian bootstrap)"
    else
      ""
    ate_scale <- if (time_scale) " [time ratio]" else ""
    cat("\n", ate_label, ate_scale, ": ", round(x$ate$mean, digits),
        "  ", ci_pct, " CI", ci_suffix,
        ": [", round(x$ate$lower, digits), ", ",
               round(x$ate$upper, digits), "]\n", sep = "")
  }

  n_show <- min(n_head, x$n)
  hdr    <- sprintf("  %5s  %8s  %8s  %8s\n", "", "mean", "lower", "upper")
  sep    <- sprintf("  %5s  %8s  %8s  %8s\n", "",
                    strrep("-", 8), strrep("-", 8), strrep("-", 8))
  row_fn <- function(i, lst)
    sprintf("  [%3d]  %8.*f  %8.*f  %8.*f\n",
            i, digits, lst$mean[i], digits, lst$lower[i], digits, lst$upper[i])

  print_block <- function(lst, label) {
    cat("\n", label, ":\n", sep = "")
    cat(hdr); cat(sep)
    for (i in seq_len(n_show)) cat(row_fn(i, lst))
    if (x$n > n_head) cat("  ... (", x$n - n_head, " more)\n", sep = "")
  }

  print_block(x$prognostic,
              if (time_scale) "Prognostic (E[T0])" else "Prognostic (mu)")
  print_block(x$cate,
              if (time_scale) "CATE (time ratio, exp(tau))" else "CATE (tau)")
  print_block(x$total,
              if (time_scale) "Total (E[T])" else "Total (mu + tau)")

  cat("\n")
  invisible(x)
}

#' Summarise a CausalShrinkageForestPrediction object
#'
#' Returns distributional summaries (min, Q1, median, max) of the posterior
#' mean predictions and credible interval bounds across all test observations,
#' separately for the prognostic function, CATE, and total outcome.
#'
#' @param object A \code{CausalShrinkageForestPrediction} object.
#' @param ... Currently unused.
#' @return A \code{summary.CausalShrinkageForestPrediction} object.
#' @seealso \code{\link{predict.CausalShrinkageForest}},
#'   \code{\link{print.summary.CausalShrinkageForestPrediction}}
#' @export
summary.CausalShrinkageForestPrediction <- function(object, ...) {
  summ_pred <- function(lst) list(
    mean  = quantile(lst$mean,  c(0, 0.25, 0.5, 1)),
    lower = quantile(lst$lower, c(0, 0.25, 0.5, 1)),
    upper = quantile(lst$upper, c(0, 0.25, 0.5, 1))
  )
  out <- list(
    n            = object$n,
    level        = object$level,
    outcome_type = object$outcome_type,
    timescale    = object$timescale,
    prognostic   = summ_pred(object$prognostic),
    cate         = summ_pred(object$cate),
    total        = summ_pred(object$total)
  )
  class(out) <- "summary.CausalShrinkageForestPrediction"
  out
}

#' Print a CausalShrinkageForestPrediction summary
#'
#' Displays distributional summaries (min, Q1, median, max) of the posterior
#' mean predictions and credible interval bounds, separately for the prognostic
#' function, CATE, and total outcome.
#'
#' @param x A \code{summary.CausalShrinkageForestPrediction} object.
#' @param digits Number of decimal places. Default \code{3}.
#' @param ... Currently unused.
#' @return Invisibly returns \code{x}.
#' @seealso \code{\link{summary.CausalShrinkageForestPrediction}}
#' @export
print.summary.CausalShrinkageForestPrediction <- function(x, digits = 3, ...) {

  ci_pct     <- paste0(round(x$level * 100), "%")
  time_scale <- x$outcome_type %in% c("right-censored", "interval-censored") && x$timescale == "time"

  cat("\n")
  cat("CausalShrinkageForest prediction summary\n")
  cat("-----------------------------------------\n")
  lbl <- function(s) sprintf("%-22s", s)
  cat(lbl("Observations:"),      x$n,    "\n", sep = "")
  cat(lbl("Credible interval:"), ci_pct, "\n", sep = "")

  hdr <- sprintf("  %-8s  %8s  %8s  %8s\n", "", "mean", "lower", "upper")
  sep <- sprintf("  %-8s  %8s  %8s  %8s\n", "",
                 strrep("-", 8), strrep("-", 8), strrep("-", 8))

  print_block <- function(lst, label) {
    cat("\n", label, ":\n", sep = "")
    cat(hdr); cat(sep)
    nms <- c("Min.", "Q1", "Median", "Max.")
    for (i in seq_along(nms))
      cat(sprintf("  %-8s  %8.*f  %8.*f  %8.*f\n",
                  nms[i], digits, lst$mean[i],
                  digits, lst$lower[i], digits, lst$upper[i]))
  }

  print_block(x$prognostic,
              if (time_scale) "Prognostic (E[T0])" else "Prognostic (mu)")
  print_block(x$cate,
              if (time_scale) "CATE (time ratio, exp(tau))" else "CATE (tau)")
  print_block(x$total,
              if (time_scale) "Total (E[T])" else "Total (mu + tau)")

  cat("\n")
  invisible(x)
}

#' Processed TCGA PAAD dataset (pdac)
#'
#' A reduced and cleaned subset of the TCGA pancreatic ductal adenocarcinoma (PAAD)
#' dataset, derived from The Cancer Genome Atlas (TCGA) PAAD cohort. This version,
#' `pdac`, is smaller and simplified for practical analyses and package examples.
#'
#' @details
#' This dataset was originally compiled and curated in the open-source \code{pdacR}
#' package by Torre-Healy et al. (2023), which harmonized and integrated the TCGA
#' PAAD gene expression and clinical data. The current version further reduces and
#' simplifies the data for efficient modeling demonstrations and survival analyses.
#'
#' The data frame includes:
#' \itemize{
#'   \item \strong{time}: Overall survival time in months.
#'   \item \strong{status}: Event indicator; 1 = event occurred, 0 = censored.
#'   \item \strong{treatment}: Binary treatment indicator; 1 = radiation therapy, 0 = control.
#'   \item \strong{age}: Age at initial pathologic diagnosis (numeric).
#'   \item \strong{sex}: Binary sex indicator; 1 = male, 0 = female.
#'   \item \strong{grade}: Tumor differentiation grade (ordinal; 1 = well, 2 = moderate, 3 = poor, 4 = undifferentiated).
#'   \item \strong{tumor.cellularity}: Tumor cellularity estimate (numeric).
#'   \item \strong{tumor.purity}: Tumor purity class (binary; 1 = high, 0 = low).
#'   \item \strong{absolute.purity}: Absolute purity estimate (numeric).
#'   \item \strong{moffitt.cluster}: Moffitt transcriptional subtype (binary; 1 = basal-like, 0 = classical).
#'   \item \strong{meth.leukocyte.percent}: DNA methylation leukocyte estimate (numeric).
#'   \item \strong{meth.purity.mode}: DNA methylation purity mode (numeric).
#'   \item \strong{stage}: Nodal stage indicator (binary; 1 = n1, 0 = n0).
#'   \item \strong{lymph.nodes}: Number of lymph nodes examined (numeric).
#'   \item \strong{Driver gene columns}: Expression values of key driver genes (e.g., KRAS, TP53, CDKN2A, SMAD4, BRCA1, BRCA2).
#'   \item \strong{Other gene columns}: Expression values of ~3,000 most variable non-driver genes (based on median absolute deviation).
#' }
#'
#' @format A data frame with rows corresponding to patients and columns as described above.
#'
#' @source \doi{10.1016/j.ccell.2017.07.007}
#'
#' @references
#' \itemize{
#'   \item Raphael BJ, et al. "Integrated genomic characterization of pancreatic ductal adenocarcinoma." Cancer Cell. 2017 Aug 14;32(2):185–203.e13. PMID: 28810144.
#'   \item Torre-Healy LA, Kawalerski RR, Oh K, et al. "Open-source curation of a pancreatic ductal adenocarcinoma gene expression analysis platform (pdacR) supports a two-subtype model." Communications Biology. 2023; https://doi.org/10.1038/s42003-023-04461-6.
#'   \item The Cancer Genome Atlas (TCGA), PAAD project, DbGaP: phs000178.
#' }
"pdac"


#' Semi-synthesised TCGA Ovarian Cancer Dataset
#'
#' Gene expression and clinical covariates for ovarian cancer patients from
#' The Cancer Genome Atlas (TCGA-OV), combined with semi-synthetic
#' survival outcomes and treatment assignment. Real covariates (age,
#' FIGO stage, tumor grade, gene expression) are retained; survival
#' times, event indicator, and treatment assignment are simulated from a
#' known data-generating process so that the true treatment effect is
#' available for validation (see \code{\link{ovarian_truth}}).
#'
#' @format A data frame with 357 rows (patients) and 1007 columns:
#' \itemize{
#'   \item \strong{OS_time}: Numeric. Observed survival time in days (simulated).
#'   \item \strong{OS_event}: Integer. Event indicator (simulated).
#'     1 = event observed, 0 = right-censored.
#'   \item \strong{treatment}: Integer. Simulated treatment assignment.
#'     1 = carboplatin, 0 = cisplatin. Driven primarily by
#'     \code{year_of_diagnosis} as an instrumental variable
#'     (cisplatin era pre-~2000, carboplatin after).
#'   \item \strong{age}: Integer. Age at initial pathologic diagnosis in years.
#'   \item \strong{figo_stage}: Integer. FIGO stage coded as 2 = Stage II,
#'     3 = Stage III, 4 = Stage IV.
#'   \item \strong{tumor_grade}: Integer. Histologic tumor grade coded as
#'     2 = G2, 3 = G3, 4 = G4. Rows with GX (unknown grade) were
#'     excluded.
#'   \item \strong{year_of_diagnosis}: Integer. Year of initial pathologic diagnosis
#'     (approx. 1992--2013). Used as an instrumental variable for
#'     treatment assignment in the DGP.
#'   \item \strong{right_time, left_time}: Numeric. Interval-censoring bounds
#'     derived from the simulated survival times, suitable for passing
#'     to the package's interval-censored survival interface
#'     (\code{right_time = Inf} for right-censored observations,
#'     \code{left_time == right_time} for exact events).
#'   \item \strong{year_of_diagnosis.1}: Integer. Duplicate of
#'     \code{year_of_diagnosis} left in place from the data-assembly
#'     join; retained for reproducibility and may be ignored.
#'   \item \strong{ENSG...}: Numeric. Log2(TPM + 1) normalised gene expression
#'     levels for 997 Ensembl genes (columns named by versioned Ensembl
#'     gene IDs, e.g. \code{ENSG00000270372.1}). Genes were selected as
#'     the most variable transcripts across TCGA-OV samples, ranked by
#'     median absolute deviation (MAD).
#' }
#' @details
#' RNA-seq data were downloaded from the GDC portal using the
#' \code{TCGAbiolinks} package (STAR - Counts workflow). Expression values
#' were normalised to TPM and log2-transformed as log2(TPM + 1). Genes with
#' median TPM <= 1 across all samples were removed prior to MAD filtering.
#' Clinical data were obtained from the BCR Biotab clinical supplement.
#' Treatment assignment was derived from the drug table
#' (\code{clinical_drug_ov}), restricted to adjuvant (first-line) treatment
#' records. Samples were matched between expression and clinical data using
#' the 12-character TCGA patient barcode.
#'
#' @source \url{https://portal.gdc.cancer.gov/projects/TCGA-OV}
#'
#' @references
#' Cancer Genome Atlas Research Network (2011). Integrated genomic analyses
#' of ovarian carcinoma. \emph{Nature}, 474, 609--615.
#' \doi{10.1038/nature10166}
#'
#' Colaprico, A. et al. (2016). TCGAbiolinks: an R/Bioconductor package for
#' integrative analysis with GDC data. \emph{Nucleic Acids Research}, 44(8).
#' \doi{10.1093/nar/gkv1507}
#'
#' @examples
#' data(ovarian)
#'
#' # Dimensions: patients x (6 clinical + 2000 gene columns)
#' dim(ovarian)
#'
#' # Survival outcome
#' head(ovarian[, c("OS_time", "OS_event", "treatment")])
#'
#' # KM plot by treatment
#' if (requireNamespace("survival", quietly = TRUE)) {
#'   library(survival)
#'   fit <- survfit(Surv(OS_time, OS_event) ~ treatment, data = ovarian)
#'   plot(fit, col = c("blue", "red"), xlab = "Time (days)", ylab = "Survival")
#'   legend("topright", c("Carboplatin", "Cisplatin"), col = c("blue", "red"), lty = 1)
#' }
"ovarian"


#' Ground-truth quantities for the semi-synthesised ovarian dataset
#'
#' The simulated quantities that correspond to the
#' \code{\link{ovarian}} dataset. Because the ovarian outcomes and
#' treatment assignment are generated from a known data-generating
#' process, the underlying potential outcomes, prognostic function,
#' conditional treatment effect, and propensity score are available for
#' validating estimators of treatment effects under right- and
#' interval-censored survival.
#'
#' @format A data frame with one row per patient in \code{\link{ovarian}}
#'   and the following columns:
#' \describe{
#'   \item{true_log_T}{Numeric. True (uncensored) survival time on the
#'     log scale.}
#'   \item{true_T}{Numeric. True (uncensored) survival time on the
#'     original scale.}
#'   \item{true_mu}{Numeric. True prognostic function \eqn{\mu(x)}
#'     (expected log survival time at the reference treatment).}
#'   \item{true_tau}{Numeric. True conditional average treatment effect
#'     \eqn{\tau(x)} on the log-survival scale.}
#'   \item{true_propensity}{Numeric. True propensity for the treated
#'     group (carboplatin) used to simulate the observed assignment.}
#' }
#'
#' @seealso \code{\link{ovarian}} for the observed semi-synthesised data.
#'
#' @examples
#' data(ovarian)
#' data(ovarian_truth)
#' stopifnot(nrow(ovarian) == nrow(ovarian_truth))
#' # True (population) average treatment effect on the log-survival scale:
#' mean(ovarian_truth$true_tau)
"ovarian_truth"

#' Causal Horseshoe Forests
#'
#' This function fits a (Bayesian) Causal Horseshoe Forest.
#' It can be used for estimation of conditional average treatments effects of
#' survival data given high-dimensional covariates. The outcome is decomposed in
#' a prognostic part (control) and a treatment effect part. For both of these,
#' we specify a Horseshoe Trees regression function.
#' Supports continuous, right-censored, and interval-censored outcomes.
#'
#' @param y Outcome vector. For survival, represents follow-up times (can be on
#' original or log scale depending on \code{timescale}). Set to \code{NULL}
#' when using \code{outcome_type = "interval-censored"}, as values are derived
#' from \code{left_time} and \code{right_time}.
#' @param status Optional event indicator vector (1 = event occurred,
#' 0 = censored). Required when \code{outcome_type = "right-censored"}.
#' For interval-censored outcomes, this is derived automatically from
#' \code{left_time} and \code{right_time}.
#' @param X_train_control Covariate matrix for the control forest. Rows 
#' correspond to samples, columns to covariates.
#' @param X_train_treat Covariate matrix for the treatment forest. Rows 
#' correspond to samples, columns to covariates.
#' @param treatment_indicator_train Vector indicating treatment assignment for 
#' training samples (1 = treated, 0 = control).
#' @param X_test_control Optional test covariate matrix for control forest. If 
#' \code{NULL}, defaults to column means of \code{X_train_control}.
#' @param X_test_treat Optional test covariate matrix for treatment forest. If 
#' \code{NULL}, defaults to column means of \code{X_train_treat}.
#' @param treatment_indicator_test Optional vector indicating treatment 
#' assignment for test samples.
#' @param left_time Optional numeric vector of left (lower) time boundaries.
#' Required when \code{outcome_type = "interval-censored"}. Exact events
#' have \code{left_time == right_time}; right-censored observations have
#' \code{right_time = Inf}; interval-censored observations have finite
#' \code{left_time < right_time}.
#' @param right_time Optional numeric vector of right (upper) time boundaries.
#' Required when \code{outcome_type = "interval-censored"}. Use \code{Inf}
#' for right-censored observations.
#' @param outcome_type Type of outcome: one of \code{"continuous"},
#' \code{"right-censored"}, or \code{"interval-censored"}.
#' Default is \code{"continuous"}.
#' @param timescale For survival outcomes: either \code{"time"} (original time
#' scale, log-transformed internally) or \code{"log"} (already log-transformed).
#' Used when \code{outcome_type} is \code{"right-censored"} or
#' \code{"interval-censored"}.
#' @param number_of_trees Number of trees in each forest. Default is 200.
#' @param k Horseshoe prior scale hyperparameter. Default is 0.1. Controls 
#' global-local shrinkage on step heights.
#' @param power Power parameter for tree structure prior. Default is 2.0.
#' @param base Base parameter for tree structure prior. Default is 0.95.
#' @param p_grow Probability of proposing a grow move. Default is 0.4.
#' @param p_prune Probability of proposing a prune move. Default is 0.4.
#' @param nu Degrees of freedom for the error variance prior. Default is 3.
#' @param q Quantile parameter for error variance prior. Default is 0.90.
#' @param sigma Optional known standard deviation of the outcome. If 
#' \code{NULL}, estimated from data.
#' @param N_post Number of posterior samples to store. Default is 5000.
#' @param N_burn Number of burn-in iterations. Default is 5000.
#' @param delayed_proposal Number of delayed iterations before proposal updates.
#'  Default is 5.
#' @param store_posterior_sample Logical; whether to store posterior samples of
#' predictions. Default is \code{FALSE}.
#' @param treatment_coding Treatment coding scheme for the two-forest model.
#'   One of \code{"centered"} (default), \code{"binary"}, \code{"adaptive"},
#'   or \code{"invariant"}.
#'   \code{"centered"} uses \eqn{b_i \in \{-1/2, 1/2\}};
#'   \code{"binary"} uses \eqn{b_i \in \{0, 1\}};
#'   \code{"adaptive"} uses \eqn{b_i = A_i - \hat{e}(x_i)} where
#'   \eqn{\hat{e}(x_i)} is the estimated propensity score;
#'   \code{"invariant"} treats \eqn{b_0, b_1} as parameters estimated within
#'   the Gibbs sampler with \eqn{b_j \sim N(0, 1/2)} priors, yielding a
#'   parameterisation-invariant model (Hahn et al., 2020, Section 5.2).
#' @param propensity Optional numeric vector of propensity scores
#'   \eqn{\hat{e}(x_i)} for training observations. Required when
#'   \code{treatment_coding = "adaptive"}.
#' @param propensity_test Optional numeric vector of propensity scores for
#'   test observations. Only used when \code{treatment_coding = "adaptive"}.
#'   Defaults to \code{0.5} for all test observations if not provided.
#' @param n_chains Number of independent MCMC chains to run. Default is
#'   \code{1} (standard single-chain behaviour). When \code{n_chains > 1} the
#'   chains are run in parallel via \code{parallel::mclapply} and their
#'   posterior samples are pooled into a single \code{CausalShrinkageForest}
#'   object, so all existing \code{print} and \code{summary} methods work
#'   without modification. On Windows, \code{mclapply} falls back to
#'   sequential execution.
#' @param verbose Logical; whether to print verbose output during sampling.
#' Default is \code{TRUE}.
#'
#' @return An S3 object of class \code{"CausalShrinkageForest"} containing:
#' \describe{
#'   \item{train_predictions}{Posterior mean predictions on training data 
#'   (combined forest).}
#'   \item{test_predictions}{Posterior mean predictions on test data 
#'   (combined forest).}
#'   \item{train_predictions_control}{Estimated control outcomes on training 
#'   data.}
#'   \item{test_predictions_control}{Estimated control outcomes on test data.}
#'   \item{train_predictions_treat}{Estimated treatment effects on training 
#'   data.}
#'   \item{test_predictions_treat}{Estimated treatment effects on test data.}
#'   \item{sigma}{Vector of posterior samples for the error standard deviation.}
#'   \item{acceptance_ratio_control}{Average acceptance ratio in control 
#'   forest.}
#'   \item{acceptance_ratio_treat}{Average acceptance ratio in treatment 
#'   forest.}
#'   \item{train_predictions_sample_control}{Matrix of posterior samples for 
#'   control predictions (if \code{store_posterior_sample = TRUE}).}
#'   \item{test_predictions_sample_control}{Matrix of posterior samples for 
#'   control predictions (if \code{store_posterior_sample = TRUE}).}
#'   \item{train_predictions_sample_treat}{Matrix of posterior samples for 
#'   treatment effects (if \code{store_posterior_sample = TRUE}).}
#'   \item{test_predictions_sample_treat}{Matrix of posterior samples for 
#'   treatment effects (if \code{store_posterior_sample = TRUE}).}
#' }
#'
#' @details
#' The model separately regularizes the control and treatment trees using
#' Horseshoe priors with global-local shrinkage on the step heights.
#' This approach is designed for robust estimation of heterogeneous treatment
#' effects in high-dimensional settings.
#' It supports continuous, right-censored, and interval-censored survival
#' outcomes. For interval-censored data, provide \code{left_time} and
#' \code{right_time} instead of \code{y} and \code{status}; the event
#' indicators are derived internally following the
#' \code{survival::Surv(type = "interval2")} convention.
#' 
#' @examples
#' # Example: Continuous outcome and homogeneous treatment effect
#' n <- 50
#' p <- 3
#' X_control <- matrix(runif(n * p), ncol = p)
#' X_treat <- matrix(runif(n * p), ncol = p)
#' treatment <- rbinom(n, 1, 0.5)
#' tau <- 2
#' y <- X_control[, 1] + (0.5 - treatment) * tau + rnorm(n)
#'
#' fit <- CausalHorseForest(
#'   y = y,
#'   X_train_control = X_control,
#'   X_train_treat = X_treat,
#'   treatment_indicator_train = treatment,
#'   outcome_type = "continuous",
#'   number_of_trees = 5,
#'   N_post = 10,
#'   N_burn = 5,
#'   store_posterior_sample = TRUE,
#'   verbose = FALSE
#' )
#'
#' \donttest{
#' ## Example: Right-censored survival outcome
#' # Set data dimensions
#' n <- 100
#' p <- 1000
#' 
#' # Generate covariates
#' X <- matrix(runif(n * p), ncol = p)
#' X_treat <- X
#' treatment <- rbinom(n, 1, pnorm(X[, 1] - 1/2))
#' 
#' # Generate true survival times depending on X and treatment
#' linpred <- X[, 1] - X[, 2] + (treatment - 0.5) * (1 + X[, 2] / 2 + X[, 3] / 3 
#'                                                   + X[, 4] / 4)
#' true_time <- linpred + rnorm(n, 0, 0.5)
#' 
#' # Generate censoring times
#' censor_time <- log(rexp(n, rate = 1 / 5))
#' 
#' # Observed times and event indicator
#' time_obs <- pmin(true_time, censor_time)
#' status <- as.numeric(true_time == time_obs)
#' 
#' # Estimate propensity score using HorseTrees
#' fit_prop <- HorseTrees(
#'   y = treatment,
#'   X_train = X,
#'   outcome_type = "binary",
#'   number_of_trees = 200,
#'   N_post = 1000,
#'   N_burn = 1000
#' )
#' 
#' # Retrieve estimated probability of treatment (propensity score)
#' propensity <- fit_prop$train_probabilities
#' 
#' # Combine propensity score with covariates for control forest
#' X_control <- cbind(propensity, X)
#' 
#' # Fit the Causal Horseshoe Forest for survival outcome
#' fit_surv <- CausalHorseForest(
#'   y = time_obs,
#'   status = status,
#'   X_train_control = X_control,
#'   X_train_treat = X_treat,
#'   treatment_indicator_train = treatment,
#'   outcome_type = "right-censored",
#'   timescale = "log",
#'   number_of_trees = 200,
#'   k = 0.1,
#'   N_post = 1000,
#'   N_burn = 1000,
#'   store_posterior_sample = TRUE
#' )
#' 
#' ## Evaluate and summarize results
#' 
#' # Evaluate C-index if survival package is available
#' if (requireNamespace("survival", quietly = TRUE)) {
#'   predicted_survtime <- fit_surv$train_predictions
#'   cindex_result <- survival::concordance(survival::Surv(time_obs, status) ~ predicted_survtime)
#'   c_index <- cindex_result$concordance
#'   cat("C-index:", round(c_index, 3), "\n")
#' } else {
#'   cat("Package 'survival' not available. Skipping C-index computation.\n")
#' }
#' 
#' # Compute posterior ATE samples
#' ate_samples <- rowMeans(fit_surv$train_predictions_sample_treat)
#' mean_ate <- mean(ate_samples)
#' ci_95 <- quantile(ate_samples, probs = c(0.025, 0.975))
#' 
#' cat("Posterior mean ATE:", round(mean_ate, 3), "\n")
#' cat("95% credible interval: [", round(ci_95[1], 3), ", ", round(ci_95[2], 3), "]\n", sep = "")
#' 
#' # Plot histogram of ATE samples
#' hist(
#'   ate_samples,
#'   breaks = 30,
#'   col = "steelblue",
#'   freq = FALSE,
#'   border = "white",
#'   xlab = "Average Treatment Effect (ATE)",
#'   main = "Posterior distribution of ATE"
#' )
#' abline(v = mean_ate, col = "orange3", lwd = 2)
#' abline(v = ci_95, col = "orange3", lty = 2, lwd = 2)
#' abline(v = 1.541667, col = "darkred", lwd = 2)
#' legend(
#'   "topright",
#'   legend = c("Mean", "95% CI", "Truth"),
#'   col = c("orange3", "orange3", "red"),
#'   lty = c(1, 2, 1),
#'   lwd = 2
#' )
#' 
#' ## Plot individual CATE estimates
#' 
#' # Summarize posterior distribution per patient
#' posterior_matrix <- fit_surv$train_predictions_sample_treat
#' posterior_mean <- colMeans(posterior_matrix)
#' posterior_ci <- apply(posterior_matrix, 2, quantile, probs = c(0.025, 0.975))
#' 
#' df_cate <- data.frame(
#'   mean = posterior_mean,
#'   lower = posterior_ci[1, ],
#'   upper = posterior_ci[2, ]
#' )
#' 
#' # Sort patients by posterior mean CATE
#' df_cate_sorted <- df_cate[order(df_cate$mean), ]
#' n_patients <- nrow(df_cate_sorted)
#' 
#' # Create the plot
#' plot(
#'   x = df_cate_sorted$mean,
#'   y = 1:n_patients,
#'   type = "n",
#'   xlab = "CATE per patient (95% credible interval)",
#'   ylab = "Patient index (sorted)",
#'   main = "Posterior CATE estimates",
#'   xlim = range(df_cate_sorted$lower, df_cate_sorted$upper)
#' )
#' 
#' # Add CATE intervals
#' segments(
#'   x0 = df_cate_sorted$lower,
#'   x1 = df_cate_sorted$upper,
#'   y0 = 1:n_patients,
#'   y1 = 1:n_patients,
#'   col = "steelblue"
#' )
#' 
#' # Add mean points
#' points(df_cate_sorted$mean, 1:n_patients, pch = 16, col = "orange3", lwd = 0.1)
#' 
#' # Add reference line at 0
#' abline(v = 0, col = "black", lwd = 2)
#' 
#' }
#' 
#' @seealso
#' Model family: \code{\link{HorseTrees}} (non-causal, horseshoe prior),
#' \code{\link{ShrinkageTrees}} (non-causal, flexible prior),
#' \code{\link{CausalShrinkageForest}} (causal, flexible prior).
#'
#' Survival wrappers: \code{\link{SurvivalBCF}}, \code{\link{SurvivalShrinkageBCF}}.
#'
#' S3 methods: \code{\link{print.CausalShrinkageForest}},
#' \code{\link{summary.CausalShrinkageForest}},
#' \code{\link{predict.CausalShrinkageForest}},
#' \code{\link{plot.CausalShrinkageForest}}.
#'
#' @importFrom Rcpp evalCpp
#' @useDynLib ShrinkageTrees, .registration = TRUE
#' @importFrom stats sd qchisq runif
#' @importFrom parallel mclapply detectCores
#' @export
CausalHorseForest <- function(y = NULL,
                              status = NULL,
                              X_train_control,
                              X_train_treat,
                              treatment_indicator_train,
                              X_test_control = NULL,
                              X_test_treat = NULL,
                              treatment_indicator_test = NULL,
                              left_time = NULL,
                              right_time = NULL,
                              outcome_type = "continuous",
                              timescale = "time",
                              number_of_trees = 200,
                              k = 0.1,
                              power = 2.0,
                              base = 0.95,
                              p_grow = 0.4,
                              p_prune = 0.4,
                              nu = 3,
                              q = 0.90,
                              sigma = NULL,
                              N_post = 5000,
                              N_burn = 5000,
                              delayed_proposal = 5,
                              store_posterior_sample = FALSE,
                              treatment_coding = "centered",
                              propensity = NULL,
                              propensity_test = NULL,
                              n_chains = 1,
                              verbose = TRUE) {

  # ── Multi-chain dispatch ──────────────────────────────────────────────────
  if (n_chains > 1) {
    mc <- match.call()

    chain_args <- list(
      y                         = y,
      status                    = status,
      X_train_control           = X_train_control,
      X_train_treat             = X_train_treat,
      treatment_indicator_train = treatment_indicator_train,
      X_test_control            = X_test_control,
      X_test_treat              = X_test_treat,
      treatment_indicator_test  = treatment_indicator_test,
      left_time                 = left_time,
      right_time                = right_time,
      outcome_type              = outcome_type,
      timescale                 = timescale,
      number_of_trees           = number_of_trees,
      k                         = k,
      power                     = power,
      base                      = base,
      p_grow                    = p_grow,
      p_prune                   = p_prune,
      nu                        = nu,
      q                         = q,
      sigma                     = sigma,
      N_post                    = N_post,
      N_burn                    = N_burn,
      delayed_proposal          = delayed_proposal,
      store_posterior_sample    = store_posterior_sample,
      treatment_coding          = treatment_coding,
      propensity                = propensity,
      propensity_test           = propensity_test,
      n_chains                  = 1,
      verbose                   = FALSE
    )

    n_cores <- .resolve_cores(n_chains)
    if (verbose)
      message("Running ", n_chains, " chains (", n_cores, " cores) ...")

    chains <- parallel::mclapply(
      seq_len(n_chains),
      function(i) do.call(CausalHorseForest, chain_args),
      mc.cores = n_cores
    )

    combined      <- .combine_causal_chains(chains)
    combined$call <- mc
    return(combined)
  }

  # Check outcome_type value
  allowed_types <- c("continuous", "right-censored", "interval-censored")
  if (!outcome_type %in% allowed_types) {
    stop("Invalid outcome_type. Please choose 'continuous', 'right-censored',
         or 'interval-censored'.")
  }

  # Check that y is provided for non-interval-censored types
  if (is.null(y) && outcome_type != "interval-censored") {
    stop("'y' is required when outcome_type is not 'interval-censored'.")
  }

  # Check interval-censored arguments
  if (outcome_type == "interval-censored") {
    if (is.null(left_time) || is.null(right_time))
      stop("outcome_type = 'interval-censored' requires 'left_time' and 'right_time'.")
    if (length(left_time) != length(right_time))
      stop("'left_time' and 'right_time' must have the same length.")
    if (any(left_time > right_time))
      stop("All 'left_time' values must be <= corresponding 'right_time' values.")
    if (timescale == "time" && any(left_time <= 0))
      stop("left_time contains non-positive values, but timescale = 'time' requires strictly positive times.")
  }

  # Check consistency with status argument
  if (outcome_type == "right-censored" && is.null(status)) {
    stop("You specified outcome_type = 'right-censored', but did not provide a
         'status' vector.")
  }
  if (!is.null(status) && !is.null(y) && length(status) != length(y)) {
    stop("The length of 'status' must match the length of 'y'.")
  }

  if (!outcome_type %in% c("right-censored", "interval-censored") && !is.null(status)) {
    warning("You provided a 'status' vector, but outcome_type is not
            'right-censored' or 'interval-censored'. The 'status' vector will be ignored.")
  }

  # Check survival data and timescale
  if (outcome_type == "right-censored" && timescale == "time" && any(y < 0)) {
    stop("Outcome contains negative values, but timescale = 'time' for survival
         data requires non-negative times.")
  }
  
  # Coerce to matrix so nrow/ncol are always defined
  if (!is.matrix(X_train_control)) X_train_control <- as.matrix(X_train_control)
  if (!is.matrix(X_train_treat))   X_train_treat   <- as.matrix(X_train_treat)

  # Retrieve dimensions of training data
  n_train   <- nrow(X_train_control)
  p_control <- ncol(X_train_control)
  p_treat   <- ncol(X_train_treat)

  # Check matching row numbers
  n_obs <- if (outcome_type == "interval-censored") length(left_time) else length(y)
  if (nrow(X_train_control) != n_obs) {
    stop("X_train_control rows must match length of outcome data.")
  }
  if (nrow(X_train_treat) != n_obs) {
    stop("X_train_treat rows must match length of outcome data.")
  }
  if (length(treatment_indicator_train) != n_obs) {
    stop("treatment_indicator_train must match length of outcome data.")
  }
  if (!all(treatment_indicator_train %in% c(0, 1))) {
    stop("treatment_indicator_train must contain only 0 and 1.")
  }

  # Validate treatment_coding
  treatment_coding <- match.arg(treatment_coding, c("centered", "binary", "adaptive", "invariant"))
  if (treatment_coding == "adaptive" && is.null(propensity)) {
    stop("propensity scores are required when treatment_coding = 'adaptive'.")
  }
  if (!is.null(propensity) && length(propensity) != n_obs) {
    stop("Length of 'propensity' must match the number of training observations.")
  }

  # Prepare propensity vectors for C++
  propensity_train_cpp <- if (!is.null(propensity)) as.numeric(propensity) else numeric(n_obs)

  # If test provided, check
  test_provided_flag <- !is.null(X_test_control) && !is.null(X_test_treat)
  if (!is.null(X_test_control) && !is.null(X_test_treat)) {
    n_test <- nrow(X_test_control)
    if (ncol(X_test_control) != p_control || ncol(X_test_treat) != p_treat) {
      stop("Number of columns in X_test_control or X_test_treat does not match 
           training data.")
    }
    if (!is.null(treatment_indicator_test) && 
        length(treatment_indicator_test) != n_test) {
      stop("treatment_indicator_test length must match number of test rows.")
    }
    X_test_control <- as.numeric(t(X_test_control))
    X_test_treat <- as.numeric(t(X_test_treat))
  } else {
    n_test <- 1
    X_test_control <- as.numeric(colMeans(X_train_control))
    X_test_treat <- as.numeric(colMeans(X_train_treat))
    treatment_indicator_test <- as.integer(rep(1, n_test))
  }
  
  # Treatment indicators as integer
  treatment_indicator_train <- as.integer(treatment_indicator_train)
  if (is.null(treatment_indicator_test)) {
    treatment_indicator_test <- as.integer(rep(1, n_test))
  } else {
    treatment_indicator_test <- as.integer(treatment_indicator_test)
  }

  # Prepare propensity for test data
  propensity_test_cpp <- if (!is.null(propensity_test)) {
    as.numeric(propensity_test)
  } else {
    rep(0.5, n_test)
  }

  prior_type_control_user <- "horseshoe"
  prior_type_treat_user <- "horseshoe"
  
  # Force data types to be numeric and plain arrays
  N_post <- as.integer(N_post)[1]
  N_burn <- as.integer(N_burn)[1]
  power <- as.numeric(power)[1]
  base <- as.numeric(base)[1]
  p_grow <- as.numeric(p_grow)[1]
  p_prune <- as.numeric(p_prune)[1]
  X_control_mat <- X_train_control        # preserve matrix for predict() / vi plots
  X_treat_mat   <- X_train_treat
  X_train_treat <- as.numeric(t(X_train_treat))
  X_train_control <- as.numeric(t(X_train_control))

  # For interval-censored: derive y/status/ic_indicator from left_time/right_time
  # before saving originals
  if (outcome_type == "interval-censored") {
    left_time <- as.numeric(left_time)
    right_time <- as.numeric(right_time)

    status <- as.integer(left_time == right_time)
    ic_indicator <- as.integer(left_time < right_time & is.finite(right_time))

    y <- ifelse(status == 1, left_time,
                ifelse(ic_indicator == 1, (left_time + right_time) / 2,
                       left_time))

    # C++ expects finite boundary for right-censored obs
    right_time[!is.finite(right_time)] <- left_time[!is.finite(right_time)]
  }

  # Save originals for data slot (used by predict())
  y_horse_raw      <- y
  status_horse_raw <- status
  left_time_raw    <- if (outcome_type == "interval-censored") left_time else NULL
  right_time_raw   <- if (outcome_type == "interval-censored") right_time else NULL

  if (outcome_type == "right-censored") {

    # Convert y to numeric for C++ compatibility
    y <- as.numeric(y)

    # Log-transform survival times if timescale = "time"
    if (timescale == "time") {
      y <- log(y)
    }

    # Obtain estimated mean and standard deviation using censored_info()
    cens_inf <- censored_info(y, status)
    
    # Center the data using estimated mean
    y_mean <- cens_inf$mu
    y <- y - y_mean
    
    # Determine sigma (timescale parameter) and whether it is known
    if (is.null(sigma)) {
      sigma_hat <- cens_inf$sd
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma
      sigma_known <- TRUE
    }
    
    # Standardize the centered data
    y <- y / sigma_hat
    
    # Set survival flag
    survival <- TRUE
    
    # Compute lambda parameter for error distribution prior
    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu
    
    # Fit a Causal Horseshoe Forest
    fit <- CausalHorseForest_cpp(
      nSEXP = n_train,
      p_treatSEXP = p_treat,
      p_controlSEXP = p_control,
      X_train_treatSEXP = X_train_treat,
      X_train_controlSEXP = X_train_control,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = numeric(n_train),
      observed_right_timeSEXP = y + 0,
      interval_censoring_indicatorSEXP = numeric(n_train),
      treatment_indicatorSEXP = treatment_indicator_train,
      n_testSEXP = n_test,
      X_test_controlSEXP = X_test_control,
      X_test_treatSEXP = X_test_treat,
      treatment_indicator_testSEXP = treatment_indicator_test,
      no_trees_treatSEXP = number_of_trees,
      power_treatSEXP = power,
      base_treatSEXP = base,
      p_grow_treatSEXP = p_grow,
      p_prune_treatSEXP = p_prune,
      omega_treatSEXP = 1/2,
      prior_type_treatSEXP = "horseshoe",
      param1_treatSEXP = k / sqrt(number_of_trees),
      param2_treatSEXP = k / sqrt(number_of_trees),
      reversible_treatSEXP = TRUE,
      dirichlet_bool_treatSEXP = FALSE,
      a_dirichlet_treatSEXP = 1.0,
      b_dirichlet_treatSEXP = 1.0, 
      rho_dirichlet_treatSEXP = 1.0,  
      no_trees_controlSEXP = number_of_trees,
      power_controlSEXP = power,
      base_controlSEXP = base,
      p_grow_controlSEXP = p_grow,
      p_prune_controlSEXP = p_prune,
      omega_controlSEXP = 1/2,
      prior_type_controlSEXP = "horseshoe",
      param1_controlSEXP = k / sqrt(number_of_trees),
      param2_controlSEXP = k / sqrt(number_of_trees),
      reversible_controlSEXP = TRUE,
      dirichlet_bool_controlSEXP = FALSE,
      a_dirichlet_controlSEXP = 1.0,
      b_dirichlet_controlSEXP = 1.0, 
      rho_dirichlet_controlSEXP = 1.0,  
      sigma_knownSEXP = sigma_known,
      sigmaSEXP = sigma_hat,
      lambdaSEXP = lambda,
      nuSEXP = nu,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      store_posterior_sample_controlSEXP = store_posterior_sample,
      store_posterior_sample_treatSEXP = store_posterior_sample,
      verboseSEXP = verbose,
      treatment_codingSEXP = treatment_coding,
      propensity_trainSEXP = propensity_train_cpp,
      propensity_testSEXP = propensity_test_cpp
    )
   
    if (timescale == "time") {
      
      # Total
      fit$train_predictions <- exp(fit$train_predictions * sigma_hat + y_mean)
      fit$test_predictions <- exp(fit$test_predictions * sigma_hat + y_mean)
      
      # Control
      fit$train_predictions_control <- exp(fit$train_predictions_control * 
                                             sigma_hat + y_mean)
      fit$test_predictions_control <- exp(fit$test_predictions_control * 
                                            sigma_hat + y_mean)
      
      # Treatment
      fit$train_predictions_treat <- exp(fit$train_predictions_treat * 
                                           sigma_hat)
      fit$test_predictions_treat <- exp(fit$test_predictions_treat * sigma_hat)
      
      # Posterior samples
      if (store_posterior_sample) { #
        fit$train_predictions_sample_control <- 
          exp(fit$train_predictions_sample_control * sigma_hat + y_mean)
        fit$test_predictions_sample_control <- 
          exp(fit$test_predictions_sample_control * sigma_hat + y_mean)
        fit$train_predictions_sample_treat <- 
          exp(fit$train_predictions_sample_treat * sigma_hat)
        fit$test_predictions_sample_treat <- 
          exp(fit$test_predictions_sample_treat * sigma_hat)
      }
    } else {
      # Total
      fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
      fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
      
      # Control
      fit$train_predictions_control <- 
        fit$train_predictions_control * sigma_hat + y_mean
      fit$test_predictions_control <- 
        fit$test_predictions_control * sigma_hat + y_mean
      
      # Treatment
      fit$train_predictions_treat <- fit$train_predictions_treat * sigma_hat
      fit$test_predictions_treat <- fit$test_predictions_treat * sigma_hat
      
      # Posterior samples
      if (store_posterior_sample) {
        fit$train_predictions_sample_control <- 
          fit$train_predictions_sample_control * sigma_hat + y_mean
        fit$test_predictions_sample_control <- 
          fit$test_predictions_sample_control * sigma_hat + y_mean
        fit$train_predictions_sample_treat <- 
          fit$train_predictions_sample_treat * sigma_hat
        fit$test_predictions_sample_treat <- 
          fit$test_predictions_sample_treat * sigma_hat
      }
    }
    
  # If interval-censored
  } else if (outcome_type == "interval-censored") {

    y <- as.numeric(y)

    # Log-transform if timescale = "time"
    if (timescale == "time") {
      y <- log(y)
      left_time <- log(left_time)
      right_time <- log(right_time)
    }

    # Estimate mu and sd
    cens_inf <- censored_info(y, status, left_time = left_time,
                              right_time = right_time, ic_indicator = ic_indicator)

    y_mean <- cens_inf$mu
    y <- y - y_mean
    left_time <- left_time - y_mean
    right_time <- right_time - y_mean

    if (is.null(sigma)) {
      sigma_hat <- cens_inf$sd
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma
      sigma_known <- TRUE
    }

    y <- y / sigma_hat
    left_time <- left_time / sigma_hat
    right_time <- right_time / sigma_hat

    survival <- TRUE

    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu

    fit <- CausalHorseForest_cpp(
      nSEXP = n_train,
      p_treatSEXP = p_treat,
      p_controlSEXP = p_control,
      X_train_treatSEXP = X_train_treat,
      X_train_controlSEXP = X_train_control,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = left_time,
      observed_right_timeSEXP = right_time,
      interval_censoring_indicatorSEXP = ic_indicator,
      treatment_indicatorSEXP = treatment_indicator_train,
      n_testSEXP = n_test,
      X_test_controlSEXP = X_test_control,
      X_test_treatSEXP = X_test_treat,
      treatment_indicator_testSEXP = treatment_indicator_test,
      no_trees_treatSEXP = number_of_trees,
      power_treatSEXP = power,
      base_treatSEXP = base,
      p_grow_treatSEXP = p_grow,
      p_prune_treatSEXP = p_prune,
      omega_treatSEXP = 1/2,
      prior_type_treatSEXP = "horseshoe",
      param1_treatSEXP = k / sqrt(number_of_trees),
      param2_treatSEXP = k / sqrt(number_of_trees),
      reversible_treatSEXP = TRUE,
      dirichlet_bool_treatSEXP = FALSE,
      a_dirichlet_treatSEXP = 1.0,
      b_dirichlet_treatSEXP = 1.0,
      rho_dirichlet_treatSEXP = 1.0,
      no_trees_controlSEXP = number_of_trees,
      power_controlSEXP = power,
      base_controlSEXP = base,
      p_grow_controlSEXP = p_grow,
      p_prune_controlSEXP = p_prune,
      omega_controlSEXP = 1/2,
      prior_type_controlSEXP = "horseshoe",
      param1_controlSEXP = k / sqrt(number_of_trees),
      param2_controlSEXP = k / sqrt(number_of_trees),
      reversible_controlSEXP = TRUE,
      dirichlet_bool_controlSEXP = FALSE,
      a_dirichlet_controlSEXP = 1.0,
      b_dirichlet_controlSEXP = 1.0,
      rho_dirichlet_controlSEXP = 1.0,
      sigma_knownSEXP = sigma_known,
      sigmaSEXP = sigma_hat,
      lambdaSEXP = lambda,
      nuSEXP = nu,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      store_posterior_sample_controlSEXP = store_posterior_sample,
      store_posterior_sample_treatSEXP = store_posterior_sample,
      verboseSEXP = verbose,
      treatment_codingSEXP = treatment_coding,
      propensity_trainSEXP = propensity_train_cpp,
      propensity_testSEXP = propensity_test_cpp
    )

    # Back-transform (identical to right-censored)
    if (timescale == "time") {
      fit$train_predictions <- exp(fit$train_predictions * sigma_hat + y_mean)
      fit$test_predictions <- exp(fit$test_predictions * sigma_hat + y_mean)
      fit$train_predictions_control <- exp(fit$train_predictions_control *
                                             sigma_hat + y_mean)
      fit$test_predictions_control <- exp(fit$test_predictions_control *
                                            sigma_hat + y_mean)
      fit$train_predictions_treat <- exp(fit$train_predictions_treat * sigma_hat)
      fit$test_predictions_treat <- exp(fit$test_predictions_treat * sigma_hat)
      if (store_posterior_sample) {
        fit$train_predictions_sample_control <-
          exp(fit$train_predictions_sample_control * sigma_hat + y_mean)
        fit$test_predictions_sample_control <-
          exp(fit$test_predictions_sample_control * sigma_hat + y_mean)
        fit$train_predictions_sample_treat <-
          exp(fit$train_predictions_sample_treat * sigma_hat)
        fit$test_predictions_sample_treat <-
          exp(fit$test_predictions_sample_treat * sigma_hat)
      }
    } else {
      fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
      fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
      fit$train_predictions_control <-
        fit$train_predictions_control * sigma_hat + y_mean
      fit$test_predictions_control <-
        fit$test_predictions_control * sigma_hat + y_mean
      fit$train_predictions_treat <- fit$train_predictions_treat * sigma_hat
      fit$test_predictions_treat <- fit$test_predictions_treat * sigma_hat
      if (store_posterior_sample) {
        fit$train_predictions_sample_control <-
          fit$train_predictions_sample_control * sigma_hat + y_mean
        fit$test_predictions_sample_control <-
          fit$test_predictions_sample_control * sigma_hat + y_mean
        fit$train_predictions_sample_treat <-
          fit$train_predictions_sample_treat * sigma_hat
        fit$test_predictions_sample_treat <-
          fit$test_predictions_sample_treat * sigma_hat
      }
    }

     # Otherwise, continuous
  } else {

    # Force outcome to plain numeric vector
    y <- as.numeric(y)

    # Create dummy status vector (not used for continuous)
    status <- rep(1, n_train)
    survival <- FALSE
    
    # Determine prior guess of sigma 
    if (is.null(sigma)) {
      sigma_hat <- sd(y)      # Estimate sigma from data
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma      # Use provided sigma
      sigma_known <- TRUE
    }
    
    # Compute hyperparameters of error variance prior
    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu
    
    # Compute mean and standardize  the outcome
    y_mean <- mean(y)
    y <- y - y_mean
    y <- y / sigma_hat  # Standardize y
    
    # Fit a Causal Horseshoe Forest
    fit <- CausalHorseForest_cpp(
      nSEXP = n_train,
      p_treatSEXP = p_treat,
      p_controlSEXP = p_control,
      X_train_treatSEXP = X_train_treat,
      X_train_controlSEXP = X_train_control,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = numeric(n_train),
      observed_right_timeSEXP = y + 0,
      interval_censoring_indicatorSEXP = numeric(n_train),
      treatment_indicatorSEXP = treatment_indicator_train,
      n_testSEXP = n_test,
      X_test_controlSEXP = X_test_control,
      X_test_treatSEXP = X_test_treat,
      treatment_indicator_testSEXP = treatment_indicator_test,
      no_trees_treatSEXP = number_of_trees,
      power_treatSEXP = power,
      base_treatSEXP = base,
      p_grow_treatSEXP = p_grow,
      p_prune_treatSEXP = p_prune,
      omega_treatSEXP = 1/2,
      prior_type_treatSEXP = "horseshoe",
      param1_treatSEXP = k / sqrt(number_of_trees),
      param2_treatSEXP = k / sqrt(number_of_trees),
      reversible_treatSEXP = TRUE,
      dirichlet_bool_treatSEXP = FALSE,
      a_dirichlet_treatSEXP = 1.0,
      b_dirichlet_treatSEXP = 1.0,
      rho_dirichlet_treatSEXP = 1.0,
      no_trees_controlSEXP = number_of_trees,
      power_controlSEXP = power,
      base_controlSEXP = base,
      p_grow_controlSEXP = p_grow,
      p_prune_controlSEXP = p_prune,
      omega_controlSEXP = 1/2,
      prior_type_controlSEXP = "horseshoe",
      param1_controlSEXP = k / sqrt(number_of_trees),
      param2_controlSEXP = k / sqrt(number_of_trees),
      reversible_controlSEXP = TRUE,
      dirichlet_bool_controlSEXP = FALSE,
      a_dirichlet_controlSEXP = 1.0,
      b_dirichlet_controlSEXP = 1.0,
      rho_dirichlet_controlSEXP = 1.0,
      sigma_knownSEXP = sigma_known,
      sigmaSEXP = sigma_hat,
      lambdaSEXP = lambda,
      nuSEXP = nu,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      store_posterior_sample_controlSEXP = store_posterior_sample,
      store_posterior_sample_treatSEXP = store_posterior_sample,
      verboseSEXP = verbose,
      treatment_codingSEXP = treatment_coding,
      propensity_trainSEXP = propensity_train_cpp,
      propensity_testSEXP = propensity_test_cpp
    )
    
    # Total
    fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
    fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
    
    # Control
    fit$train_predictions_control <- fit$train_predictions_control * 
      sigma_hat + y_mean
    fit$test_predictions_control <- fit$test_predictions_control * 
      sigma_hat + y_mean
    
    # Treatment
    fit$train_predictions_treat <- fit$train_predictions_treat * sigma_hat 
    fit$test_predictions_treat <- fit$test_predictions_treat * sigma_hat 
    
    # Posterior samples
    if (store_posterior_sample) {
      fit$train_predictions_sample <- 
        fit$train_predictions_sample *  sigma_hat + y_mean
      fit$test_predictions_sample <- 
        fit$test_predictions_sample * sigma_hat +  y_mean
      fit$train_predictions_sample_control <- 
        fit$train_predictions_sample_control * sigma_hat + y_mean
      fit$test_predictions_sample_control <- 
        fit$test_predictions_sample_control * sigma_hat + y_mean
      fit$train_predictions_sample_treat <- 
        fit$train_predictions_sample_treat * sigma_hat 
      fit$test_predictions_sample_treat <- 
        fit$test_predictions_sample_treat * sigma_hat 
    }
    
  }
  
  # remove burn-in of sigma
  if (!sigma_known) {
    fit$sigma <- fit$sigma[-seq_len(N_burn)]
  } 

  prior_type_control_user <- "horseshoe"
  prior_type_treat_user <- "horseshoe"
  dirichlet_bool_control <- FALSE
  dirichlet_bool_treat <- FALSE
  prior_type_control_cpp <- "horseshoe"
  prior_type_treat_cpp <- "horseshoe"

  obj <- NewCausalShrinkageForest(
    fit = fit,
    call = match.call(),
    outcome_type = outcome_type,
    timescale = timescale,
    n_train = n_train,
    p_control = p_control,
    p_treat = p_treat,
    n_test = n_test,
    test_provided = test_provided_flag,
    number_of_trees_control = number_of_trees,
    number_of_trees_treat = number_of_trees,
    N_post = N_post,
    N_burn = N_burn,
    store_posterior_sample = store_posterior_sample,
    sigma_hat = sigma_hat,
    sigma_known = sigma_known,
    y_mean = y_mean,
    prior_type_control_user = prior_type_control_user,
    prior_type_control_cpp = prior_type_control_cpp,
    prior_type_treat_user = prior_type_treat_user,
    prior_type_treat_cpp = prior_type_treat_cpp,
    dirichlet_bool_control = dirichlet_bool_control,
    dirichlet_bool_treat = dirichlet_bool_treat,
    # Data stored for predict()
    y_train                   = y_horse_raw,
    X_train_control           = X_control_mat,
    X_train_treat             = X_treat_mat,
    treatment_indicator_train = treatment_indicator_train,
    status_train              = if (outcome_type %in% c("right-censored", "interval-censored")) status_horse_raw else NULL,
    left_time_train           = left_time_raw,
    right_time_train          = right_time_raw,
    ic_indicator_train        = if (outcome_type == "interval-censored") ic_indicator else NULL,
    # Hyperparameters stored for predict()
    p_grow           = p_grow,
    p_prune          = p_prune,
    delayed_proposal = delayed_proposal,
    nu               = nu,
    lambda           = lambda,
    power_control    = power,
    base_control     = base,
    param1_control   = k / sqrt(number_of_trees),
    param2_control   = k / sqrt(number_of_trees),
    omega_control    = 1/2,
    reversible_control   = TRUE,
    a_dirichlet_control  = 0.5,
    b_dirichlet_control  = 1.0,
    rho_dirichlet_control = p_control,
    power_treat      = power,
    base_treat       = base,
    param1_treat     = k / sqrt(number_of_trees),
    param2_treat     = k / sqrt(number_of_trees),
    omega_treat      = 1/2,
    reversible_treat     = TRUE,
    a_dirichlet_treat    = 0.5,
    b_dirichlet_treat    = 1.0,
    rho_dirichlet_treat  = p_treat,
    treatment_coding     = treatment_coding,
    propensity_train     = propensity
  )

  return(obj)
}

# ── Helper: generate interval-censored data ──────────────────────────────────

generate_ic_data <- function(n = 50, p = 3) {
  X <- matrix(rnorm(n * p), ncol = p)
  true_time <- rexp(n, rate = exp(0.3 * X[, 1]))

  set.seed(42)
  obs_type <- sample(c("exact", "interval", "right"),
                     n, replace = TRUE, prob = c(0.4, 0.3, 0.3))

  left_time  <- numeric(n)
  right_time <- numeric(n)
  for (i in seq_len(n)) {
    if (obs_type[i] == "exact") {
      left_time[i]  <- true_time[i]
      right_time[i] <- true_time[i]
    } else if (obs_type[i] == "interval") {
      w <- runif(1, 0.1, 0.5) * true_time[i]
      left_time[i]  <- true_time[i] - w / 2
      right_time[i] <- true_time[i] + w / 2
    } else {
      left_time[i]  <- true_time[i] * runif(1, 0.3, 0.9)
      right_time[i] <- Inf
    }
  }

  list(X = X, left_time = left_time, right_time = right_time,
       true_time = true_time, obs_type = obs_type)
}


# ── ShrinkageTrees with interval-censored survival ──────────────────────────

test_that("ShrinkageTrees works for interval-censored survival", {
  dat <- generate_ic_data(n = 50, p = 3)

  set.seed(1)
  fit <- ShrinkageTrees(
    left_time = dat$left_time,
    right_time = dat$right_time,
    X_train = dat$X,
    outcome_type = "interval-censored",
    number_of_trees = 5,
    prior_type = "horseshoe",
    local_hp = 0.1 / sqrt(5),
    global_hp = 0.1 / sqrt(5),
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "ShrinkageTrees")
  expect_length(fit$train_predictions, 50)
  expect_length(fit$sigma, 10)
  expect_true(all(fit$train_predictions > 0))
  expect_true(all(fit$sigma > 0))
  expect_false(any(is.na(fit$train_predictions)))

  # Posterior samples
  expect_equal(dim(fit$train_predictions_sample), c(10, 50))

  # Stored interval data
  expect_length(fit$data$left_time_train, 50)
  expect_length(fit$data$right_time_train, 50)
  expect_length(fit$data$ic_indicator_train, 50)

  # S3 methods
  expect_no_error(capture.output(print(fit)))
  expect_no_error(smry <- summary(fit))
  expect_type(smry, "list")
})


test_that("ShrinkageTrees interval-censored predict works", {
  dat <- generate_ic_data(n = 50, p = 3)

  set.seed(1)
  fit <- ShrinkageTrees(
    left_time = dat$left_time,
    right_time = dat$right_time,
    X_train = dat$X,
    outcome_type = "interval-censored",
    number_of_trees = 5,
    prior_type = "horseshoe",
    local_hp = 0.1 / sqrt(5),
    global_hp = 0.1 / sqrt(5),
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  X_new <- matrix(rnorm(20 * 3), ncol = 3)
  pred <- predict(fit, newdata = X_new)

  expect_s3_class(pred, "ShrinkageTreesPrediction")
  expect_length(pred$mean, 20)
  expect_true(all(pred$mean > 0))
  expect_true(all(is.finite(pred$mean)))
  expect_true(all(pred$lower <= pred$upper))
  expect_true(!is.null(pred$predictions_sample))
  expect_true(!is.null(pred$sigma))
})


# ── CausalShrinkageForest with interval-censored survival ───────────────────

test_that("CausalShrinkageForest works for interval-censored survival", {
  n <- 50; p <- 3
  X <- matrix(rnorm(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)

  true_time <- rexp(n, rate = exp(0.3 * X[, 1] + treatment * (-0.3)))

  set.seed(42)
  obs_type <- sample(c("exact", "interval", "right"),
                     n, replace = TRUE, prob = c(0.4, 0.3, 0.3))
  left_time  <- numeric(n)
  right_time <- numeric(n)
  for (i in seq_len(n)) {
    if (obs_type[i] == "exact") {
      left_time[i]  <- true_time[i]
      right_time[i] <- true_time[i]
    } else if (obs_type[i] == "interval") {
      w <- runif(1, 0.1, 0.5) * true_time[i]
      left_time[i]  <- true_time[i] - w / 2
      right_time[i] <- true_time[i] + w / 2
    } else {
      left_time[i]  <- true_time[i] * runif(1, 0.3, 0.9)
      right_time[i] <- Inf
    }
  }

  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    left_time = left_time,
    right_time = right_time,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "interval-censored",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat = lh, global_hp_treat = lh,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_length(fit$train_predictions, n)
  expect_length(fit$sigma, 10)
  expect_true(all(fit$sigma > 0))
  expect_false(any(is.na(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))

  # CATE should be available
  expect_length(fit$train_predictions_treat, n)
  expect_length(fit$train_predictions_control, n)

  # S3 methods
  expect_no_error(capture.output(print(fit)))
  expect_no_error(smry <- summary(fit))
  expect_type(smry, "list")
})


test_that("CausalHorseForest works for interval-censored survival", {
  n <- 50; p <- 3
  X <- matrix(rnorm(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)

  true_time <- rexp(n, rate = exp(0.3 * X[, 1] + treatment * (-0.3)))

  set.seed(42)
  obs_type <- sample(c("exact", "interval", "right"),
                     n, replace = TRUE, prob = c(0.4, 0.3, 0.3))
  left_time  <- numeric(n)
  right_time <- numeric(n)
  for (i in seq_len(n)) {
    if (obs_type[i] == "exact") {
      left_time[i]  <- true_time[i]
      right_time[i] <- true_time[i]
    } else if (obs_type[i] == "interval") {
      w <- runif(1, 0.1, 0.5) * true_time[i]
      left_time[i]  <- true_time[i] - w / 2
      right_time[i] <- true_time[i] + w / 2
    } else {
      left_time[i]  <- true_time[i] * runif(1, 0.3, 0.9)
      right_time[i] <- Inf
    }
  }

  set.seed(1)
  fit <- CausalHorseForest(
    left_time = left_time,
    right_time = right_time,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "interval-censored",
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_length(fit$train_predictions, n)
  expect_length(fit$sigma, 10)
  expect_true(all(fit$sigma > 0))
  expect_false(any(is.na(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))

  # S3 methods
  expect_no_error(capture.output(print(fit)))
  expect_no_error(smry <- summary(fit))
})

test_that("CausalHorseForest works for continuous outcome", {

  # Generate data
  n <- 50
  p <- 3
  X_control <- matrix(runif(n * p), ncol = p)
  X_treat   <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)

  # True treatment effect
  tau <- 2

  # Outcome: baseline + treatment effect + noise
  y <- X_control[, 1] + treatment * tau + rnorm(n)

  # Test data
  X_test_control <- matrix(runif(n * p), ncol = p)
  X_test_treat   <- matrix(runif(n * p), ncol = p)
  treatment_test <- rbinom(n, 1, 0.5)

  # Fit the model
  set.seed(1)
  fit <- CausalHorseForest(
    y = y,
    X_train_control = X_control,
    X_train_treat = X_treat,
    treatment_indicator_train = treatment,
    X_test_control = X_test_control,
    X_test_treat = X_test_treat,
    treatment_indicator_test = treatment_test,
    outcome_type = "continuous",
    number_of_trees = 5,
    N_post = 10,
    N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )
  
  # --- Basic list checks ---
  expect_type(fit, "list")
  
  # Check scalar outputs
  expect_length(fit$acceptance_ratio_control, 1)
  expect_length(fit$acceptance_ratio_treat, 1)
  expect_true(all(fit$acceptance_ratio_control >= 0 & fit$acceptance_ratio_control <= 1))
  expect_true(all(fit$acceptance_ratio_treat >= 0 & fit$acceptance_ratio_treat <= 1))
  
  # --- Check main predictions ---
  expect_length(fit$train_predictions, n)
  expect_length(fit$test_predictions, n)
  expect_length(fit$train_predictions_control, n)
  expect_length(fit$test_predictions_control, n)
  expect_length(fit$train_predictions_treat, n)
  expect_length(fit$test_predictions_treat, n)
  
  # --- Check sigma vector ---
  expect_length(fit$sigma, 10)
  expect_true(all(fit$sigma > 0))
  
  # --- Posterior samples (control) ---
  expect_equal(dim(fit$train_predictions_sample_control), c(10, n))
  expect_equal(dim(fit$test_predictions_sample_control), c(10, n))
  
  # --- Posterior samples (treat) ---
  expect_equal(dim(fit$train_predictions_sample_treat), c(10, n))
  expect_equal(dim(fit$test_predictions_sample_treat), c(10, n))
  
  # --- Numerical sanity checks ---
  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_false(any(is.nan(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  
  # Predictions are not all zero
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$test_predictions) > 0)
  expect_true(sd(fit$train_predictions_control) > 0)
  expect_true(sd(fit$test_predictions_control) > 0)
  expect_true(sd(fit$train_predictions_treat) > 0)
  expect_true(sd(fit$test_predictions_treat) > 0)
  
  # Posterior samples non-constant
  expect_true(sd(fit$train_predictions_sample_control) > 0)
  expect_true(sd(fit$test_predictions_sample_control) > 0)
  expect_true(sd(fit$train_predictions_sample_treat) > 0)
  expect_true(sd(fit$test_predictions_sample_treat) > 0)
  
  # --- Check reproducibility ---
  set.seed(1)
  fit2 <- CausalHorseForest(
    y = y,
    X_train_control = X_control,
    X_train_treat = X_treat,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees = 5,
    N_post = 10,
    N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )
  expect_equal(fit$train_predictions, fit2$train_predictions)
})


test_that("CausalHorseForest works for survival outcome", {
  
  # Generate data
  n <- 50
  p <- 3
  X_control <- matrix(runif(n * p), ncol = p)
  X_treat   <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  
  # True treatment effect on log-time (AFT interpretation)
  tau <- -0.5  # Negative tau => accelerates survival (protective)
  
  # Baseline log survival time
  log_T0 <- X_control[, 1] + rnorm(n)
  
  # Treatment effect
  log_T <- log_T0 + treatment * tau
  
  # Transform to actual survival time
  Ti <- exp(log_T)
  
  # Administrative censoring times (e.g., follow-up cutoff)
  C <- rexp(n, rate = 0.1)  # example: random censoring times
  
  # Observed time and status indicator
  time <- pmin(Ti, C)
  status <- as.integer(T <= C)  # 1 = event (uncensored), 0 = censored
  
  # Test data
  X_test_control <- matrix(runif(n * p), ncol = p)
  X_test_treat   <- matrix(runif(n * p), ncol = p)
  treatment_test <- rbinom(n, 1, 0.5)
  
  # Fit the model
  set.seed(1)
  fit <- CausalHorseForest(
    y = time,
    status = status,
    X_train_control = X_control,
    X_train_treat = X_treat,
    treatment_indicator_train = treatment,
    X_test_control = X_test_control,
    X_test_treat = X_test_treat,
    treatment_indicator_test = treatment_test,
    outcome_type = "right-censored",
    number_of_trees = 5,
    N_post = 10,
    N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )
  
  # --- Basic list checks ---
  expect_type(fit, "list")
  
  # Check scalar outputs
  expect_length(fit$acceptance_ratio_control, 1)
  expect_length(fit$acceptance_ratio_treat, 1)
  expect_true(all(fit$acceptance_ratio_control >= 0 & fit$acceptance_ratio_control <= 1))
  expect_true(all(fit$acceptance_ratio_treat >= 0 & fit$acceptance_ratio_treat <= 1))
  
  # --- Check main predictions ---
  expect_length(fit$train_predictions, n)
  expect_length(fit$test_predictions, n)
  expect_length(fit$train_predictions_control, n)
  expect_length(fit$test_predictions_control, n)
  expect_length(fit$train_predictions_treat, n)
  expect_length(fit$test_predictions_treat, n)
  
  # --- Check sigma vector ---
  expect_length(fit$sigma, 10)
  expect_true(all(fit$sigma > 0))
  
  # --- Posterior samples (control) ---
  expect_equal(dim(fit$train_predictions_sample_control), c(10, n))
  expect_equal(dim(fit$test_predictions_sample_control), c(10, n))
  
  # --- Posterior samples (treat) ---
  expect_equal(dim(fit$train_predictions_sample_treat), c(10, n))
  expect_equal(dim(fit$test_predictions_sample_treat), c(10, n))
  
  # --- Numerical sanity checks ---
  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_false(any(is.nan(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  
  # Predictions are not all zero
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$test_predictions) > 0)
  expect_true(sd(fit$train_predictions_control) > 0)
  expect_true(sd(fit$test_predictions_control) > 0)
  expect_true(sd(fit$train_predictions_treat) > 0)
  expect_true(sd(fit$test_predictions_treat) > 0)
  
  # Posterior samples non-constant
  expect_true(sd(fit$train_predictions_sample_control) > 0)
  expect_true(sd(fit$test_predictions_sample_control) > 0)
  expect_true(sd(fit$train_predictions_sample_treat) > 0)
  expect_true(sd(fit$test_predictions_sample_treat) > 0)
  
  # --- Check reproducibility ---
  set.seed(1)
  fit2 <- CausalHorseForest(
    y = time,
    status = status,
    X_train_control = X_control,
    X_train_treat = X_treat,
    treatment_indicator_train = treatment,
    outcome_type = "right-censored",
    number_of_trees = 5,
    N_post = 10,
    N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )
  expect_equal(fit$train_predictions, fit2$train_predictions)
})


# ── treatment_coding ──────────────────────────────────────────────────────────

test_that("CausalHorseForest works with treatment_coding = 'binary'", {
  n <- 50; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)

  set.seed(1)
  fit <- CausalHorseForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    treatment_coding = "binary",
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_equal(fit$treatment_coding, "binary")
  expect_length(fit$train_predictions, n)
  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  expect_true(sd(fit$train_predictions) > 0)
})

test_that("CausalHorseForest works with treatment_coding = 'adaptive'", {
  n <- 50; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, X[, 1])
  y <- X[, 1] + treatment * 2 + rnorm(n)
  propensity <- X[, 1]  # true propensity

  set.seed(1)
  fit <- CausalHorseForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    treatment_coding = "adaptive",
    propensity = propensity,
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_equal(fit$treatment_coding, "adaptive")
  expect_length(fit$train_predictions, n)
  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  expect_true(sd(fit$train_predictions) > 0)
})

test_that("CausalHorseForest errors when adaptive coding used without propensity", {
  n <- 30; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)

  expect_error(
    CausalHorseForest(
      y = y,
      X_train_control = X, X_train_treat = X,
      treatment_indicator_train = treatment,
      outcome_type = "continuous",
      treatment_coding = "adaptive",
      number_of_trees = 5,
      N_post = 10, N_burn = 5,
      verbose = FALSE
    ),
    "propensity"
  )
})

test_that("CausalHorseForest works with treatment_coding = 'invariant'", {
  n <- 50; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)

  set.seed(1)
  fit <- CausalHorseForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    treatment_coding = "invariant",
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_equal(fit$treatment_coding, "invariant")
  expect_length(fit$train_predictions, n)

  # b0 and b1 posterior draws should be returned
  expect_length(fit$b0, 10)
  expect_length(fit$b1, 10)
  expect_true(all(is.finite(fit$b0)))
  expect_true(all(is.finite(fit$b1)))

  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  expect_true(sd(fit$train_predictions) > 0)
})


# ── S3 class and methods ──────────────────────────────────────────────────────

test_that("CausalHorseForest returns CausalShrinkageForest S3 object with working methods", {
  n <- 30; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)

  set.seed(1)
  fit <- CausalHorseForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_no_error(capture.output(print(fit)))
  expect_no_error(smry <- summary(fit))
  expect_type(smry, "list")
  expect_true(!is.null(smry$sigma))
})


# ── Multi-chain (n_chains) ────────────────────────────────────────────────────

test_that("CausalHorseForest n_chains > 1 pools chains correctly", {
  n <- 30; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)

  set.seed(1)
  fit <- CausalHorseForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees = 5,
    N_post = 10, N_burn = 5,
    n_chains = 2,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_length(fit$sigma, 20)
  expect_equal(fit$mcmc$n_chains, 2)
  expect_length(fit$chains$acceptance_ratios_control, 2)
  expect_length(fit$chains$acceptance_ratios_treat,   2)
})

#' Resolve the number of worker cores for multi-chain MCMC
#'
#' Returns a safe core count that never exceeds CRAN's two-core policy
#' when running under `R CMD check`, honours the `OMP_THREAD_LIMIT`
#' environment variable, and falls back to sequential execution on Windows
#' (where `mclapply` cannot fork).
#'
#' @param n_chains Requested number of chains.
#' @return Integer number of cores to use.
#' @keywords internal
#' @noRd
.resolve_cores <- function(n_chains) {
  if (.Platform$OS.type == "windows") return(1L)
  cap <- if (nzchar(Sys.getenv("_R_CHECK_LIMIT_CORES_"))) 2L else Inf
  tl <- suppressWarnings(as.integer(Sys.getenv("OMP_THREAD_LIMIT", "")))
  if (!is.na(tl) && tl > 0) cap <- min(cap, tl)
  detected <- tryCatch(parallel::detectCores(logical = FALSE),
                       error = function(e) 1L)
  if (is.na(detected) || detected < 1L) detected <- 1L
  as.integer(max(1L, min(n_chains, detected, cap)))
}


#' Compute mean estimate for censored data
#'
#' Estimates the mean and standard deviation for right-censored or
#' interval-censored survival data. Uses the `survival` package if available,
#' otherwise falls back to the naive mean among observed events.
#'
#' @param y Numeric vector of (log-transformed) survival times.
#' @param status Numeric vector; event indicator (1 = event, 0 = censored).
#' @param left_time Optional numeric vector of left boundaries (for interval censoring).
#' @param right_time Optional numeric vector of right boundaries (for interval censoring).
#' @param ic_indicator Optional numeric vector; 1 = interval-censored, 0 = right-censored/exact.
#'
#' @return A list with elements:
#'   \item{mu}{Estimated mean of survival times.}
#'   \item{sd}{Estimated standard deviation of survival times.}
#'   \item{min}{Estimated minimum of survival times.}
#'   \item{max}{Estimated maximum of survival times.}
#'
#' @importFrom stats dnorm pnorm
#' @noRd
censored_info <- function(y, status, left_time = NULL, right_time = NULL,
                          ic_indicator = NULL) {

  if (requireNamespace("survival", quietly = TRUE)) {

    if (!is.null(left_time) && !is.null(right_time)) {
      # Interval-censored path: use Surv(time, time2, type = "interval2")
      surv_obj <- survival::Surv(left_time, right_time, type = "interval2")
      fit <- survival::survreg(surv_obj ~ 1, dist = "gaussian")

      mu <- as.numeric(fit$coefficients[1])
      sd <- as.numeric(fit$scale)

      # Impute censored values for min/max estimation
      imputed_y <- y

      # Right-censored observations: impute with Mills ratio
      if (!is.null(ic_indicator)) {
        rc_idx <- which(status == 0 & ic_indicator == 0)
        if (length(rc_idx) > 0) {
          a <- (y[rc_idx] - mu) / sd
          lambda_val <- dnorm(a) / (1 - pnorm(a))
          imputed_y[rc_idx] <- mu + sd * lambda_val
        }
      }

      min_val <- min(imputed_y, na.rm = TRUE)
      max_val <- max(imputed_y, na.rm = TRUE)

    } else {
      # Right-censored path (existing)
      fit <- survival::survreg(survival::Surv(y, status) ~ 1, dist = "gaussian")

      mu <- as.numeric(fit$coefficients[1])
      sd <- as.numeric(fit$scale)

      # ---- IMPUTE censored values ----
      cens_idx <- which(status == 0)
      imputed_y <- y

      if (length(cens_idx) > 0) {
        a <- (y[cens_idx] - mu) / sd
        lambda <- dnorm(a) / (1 - pnorm(a))     # Mills ratio
        imputed_y[cens_idx] <- mu + sd * lambda # Replace censored with E[Y | Y > c]
      }

      # min/max after imputation
      min_val <- min(imputed_y, na.rm = TRUE)
      max_val <- max(imputed_y, na.rm = TRUE)
    }

  } else {

    # Fallback: use only uncensored observations
    min_val <- min(y, na.rm = TRUE)
    max_val <- max(y, na.rm = TRUE)

    mu <- mean(y[status == 1], na.rm = TRUE)
    sd <- sd(y[status == 1], na.rm = TRUE)
  }

  return(list(
    mu = mu,
    sd = sd,
    min = min_val,
    max = max_val
  ))
}

# Internal helper: pool a list of ShrinkageTrees objects (one per chain) into
# a single ShrinkageTrees object with concatenated/averaged posteriors.
.combine_chains <- function(chains) {
  n_chains <- length(chains)
  out      <- chains[[1]]   # use first chain as structural template

  pool_matrix <- function(field) {
    mats <- lapply(chains, `[[`, field)
    if (all(vapply(mats, is.null, logical(1)))) return(NULL)
    do.call(rbind, mats)
  }
  pool_vector <- function(field) {
    vecs <- lapply(chains, `[[`, field)
    if (all(vapply(vecs, is.null, logical(1)))) return(NULL)
    unlist(vecs)
  }

  # ── Posterior sample matrices (rows = MCMC iterations) ───────────────────
  out$train_predictions_sample <- pool_matrix("train_predictions_sample")
  out$test_predictions_sample  <- pool_matrix("test_predictions_sample")

  # ── Posterior means: recompute from pooled samples when available ─────────
  if (!is.null(out$train_predictions_sample)) {
    out$train_predictions <- colMeans(out$train_predictions_sample)
    out$test_predictions  <- colMeans(out$test_predictions_sample)
  } else {
    # store_posterior_sample = FALSE: average the per-chain posterior means
    out$train_predictions <- Reduce("+", lapply(chains, `[[`, "train_predictions")) / n_chains
    out$test_predictions  <- Reduce("+", lapply(chains, `[[`, "test_predictions"))  / n_chains
  }

  # ── Sigma posterior samples ───────────────────────────────────────────────
  out$sigma <- pool_vector("sigma")

  # ── Acceptance ratios (callers always use mean(); we concatenate) ─────────
  out$acceptance_ratio <- pool_vector("acceptance_ratio")

  # ── Binary outcome: pool probability samples ──────────────────────────────
  out$train_probabilities_sample <- pool_matrix("train_probabilities_sample")
  out$test_probabilities_sample  <- pool_matrix("test_probabilities_sample")
  if (!is.null(out$train_probabilities_sample)) {
    out$train_probabilities <- colMeans(out$train_probabilities_sample)
    out$test_probabilities  <- colMeans(out$test_probabilities_sample)
  } else if (!is.null(chains[[1]]$train_probabilities)) {
    out$train_probabilities <- Reduce("+", lapply(chains, `[[`, "train_probabilities")) / n_chains
    out$test_probabilities  <- Reduce("+", lapply(chains, `[[`, "test_probabilities"))  / n_chains
  }

  # ── Variable importance: rbind iteration matrices so vi helpers still work ─
  out$split_probs        <- pool_matrix("split_probs")
  out$store_split_counts <- pool_matrix("store_split_counts")

  # ── Chain-level diagnostics ───────────────────────────────────────────────
  out$chains <- list(
    n_chains         = n_chains,
    acceptance_ratios = vapply(chains, function(ch) mean(ch$acceptance_ratio), numeric(1))
  )

  # Record n_chains in mcmc slot (N_post stays per-chain for predict())
  out$mcmc$n_chains <- n_chains

  out
}

# Internal helper: pool a list of CausalShrinkageForest objects (one per chain)
# into a single object with concatenated/averaged posteriors.
.combine_causal_chains <- function(chains) {
  n_chains <- length(chains)
  out      <- chains[[1]]

  pool_matrix <- function(field) {
    mats <- lapply(chains, `[[`, field)
    if (all(vapply(mats, is.null, logical(1)))) return(NULL)
    do.call(rbind, mats)
  }
  pool_vector <- function(field) {
    vecs <- lapply(chains, `[[`, field)
    if (all(vapply(vecs, is.null, logical(1)))) return(NULL)
    unlist(vecs)
  }
  avg <- function(field) Reduce("+", lapply(chains, `[[`, field)) / n_chains

  # ── Posterior sample matrices ─────────────────────────────────────────────
  out$train_predictions_sample_control <- pool_matrix("train_predictions_sample_control")
  out$test_predictions_sample_control  <- pool_matrix("test_predictions_sample_control")
  out$train_predictions_sample_treat   <- pool_matrix("train_predictions_sample_treat")
  out$test_predictions_sample_treat    <- pool_matrix("test_predictions_sample_treat")

  # ── Posterior means ───────────────────────────────────────────────────────
  if (!is.null(out$train_predictions_sample_control)) {
    out$train_predictions_control <- colMeans(out$train_predictions_sample_control)
    out$test_predictions_control  <- colMeans(out$test_predictions_sample_control)
    out$train_predictions_treat   <- colMeans(out$train_predictions_sample_treat)
    out$test_predictions_treat    <- colMeans(out$test_predictions_sample_treat)
  } else {
    out$train_predictions_control <- avg("train_predictions_control")
    out$test_predictions_control  <- avg("test_predictions_control")
    out$train_predictions_treat   <- avg("train_predictions_treat")
    out$test_predictions_treat    <- avg("test_predictions_treat")
  }
  # Total is always averaged per-chain (control + treat are on different scales)
  out$train_predictions <- avg("train_predictions")
  out$test_predictions  <- avg("test_predictions")

  # ── Sigma ─────────────────────────────────────────────────────────────────
  out$sigma <- pool_vector("sigma")

  # ── Acceptance ratios ─────────────────────────────────────────────────────
  out$acceptance_ratio_control <- pool_vector("acceptance_ratio_control")
  out$acceptance_ratio_treat   <- pool_vector("acceptance_ratio_treat")

  # ── Variable importance ───────────────────────────────────────────────────
  out$split_probs_control        <- pool_matrix("split_probs_control")
  out$store_split_counts_control <- pool_matrix("store_split_counts_control")
  out$split_probs_treat          <- pool_matrix("split_probs_treat")
  out$store_split_counts_treat   <- pool_matrix("store_split_counts_treat")

  # ── Chain-level diagnostics ───────────────────────────────────────────────
  out$chains <- list(
    n_chains                  = n_chains,
    acceptance_ratios_control = vapply(chains,
      function(ch) mean(ch$acceptance_ratio_control), numeric(1)),
    acceptance_ratios_treat   = vapply(chains,
      function(ch) mean(ch$acceptance_ratio_treat),   numeric(1))
  )

  out$mcmc$n_chains <- n_chains
  out
}

#' Posterior ATE samples from a CATE draw matrix
#'
#' @param tau_samples S x n matrix of posterior CATE draws.
#' @param bayesian_bootstrap If \code{TRUE}, reweight columns at each iteration
#'   with Dirichlet(1, ..., 1) weights, giving posterior draws of the
#'   population ATE. If \code{FALSE}, use equal 1/n weights (mixed ATE,
#'   conditional on the observed covariates).
#' @return Length-S numeric vector of posterior ATE draws.
#' @importFrom stats rexp
#' @noRd
.ate_samples <- function(tau_samples, bayesian_bootstrap = TRUE) {
  if (!bayesian_bootstrap) return(rowMeans(tau_samples))
  S <- nrow(tau_samples)
  n <- ncol(tau_samples)
  W <- matrix(rexp(S * n), nrow = S, ncol = n)
  W <- W / rowSums(W)
  rowSums(W * tau_samples)
}

test_that("CausalShrinkageForest works for continuous outcome", {
  
  n <- 50
  p <- 3
  X <- matrix(runif(n * p), ncol = p)
  X_treat <- X_control <- X
  treatment <- rbinom(n, 1, X[, 1])
  tau <- 2
  y <- X[, 1] + (0.5 - treatment) * tau + rnorm(n)
  
  # Fit the model
  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X_control,
    X_train_treat = X_treat,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_treat = 5,
    number_of_trees_control = 5,
    prior_type_control = "horseshoe",
    prior_type_treat = "horseshoe",
    local_hp_control = 0.1 / sqrt(5),
    local_hp_treat = 0.1 / sqrt(5),
    global_hp_control = 0.1 / sqrt(5),
    global_hp_treat = 0.1 / sqrt(5),
    N_post = 10,
    N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )
  
  # Basic checks
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_length(fit$train_predictions, n)
  expect_length(fit$sigma, 10)  # N_post + N_burn = 10 + 5
  
  # Numerical checks
  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  
  # Check acceptance ratios
  expect_length(fit$acceptance_ratio_control, 1)
  expect_length(fit$acceptance_ratio_treat, 1)
  expect_true(fit$acceptance_ratio_control >= 0 & fit$acceptance_ratio_control <= 1)
  expect_true(fit$acceptance_ratio_treat >= 0 & fit$acceptance_ratio_treat <= 1)
  
  # Check posterior samples
  expect_equal(dim(fit$train_predictions_sample_control), c(10, n))
  expect_equal(dim(fit$train_predictions_sample_treat), c(10, n))
  
  # Check predictions are not all constant
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_control) > 0)
  expect_true(sd(fit$train_predictions_treat) > 0)
  
  # Check reproducibility
  set.seed(1)
  fit2 <- CausalShrinkageForest(
    y = y,
    X_train_control = X_control,
    X_train_treat = X_treat,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_treat = 5,
    number_of_trees_control = 5,
    prior_type_control = "horseshoe",
    prior_type_treat = "horseshoe",
    local_hp_control = 0.1 / sqrt(5),
    local_hp_treat = 0.1 / sqrt(5),
    global_hp_control = 0.1 / sqrt(5),
    global_hp_treat = 0.1 / sqrt(5),
    N_post = 10,
    N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )
  expect_equal(fit$train_predictions, fit2$train_predictions)
})


test_that("CausalShrinkageForest works for right-censored survival outcome", {
  
  n <- 50
  p <- 3
  X <- matrix(runif(n * p), ncol = p)
  X_treat <- X_control <- X
  treatment <- rbinom(n, 1, 0.5)
  tau <- -0.5  # protective effect
  
  log_T0 <- X[, 1] + rnorm(n)
  log_T <- log_T0 + treatment * tau
  true_time <- exp(log_T)
  
  censor_time <- rexp(n, rate = 0.1)
  time_obs <- pmin(true_time, censor_time)
  status <- as.integer(true_time <= censor_time)
  
  # Fit the model
  set.seed(1)
  fit <- CausalShrinkageForest(
    y = time_obs,
    status = status,
    X_train_control = X_control,
    X_train_treat = X_treat,
    treatment_indicator_train = treatment,
    outcome_type = "right-censored",
    timescale = "log",
    number_of_trees_treat = 5,
    number_of_trees_control = 5,
    prior_type_control = "horseshoe",
    prior_type_treat = "horseshoe",
    local_hp_control = 0.1 / sqrt(5),
    local_hp_treat = 0.1 / sqrt(5),
    global_hp_control = 0.1 / sqrt(5),
    global_hp_treat = 0.1 / sqrt(5),
    N_post = 10,
    N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )
  
  # Basic checks
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_length(fit$train_predictions, n)
  expect_length(fit$sigma, 10)
  
  # Numerical checks
  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  
  # Check acceptance ratios
  expect_length(fit$acceptance_ratio_control, 1)
  expect_length(fit$acceptance_ratio_treat, 1)
  expect_true(fit$acceptance_ratio_control >= 0 & fit$acceptance_ratio_control <= 1)
  expect_true(fit$acceptance_ratio_treat >= 0 & fit$acceptance_ratio_treat <= 1)
  
  # Check posterior samples
  expect_equal(dim(fit$train_predictions_sample_control), c(10, n))
  expect_equal(dim(fit$train_predictions_sample_treat), c(10, n))
  
  # Check predictions are not all constant
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_control) > 0)
  expect_true(sd(fit$train_predictions_treat) > 0)
  
  # Check reproducibility
  set.seed(1)
  fit2 <- CausalShrinkageForest(
    y = time_obs,
    status = status,
    X_train_control = X_control,
    X_train_treat = X_treat,
    treatment_indicator_train = treatment,
    outcome_type = "right-censored",
    timescale = "log",
    number_of_trees_treat = 5,
    number_of_trees_control = 5,
    prior_type_control = "horseshoe",
    prior_type_treat = "horseshoe",
    local_hp_control = 0.1 / sqrt(5),
    local_hp_treat = 0.1 / sqrt(5),
    global_hp_control = 0.1 / sqrt(5),
    global_hp_treat = 0.1 / sqrt(5),
    N_post = 10,
    N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )
  expect_equal(fit$train_predictions, fit2$train_predictions)
})


test_that("CausalShrinkageForest works for continuous outcome with half-cauchy prior", {
  
  n <- 50
  p <- 3
  X <- matrix(runif(n * p), ncol = p)
  X_treat <- X_control <- X
  treatment <- rbinom(n, 1, X[, 1])
  tau <- 2
  y <- X[, 1] + (0.5 - treatment) * tau + rnorm(n)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X_control,
    X_train_treat = X_treat,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_treat = 5,
    number_of_trees_control = 5,
    prior_type_control = "half-cauchy",
    prior_type_treat = "half-cauchy",
    local_hp_control = 1 / sqrt(5),
    local_hp_treat = 1 / sqrt(5),
    N_post = 10,
    N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )
  
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_length(fit$train_predictions, n)
  expect_length(fit$sigma, 10)
  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_control) > 0)
  expect_true(sd(fit$train_predictions_treat) > 0)
  expect_equal(dim(fit$train_predictions_sample_control), c(10, n))
  expect_equal(dim(fit$train_predictions_sample_treat), c(10, n))
})

test_that("CausalShrinkageForest works for continuous outcome with horseshoe_fw prior", {
  
  n <- 50
  p <- 3
  X <- matrix(runif(n * p), ncol = p)
  X_treat <- X_control <- X
  treatment <- rbinom(n, 1, X[, 1])
  tau <- 2
  y <- X[, 1] + (0.5 - treatment) * tau + rnorm(n)
  
  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X_control,
    X_train_treat = X_treat,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_treat = 5,
    number_of_trees_control = 5,
    prior_type_control = "horseshoe_fw",
    prior_type_treat = "horseshoe_fw",
    local_hp_control = 0.1 / sqrt(5),
    local_hp_treat = 0.1 / sqrt(5),
    global_hp_control = 0.1 / sqrt(5),
    global_hp_treat = 0.1 / sqrt(5),
    N_post = 10,
    N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )
  
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_true("forestwide_shrinkage_control" %in% names(fit))
  expect_true("forestwide_shrinkage_treat" %in% names(fit))
  
  expect_length(fit$train_predictions, n)
  expect_length(fit$sigma, 10)
  
  # Check forestwide shrinkage vectors
  expect_true(is.numeric(fit$forestwide_shrinkage_control))
  expect_true(is.numeric(fit$forestwide_shrinkage_treat))
  expect_length(fit$forestwide_shrinkage_control, 10)
  expect_length(fit$forestwide_shrinkage_treat, 10)
  
  # Sanity checks on numerical values
  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_control) > 0)
  expect_true(sd(fit$train_predictions_treat) > 0)
  
  expect_equal(dim(fit$train_predictions_sample_control), c(10, n))
  expect_equal(dim(fit$train_predictions_sample_treat), c(10, n))
  
  # Forestwide shrinkage values should be non-negative
  expect_true(all(fit$forestwide_shrinkage_control >= 0))
  expect_true(all(fit$forestwide_shrinkage_treat >= 0))
})


# ── treatment_coding ──────────────────────────────────────────────────────────

test_that("CausalShrinkageForest works with treatment_coding = 'binary'", {
  n <- 50; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)
  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    treatment_coding = "binary",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat  = lh, global_hp_treat  = lh,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_equal(fit$treatment_coding, "binary")
  expect_length(fit$train_predictions, n)
  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  expect_true(sd(fit$train_predictions) > 0)
})

test_that("CausalShrinkageForest works with treatment_coding = 'adaptive'", {
  n <- 50; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, X[, 1])
  y <- X[, 1] + treatment * 2 + rnorm(n)
  propensity <- X[, 1]  # true propensity
  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    treatment_coding = "adaptive",
    propensity = propensity,
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat  = lh, global_hp_treat  = lh,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_equal(fit$treatment_coding, "adaptive")
  expect_length(fit$train_predictions, n)
  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  expect_true(sd(fit$train_predictions) > 0)
})

test_that("CausalShrinkageForest errors when adaptive coding used without propensity", {
  n <- 30; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)
  lh <- 0.1 / sqrt(5)

  expect_error(
    CausalShrinkageForest(
      y = y,
      X_train_control = X, X_train_treat = X,
      treatment_indicator_train = treatment,
      outcome_type = "continuous",
      treatment_coding = "adaptive",
      number_of_trees_control = 5, number_of_trees_treat = 5,
      prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
      local_hp_control = lh, global_hp_control = lh,
      local_hp_treat  = lh, global_hp_treat  = lh,
      N_post = 10, N_burn = 5,
      verbose = FALSE
    ),
    "propensity"
  )
})

test_that("CausalShrinkageForest works with treatment_coding = 'invariant'", {
  n <- 50; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)
  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    treatment_coding = "invariant",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat  = lh, global_hp_treat  = lh,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_equal(fit$treatment_coding, "invariant")
  expect_length(fit$train_predictions, n)

  # b0 and b1 posterior draws should be returned
  expect_length(fit$b0, 10)
  expect_length(fit$b1, 10)
  expect_true(all(is.finite(fit$b0)))
  expect_true(all(is.finite(fit$b1)))

  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  expect_true(sd(fit$train_predictions) > 0)
})


# ── S3 class and methods ──────────────────────────────────────────────────────

test_that("CausalShrinkageForest returns CausalShrinkageForest S3 object with working methods", {
  n <- 30; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, X[, 1])
  y <- X[, 1] + (0.5 - treatment) * 2 + rnorm(n)
  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat  = lh, global_hp_treat  = lh,
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_no_error(capture.output(print(fit)))
  expect_no_error(smry <- summary(fit))
  expect_type(smry, "list")
  expect_true(!is.null(smry$sigma))
  # ATE should be available since store_posterior_sample = TRUE
  expect_true(!is.null(smry$treatment_effect$ate))
  expect_true(!is.null(smry$treatment_effect$ate_lower))
})


# ── Bayesian bootstrap PATE ───────────────────────────────────────────────────

test_that("bayesian_bootstrap widens the ATE credible interval", {
  n <- 40; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)
  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat  = lh, global_hp_treat  = lh,
    N_post = 200, N_burn = 50,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  set.seed(2); smry_pate <- summary(fit, bayesian_bootstrap = TRUE)
  smry_mate <- summary(fit, bayesian_bootstrap = FALSE)

  te_p <- smry_pate$treatment_effect
  te_m <- smry_mate$treatment_effect

  expect_true(isTRUE(te_p$bayesian_bootstrap))
  expect_false(isTRUE(te_m$bayesian_bootstrap))

  # MATE CI matches legacy rowMeans behaviour.
  ate_rowmean <- rowMeans(fit$train_predictions_sample_treat)
  expect_equal(te_m$ate_lower, unname(quantile(ate_rowmean, 0.025)))
  expect_equal(te_m$ate_upper, unname(quantile(ate_rowmean, 0.975)))

  # Posterior means track each other closely.
  expect_equal(te_p$ate, te_m$ate, tolerance = 0.2)
})


test_that(".ate_samples PATE variance decomposes correctly and beats MATE on a fixed matrix", {
  # Construct a tau matrix with clear between-observation spread so the
  # Dirichlet reweighting has room to add variance beyond the MCMC variation.
  set.seed(42)
  S <- 5000
  n <- 50
  tau_iter  <- rnorm(S, mean = 0.5, sd = 0.1)             # iteration-level signal
  tau_obs   <- rnorm(n, mean = 0,   sd = 1.0)             # across-observation spread
  tau <- matrix(tau_iter, S, n) + matrix(tau_obs, S, n, byrow = TRUE)

  mate <- ShrinkageTrees:::.ate_samples(tau, bayesian_bootstrap = FALSE)
  pate <- ShrinkageTrees:::.ate_samples(tau, bayesian_bootstrap = TRUE)

  # MATE equals rowMeans exactly.
  expect_equal(mate, rowMeans(tau))

  # With large S the PATE variance is (very reliably) larger than MATE variance,
  # because the Dirichlet reweighting injects extra variability from F_X.
  expect_gt(var(pate), var(mate))

  # Posterior means agree in expectation.
  expect_equal(mean(pate), mean(mate), tolerance = 0.01)
})


test_that("bayesian_bootstrap_ate helper returns coherent PATE and MATE", {
  n <- 40; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment * 2 + rnorm(n)
  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat  = lh, global_hp_treat  = lh,
    N_post = 200, N_burn = 50,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  set.seed(3)
  bb <- bayesian_bootstrap_ate(fit)

  expect_named(bb, c("pate_mean", "pate_ci", "pate_samples",
                     "mate_mean", "mate_ci", "mate_samples", "n", "S"))
  expect_equal(bb$n, n)
  expect_equal(bb$S, 200)
  expect_length(bb$pate_samples, 200)
  expect_length(bb$mate_samples, 200)
  expect_equal(bb$mate_samples, rowMeans(fit$train_predictions_sample_treat))
  expect_true(bb$pate_ci$lower <= bb$pate_mean)
  expect_true(bb$pate_mean <= bb$pate_ci$upper)
  expect_equal(bb$pate_mean, bb$mate_mean, tolerance = 0.2)
})


test_that("bayesian_bootstrap_ate errors without stored posterior samples", {
  n <- 30; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  y <- X[, 1] + treatment + rnorm(n)
  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat  = lh, global_hp_treat  = lh,
    N_post = 10, N_burn = 5,
    store_posterior_sample = FALSE,
    verbose = FALSE
  )

  expect_error(bayesian_bootstrap_ate(fit), "store_posterior_sample")
  expect_error(bayesian_bootstrap_ate(list()), "CausalShrinkageForest")
})


# ── Multi-chain (n_chains) ────────────────────────────────────────────────────

test_that("CausalShrinkageForest n_chains > 1 pools chains correctly", {
  n <- 30; p <- 3
  X <- matrix(runif(n * p), ncol = p)
  treatment <- rbinom(n, 1, X[, 1])
  y <- X[, 1] + (0.5 - treatment) * 2 + rnorm(n)
  lh <- 0.1 / sqrt(5)

  set.seed(1)
  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = treatment,
    outcome_type = "continuous",
    number_of_trees_control = 5, number_of_trees_treat = 5,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = lh, global_hp_control = lh,
    local_hp_treat  = lh, global_hp_treat  = lh,
    N_post = 10, N_burn = 5,
    n_chains = 2,
    verbose = FALSE
  )

  expect_s3_class(fit, "CausalShrinkageForest")
  expect_length(fit$sigma, 20)
  expect_equal(fit$mcmc$n_chains, 2)
  expect_length(fit$chains$acceptance_ratios_control, 2)
  expect_length(fit$chains$acceptance_ratios_treat,   2)
})

#' SurvivalBART
#'
#' Fits an Accelerated Failure Time (AFT) model using the classical
#' Bayesian Additive Regression Trees (BART) prior:
#' \eqn{\log(Y) = f(x) + \varepsilon}.
#' Supports both right-censored and interval-censored survival outcomes.
#'
#' @param time Outcome vector of (non-negative) survival times. Required for
#'   right-censored outcomes; set to \code{NULL} when using interval censoring.
#' @param status Event indicator (1 = event, 0 = censored). Required for
#'   right-censored outcomes; derived automatically for interval censoring.
#' @param X_train Design matrix for training data.
#' @param X_test Optional test matrix. If NULL, predictions are computed at
#'   the column means of \code{X_train}.
#' @param timescale Either \code{"time"} (log-transform internally) or
#'   \code{"log"} (already log-transformed).
#' @param number_of_trees Number of trees in the ensemble. Default is 200.
#' @param k Scaling constant used to calibrate the prior variance of the
#'   step heights.
#' @param N_post Number of posterior samples to store.
#' @param N_burn Number of burn-in iterations.
#' @param store_posterior_sample Logical; if \code{TRUE} (default), store
#'   the full \eqn{N_\text{post} \times n} matrix of posterior predictions.
#'   Required for \code{predict()}, \code{plot(type = "survival")} with full
#'   posterior credible bands, and custom posterior analyses. Set to
#'   \code{FALSE} to save memory when only posterior means are needed.
#' @param verbose Logical; print sampling progress.
#' @param left_time Optional numeric vector of left (lower) time boundaries
#'   for interval-censored data. Exact events have
#'   \code{left_time == right_time}; right-censored observations have
#'   \code{right_time = Inf}; interval-censored observations have finite
#'   \code{left_time < right_time}. When provided together with
#'   \code{right_time}, the model is fitted with
#'   \code{outcome_type = "interval-censored"} and \code{time}/\code{status}
#'   are ignored.
#' @param right_time Optional numeric vector of right (upper) time boundaries.
#'   Use \code{Inf} for right-censored observations.
#' @param ... Additional arguments passed to \code{\link{ShrinkageTrees}}
#'   to override default hyperparameters.
#'
#' @details
#' This function provides a survival-specific interface for classical BART
#' under an AFT formulation for right-censored or interval-censored outcomes.
#'
#' For right-censored data, supply \code{time} and \code{status}.
#' For interval-censored data, supply \code{left_time} and \code{right_time}
#' instead; event indicators are derived internally following the
#' \code{survival::Surv(type = "interval2")} convention.
#'
#' Structural regularisation is induced through the standard Gaussian
#' leaf prior and tree depth prior of Chipman, George & McCulloch (2010).
#'
#' Users requiring alternative shrinkage priors (e.g., Horseshoe or
#' Dirichlet splitting priors) should use \code{\link{ShrinkageTrees}}
#' directly.
#'
#' @references
#' Chipman, H. A., George, E. I., & McCulloch, R. E. (2010).
#' Bayesian Additive Regression Trees.
#' Annals of Applied Statistics.
#' 
#' @seealso
#' Related models: \code{\link{SurvivalDART}} (Dirichlet sparsity),
#' \code{\link{HorseTrees}} (horseshoe prior),
#' \code{\link{ShrinkageTrees}} (general shrinkage priors).
#'
#' S3 methods: \code{\link{print.ShrinkageTrees}},
#' \code{\link{summary.ShrinkageTrees}},
#' \code{\link{predict.ShrinkageTrees}},
#' \code{\link{plot.ShrinkageTrees}}.
#'
#' @return
#' An object of class \code{"ShrinkageTrees"} fitted under a classical
#' BART prior within an AFT formulation.
#'
#' See \code{\link{ShrinkageTrees}} for a full description of returned components
#'
#' @examples
#' set.seed(1)
#' n <- 30; p <- 5
#' X <- matrix(rnorm(n * p), ncol = p)
#' time <- rexp(n, rate = exp(0.5 * X[, 1]))
#' status <- rbinom(n, 1, 0.7)
#'
#' fit <- SurvivalBART(time = time, status = status, X_train = X,
#'                     number_of_trees = 5, N_post = 50, N_burn = 25,
#'                     verbose = FALSE)
#'
#' # S3 methods
#' print(fit)
#' smry <- summary(fit)
#'
#' # Posterior predictions on new data
#' X_new <- matrix(rnorm(10 * p), ncol = p)
#' pred <- predict(fit, newdata = X_new)
#' print(pred)
#'
#' # Diagnostic plot (requires ggplot2)
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   plot(fit, type = "trace")
#'
#'   # Posterior survival curves for training data
#'   plot(fit, type = "survival")
#'
#'   # Posterior predictive survival curves for new data
#'   plot(pred, type = "survival")
#'   plot(pred, type = "survival", obs = c(1, 5))
#' }
#'
#' # Interval-censored example
#' set.seed(11)
#' n <- 30; p <- 5
#' X <- matrix(rnorm(n * p), ncol = p)
#' true_t <- rexp(n, rate = exp(0.5 * X[, 1]))
#' left_t  <- true_t * runif(n, 0.5, 1)
#' right_t <- true_t * runif(n, 1, 1.5)
#' exact <- sample(n, 10); left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
#' rc <- sample(setdiff(seq_len(n), exact), 5); right_t[rc] <- Inf
#'
#' fit_ic <- SurvivalBART(left_time = left_t, right_time = right_t,
#'                         X_train = X, number_of_trees = 5,
#'                         N_post = 50, N_burn = 25, verbose = FALSE)
#'
#' @export
SurvivalBART <- function(
  time = NULL,
  status = NULL,
  X_train,
  X_test = NULL,
  timescale = "time",
  number_of_trees = 200,
  k = 2.0,
  N_post = 1000,
  N_burn = 1000,
  store_posterior_sample = TRUE,
  verbose = TRUE,
  left_time = NULL,
  right_time = NULL,
  ...
) {

  interval_censored <- !is.null(left_time) && !is.null(right_time)

  if (interval_censored) {
    if (any(left_time <= 0) && timescale == "time")
      stop("Survival times must be strictly positive when timescale = 'time'.")

    # Derive representative values for local_hp calibration
    y_proxy <- ifelse(left_time == right_time, left_time,
                      ifelse(is.finite(right_time),
                             (left_time + right_time) / 2, left_time))

    if (timescale == "time") {
      local_hp <- (max(log(y_proxy)) - min(log(y_proxy))) /
        (k * 2 * sqrt(number_of_trees))
    } else {
      local_hp <- (max(y_proxy) - min(y_proxy)) /
        (k * 2 * sqrt(number_of_trees))
    }

    fit <- ShrinkageTrees(
            left_time = left_time,
            right_time = right_time,
            X_train = X_train,
            X_test = X_test,
            outcome_type = "interval-censored",
            timescale = timescale,
            number_of_trees = number_of_trees,
            prior_type = "standard",
            local_hp = local_hp,
            N_post = N_post,
            N_burn = N_burn,
            store_posterior_sample = store_posterior_sample,
            verbose = verbose,
            ...
    )
  } else {
    if (is.null(time) || is.null(status))
      stop("'time' and 'status' are required for right-censored outcomes.")

    if (any(time <= 0) && timescale == "time")
      stop("Survival times must be strictly positive when timescale = 'time'.")

    if (timescale == "time") {
      local_hp <- (max(log(time)) - min(log(time))) /
        (k * 2 * sqrt(number_of_trees))
    } else {
      local_hp <- (max(time) - min(time)) /
        (k * 2 * sqrt(number_of_trees))
    }

    fit <- ShrinkageTrees(
            y = time,
            status = status,
            X_train = X_train,
            X_test = X_test,
            outcome_type = "right-censored",
            timescale = timescale,
            number_of_trees = number_of_trees,
            prior_type = "standard",
            local_hp = local_hp,
            N_post = N_post,
            N_burn = N_burn,
            store_posterior_sample = store_posterior_sample,
            verbose = verbose,
            ...
    )
  }

  fit$wrapper <- "SurvivalBART"
  return(fit)
}


#' SurvivalDART
#'
#' Fits an Accelerated Failure Time (AFT) model using the Dirichlet splitting
#' prior (DART), which induces structural sparsity through a Beta-Dirichlet
#' hierarchy on splitting probabilities.
#' Supports both right-censored and interval-censored survival outcomes.
#'
#' @inheritParams SurvivalBART
#' @param number_of_trees Number of trees in the ensemble. Default is 200.
#' @param a_dirichlet,b_dirichlet Beta hyperparameters controlling sparsity
#'   in the Dirichlet splitting rule.
#' @param rho_dirichlet Expected number of active predictors. If NULL,
#'   defaults to the number of covariates in \code{X_train}.
#' @param k Scaling constant used to calibrate the prior variance of the
#'   step heights.
#' @param ... Additional arguments passed to \code{\link{ShrinkageTrees}}
#'   to override default hyperparameters.
#'
#' @details
#' This function provides a survival-specific wrapper for DART under an
#' AFT formulation for right-censored or interval-censored outcomes.
#'
#' For right-censored data, supply \code{time} and \code{status}.
#' For interval-censored data, supply \code{left_time} and \code{right_time}
#' instead; event indicators are derived internally following the
#' \code{survival::Surv(type = "interval2")} convention.
#'
#' Structural regularisation is induced through a Dirichlet prior on
#' splitting probabilities, encouraging sparse feature usage in
#' high-dimensional settings.
#'
#' Users requiring alternative shrinkage priors on the leaf parameters
#' (e.g., Horseshoe or half-Cauchy priors) should use
#' \code{\link{ShrinkageTrees}} directly.
#'
#' @seealso
#' Related models: \code{\link{SurvivalBART}} (standard BART prior),
#' \code{\link{ShrinkageTrees}} (general shrinkage priors).
#'
#' S3 methods: \code{\link{print.ShrinkageTrees}},
#' \code{\link{summary.ShrinkageTrees}},
#' \code{\link{predict.ShrinkageTrees}},
#' \code{\link{plot.ShrinkageTrees}}.
#'
#' @return
#' An object of class \code{"ShrinkageTrees"} fitted under a Dirichlet
#' splitting prior (DART) within an AFT formulation.
#'
#' See \code{\link{ShrinkageTrees}} for a full description of returned components.
#'
#' @examples
#' set.seed(2)
#' n <- 30; p <- 5
#' X <- matrix(rnorm(n * p), ncol = p)
#' time <- rexp(n, rate = exp(0.5 * X[, 1]))
#' status <- rbinom(n, 1, 0.7)
#'
#' fit <- SurvivalDART(time = time, status = status, X_train = X,
#'                     number_of_trees = 5, N_post = 50, N_burn = 25,
#'                     verbose = FALSE)
#'
#' # S3 methods
#' print(fit)
#' smry <- summary(fit)
#'
#' # Posterior predictions on new data
#' X_new <- matrix(rnorm(10 * p), ncol = p)
#' pred <- predict(fit, newdata = X_new)
#' print(pred)
#'
#' # Variable importance and survival plots (requires ggplot2)
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   plot(fit, type = "vi", n_vi = 5)
#'
#'   # Posterior survival curves for training data
#'   plot(fit, type = "survival")
#'
#'   # Posterior predictive survival curves for new data
#'   plot(pred, type = "survival")
#'   plot(pred, type = "survival", obs = c(1, 5))
#' }
#'
#' # Interval-censored example
#' set.seed(12)
#' n <- 30; p <- 5
#' X <- matrix(rnorm(n * p), ncol = p)
#' true_t <- rexp(n, rate = exp(0.5 * X[, 1]))
#' left_t  <- true_t * runif(n, 0.5, 1)
#' right_t <- true_t * runif(n, 1, 1.5)
#' exact <- sample(n, 10); left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
#' rc <- sample(setdiff(seq_len(n), exact), 5); right_t[rc] <- Inf
#'
#' fit_ic <- SurvivalDART(left_time = left_t, right_time = right_t,
#'                         X_train = X, number_of_trees = 5,
#'                         N_post = 50, N_burn = 25, verbose = FALSE)
#'
#' @export
SurvivalDART <- function(
  time = NULL,
  status = NULL,
  X_train,
  X_test = NULL,
  timescale = "time",
  number_of_trees = 200,
  a_dirichlet = 0.5,
  b_dirichlet = 1.0,
  rho_dirichlet = NULL,
  k = 2.0,
  N_post = 1000,
  N_burn = 1000,
  store_posterior_sample = TRUE,
  verbose = TRUE,
  left_time = NULL,
  right_time = NULL,
  ...
) {

  interval_censored <- !is.null(left_time) && !is.null(right_time)

  if (is.null(rho_dirichlet)) {
    rho_dirichlet <- ncol(X_train)
  }

  if (interval_censored) {
    if (any(left_time <= 0) && timescale == "time")
      stop("Survival times must be strictly positive when timescale = 'time'.")

    # Derive representative values for local_hp calibration
    y_proxy <- ifelse(left_time == right_time, left_time,
                      ifelse(is.finite(right_time),
                             (left_time + right_time) / 2, left_time))

    if (timescale == "time") {
      local_hp <- (max(log(y_proxy)) - min(log(y_proxy))) /
        (k * 2 * sqrt(number_of_trees))
    } else {
      local_hp <- (max(y_proxy) - min(y_proxy)) /
        (k * 2 * sqrt(number_of_trees))
    }

    fit <- ShrinkageTrees(
            left_time = left_time,
            right_time = right_time,
            X_train = X_train,
            X_test = X_test,
            outcome_type = "interval-censored",
            timescale = timescale,
            number_of_trees = number_of_trees,
            prior_type = "dirichlet",
            local_hp = local_hp,
            a_dirichlet = a_dirichlet,
            b_dirichlet = b_dirichlet,
            rho_dirichlet = rho_dirichlet,
            N_post = N_post,
            N_burn = N_burn,
            store_posterior_sample = store_posterior_sample,
            verbose = verbose,
            ...
          )
  } else {
    if (is.null(time) || is.null(status))
      stop("'time' and 'status' are required for right-censored outcomes.")

    if (any(time <= 0) && timescale == "time")
      stop("Survival times must be strictly positive when timescale = 'time'.")

    if (timescale == "time") {
      local_hp <- (max(log(time)) - min(log(time))) /
        (k * 2 * sqrt(number_of_trees))
    } else {
      local_hp <- (max(time) - min(time)) /
        (k * 2 * sqrt(number_of_trees))
    }

    fit <- ShrinkageTrees(
            y = time,
            status = status,
            X_train = X_train,
            X_test = X_test,
            outcome_type = "right-censored",
            timescale = timescale,
            number_of_trees = number_of_trees,
            prior_type = "dirichlet",
            local_hp = local_hp,
            a_dirichlet = a_dirichlet,
            b_dirichlet = b_dirichlet,
            rho_dirichlet = rho_dirichlet,
            N_post = N_post,
            N_burn = N_burn,
            store_posterior_sample = store_posterior_sample,
            verbose = verbose,
            ...
          )
  }

  fit$wrapper <- "SurvivalDART"
  return(fit)
}


#' SurvivalBCF (Bayesian Causal Forest for survival data)
#'
#' Fits an Accelerated Failure Time (AFT) version of Bayesian Causal Forest (BCF):
#' \eqn{Y = \mu(x) + W \tau(x) + \varepsilon}, where separate forests are used
#' for the prognostic (control) function \eqn{\mu(x)} and the treatment effect
#' function \eqn{\tau(x)}.
#'
#' This wrapper provides a survival-specific implementation using classical
#' BART-style priors for both forests. Supports both right-censored and
#' interval-censored survival outcomes.
#'
#' @inheritParams SurvivalBART
#' @param treatment Treatment indicator (0/1) for training data.
#' @param number_of_trees_control Number of trees in the control forest.
#'   Default is 200.
#' @param number_of_trees_treat Number of trees in the treatment forest.
#'   Default is 50.
#' @param power_control,base_control Tree-structure prior parameters for the
#'   control forest.
#' @param power_treat,base_treat Tree-structure prior parameters for the
#'   treatment forest.
#' @param ... Additional arguments passed to \code{\link{CausalShrinkageForest}}
#'   to override default hyperparameters.
#' @param propensity Optional vector of propensity scores. If provided,
#' it is appended to the control forest design matrix. Required when
#' \code{treatment_coding = "adaptive"}.
#' @param treatment_coding Character string specifying how the treatment
#' indicator enters the model. One of \code{"centered"} (default, maps to
#' \eqn{-1/2} and \eqn{1/2}), \code{"binary"} (maps to \eqn{0} and \eqn{1}),
#' \code{"adaptive"} (maps to \eqn{z_i - \hat{e}(x_i)}, where
#' \eqn{\hat{e}(x_i)} is the propensity score), or \code{"invariant"}
#' (parameter-expanded coding with \eqn{b_0, b_1 \sim N(0, 1/2)} estimated
#' within the Gibbs sampler; Hahn et al., 2020, Section 5.2).
#'
#' @details
#' This function implements a simplified AFT-BCF model for right-censored or
#' interval-censored survival outcomes. Structural regularisation is induced
#' through classical BART priors on the tree structure and leaf parameters.
#'
#' For right-censored data, supply \code{time} and \code{status}.
#' For interval-censored data, supply \code{left_time} and \code{right_time}
#' instead; event indicators are derived internally following the
#' \code{survival::Surv(type = "interval2")} convention.
#'
#' Users requiring alternative shrinkage priors (e.g., Horseshoe or Dirichlet
#' splitting priors) should use \code{\link{SurvivalShrinkageBCF}} or call
#' \code{\link{CausalShrinkageForest}} directly.
#'
#' @references
#' Hahn, P. R., Murray, J. S., & Carvalho, C. M. (2020).
#' Bayesian regression tree models for causal inference:
#' Regularization, confounding, and heterogeneous effects.
#' Bayesian Analysis.
#'
#' @seealso
#' Related models: \code{\link{SurvivalShrinkageBCF}} (Dirichlet sparsity),
#' \code{\link{CausalHorseForest}} (horseshoe prior),
#' \code{\link{CausalShrinkageForest}} (general shrinkage priors).
#'
#' S3 methods: \code{\link{print.CausalShrinkageForest}},
#' \code{\link{summary.CausalShrinkageForest}},
#' \code{\link{predict.CausalShrinkageForest}},
#' \code{\link{plot.CausalShrinkageForest}}.
#'
#' @return
#' An object of class \code{"CausalShrinkageForest"} corresponding to a
#' survival BCF model under classical BART priors.
#'
#' See \code{\link{CausalShrinkageForest}} for returned components.
#'
#' @examples
#' set.seed(3)
#' n <- 30; p <- 5
#' X <- matrix(rnorm(n * p), ncol = p)
#' treatment <- rbinom(n, 1, 0.5)
#' log_T <- X[, 1] + treatment * (-0.5) + rnorm(n)
#' time <- exp(log_T)
#' status <- rbinom(n, 1, 0.7)
#'
#' fit <- SurvivalBCF(time = time, status = status, X_train = X,
#'                    treatment = treatment,
#'                    number_of_trees_control = 5,
#'                    number_of_trees_treat = 5,
#'                    N_post = 50, N_burn = 25,
#'                    verbose = FALSE)
#'
#' # S3 methods
#' print(fit)
#' smry <- summary(fit)
#'
#' # Posterior ATE
#' cat("ATE:", round(smry$treatment_effect$ate, 3), "\n")
#'
#' # Diagnostic and treatment-effect plots (requires ggplot2)
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   plot(fit, type = "trace")
#'   plot(fit, type = "ate")
#'   plot(fit, type = "cate")
#' }
#'
#' # Interval-censored causal example
#' set.seed(13)
#' n <- 30; p <- 5
#' X <- matrix(rnorm(n * p), ncol = p)
#' treatment <- rbinom(n, 1, 0.5)
#' true_t <- exp(X[, 1] + treatment * (-0.5) + rnorm(n))
#' left_t  <- true_t * runif(n, 0.5, 1)
#' right_t <- true_t * runif(n, 1, 1.5)
#' exact <- sample(n, 10); left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
#' rc <- sample(setdiff(seq_len(n), exact), 5); right_t[rc] <- Inf
#'
#' fit_ic <- SurvivalBCF(left_time = left_t, right_time = right_t,
#'                        X_train = X, treatment = treatment,
#'                        number_of_trees_control = 5,
#'                        number_of_trees_treat = 5,
#'                        N_post = 50, N_burn = 25, verbose = FALSE)
#'
#' @export
SurvivalBCF <- function(
  time = NULL,
  status = NULL,
  X_train,
  treatment,
  timescale = "time",
  propensity = NULL,
  treatment_coding = "centered",
  number_of_trees_control = 200,
  number_of_trees_treat = 50,
  power_control = 2,
  base_control = 0.95,
  power_treat = 3,
  base_treat = 0.25,
  N_post = 1000,
  N_burn = 1000,
  store_posterior_sample = TRUE,
  verbose = TRUE,
  left_time = NULL,
  right_time = NULL,
  ...
) {

  interval_censored <- !is.null(left_time) && !is.null(right_time)

  if (!interval_censored && (is.null(time) || is.null(status)))
    stop("'time' and 'status' are required for right-censored outcomes.")

  if (interval_censored && any(left_time <= 0) && timescale == "time")
    stop("Survival times must be strictly positive when timescale = 'time'.")

  if (!interval_censored && any(time <= 0) && timescale == "time")
    stop("Survival times must be strictly positive when timescale = 'time'.")

  if (length(treatment) != nrow(X_train))
    stop("Length of treatment must match number of training observations.")

  if (!is.null(propensity)) {
    X_train_control <- cbind(propensity, X_train)
  } else {
    X_train_control <- X_train
  }

  if (interval_censored) {
    # Derive status/ic_indicator/y from boundaries for censored_info
    ic_status    <- as.integer(left_time == right_time)
    ic_indicator <- as.integer(left_time < right_time & is.finite(right_time))
    y_derived    <- ifelse(ic_status == 1, left_time,
                           ifelse(ic_indicator == 1,
                                  (left_time + right_time) / 2, left_time))

    info <- censored_info(y_derived, ic_status,
                          left_time = left_time, right_time = right_time,
                          ic_indicator = ic_indicator)
    sd_time <- info$sd

    fit <- CausalShrinkageForest(
      left_time = left_time,
      right_time = right_time,
      X_train_control = X_train_control,
      X_train_treat = X_train,
      treatment_indicator_train = treatment,
      outcome_type = "interval-censored",
      timescale = timescale,

      prior_type_control = "standard-halfcauchy",
      prior_type_treat = "standard",

      local_hp_treat = 1 / (sd_time * sqrt(number_of_trees_treat) * 0.674),

      number_of_trees_control = number_of_trees_control,
      number_of_trees_treat = number_of_trees_treat,

      power_control = power_control,
      base_control = base_control,
      power_treat = power_treat,
      base_treat = base_treat,

      treatment_coding = treatment_coding,
      propensity = propensity,

      N_post = N_post,
      N_burn = N_burn,
      store_posterior_sample = store_posterior_sample,
      verbose = verbose,
      ...
    )
  } else {
    info <- censored_info(time, status)
    sd_time <- info$sd

    fit <- CausalShrinkageForest(
      y = time,
      status = status,
      X_train_control = X_train_control,
      X_train_treat = X_train,
      treatment_indicator_train = treatment,
      outcome_type = "right-censored",
      timescale = timescale,

      prior_type_control = "standard-halfcauchy",
      prior_type_treat = "standard",

      local_hp_treat = 1 / (sd_time * sqrt(number_of_trees_treat) * 0.674),

      number_of_trees_control = number_of_trees_control,
      number_of_trees_treat = number_of_trees_treat,

      power_control = power_control,
      base_control = base_control,
      power_treat = power_treat,
      base_treat = base_treat,

      treatment_coding = treatment_coding,
      propensity = propensity,

      N_post = N_post,
      N_burn = N_burn,
      store_posterior_sample = store_posterior_sample,
      verbose = verbose,
      ...
    )
  }

  fit$wrapper <- "SurvivalBCF"
  return(fit)
}

#' SurvivalShrinkageBCF (Shrinkage Bayesian Causal Forest for survival data)
#'
#' Fits a survival version of a Bayesian Causal Forest (BCF) under an
#' accelerated failure time (AFT) model, combining Dirichlet splitting
#' priors with global-local shrinkage. Supports both right-censored and
#' interval-censored survival outcomes.
#'
#' This wrapper extends \code{\link{SurvivalBCF}} by incorporating
#' Dirichlet sparsity in both the prognostic (control) and treatment
#' forests, while applying additional shrinkage to the control forest
#' via a half-Cauchy prior.
#'
#' @inheritParams SurvivalBCF
#' @param a_dir First shape parameter of the Beta prior controlling the 
#' sparsity level in the Dirichlet splitting rule.
#' @param b_dir Second shape parameter of the Beta prior controlling the 
#' sparsity level in the Dirichlet splitting rule.
#'
#' @details
#' The SurvivalShrinkageBCF model decomposes the outcome as
#' \deqn{
#'   \log T = \mu(x) + a \cdot \tau(x) + \varepsilon,
#' }
#' where \eqn{\mu(x)} represents the prognostic (control) component and 
#' \eqn{\tau(x)} the heterogeneous treatment effect.
#'
#' In contrast to \code{SurvivalBCF}, this function:
#' \itemize{
#'   \item Applies a Dirichlet splitting prior to both forests, inducing 
#'   structural sparsity in variable selection.
#'   \item Combines Dirichlet sparsity with additional half-Cauchy shrinkage 
#'   in the control forest.
#' }
#'
#' The Dirichlet prior follows the sparse splitting framework of Linero (2018),
#' where splitting probabilities are governed by a Beta-Dirichlet hierarchy.
#' The sparsity level is controlled by \code{a_dir} and \code{b_dir}.
#'
#' Survival outcomes are modeled using an AFT formulation with censoring
#' handled via data augmentation. Both right-censored and interval-censored
#' data are supported. For interval-censored data, supply \code{left_time}
#' and \code{right_time} instead of \code{time} and \code{status}.
#'
#' @references
#' Caron, A., Baio, G., & Manolopoulou, I. (2022).
#' Shrinkage Bayesian Causal Forests for Heterogeneous Treatment Effects Estimation.
#' Journal of Computational and Graphical Statistics, 31(4), 1202--1214.
#' https://doi.org/10.1080/10618600.2022.2067549
#'
#' @seealso
#' Related models: \code{\link{SurvivalBCF}} (standard BART priors),
#' \code{\link{CausalShrinkageForest}} (general shrinkage priors),
#' \code{\link{CausalHorseForest}} (horseshoe prior).
#'
#' S3 methods: \code{\link{print.CausalShrinkageForest}},
#' \code{\link{summary.CausalShrinkageForest}},
#' \code{\link{predict.CausalShrinkageForest}},
#' \code{\link{plot.CausalShrinkageForest}}.
#'
#' @return
#' An object of class \code{"CausalShrinkageForest"} fitted with
#' Dirichlet splitting priors and additional shrinkage.
#'
#' @examples
#' set.seed(4)
#' n <- 30; p <- 5
#' X <- matrix(rnorm(n * p), ncol = p)
#' treatment <- rbinom(n, 1, 0.5)
#' log_T <- X[, 1] + treatment * (-0.5) + rnorm(n)
#' time <- exp(log_T)
#' status <- rbinom(n, 1, 0.7)
#'
#' fit <- SurvivalShrinkageBCF(time = time, status = status, X_train = X,
#'                              treatment = treatment,
#'                              number_of_trees_control = 5,
#'                              number_of_trees_treat = 5,
#'                              N_post = 50, N_burn = 25,
#'                              verbose = FALSE)
#'
#' # S3 methods
#' print(fit)
#' smry <- summary(fit)
#'
#' # Posterior ATE with 95% credible interval
#' cat("ATE:", round(smry$treatment_effect$ate, 3), "\n")
#'
#' # Diagnostic and treatment-effect plots (requires ggplot2)
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   plot(fit, type = "trace")
#'   plot(fit, type = "cate")
#' }
#'
#' # Interval-censored causal example
#' set.seed(14)
#' n <- 30; p <- 5
#' X <- matrix(rnorm(n * p), ncol = p)
#' treatment <- rbinom(n, 1, 0.5)
#' true_t <- exp(X[, 1] + treatment * (-0.5) + rnorm(n))
#' left_t  <- true_t * runif(n, 0.5, 1)
#' right_t <- true_t * runif(n, 1, 1.5)
#' exact <- sample(n, 10); left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
#' rc <- sample(setdiff(seq_len(n), exact), 5); right_t[rc] <- Inf
#'
#' fit_ic <- SurvivalShrinkageBCF(left_time = left_t, right_time = right_t,
#'                                 X_train = X, treatment = treatment,
#'                                 number_of_trees_control = 5,
#'                                 number_of_trees_treat = 5,
#'                                 N_post = 50, N_burn = 25, verbose = FALSE)
#'
#' @export
SurvivalShrinkageBCF <- function(
  time = NULL,
  status = NULL,
  X_train,
  treatment,
  timescale = "time",
  propensity = NULL,
  treatment_coding = "centered",
  a_dir = 0.5,
  b_dir = 1.0,
  number_of_trees_control = 200,
  number_of_trees_treat = 50,
  power_control = 2,
  base_control = 0.95,
  power_treat = 3,
  base_treat = 0.25,
  N_post = 1000,
  N_burn = 1000,
  store_posterior_sample = TRUE,
  verbose = TRUE,
  left_time = NULL,
  right_time = NULL,
  ...
) {

  interval_censored <- !is.null(left_time) && !is.null(right_time)

  if (!interval_censored && (is.null(time) || is.null(status)))
    stop("'time' and 'status' are required for right-censored outcomes.")

  if (interval_censored && any(left_time <= 0) && timescale == "time")
    stop("Survival times must be strictly positive when timescale = 'time'.")

  if (!interval_censored && any(time <= 0) && timescale == "time")
    stop("Survival times must be strictly positive when timescale = 'time'.")

  if (length(treatment) != nrow(X_train))
    stop("Length of treatment must match number of training observations.")

  if (!is.null(propensity)) {
    X_control <- cbind(propensity, X_train)
    rho_dir_control <- ncol(X_control)
    rho_dir_treat <- ncol(X_train)
  } else {
    X_control <- X_train
    rho_dir_control <- ncol(X_train)
    rho_dir_treat <- ncol(X_train)
  }

  if (interval_censored) {
    # Derive status/ic_indicator/y from boundaries for censored_info
    ic_status    <- as.integer(left_time == right_time)
    ic_indicator <- as.integer(left_time < right_time & is.finite(right_time))
    y_derived    <- ifelse(ic_status == 1, left_time,
                           ifelse(ic_indicator == 1,
                                  (left_time + right_time) / 2, left_time))

    info <- censored_info(y_derived, ic_status,
                          left_time = left_time, right_time = right_time,
                          ic_indicator = ic_indicator)
    sd_time <- info$sd

    fit <- CausalShrinkageForest(
      left_time = left_time,
      right_time = right_time,
      X_train_control = X_control,
      X_train_treat = X_train,
      treatment_indicator_train = treatment,
      outcome_type = "interval-censored",
      timescale = timescale,

      prior_type_control = "dirichlet-halfcauchy",
      prior_type_treat = "dirichlet",

      local_hp_treat = 1 / (sd_time * sqrt(number_of_trees_treat) * 0.674),

      number_of_trees_control = number_of_trees_control,
      number_of_trees_treat = number_of_trees_treat,

      power_control = power_control,
      base_control = base_control,
      power_treat = power_treat,
      base_treat = base_treat,

      a_dirichlet_control = a_dir,
      b_dirichlet_control = b_dir,
      rho_dirichlet_control = rho_dir_control,
      a_dirichlet_treat = a_dir,
      b_dirichlet_treat = b_dir,
      rho_dirichlet_treat = rho_dir_treat,

      treatment_coding = treatment_coding,
      propensity = propensity,

      N_post = N_post,
      N_burn = N_burn,
      store_posterior_sample = store_posterior_sample,
      verbose = verbose,
      ...
    )
  } else {
    info <- censored_info(time, status)
    sd_time <- info$sd

    fit <- CausalShrinkageForest(
      y = time,
      status = status,
      X_train_control = X_control,
      X_train_treat = X_train,
      treatment_indicator_train = treatment,
      outcome_type = "right-censored",
      timescale = timescale,

      prior_type_control = "dirichlet-halfcauchy",
      prior_type_treat = "dirichlet",

      local_hp_treat = 1 / (sd_time * sqrt(number_of_trees_treat) * 0.674),

      number_of_trees_control = number_of_trees_control,
      number_of_trees_treat = number_of_trees_treat,

      power_control = power_control,
      base_control = base_control,
      power_treat = power_treat,
      base_treat = base_treat,

      a_dirichlet_control = a_dir,
      b_dirichlet_control = b_dir,
      rho_dirichlet_control = rho_dir_control,
      a_dirichlet_treat = a_dir,
      b_dirichlet_treat = b_dir,
      rho_dirichlet_treat = rho_dir_treat,

      treatment_coding = treatment_coding,
      propensity = propensity,

      N_post = N_post,
      N_burn = N_burn,
      store_posterior_sample = store_posterior_sample,
      verbose = verbose,
      ...
    )
  }

  fit$wrapper <- "SurvivalShrinkageBCF"
  return(fit)
}

test_that("SurvivalBCF works for right-censored survival outcome", {
  
  n <- 50
  p <- 3
  
  # Generate covariates
  set.seed(123)
  X <- matrix(rnorm(n * p), ncol = p)
  treatment <- rbinom(n, 1, 0.5)
  
  # Generate AFT-style log survival times
  tau <- -0.5   # protective effect
  log_T0 <- 0.5 * X[, 1] + rnorm(n)
  log_T  <- log_T0 + treatment * tau
  true_time <- exp(log_T)
  
  # Censoring
  censor_time <- rexp(n, rate = 0.2)
  time_obs <- pmin(true_time, censor_time)
  status <- as.integer(true_time <= censor_time)
  
  # Fit model
  set.seed(1)
  fit <- SurvivalBCF(
    time = time_obs,
    status = status,
    X_train = X,
    treatment = treatment,
    timescale = "log",
    N_post = 10,
    N_burn = 5,
    verbose = FALSE
  )
  
  # --------------------
  # Basic structure
  # --------------------
  expect_type(fit, "list")
  
  expect_true("train_predictions" %in% names(fit))
  expect_true("train_predictions_control" %in% names(fit))
  expect_true("train_predictions_treat" %in% names(fit))
  expect_true("sigma" %in% names(fit))
  
  # --------------------
  # Dimensions
  # --------------------
  expect_length(fit$train_predictions, n)
  expect_length(fit$train_predictions_control, n)
  expect_length(fit$train_predictions_treat, n)
  expect_length(fit$sigma, 10)
  
  # --------------------
  # Numerical sanity
  # --------------------
  fit_numeric <- unlist(fit[vapply(fit, is.numeric, logical(1))])
  expect_false(any(is.na(fit_numeric)))
  expect_true(all(is.finite(fit_numeric)))
  expect_true(all(fit$sigma > 0))
  
  # --------------------
  # Posterior samples
  # --------------------
  expect_true("train_predictions_sample_control" %in% names(fit))
  expect_true("train_predictions_sample_treat" %in% names(fit))
  
  expect_equal(dim(fit$train_predictions_sample_control), c(10, n))
  expect_equal(dim(fit$train_predictions_sample_treat), c(10, n))
  
  # --------------------
  # Non-degeneracy
  # --------------------
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_control) > 0)
  expect_true(sd(fit$train_predictions_treat) > 0)
  
  # --------------------
  # Reproducibility
  # --------------------
  set.seed(1)
  fit2 <- SurvivalBCF(
    time = time_obs,
    status = status,
    X_train = X,
    treatment = treatment,
    timescale = "log",
    N_post = 10,
    N_burn = 5,
    verbose = FALSE
  )
  
  expect_equal(fit$train_predictions, fit2$train_predictions)
})

test_that("SurvivalDART works for right-censored survival outcome", {
  
  # Generate covariates
  set.seed(123)
  X <- matrix(rnorm(50 * 4), ncol = 4)
  X_test <- matrix(rnorm(50 * 4), ncol = 4)
  
  # Generate survival times (AFT-type data generating mechanism)
  linpred <- 0.5 * X[, 1] - 0.3 * X[, 2]
  true_time <- rexp(50, rate = exp(linpred))
  
  # Generate censoring
  censor_time <- rexp(50, rate = 0.5)
  
  # Observed time and status
  time_obs <- pmin(true_time, censor_time)
  status <- as.numeric(true_time <= censor_time)
  
  # Fit model
  set.seed(1)
  fit <- SurvivalDART(
    time = time_obs,
    status = status,
    X_train = X,
    X_test = X_test,
    timescale = "time",
    number_of_trees = 50,
    a_dirichlet = 0.5,
    b_dirichlet = 1.0,
    rho_dirichlet = ncol(X),
    k = 2.0,
    N_post = 10,
    N_burn = 5,
    verbose = FALSE
  )
  
  # Basic structure
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_true("test_predictions" %in% names(fit))
  expect_true("sigma" %in% names(fit))
  
  # Dimensions
  expect_length(fit$train_predictions, 50)
  expect_length(fit$test_predictions, 50)
  expect_length(fit$sigma, 10)
  
  # Numerical sanity
  expect_false(any(is.na(fit$train_predictions)))
  expect_false(any(is.nan(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))
  expect_true(all(fit$sigma > 0))
  
  # Posterior samples
  expect_true("train_predictions_sample" %in% names(fit))
  expect_true("test_predictions_sample" %in% names(fit))
  expect_equal(dim(fit$train_predictions_sample), c(10, 50))
  expect_equal(dim(fit$test_predictions_sample), c(10, 50))
  
  # Predictions not degenerate
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$test_predictions) > 0)
  expect_true(sd(fit$train_predictions_sample) > 0)
  expect_true(sd(fit$test_predictions_sample) > 0)
  
  # Reproducibility
  set.seed(1)
  fit2 <- SurvivalDART(
    time = time_obs,
    status = status,
    X_train = X,
    X_test = X_test,
    timescale = "time",
    number_of_trees = 50,
    a_dirichlet = 0.5,
    b_dirichlet = 1.0,
    rho_dirichlet = ncol(X),
    k = 2.0,
    N_post = 10,
    N_burn = 5,
    verbose = FALSE
  )
  
  expect_equal(fit$train_predictions, fit2$train_predictions)
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

CausalHorseForest_cpp <- function(nSEXP, p_treatSEXP, p_controlSEXP, X_train_treatSEXP, X_train_controlSEXP, ySEXP, status_indicatorSEXP, is_survivalSEXP, observed_left_timeSEXP, observed_right_timeSEXP, interval_censoring_indicatorSEXP, treatment_indicatorSEXP, n_testSEXP, X_test_controlSEXP, X_test_treatSEXP, treatment_indicator_testSEXP, no_trees_treatSEXP, power_treatSEXP, base_treatSEXP, p_grow_treatSEXP, p_prune_treatSEXP, omega_treatSEXP, prior_type_treatSEXP, param1_treatSEXP, param2_treatSEXP, reversible_treatSEXP, dirichlet_bool_treatSEXP, a_dirichlet_treatSEXP, b_dirichlet_treatSEXP, rho_dirichlet_treatSEXP, no_trees_controlSEXP, power_controlSEXP, base_controlSEXP, p_grow_controlSEXP, p_prune_controlSEXP, omega_controlSEXP, prior_type_controlSEXP, param1_controlSEXP, param2_controlSEXP, reversible_controlSEXP, dirichlet_bool_controlSEXP, a_dirichlet_controlSEXP, b_dirichlet_controlSEXP, rho_dirichlet_controlSEXP, sigma_knownSEXP, sigmaSEXP, lambdaSEXP, nuSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, store_posterior_sample_controlSEXP, store_posterior_sample_treatSEXP, verboseSEXP, treatment_codingSEXP, propensity_trainSEXP, propensity_testSEXP) {
    .Call(`_ShrinkageTrees_CausalHorseForest_cpp`, nSEXP, p_treatSEXP, p_controlSEXP, X_train_treatSEXP, X_train_controlSEXP, ySEXP, status_indicatorSEXP, is_survivalSEXP, observed_left_timeSEXP, observed_right_timeSEXP, interval_censoring_indicatorSEXP, treatment_indicatorSEXP, n_testSEXP, X_test_controlSEXP, X_test_treatSEXP, treatment_indicator_testSEXP, no_trees_treatSEXP, power_treatSEXP, base_treatSEXP, p_grow_treatSEXP, p_prune_treatSEXP, omega_treatSEXP, prior_type_treatSEXP, param1_treatSEXP, param2_treatSEXP, reversible_treatSEXP, dirichlet_bool_treatSEXP, a_dirichlet_treatSEXP, b_dirichlet_treatSEXP, rho_dirichlet_treatSEXP, no_trees_controlSEXP, power_controlSEXP, base_controlSEXP, p_grow_controlSEXP, p_prune_controlSEXP, omega_controlSEXP, prior_type_controlSEXP, param1_controlSEXP, param2_controlSEXP, reversible_controlSEXP, dirichlet_bool_controlSEXP, a_dirichlet_controlSEXP, b_dirichlet_controlSEXP, rho_dirichlet_controlSEXP, sigma_knownSEXP, sigmaSEXP, lambdaSEXP, nuSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, store_posterior_sample_controlSEXP, store_posterior_sample_treatSEXP, verboseSEXP, treatment_codingSEXP, propensity_trainSEXP, propensity_testSEXP)
}

HorseTrees_cpp <- function(nSEXP, pSEXP, n_testSEXP, X_trainSEXP, ySEXP, status_indicatorSEXP, is_survivalSEXP, observed_left_timeSEXP, observed_right_timeSEXP, interval_censoring_indicatorSEXP, X_testSEXP, number_of_treesSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, powerSEXP, baseSEXP, p_growSEXP, p_pruneSEXP, nuSEXP, lambdaSEXP, dirichlet_boolSEXP, a_dirichletSEXP, b_dirichletSEXP, rho_dirichletSEXP, sigmaSEXP, sigma_knownSEXP, omegaSEXP, param1SEXP, param2SEXP, prior_typeSEXP, reversibleSEXP, store_parametersSEXP, store_posterior_sampleSEXP, verboseSEXP) {
    .Call(`_ShrinkageTrees_HorseTrees_cpp`, nSEXP, pSEXP, n_testSEXP, X_trainSEXP, ySEXP, status_indicatorSEXP, is_survivalSEXP, observed_left_timeSEXP, observed_right_timeSEXP, interval_censoring_indicatorSEXP, X_testSEXP, number_of_treesSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, powerSEXP, baseSEXP, p_growSEXP, p_pruneSEXP, nuSEXP, lambdaSEXP, dirichlet_boolSEXP, a_dirichletSEXP, b_dirichletSEXP, rho_dirichletSEXP, sigmaSEXP, sigma_knownSEXP, omegaSEXP, param1SEXP, param2SEXP, prior_typeSEXP, reversibleSEXP, store_parametersSEXP, store_posterior_sampleSEXP, verboseSEXP)
}

probitHorseTrees_cpp <- function(nSEXP, pSEXP, n_testSEXP, X_trainSEXP, ySEXP, X_testSEXP, number_of_treesSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, powerSEXP, baseSEXP, p_growSEXP, p_pruneSEXP, omegaSEXP, latent_thresholdSEXP, param1SEXP, param2SEXP, prior_typeSEXP, reversibleSEXP, store_posterior_sampleSEXP, verboseSEXP) {
    .Call(`_ShrinkageTrees_probitHorseTrees_cpp`, nSEXP, pSEXP, n_testSEXP, X_trainSEXP, ySEXP, X_testSEXP, number_of_treesSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, powerSEXP, baseSEXP, p_growSEXP, p_pruneSEXP, omegaSEXP, latent_thresholdSEXP, param1SEXP, param2SEXP, prior_typeSEXP, reversibleSEXP, store_posterior_sampleSEXP, verboseSEXP)
}

#' General Shrinkage Regression Trees (ShrinkageTrees)
#'
#' Fits a Bayesian Shrinkage Tree model with flexible global-local priors on the
#' step heights. This function generalizes \code{\link{HorseTrees}} by allowing
#' different global-local shrinkage priors on the step heights.
#' Supports continuous, binary, right-censored, and interval-censored outcomes.
#'
#' @param y Outcome vector. Numeric. Can represent continuous outcomes, binary
#' outcomes (0/1), or follow-up times for survival data. Set to \code{NULL}
#' when using \code{outcome_type = "interval-censored"}, as values are derived
#' from \code{left_time} and \code{right_time}.
#' @param status Optional censoring indicator vector (1 = event occurred,
#' 0 = censored). Required if \code{outcome_type = "right-censored"}.
#' For interval-censored outcomes, this is derived automatically from
#' \code{left_time} and \code{right_time}.
#' @param X_train Covariate matrix for training. Each row corresponds to an
#' observation, and each column to a covariate.
#' @param X_test Optional covariate matrix for test data. If NULL, defaults to
#' the mean of the training covariates.
#' @param left_time Optional numeric vector of left (lower) time boundaries.
#' Required when \code{outcome_type = "interval-censored"}. Exact events
#' have \code{left_time == right_time}; right-censored observations have
#' \code{right_time = Inf}; interval-censored observations have finite
#' \code{left_time < right_time}.
#' @param right_time Optional numeric vector of right (upper) time boundaries.
#' Required when \code{outcome_type = "interval-censored"}. Use \code{Inf}
#' for right-censored observations.
#' @param outcome_type Type of outcome. One of \code{"continuous"},
#' \code{"binary"}, \code{"right-censored"}, or \code{"interval-censored"}.
#' @param timescale Indicates the scale of follow-up times. Options are
#' \code{"time"} (nonnegative follow-up times, will be log-transformed
#' internally) or \code{"log"} (already log-transformed). Used when
#' \code{outcome_type} is \code{"right-censored"} or
#' \code{"interval-censored"}.
#' @param number_of_trees Number of trees in the ensemble. Default is 200.
#' @param prior_type Type of prior on the step heights. Options include
#' `"horseshoe"`, `"horseshoe_fw"`, `"half-cauchy"`,
#' `"standard"` and `"dirichlet"`.
#' @param local_hp Local hyperparameter controlling shrinkage on individual
#' step heights. Should typically be set smaller than 1 / sqrt(number_of_trees).
#' Required for `prior_type = "standard"`.
#' @param global_hp Global hyperparameter controlling overall shrinkage.
#' Must be specified for Horseshoe-type priors; ignored for 
#' `prior_type = "half-cauchy"` or `"standard"`.
#' @param power Power parameter for the tree structure prior. Default is 2.0.
#' @param base Base parameter for the tree structure prior. Default is 0.95.
#' @param p_grow Probability of proposing a grow move. Default is 0.4.
#' @param p_prune Probability of proposing a prune move. Default is 0.4.
#' @param a_dirichlet First shape parameter of the Beta prior used in the
#' Dirichlet-Sparse splitting rule. Together with `b_dirichlet`, it
#' controls the expected sparsity level. Only when `prior_type = "dirichlet"`.
#' @param b_dirichlet Second shape parameter of the Beta prior for the
#' sparsity level. Larger values shrink splitting probabilities more strongly
#' toward uniform sparsity. Only when `prior_type = "dirichlet"`.
#' @param rho_dirichlet Sparsity hyperparameter. If left NULL, it defaults to
#' the number of covariates. Only when `prior_type = "dirichlet"`.
#' @param nu Degrees of freedom for the error distribution prior. Default is 3.
#' @param q Quantile hyperparameter for the error variance prior. Default is 0.90.
#' @param sigma Optional known value for error standard deviation. If NULL,
#' estimated from data.
#' @param N_post Number of posterior samples to store. Default is 1000.
#' @param N_burn Number of burn-in iterations. Default is 1000.
#' @param delayed_proposal Number of delayed iterations before proposal. Only
#' for reversible updates. Default is 5.
#' @param store_posterior_sample Logical; whether to store posterior samples for
#' each iteration. Default is TRUE.
#' @param n_chains Number of independent MCMC chains to run. Default is
#'   \code{1} (standard single-chain behaviour). When \code{n_chains > 1} the
#'   chains are run in parallel via \code{parallel::mclapply} and their
#'   posterior samples are pooled into a single \code{ShrinkageTrees} object,
#'   so all existing \code{print}, \code{summary}, and \code{predict} methods
#'   work without modification. On Windows, \code{mclapply} falls back to
#'   sequential execution.
#' @param verbose Logical; whether to print verbose output. Default is TRUE.
#'
#' @return An S3 object of class "ShrinkageTrees" containing the following elements:
#' \describe{
#'   \item{train_predictions}{Vector of posterior mean predictions on the
#'   training data.}
#'   \item{test_predictions}{Vector of posterior mean predictions on the test
#'   data (or on mean covariate vector if `X_test` not provided).}
#'   \item{sigma}{Vector of posterior samples of the error variance.}
#'   \item{acceptance_ratio}{Average acceptance ratio across trees during
#'   sampling.}
#'   \item{train_predictions_sample}{Matrix of posterior samples of training
#'   predictions (iterations in rows, observations in columns). Present only if
#'   `store_posterior_sample = TRUE`.}
#'   \item{test_predictions_sample}{Matrix of posterior samples of test
#'   predictions. Present only if `store_posterior_sample = TRUE`.}
#'   \item{train_probabilities}{Vector of posterior mean probabilities on the
#'   training data (only for `outcome_type = "binary"`).}
#'   \item{test_probabilities}{Vector of posterior mean probabilities on the
#'   test data (only for `outcome_type = "binary"`).}
#'   \item{train_probabilities_sample}{Matrix of posterior samples of training
#'   probabilities (only for `outcome_type = "binary"` and if
#'   `store_posterior_sample = TRUE`).}
#'   \item{test_probabilities_sample}{Matrix of posterior samples of test
#'   probabilities (only for `outcome_type = "binary"` and if
#'   `store_posterior_sample = TRUE`).}
#' }
#'
#' @details
#' This function is a flexible generalization of \code{HorseTrees}. 
#' Instead of using a single Horseshoe prior, it allows specifying different
#' global–local shrinkage configurations for the tree step heights. Further
#' methodological details on the Horseshoe Forest framework can be found in 
#' Jacobs, van Wieringen & van der Pas (2025).
#'
#' The \code{horseshoe} prior is the fully Bayesian global-local shrinkage
#' prior, where both the global and local shrinkage parameters are assigned
#' half-Cauchy distributions with scale hyperparameters \code{global_hp} and
#' \code{local_hp}, respectively. The global shrinkage parameter is defined
#' separately for each tree, allowing adaptive regularization per tree.
#'
#' The \code{horseshoe_fw} prior (forest-wide horseshoe) is similar to
#' \code{horseshoe}, except that the global shrinkage parameter is shared
#' across all trees in the forest simultaneously. 
#'
#' The \code{half-cauchy} prior considers only local shrinkage and does not
#' include a global shrinkage component. It places a half-Cauchy prior on each
#' local shrinkage parameter with scale hyperparameter \code{local_hp}.
#' 
#' The \code{standard} prior (Chipman, George & McCulloch, 2010) corresponds to
#' the classical BART specification, where step heights are given a normal 
#' prior with variance scaled by the number of trees. This prior does not
#' introduce a global shrinkage parameter and does not use global–local 
#' structure.
#'
#' The \code{dirichlet} prior implements the Dirichlet–Sparse splitting rule of 
#' Linero (2018), in which splitting probabilities follow a Dirichlet prior 
#' whose concentration is controlled by a Beta sparsity parameter 
#' (\code{a_dirichlet}, \code{b_dirichlet}) and an expected sparsity level 
#' \code{rho_dirichlet}. 
#'
#' @references
#' Jacobs, T., van Wieringen, W. N., & van der Pas, S. L. (2025).  
#' *Horseshoe Forests for High-Dimensional Causal Survival Analysis.*  
#' arXiv:2507.22004. https://doi.org/10.48550/arXiv.2507.22004
#' Chipman, H. A., George, E. I., & McCulloch, R. E. (2010). 
#' *BART: Bayesian additive regression trees.* Annals of Applied Statistics.
#'
#' Linero, A. R. (2018). *Bayesian regression trees for high-dimensional 
#' prediction and variable selection.* Journal of the American Statistical 
#' Association.
#' 
#' @examples
#' # Example: Continuous outcome with ShrinkageTrees, two priors
#' n <- 50
#' p <- 3
#' X <- matrix(runif(n * p), ncol = p)
#' X_test <- matrix(runif(n * p), ncol = p)
#' y <- X[, 1] + rnorm(n)
#'
#' # Fit ShrinkageTrees with standard horseshoe prior
#' fit_horseshoe <- ShrinkageTrees(y = y,
#'                                 X_train = X,
#'                                 X_test = X_test,
#'                                 outcome_type = "continuous",
#'                                 number_of_trees = 5,
#'                                 prior_type = "horseshoe",
#'                                 local_hp = 0.1 / sqrt(5),
#'                                 global_hp = 0.1 / sqrt(5),
#'                                 N_post = 10,
#'                                 N_burn = 5,
#'                                 store_posterior_sample = TRUE,
#'                                 verbose = FALSE)
#'
#' # Fit ShrinkageTrees with half-Cauchy prior
#' fit_halfcauchy <- ShrinkageTrees(y = y,
#'                                  X_train = X,
#'                                  X_test = X_test,
#'                                  outcome_type = "continuous",
#'                                  number_of_trees = 5,
#'                                  prior_type = "half-cauchy",
#'                                  local_hp = 1 / sqrt(5),
#'                                  N_post = 10,
#'                                  N_burn = 5,
#'                                  store_posterior_sample = TRUE,
#'                                  verbose = FALSE)
#'
#' # Posterior mean predictions
#' pred_horseshoe <- colMeans(fit_horseshoe$train_predictions_sample)
#' pred_halfcauchy <- colMeans(fit_halfcauchy$train_predictions_sample)
#'
#' # Posteriors of the mean (global average prediction)
#' post_mean_horseshoe <- rowMeans(fit_horseshoe$train_predictions_sample)
#' post_mean_halfcauchy <- rowMeans(fit_halfcauchy$train_predictions_sample)
#'
#' # Posterior mean prediction averages
#' mean_pred_horseshoe <- mean(post_mean_horseshoe)
#' mean_pred_halfcauchy <- mean(post_mean_halfcauchy)
#'
#' # Example: Interval-censored survival outcome
#' n <- 50; p <- 3
#' X_ic <- matrix(rnorm(n * p), ncol = p)
#' true_t <- rexp(n, rate = exp(-X_ic[, 1]))
#' left_t  <- true_t * runif(n, 0.5, 1)
#' right_t <- true_t * runif(n, 1, 1.5)
#' exact <- sample(n, 15)
#' left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
#' rc <- sample(setdiff(seq_len(n), exact), 10); right_t[rc] <- Inf
#'
#' fit_ic <- ShrinkageTrees(left_time = left_t, right_time = right_t,
#'                          X_train = X_ic,
#'                          outcome_type = "interval-censored",
#'                          prior_type = "horseshoe",
#'                          local_hp = 0.1 / sqrt(5),
#'                          global_hp = 0.1 / sqrt(5),
#'                          number_of_trees = 5,
#'                          N_post = 10, N_burn = 5,
#'                          verbose = FALSE)
#'
#' @seealso
#' Model family: \code{\link{HorseTrees}} (horseshoe prior),
#' \code{\link{CausalHorseForest}} (causal inference),
#' \code{\link{CausalShrinkageForest}} (causal, flexible prior).
#'
#' Survival wrappers: \code{\link{SurvivalBART}}, \code{\link{SurvivalDART}}.
#'
#' S3 methods: \code{\link{print.ShrinkageTrees}},
#' \code{\link{summary.ShrinkageTrees}},
#' \code{\link{predict.ShrinkageTrees}},
#' \code{\link{plot.ShrinkageTrees}}.
#'
#' @importFrom Rcpp evalCpp
#' @useDynLib ShrinkageTrees, .registration = TRUE
#' @importFrom stats sd qchisq qnorm pnorm runif
#' @importFrom parallel mclapply detectCores
#' @export
ShrinkageTrees <- function(y = NULL,
                           status = NULL,
                           X_train,
                           X_test = NULL,
                           left_time = NULL,
                           right_time = NULL,
                           outcome_type = "continuous",
                           timescale = "time",
                           number_of_trees = 200,
                           prior_type = "horseshoe",
                           local_hp = NULL,
                           global_hp = NULL,
                           a_dirichlet = 0.5,
                           b_dirichlet = 1.0,
                           rho_dirichlet = NULL,
                           power = 2.0,
                           base = 0.95,
                           p_grow = 0.4,
                           p_prune = 0.4,
                           nu = 3,
                           q = 0.90,
                           sigma = NULL,
                           N_post = 1000,
                           N_burn = 1000,
                           delayed_proposal = 5,
                           store_posterior_sample = TRUE,
                           n_chains = 1,
                           verbose = TRUE) {

  # ── Multi-chain dispatch ──────────────────────────────────────────────────
  if (n_chains > 1) {
    mc <- match.call()

    # Collect all evaluated arguments for the per-chain calls
    chain_args <- list(
      y                      = y,
      status                 = status,
      X_train                = X_train,
      X_test                 = X_test,
      left_time              = left_time,
      right_time             = right_time,
      outcome_type           = outcome_type,
      timescale              = timescale,
      number_of_trees        = number_of_trees,
      prior_type             = prior_type,
      local_hp               = local_hp,
      global_hp              = global_hp,
      a_dirichlet            = a_dirichlet,
      b_dirichlet            = b_dirichlet,
      rho_dirichlet          = rho_dirichlet,
      power                  = power,
      base                   = base,
      p_grow                 = p_grow,
      p_prune                = p_prune,
      nu                     = nu,
      q                      = q,
      sigma                  = sigma,
      N_post                 = N_post,
      N_burn                 = N_burn,
      delayed_proposal       = delayed_proposal,
      store_posterior_sample = store_posterior_sample,
      n_chains               = 1,
      verbose                = FALSE
    )

    n_cores <- .resolve_cores(n_chains)
    if (verbose)
      message("Running ", n_chains, " chains (", n_cores, " cores) ...")

    chains <- parallel::mclapply(
      seq_len(n_chains),
      function(i) do.call(ShrinkageTrees, chain_args),
      mc.cores = n_cores
    )

    combined       <- .combine_chains(chains)
    combined$call  <- mc
    return(combined)
  }


  
  # Check outcome_type value
  allowed_types <- c("continuous", "binary", "right-censored", "interval-censored")
  if (!outcome_type %in% allowed_types) {
    stop("Invalid outcome_type. Please choose 'continuous', 'binary',
         'right-censored', or 'interval-censored'.")
  }

  # Check that y is provided for non-interval-censored types
  if (is.null(y) && outcome_type != "interval-censored") {
    stop("'y' is required when outcome_type is not 'interval-censored'.")
  }

  # Check interval-censored arguments
  if (outcome_type == "interval-censored") {
    if (is.null(left_time) || is.null(right_time))
      stop("outcome_type = 'interval-censored' requires 'left_time' and 'right_time'.")
    if (length(left_time) != length(right_time))
      stop("'left_time' and 'right_time' must have the same length.")
    if (any(left_time > right_time))
      stop("All 'left_time' values must be <= corresponding 'right_time' values.")
    if (timescale == "time" && any(left_time <= 0))
      stop("left_time contains non-positive values, but timescale = 'time' requires strictly positive times.")
  }
  
  # Check prior_type value
  allowed_prior <- c("horseshoe", "horseshoe_fw", "half-cauchy", "standard", "dirichlet")
  if (!prior_type %in% allowed_prior) {
    stop("Invalid prior_type. Choose 'horseshoe', 'horseshoe_fw', 'half-cauchy', 'standard' or 'dirichlet.")
  }
  
  # Prior-specific checks
  if (prior_type %in% c("horseshoe", "horseshoe_fw")) {
    if (is.null(local_hp) || is.null(global_hp)) {
      stop("For prior_type = 'horseshoe' or 'horseshoe_fw', you must provide both local_hp and global_hp.")
    }
  }
  
  if (prior_type == "half-cauchy") {
    if (is.null(local_hp)) {
      stop("For prior_type = 'half-cauchy', you must provide local_hp.")
    }
    if (!is.null(global_hp)) {
      warning("global_hp is ignored for 'half-cauchy'.")
    }
    global_hp <- 1
    prior_type <- "halfcauchy"
  }
  
  # Default reversible flag
  reversible_flag <- TRUE

  # 'standard' BART prior: local-only shrinkage, no RJ moves
  if (prior_type %in% c("standard", "dirichlet")) {
    if (is.null(local_hp)) {
      stop("For prior_type = 'standard', you must provide local_hp.")
    }
    if (!is.null(global_hp)) {
      warning("global_hp is ignored for 'standard' or 'dirichlet' prior.")
    }

    global_hp <- 1          # placeholder (ignored by C++)
    reversible_flag <- FALSE

    if (delayed_proposal > 0) {
      delayed_proposal <- 0
    }
  }

  # Check consistency with status argument
  if (outcome_type == "right-censored" && is.null(status)) {
    stop("You specified outcome_type = 'right-censored', but did not provide a
         'status' vector.")
  }
  if (!is.null(status) && !is.null(y) && length(status) != length(y)) {
    stop("The length of 'status' must match the length of 'y'.")
  }

  if (!outcome_type %in% c("right-censored", "interval-censored") && !is.null(status)) {
    warning("You provided a 'status' vector, but outcome_type is not
            'right-censored' or 'interval-censored'. The 'status' vector will be ignored.")
  }

  # Check binary data consistency
  if (outcome_type == "binary" && !all(y %in% c(0, 1))) {
    stop("For outcome_type = 'binary', y must contain only 0 and 1.")
  }
  if (outcome_type != "binary" && !is.null(y) && all(y %in% c(0, 1))) {
    warning("The outcome y contains only 0 and 1, but outcome_type is not set to
            'binary'. Consider setting outcome_type = 'binary'.")
  }

  # Check survival data and timescale
  if (outcome_type == "right-censored" && timescale == "time" && any(y < 0)) {
    stop("Outcome contains negative values, but timescale = 'time' for survival
         data requires non-negative times.")
  }

  # Retrieve dimensions of training data
  n_train <- nrow(X_train)
  p_features <- ncol(X_train)

  # Check if dimensions covariates match with outcome
  if (outcome_type == "interval-censored") {
    if (length(left_time) != n_train)
      stop("The length of 'left_time' must match the number of rows in X_train.")
  } else {
    if (length(y) != n_train)
      stop("The length of outcome vector y must match the number of rows in X_train.")
  }
  
  # Check if dimensions match with test data
  if (!is.null(X_test)) {
    n_test <- nrow(X_test)
    if (ncol(X_test) != p_features) {
      stop("The number of covariates in X_test must match X_train.")
    }
    X_test <- as.numeric(t(X_test))
  } else {
    n_test <- 1
    X_test <- as.numeric(colMeans(X_train))
  }
  
  # Force data types to be numeric and plain arrays
  N_post <- as.integer(N_post)[1]
  N_burn <- as.integer(N_burn)[1]
  power <- as.numeric(power)[1]
  base <- as.numeric(base)[1]
  p_grow <- as.numeric(p_grow)[1]
  p_prune <- as.numeric(p_prune)[1]
  X_train_mat <- X_train                 # preserve matrix for predict() / vi plots
  X_train <- as.numeric(t(X_train))
  
  # Default rho to number of features
  if (is.null(rho_dirichlet)) {
    rho_dirichlet <- p_features
  }
  if (prior_type == "dirichlet") { 
    dirichlet <- TRUE 
  } else { 
    dirichlet <- FALSE 
  }

  if (outcome_type == "right-censored") {

    # Convert y to numeric for C++ compatibility
    y <- as.numeric(y)

    # Log-transform survival times if timescale = "time"
    if (timescale == "time") {
      y <- log(y)
    }

    # Save y before centering/standardizing (for predict())
    y_train_raw <- y

    # Obtain estimated mean and standard deviation using censored_info()
    cens_inf <- censored_info(y, status)
    
    # Center the data using estimated mean
    y_mean <- cens_inf$mu
    y <- y - y_mean
    
    # Determine sigma (timescale parameter) and whether it is known
    if (is.null(sigma)) {
      sigma_hat <- cens_inf$sd
      if (prior_type == "standard" || prior_type == "dirichlet") sigma_hat <- 1
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma
      sigma_known <- TRUE
    }
    
    # Standardize the centered data
    y <- y / sigma_hat
    
    # Set survival flag
    survival <- TRUE
    
    # Compute lambda parameter for error distribution prior
    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu

    # Fit a HorseTrees model
    fit <- HorseTrees_cpp(
      nSEXP = n_train,
      pSEXP = p_features,
      n_testSEXP = n_test,
      X_trainSEXP = X_train,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = numeric(n_train),
      observed_right_timeSEXP = y + 0,
      interval_censoring_indicatorSEXP = numeric(n_train),
      X_testSEXP = X_test,
      number_of_treesSEXP = number_of_trees,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      powerSEXP = power,
      baseSEXP = base,
      p_growSEXP = p_grow,
      p_pruneSEXP = p_prune,
      nuSEXP = nu,
      lambdaSEXP = lambda,
      dirichlet_boolSEXP = dirichlet,
      a_dirichletSEXP = a_dirichlet,
      b_dirichletSEXP = b_dirichlet,
      rho_dirichletSEXP = rho_dirichlet,
      sigmaSEXP = sigma_hat,
      sigma_knownSEXP = sigma_known,
      omegaSEXP = 1,
      param1SEXP = local_hp,
      param2SEXP = global_hp,
      prior_typeSEXP = prior_type,
      reversibleSEXP = reversible_flag,
      store_parametersSEXP = FALSE,
      store_posterior_sampleSEXP = store_posterior_sample,
      verboseSEXP = verbose
    )
    
    if (timescale == "time") {
      fit$train_predictions <- exp(fit$train_predictions * sigma_hat + y_mean)
      fit$test_predictions <- exp(fit$test_predictions * sigma_hat + y_mean)
      if (store_posterior_sample) {
        fit$train_predictions_sample <- exp(fit$train_predictions_sample * sigma_hat + y_mean)
        fit$test_predictions_sample <- exp(fit$test_predictions_sample * sigma_hat + y_mean)
      }
    } else { # If timescale is "log"
      fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
      fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
      if (store_posterior_sample) { 
        fit$train_predictions_sample <- fit$train_predictions_sample * sigma_hat + y_mean
        fit$test_predictions_sample <- fit$test_predictions_sample * sigma_hat + y_mean
      }
    }
    
  # If interval-censored
  } else if (outcome_type == "interval-censored") {

    left_time <- as.numeric(left_time)
    right_time <- as.numeric(right_time)

    # Derive status: 1 if exact (left == right), 0 otherwise
    status <- as.integer(left_time == right_time)

    # Derive interval-censoring indicator: 1 if left < right and right is finite
    ic_indicator <- as.integer(left_time < right_time & is.finite(right_time))

    # Derive initial y values for MCMC
    y <- ifelse(status == 1, left_time,
                ifelse(ic_indicator == 1, (left_time + right_time) / 2,
                       left_time))

    # For right-censored obs (right_time == Inf): C++ expects finite boundary
    right_time[!is.finite(right_time)] <- left_time[!is.finite(right_time)]

    # Log-transform if timescale = "time"
    if (timescale == "time") {
      y <- log(y)
      left_time <- log(left_time)
      right_time <- log(right_time)
    }

    # Save before centering/standardizing (for predict())
    y_train_raw <- y
    left_time_raw <- left_time
    right_time_raw <- right_time

    # Estimate mu and sd
    cens_inf <- censored_info(y, status, left_time = left_time,
                              right_time = right_time, ic_indicator = ic_indicator)

    y_mean <- cens_inf$mu
    y <- y - y_mean
    left_time <- left_time - y_mean
    right_time <- right_time - y_mean

    if (is.null(sigma)) {
      sigma_hat <- cens_inf$sd
      if (prior_type == "standard" || prior_type == "dirichlet") sigma_hat <- 1
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma
      sigma_known <- TRUE
    }

    y <- y / sigma_hat
    left_time <- left_time / sigma_hat
    right_time <- right_time / sigma_hat

    survival <- TRUE

    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu

    fit <- HorseTrees_cpp(
      nSEXP = n_train,
      pSEXP = p_features,
      n_testSEXP = n_test,
      X_trainSEXP = X_train,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = left_time,
      observed_right_timeSEXP = right_time,
      interval_censoring_indicatorSEXP = ic_indicator,
      X_testSEXP = X_test,
      number_of_treesSEXP = number_of_trees,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      powerSEXP = power,
      baseSEXP = base,
      p_growSEXP = p_grow,
      p_pruneSEXP = p_prune,
      nuSEXP = nu,
      lambdaSEXP = lambda,
      dirichlet_boolSEXP = dirichlet,
      a_dirichletSEXP = a_dirichlet,
      b_dirichletSEXP = b_dirichlet,
      rho_dirichletSEXP = rho_dirichlet,
      sigmaSEXP = sigma_hat,
      sigma_knownSEXP = sigma_known,
      omegaSEXP = 1,
      param1SEXP = local_hp,
      param2SEXP = global_hp,
      prior_typeSEXP = prior_type,
      reversibleSEXP = reversible_flag,
      store_parametersSEXP = FALSE,
      store_posterior_sampleSEXP = store_posterior_sample,
      verboseSEXP = verbose
    )

    # Back-transform predictions (identical to right-censored)
    if (timescale == "time") {
      fit$train_predictions <- exp(fit$train_predictions * sigma_hat + y_mean)
      fit$test_predictions <- exp(fit$test_predictions * sigma_hat + y_mean)
      if (store_posterior_sample) {
        fit$train_predictions_sample <- exp(fit$train_predictions_sample * sigma_hat + y_mean)
        fit$test_predictions_sample <- exp(fit$test_predictions_sample * sigma_hat + y_mean)
      }
    } else {
      fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
      fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
      if (store_posterior_sample) {
        fit$train_predictions_sample <- fit$train_predictions_sample * sigma_hat + y_mean
        fit$test_predictions_sample <- fit$test_predictions_sample * sigma_hat + y_mean
      }
    }

    # If binary
  } else if (outcome_type == "binary") {
    y <- as.numeric(y)
    y_train_raw      <- y
    latent_threshold <- qnorm(mean(y))
    sigma_hat        <- 1
    sigma_known      <- TRUE
    y_mean           <- 0
    lambda           <- 0   # unused for probit
    
    fit <- probitHorseTrees_cpp(
      nSEXP = n_train,
      pSEXP = p_features,
      n_testSEXP = n_test,
      X_trainSEXP = X_train,
      ySEXP = y,
      X_testSEXP = X_test,
      number_of_treesSEXP = number_of_trees,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      powerSEXP = power,
      baseSEXP = base,
      p_growSEXP = p_grow,
      p_pruneSEXP = p_prune,
      omegaSEXP = 1,
      latent_thresholdSEXP = latent_threshold,
      param1SEXP = local_hp,
      param2SEXP = global_hp,
      prior_typeSEXP = prior_type,
      reversibleSEXP = reversible_flag,
      store_posterior_sampleSEXP = store_posterior_sample,
      verboseSEXP = verbose
    )
    
    # Add the pnorm transform
    fit$train_probabilities <- pnorm(fit$train_predictions)
    fit$test_probabilities<- pnorm(fit$test_predictions)
    if (store_posterior_sample) {
      fit$train_probabilities_sample <- pnorm(fit$train_predictions_sample)
      fit$test_probabilities_sample <- pnorm(fit$test_predictions_sample)
    }
    
    # Otherwise, continuous
  } else {

    # Force outcome to plain numeric vector
    y <- as.numeric(y)

    # Save y before centering/standardizing (for predict())
    y_train_raw <- y

    # Create dummy status vector (not used for continuous)
    status <- rep(1, n_train)
    survival <- FALSE
    
    # Determine prior guess of sigma 
    if (is.null(sigma)) {
      sigma_hat <- sd(y)      # Estimate sigma from data
      if (prior_type == "standard" || prior_type == "dirichlet") sigma_hat <- 1
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma      # Use provided sigma
      sigma_known <- TRUE
    }
    
    # Compute hyperparameters of error variance prior
    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu
    
    # Compute mean and standardize  the outcome
    y_mean <- mean(y)
    y <- y - y_mean
    y <- y / sigma_hat  # Standardize y
    
    fit <- HorseTrees_cpp(
      nSEXP = n_train,
      pSEXP = p_features,
      n_testSEXP = n_test,
      X_trainSEXP = X_train,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = numeric(n_train),
      observed_right_timeSEXP = y + 0,
      interval_censoring_indicatorSEXP = numeric(n_train),
      X_testSEXP = X_test,
      number_of_treesSEXP = number_of_trees,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      powerSEXP = power,
      baseSEXP = base,
      p_growSEXP = p_grow,
      p_pruneSEXP = p_prune,
      nuSEXP = nu,
      lambdaSEXP = lambda,
      dirichlet_boolSEXP = dirichlet,
      a_dirichletSEXP = a_dirichlet,
      b_dirichletSEXP = b_dirichlet,
      rho_dirichletSEXP = rho_dirichlet,
      sigmaSEXP = sigma_hat,
      sigma_knownSEXP = sigma_known,
      omegaSEXP = 1,
      param1SEXP = local_hp,
      param2SEXP = global_hp,
      prior_typeSEXP = prior_type,
      reversibleSEXP = reversible_flag,
      store_parametersSEXP = FALSE,
      store_posterior_sampleSEXP = store_posterior_sample,
      verboseSEXP = verbose
    )
    
    fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
    fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
    if (store_posterior_sample) {
      fit$train_predictions_sample <- fit$train_predictions_sample * sigma_hat + y_mean
      fit$test_predictions_sample <- fit$test_predictions_sample * sigma_hat + y_mean
    }
  }
  
  # remove burn-in of sigma
  if (!sigma_known) {
    fit$sigma <- fit$sigma[-seq_len(N_burn)]
  } 

  prior_type_user <- prior_type
  prior_type_cpp <- prior_type  # after your conversions

  obj <- NewShrinkageTrees(
    fit              = fit,
    call             = match.call(),
    outcome_type     = outcome_type,
    timescale        = timescale,
    prior_type_user  = prior_type_user,
    prior_type_cpp   = prior_type_cpp,
    n_train          = n_train,
    p_features       = p_features,
    n_test           = n_test,
    test_provided    = !is.null(X_test),
    number_of_trees  = number_of_trees,
    N_post           = N_post,
    N_burn           = N_burn,
    store_posterior_sample = store_posterior_sample,
    sigma_hat        = sigma_hat,
    sigma_known      = sigma_known,
    y_mean           = y_mean,
    y_train          = y_train_raw,
    X_train          = X_train_mat,
    status_train     = if (outcome_type %in% c("right-censored", "interval-censored")) status else NULL,
    left_time_train  = if (outcome_type == "interval-censored") left_time_raw else NULL,
    right_time_train = if (outcome_type == "interval-censored") right_time_raw else NULL,
    ic_indicator_train = if (outcome_type == "interval-censored") ic_indicator else NULL,
    power            = power,
    base             = base,
    p_grow           = p_grow,
    p_prune          = p_prune,
    delayed_proposal = delayed_proposal,
    reversible       = reversible_flag,
    param1           = local_hp,
    param2           = global_hp,
    omega            = 1,
    a_dirichlet      = a_dirichlet,
    b_dirichlet      = b_dirichlet,
    rho_dirichlet    = rho_dirichlet,
    nu               = nu,
    lambda           = lambda,
    latent_threshold = if (outcome_type == "binary") latent_threshold else NULL
  )

  return(obj)
}

test_that("ShrinkageTrees (horseshoe) works for continuous outcome", {
  
  # Generate data
  X <- matrix(runif(50 * 3), ncol = 3)
  X_test <- matrix(runif(50 * 3), ncol = 3)
  y <- X[, 1] + rnorm(50)
  
  # Fit the model
  set.seed(1)
  fit <- ShrinkageTrees(y = y, 
                        X_train = X,
                        X_test = X_test,
                        outcome_type = "continuous",
                        number_of_trees = 50,
                        prior_type = "horseshoe",
                        local_hp = 0.1/sqrt(50),
                        global_hp = 0.1/sqrt(50),
                        N_post = 10,
                        N_burn = 5,
                        store_posterior_sample = TRUE,
                        verbose = FALSE)
  
  # Basic checks
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_true("test_predictions" %in% names(fit))
  expect_length(fit$train_predictions, 50)
  expect_length(fit$sigma, 10)
  
  # Numerical checks
  expect_false(any(is.na(fit$train_predictions)))
  expect_false(any(is.nan(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))
  
  # Check posterior samples
  expect_true("train_predictions_sample" %in% names(fit))
  expect_true("test_predictions_sample" %in% names(fit))
  expect_equal(dim(fit$train_predictions_sample), c(10, 50))
  expect_equal(dim(fit$test_predictions_sample), c(10, 50))
  
  # Check sigma positivity
  expect_true(all(fit$sigma > 0))
  
  # Check predictions are not all zero
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_sample) > 0)
  expect_true(sd(fit$test_predictions) > 0)
  expect_true(sd(fit$test_predictions_sample) > 0)
  
  # Check reproducibility
  set.seed(1)
  fit2 <- ShrinkageTrees(y = y, 
                         X_train = X,
                         X_test = X_test,
                         outcome_type = "continuous",
                         number_of_trees = 50,
                         prior_type = "horseshoe",
                         local_hp = 0.1/sqrt(50),
                         global_hp = 0.1/sqrt(50),
                         N_post = 10,
                         N_burn = 5,
                         store_posterior_sample = TRUE,
                         verbose = FALSE)
  expect_equal(fit$train_predictions, fit2$train_predictions)
})


test_that("ShrinkageTrees (horseshoe) works for binary outcome", {
  
  # Generate data
  X <- matrix(rnorm(50 * 3), ncol = 3)
  X_test <- matrix(runif(50 * 3), ncol = 3)
  y <- ifelse(X[, 1] + rnorm(50) > 0, 1, 0)
  
  # Fit the model
  set.seed(1)
  fit <- ShrinkageTrees(y = y, 
                        X_train = X,
                        X_test = X_test,
                        outcome_type = "binary",
                        number_of_trees = 50,
                        prior_type = "horseshoe",
                        local_hp = 0.1/sqrt(50),
                        global_hp = 0.1/sqrt(50),
                        N_post = 10,
                        N_burn = 5,
                        store_posterior_sample = TRUE,
                        verbose = FALSE)
    
  # Basic checks
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_length(fit$train_predictions, 50)
  
  # Numerical checks
  expect_false(any(is.na(fit$train_predictions)))
  expect_false(any(is.nan(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))
  
  # Check posterior samples
  expect_true("train_predictions_sample" %in% names(fit))
  expect_true("test_predictions_sample" %in% names(fit))
  expect_true("train_probabilities_sample" %in% names(fit))
  expect_true("test_probabilities_sample" %in% names(fit))
  expect_equal(dim(fit$train_predictions_sample), c(10, 50))
  expect_equal(dim(fit$test_predictions_sample), c(10, 50))
  expect_equal(dim(fit$train_probabilities_sample), c(10, 50))
  expect_equal(dim(fit$test_probabilities_sample), c(10, 50))
  
  # Check estimated probabilities are within [0,1]
  expect_true(all(fit$train_probabilities >= 0 & fit$train_probabilities <= 1))
  expect_true(all(fit$test_probabilities >= 0 & fit$test_probabilities <= 1))
  
  # Check predictions are not all constant
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_sample) > 0)
  expect_true(sd(fit$test_predictions) > 0)
  expect_true(sd(fit$test_predictions_sample) > 0)
  
  # Check reproducibility
  set.seed(1)
  fit2 <- ShrinkageTrees(y = y, 
                         X_train = X,
                         X_test = X_test,
                         outcome_type = "binary",
                         number_of_trees = 50,
                         prior_type = "horseshoe",
                         local_hp = 0.1/sqrt(50),
                         global_hp = 0.1/sqrt(50),
                         N_post = 10,
                         N_burn = 5,
                         store_posterior_sample = TRUE,
                         verbose = FALSE)
  expect_equal(fit$train_predictions, fit2$train_predictions)
})

test_that("ShrinkageTrees (horseshoe) works for right-censored survival outcome", {
  
  # Generate covariates
  X <- matrix(rnorm(50 * 3), ncol = 3)
  X_test <- matrix(runif(50 * 3), ncol = 3)
  
  # Generate survival times (exponential baseline hazard depending on X[,1])
  linpred <- X[, 1]
  true_time <- rexp(50, rate = exp(linpred))
  
  # Generate censoring times
  censor_time <- rexp(50, rate = 0.5)
  
  # Observed time and status indicator
  time_obs <- pmin(true_time, censor_time)
  status <- as.numeric(true_time <= censor_time)
  
  # Fit the model
  set.seed(1)
  fit <- ShrinkageTrees(y = time_obs, 
                        X_train = X,
                        X_test = X_test,
                        outcome_type = "right-censored",
                        status = status,
                        number_of_trees = 50,
                        prior_type = "horseshoe",
                        local_hp = 0.1/sqrt(50),
                        global_hp = 0.1/sqrt(50),
                        N_post = 10,
                        N_burn = 5,
                        store_posterior_sample = TRUE,
                        verbose = FALSE)
  
  # Basic checks
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_true("test_predictions" %in% names(fit))
  expect_true("test_predictions" %in% names(fit))
  expect_length(fit$train_predictions, 50)
  expect_length(fit$sigma, 10)
  
  # Numerical checks
  expect_false(any(is.na(fit$train_predictions)))
  expect_false(any(is.nan(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))
  
  # Check posterior samples
  expect_true("train_predictions_sample" %in% names(fit))
  expect_true("test_predictions_sample" %in% names(fit))
  expect_equal(dim(fit$train_predictions_sample), c(10, 50))
  expect_equal(dim(fit$test_predictions_sample), c(10, 50))
  
  # Check sigma positivity
  expect_true(all(fit$sigma > 0))
  
  # Check predictions are not all constant
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_sample) > 0)
  expect_true(sd(fit$test_predictions) > 0)
  expect_true(sd(fit$test_predictions_sample) > 0)
  
  # Check reproducibility
  set.seed(1)
  fit2 <- ShrinkageTrees(y = time_obs, 
                         X_train = X,
                         X_test = X_test,
                         outcome_type = "right-censored",
                         status = status,
                         number_of_trees = 50,
                         prior_type = "horseshoe",
                         local_hp = 0.1/sqrt(50),
                         global_hp = 0.1/sqrt(50),
                         N_post = 10,
                         N_burn = 5,
                         store_posterior_sample = TRUE,
                         verbose = FALSE)
  expect_equal(fit$train_predictions, fit2$train_predictions)
})


test_that("ShrinkageTrees (horseshoe_fw) works for continuous outcome", {
  
  # Generate data
  X <- matrix(runif(50 * 3), ncol = 3)
  X_test <- matrix(runif(50 * 3), ncol = 3)
  y <- X[, 1] + rnorm(50)
  
  # Fit the model
  set.seed(1)
  fit <- ShrinkageTrees(y = y, 
                        X_train = X,
                        X_test = X_test,
                        outcome_type = "continuous",
                        number_of_trees = 50,
                        prior_type = "horseshoe_fw",
                        local_hp = 0.1/sqrt(50),
                        global_hp = 0.1/sqrt(50),
                        N_post = 10,
                        N_burn = 5,
                        store_posterior_sample = TRUE,
                        verbose = FALSE)
  
  # Basic checks
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_true("test_predictions" %in% names(fit))
  expect_length(fit$train_predictions, 50)
  expect_length(fit$sigma, 10)
  
  # Numerical checks
  expect_false(any(is.na(fit$train_predictions)))
  expect_false(any(is.nan(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))
  
  # Check posterior samples
  expect_true("train_predictions_sample" %in% names(fit))
  expect_true("test_predictions_sample" %in% names(fit))
  expect_equal(dim(fit$train_predictions_sample), c(10, 50))
  expect_equal(dim(fit$test_predictions_sample), c(10, 50))
  
  # Check sigma positivity
  expect_true(all(fit$sigma > 0))
  
  # Check predictions are not all zero
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_sample) > 0)
  expect_true(sd(fit$test_predictions) > 0)
  expect_true(sd(fit$test_predictions_sample) > 0)
  
  # Check forestwide shrinkage vectors
  expect_true(is.numeric(fit$forestwide_shrinkage))
  expect_length(fit$forestwide_shrinkage, 10)
  expect_true(all(fit$forestwide_shrinkage >= 0))
  
  # Check reproducibility
  set.seed(1)
  fit2 <- ShrinkageTrees(y = y, 
                         X_train = X,
                         X_test = X_test,
                         outcome_type = "continuous",
                         number_of_trees = 50,
                         prior_type = "horseshoe_fw",
                         local_hp = 0.1/sqrt(50),
                         global_hp = 0.1/sqrt(50),
                         N_post = 10,
                         N_burn = 5,
                         store_posterior_sample = TRUE,
                         verbose = FALSE)
  expect_equal(fit$train_predictions, fit2$train_predictions)
})


test_that("ShrinkageTrees (half-cauchy) works for continuous outcome", {
  
  # Generate data
  X <- matrix(runif(50 * 3), ncol = 3)
  X_test <- matrix(runif(50 * 3), ncol = 3)
  y <- X[, 1] + rnorm(50)
  
  # Fit the model
  set.seed(1)
  fit <- ShrinkageTrees(y = y, 
                        X_train = X,
                        X_test = X_test,
                        outcome_type = "continuous",
                        number_of_trees = 50,
                        prior_type = "half-cauchy",
                        local_hp = 0.1/sqrt(50),
                        N_post = 10,
                        N_burn = 5,
                        store_posterior_sample = TRUE,
                        verbose = FALSE)
  
  # Basic checks
  expect_type(fit, "list")
  expect_true("train_predictions" %in% names(fit))
  expect_true("test_predictions" %in% names(fit))
  expect_length(fit$train_predictions, 50)
  expect_length(fit$sigma, 10)
  
  # Numerical checks
  expect_false(any(is.na(fit$train_predictions)))
  expect_false(any(is.nan(fit$train_predictions)))
  expect_true(all(is.finite(fit$train_predictions)))
  
  # Check posterior samples
  expect_true("train_predictions_sample" %in% names(fit))
  expect_true("test_predictions_sample" %in% names(fit))
  expect_equal(dim(fit$train_predictions_sample), c(10, 50))
  expect_equal(dim(fit$test_predictions_sample), c(10, 50))
  
  # Check sigma positivity
  expect_true(all(fit$sigma > 0))
  
  # Check predictions are not all zero
  expect_true(sd(fit$train_predictions) > 0)
  expect_true(sd(fit$train_predictions_sample) > 0)
  expect_true(sd(fit$test_predictions) > 0)
  expect_true(sd(fit$test_predictions_sample) > 0)
  
  # Check reproducibility
  set.seed(1)
  fit2 <- ShrinkageTrees(y = y, 
                         X_train = X,
                         X_test = X_test,
                         outcome_type = "continuous",
                         number_of_trees = 50,
                         prior_type = "half-cauchy",
                         local_hp = 0.1/sqrt(50),
                         N_post = 10,
                         N_burn = 5,
                         store_posterior_sample = TRUE,
                         verbose = FALSE)
  expect_equal(fit$train_predictions, fit2$train_predictions)
})

test_that("ShrinkageTrees errors on invalid outcome_type", {
  X <- matrix(rnorm(20 * 2), ncol = 2)
  y <- rnorm(20)
  
  expect_error(
    ShrinkageTrees(
      y = y,
      X_train = X,
      outcome_type = "invalid"
    )
  )
})

test_that("X_test defaults correctly", {
  X <- matrix(rnorm(40), ncol = 2)
  y <- rnorm(20)

  fit <- ShrinkageTrees(
    y = y,
    X_train = X,
    outcome_type = "continuous",
    number_of_trees = 5,
    prior_type = "standard",
    local_hp = 0.1,
    N_post = 5,
    N_burn = 2,
    verbose = FALSE
  )

  expect_length(fit$test_predictions, 1)
})


# ── S3 class, print, summary, predict ────────────────────────────────────────

test_that("ShrinkageTrees S3 methods work", {
  X      <- matrix(runif(50 * 3), ncol = 3)
  X_test <- matrix(runif(10 * 3), ncol = 3)
  y      <- X[, 1] + rnorm(50)

  set.seed(1)
  fit <- ShrinkageTrees(
    y = y, X_train = X,
    outcome_type = "continuous",
    number_of_trees = 5,
    prior_type = "horseshoe",
    local_hp = 0.1 / sqrt(5), global_hp = 0.1 / sqrt(5),
    N_post = 10, N_burn = 5,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  expect_s3_class(fit, "ShrinkageTrees")
  expect_no_error(capture.output(print(fit)))
  expect_no_error(smry <- summary(fit))
  expect_type(smry, "list")
  expect_true(!is.null(smry$sigma))

  # predict on new data
  expect_no_error(pred <- predict(fit, newdata = X_test))
  expect_s3_class(pred, "ShrinkageTreesPrediction")
  expect_length(pred$mean, nrow(X_test))
  expect_length(pred$lower, nrow(X_test))
  expect_length(pred$upper, nrow(X_test))
  expect_true(all(pred$lower <= pred$mean))
  expect_true(all(pred$mean  <= pred$upper))

  expect_no_error(capture.output(print(pred)))
  expect_no_error(capture.output(summary(pred)))
})


# ── Multi-chain (n_chains) ────────────────────────────────────────────────────

test_that("ShrinkageTrees n_chains > 1 pools chains correctly", {
  X <- matrix(runif(50 * 3), ncol = 3)
  y <- X[, 1] + rnorm(50)

  set.seed(1)
  fit <- ShrinkageTrees(
    y = y, X_train = X,
    outcome_type = "continuous",
    number_of_trees = 5,
    prior_type = "horseshoe",
    local_hp = 0.1 / sqrt(5), global_hp = 0.1 / sqrt(5),
    N_post = 10, N_burn = 5,
    n_chains = 2,
    verbose = FALSE
  )

  expect_s3_class(fit, "ShrinkageTrees")
  expect_length(fit$sigma, 20)
  expect_equal(fit$mcmc$n_chains, 2)
  expect_length(fit$chains$acceptance_ratios, 2)
})

#' Bayesian bootstrap average treatment effect
#'
#' Post-hoc reweights the stored posterior CATE draws of a fitted causal
#' model to produce credible intervals for the \emph{population} ATE (PATE)
#' that incorporate uncertainty in the covariate distribution
#' \eqn{F_X}.
#'
#' At each MCMC iteration \eqn{s} the conditional treatment effects
#' \eqn{\tau^{(s)}(x_i)} are reweighted with
#' \eqn{(w_1^{(s)}, \dots, w_n^{(s)}) \sim \mathrm{Dir}(1, \dots, 1)}
#' to give a draw
#' \deqn{\widehat{\mathrm{PATE}}^{(s)} = \sum_{i=1}^n w_i^{(s)}\, \tau^{(s)}(x_i).}
#' The collection \eqn{\{\widehat{\mathrm{PATE}}^{(s)}\}} approximates the
#' posterior of the PATE, integrating over \eqn{\tau(\cdot)} \emph{and}
#' \eqn{F_X}. The equal-weight mixed ATE (MATE),
#' \eqn{\widehat{\mathrm{MATE}}^{(s)} = n^{-1}\sum_i \tau^{(s)}(x_i)},
#' is returned alongside for comparison.
#'
#' For reproducibility, call \code{set.seed()} before invoking the function
#' to fix the Dirichlet draws.
#'
#' @param object Either a fitted \code{CausalShrinkageForest} (from
#'   \code{\link{CausalShrinkageForest}} or \code{\link{CausalHorseForest}}
#'   with \code{store_posterior_sample = TRUE}) or a
#'   \code{CausalShrinkageForestPrediction} (from
#'   \code{\link{predict.CausalShrinkageForest}}).
#' @param alpha One minus the credible level. Default \code{0.05}
#'   (a 95 percent credible interval).
#' @return A list with
#'   \describe{
#'     \item{pate_mean, pate_ci, pate_samples}{Posterior mean, credible
#'       interval (named \code{lower} and \code{upper}), and full vector of
#'       draws of the Bayesian-bootstrap PATE.}
#'     \item{mate_mean, mate_ci, mate_samples}{Same quantities for the
#'       equal-weight mixed ATE.}
#'     \item{n, S}{Number of observations and posterior draws used.}
#'   }
#' @seealso \code{\link{summary.CausalShrinkageForest}},
#'   \code{\link{plot.CausalShrinkageForest}},
#'   \code{\link{predict.CausalShrinkageForest}}
#'
#' @examples
#' # Small toy causal model (binary outcome, for speed)
#' set.seed(1)
#' n <- 40; p <- 3
#' X <- matrix(runif(n * p), ncol = p)
#' trt <- rbinom(n, 1, 0.5)
#' y <- X[, 1] + trt * (0.5 + X[, 2]) + rnorm(n)
#'
#' fit <- CausalShrinkageForest(
#'   y = y,
#'   X_train_control = X, X_train_treat = X,
#'   treatment_indicator_train = trt,
#'   outcome_type = "continuous",
#'   number_of_trees_control = 5, number_of_trees_treat = 5,
#'   prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
#'   local_hp_control = 0.1, global_hp_control = 0.1,
#'   local_hp_treat = 0.1, global_hp_treat = 0.1,
#'   N_post = 20, N_burn = 10,
#'   store_posterior_sample = TRUE,
#'   verbose = FALSE
#' )
#'
#' bb <- bayesian_bootstrap_ate(fit, alpha = 0.05)
#' bb$pate_mean
#' bb$pate_ci
#'
#' @export
bayesian_bootstrap_ate <- function(object, alpha = 0.05) {

  if (inherits(object, "CausalShrinkageForest")) {
    tau_samples <- object$train_predictions_sample_treat
    if (is.null(tau_samples))
      stop("Posterior CATE samples not stored. Refit with ",
           "store_posterior_sample = TRUE.", call. = FALSE)
  } else if (inherits(object, "CausalShrinkageForestPrediction")) {
    tau_samples <- object$cate_samples
    if (is.null(tau_samples))
      stop("Prediction object does not contain cate_samples. ",
           "Refit and re-predict with the current package version.",
           call. = FALSE)
  } else {
    stop("`object` must be a CausalShrinkageForest or ",
         "CausalShrinkageForestPrediction.", call. = FALSE)
  }

  if (!is.numeric(alpha) || length(alpha) != 1 || alpha <= 0 || alpha >= 1)
    stop("`alpha` must be a single number in (0, 1).", call. = FALSE)

  probs <- c(alpha / 2, 1 - alpha / 2)

  mate_samples <- .ate_samples(tau_samples, bayesian_bootstrap = FALSE)
  pate_samples <- .ate_samples(tau_samples, bayesian_bootstrap = TRUE)

  ci <- function(s) {
    q <- quantile(s, probs = probs)
    list(lower = unname(q[1]), upper = unname(q[2]))
  }

  list(
    pate_mean    = mean(pate_samples),
    pate_ci      = ci(pate_samples),
    pate_samples = pate_samples,
    mate_mean    = mean(mate_samples),
    mate_ci      = ci(mate_samples),
    mate_samples = mate_samples,
    n            = ncol(tau_samples),
    S            = nrow(tau_samples)
  )
}

#' General Causal Shrinkage Forests
#'
#' Fits a (Bayesian) Causal Shrinkage Forest model for estimating heterogeneous treatment effects.
#' This function generalizes \code{\link{CausalHorseForest}} by allowing flexible
#' global-local shrinkage priors on the step heights in both the control and treatment forests.
#' It supports continuous, right-censored, and interval-censored survival outcomes.
#'
#' @param y Outcome vector. Numeric. Represents continuous outcomes or follow-up times.
#' Set to \code{NULL} when using \code{outcome_type = "interval-censored"}, as
#' values are derived from \code{left_time} and \code{right_time}.
#' @param status Optional event indicator vector (1 = event occurred, 0 = censored).
#' Required when \code{outcome_type = "right-censored"}. For interval-censored
#' outcomes, this is derived automatically from \code{left_time} and
#' \code{right_time}.
#' @param X_train_control Covariate matrix for the control forest. Rows correspond to samples,
#' columns to covariates.
#' @param X_train_treat Covariate matrix for the treatment forest.
#' @param treatment_indicator_train Vector indicating treatment assignment for training samples
#' (1 = treated, 0 = control).
#' @param X_test_control Optional covariate matrix for control forest test data. Defaults to
#' column means of \code{X_train_control} if NULL.
#' @param X_test_treat Optional covariate matrix for treatment forest test data. Defaults to
#' column means of \code{X_train_treat} if NULL.
#' @param treatment_indicator_test Optional vector indicating treatment assignment for test data.
#' @param left_time Optional numeric vector of left (lower) time boundaries.
#' Required when \code{outcome_type = "interval-censored"}. Exact events
#' have \code{left_time == right_time}; right-censored observations have
#' \code{right_time = Inf}; interval-censored observations have finite
#' \code{left_time < right_time}.
#' @param right_time Optional numeric vector of right (upper) time boundaries.
#' Required when \code{outcome_type = "interval-censored"}. Use \code{Inf}
#' for right-censored observations.
#' @param outcome_type Type of outcome: one of \code{"continuous"},
#' \code{"right-censored"}, or \code{"interval-censored"}.
#' Default is \code{"continuous"}.
#' @param timescale For survival outcomes: either \code{"time"} (original scale, log-transformed
#' internally) or \code{"log"} (already log-transformed). Default is \code{"time"}.
#' Used when \code{outcome_type} is \code{"right-censored"} or
#' \code{"interval-censored"}.
#' @param number_of_trees_control Number of trees in the control forest. Default is 200.
#' @param number_of_trees_treat Number of trees in the treatment forest. Default is 200.
#' @param prior_type_control Type of prior on control forest step heights. One of 
#' \code{"horseshoe"}, \code{"horseshoe_fw"}, or \code{"half-cauchy"}.
#' Default is \code{"horseshoe"}.
#' @param prior_type_treat Type of prior on treatment forest step heights. Same options as 
#' \code{prior_type_control}.
#' @param local_hp_control Local hyperparameter controlling shrinkage on individual steps 
#' (control forest). Required for all prior types.
#' @param local_hp_treat Local hyperparameter for treatment forest.
#' @param global_hp_control Global hyperparameter for control forest. Required for horseshoe-type
#' priors; ignored for \code{"half-cauchy"}.
#' @param global_hp_treat Global hyperparameter for treatment forest.
#' @param a_dirichlet_control First shape parameter of the Beta prior used in the
#' Dirichlet–Sparse splitting rule for the control forest. Together with 
#' `b_dirichlet_control`, it controls the expected sparsity level.
#' @param b_dirichlet_control Second shape parameter of the Beta prior for the 
#' sparsity level in the control forest. Larger values shrink splitting 
#' probabilities more strongly toward uniform sparsity.
#' @param rho_dirichlet_control Sparsity hyperparameter for the control forest.
#' Represents the *expected number of active predictors*. If left NULL, it 
#' defaults to the number of covariates in the control forest.
#' @param a_dirichlet_treat First shape parameter of the Beta prior used in the
#' Dirichlet–Sparse splitting rule for the treatment forest.
#' @param b_dirichlet_treat Second shape parameter of the Beta prior governing 
#' sparsity in the treatment forest.
#' @param rho_dirichlet_treat Sparsity hyperparameter for the treatment forest, 
#' interpreted as the expected number of active predictors. Defaults to the 
#' number of covariates in the treatment forest if not specified.
#' @param power_control Power parameter for the control forest tree structure prior splitting probability.
#' @param power_treat   Power parameter for the treatment forest tree structure prior splitting probability.
#' @param base_control  Base parameter for the control forest tree structure prior splitting probability.
#' @param base_treat    Base parameter for the treatment forest tree structure prior splitting probability.
#' @param p_grow Probability of proposing a grow move. Default is 0.5. These are fixed at 0.5 for prior_type
#' \code{"standard"} and \code{"dirichlet"}.
#' @param p_prune Probability of proposing a prune move. Default is 0.5. These are fixed at 0.5 for prior_type 
#' \code{"standard"} and \code{"dirichlet"}.
#' @param nu Degrees of freedom for the error variance prior. Default is 3.
#' @param q Quantile parameter for error variance prior. Default is 0.90.
#' @param sigma Optional known standard deviation of the outcome. If NULL, estimated from data.
#' @param N_post Number of posterior samples to store. Default is 5000.
#' @param N_burn Number of burn-in iterations. Default is 5000.
#' @param delayed_proposal Number of delayed iterations before proposal updates. Default is 5.
#' @param store_posterior_sample Logical; whether to store posterior samples of predictions.
#' Default is \code{FALSE}.
#' @param treatment_coding Treatment coding scheme for the two-forest model.
#'   One of \code{"centered"} (default), \code{"binary"}, \code{"adaptive"},
#'   or \code{"invariant"}.
#'   \code{"centered"} uses \eqn{b_i \in \{-1/2, 1/2\}};
#'   \code{"binary"} uses \eqn{b_i \in \{0, 1\}};
#'   \code{"adaptive"} uses \eqn{b_i = A_i - \hat{e}(x_i)} where
#'   \eqn{\hat{e}(x_i)} is the estimated propensity score;
#'   \code{"invariant"} treats \eqn{b_0, b_1} as parameters estimated within
#'   the Gibbs sampler with \eqn{b_j \sim N(0, 1/2)} priors, yielding a
#'   parameterisation-invariant model (Hahn et al., 2020, Section 5.2).
#' @param propensity Optional numeric vector of propensity scores
#'   \eqn{\hat{e}(x_i)} for training observations. Required when
#'   \code{treatment_coding = "adaptive"}.
#' @param propensity_test Optional numeric vector of propensity scores for
#'   test observations. Only used when \code{treatment_coding = "adaptive"}.
#'   Defaults to \code{0.5} for all test observations if not provided.
#' @param n_chains Number of independent MCMC chains to run. Default is
#'   \code{1} (standard single-chain behaviour). When \code{n_chains > 1} the
#'   chains are run in parallel via \code{parallel::mclapply} and their
#'   posterior samples are pooled into a single \code{CausalShrinkageForest}
#'   object, so all existing \code{print} and \code{summary} methods work
#'   without modification. On Windows, \code{mclapply} falls back to
#'   sequential execution.
#' @param verbose Logical; whether to print verbose output. Default is \code{TRUE}.
#'
#' @return An S3 object of class \code{"CausalShrinkageForest"} containing:
#' \describe{
#'   \item{train_predictions}{Posterior mean predictions on training data (combined forest).}
#'   \item{test_predictions}{Posterior mean predictions on test data (combined forest).}
#'   \item{train_predictions_control}{Estimated control outcomes on training data.}
#'   \item{test_predictions_control}{Estimated control outcomes on test data.}
#'   \item{train_predictions_treat}{Estimated treatment effects on training data.}
#'   \item{test_predictions_treat}{Estimated treatment effects on test data.}
#'   \item{sigma}{Vector of posterior samples for the error standard deviation.}
#'   \item{acceptance_ratio_control}{Average acceptance ratio in control forest.}
#'   \item{acceptance_ratio_treat}{Average acceptance ratio in treatment forest.}
#'   \item{train_predictions_sample_control}{Matrix of posterior samples for control predictions 
#'   (if \code{store_posterior_sample = TRUE}).}
#'   \item{test_predictions_sample_control}{Matrix of posterior samples for control predictions 
#'   (if \code{store_posterior_sample = TRUE}).}
#'   \item{train_predictions_sample_treat}{Matrix of posterior samples for treatment effects 
#'   (if \code{store_posterior_sample = TRUE}).}
#'   \item{test_predictions_sample_treat}{Matrix of posterior samples for treatment effects 
#'   (if \code{store_posterior_sample = TRUE}).}
#' }
#'
#' @details
#' This function is a flexible generalization of \code{CausalHorseForest}.
#' The Causal Shrinkage Forest model decomposes the outcome into a prognostic 
#' (control) and a treatment effect part. Each part is modeled by its own 
#' shrinkage tree ensemble, with separate flexible global-local shrinkage 
#' priors. It is particularly useful for estimating heterogeneous treatment 
#' effects in high-dimensional settings. Further
#' methodological details on the Horseshoe Forest framework can be found in 
#' Jacobs, van Wieringen & van der Pas (2025).
#'
#' The \code{horseshoe} prior is the fully Bayesian global-local shrinkage
#' prior, where both the global and local shrinkage parameters are assigned
#' half-Cauchy distributions with scale hyperparameters \code{global_hp} and
#' \code{local_hp}, respectively. The global shrinkage parameter is defined
#' separately for each tree, allowing adaptive regularization per tree.
#'
#' The \code{horseshoe_fw} prior (forest-wide horseshoe) is similar to
#' \code{horseshoe}, except that the global shrinkage parameter is shared
#' across all trees in the forest simultaneously. 
#'
#' The \code{half-cauchy} prior considers only local shrinkage and does not
#' include a global shrinkage component. It places a half-Cauchy prior on each
#' local shrinkage parameter with scale hyperparameter \code{local_hp}.
#' 
#' The \code{dirichlet} prior implements the Dirichlet–Sparse splitting rule of 
#' Linero (2018), in which splitting probabilities follow a Dirichlet prior 
#' whose concentration is controlled by a Beta sparsity parameter 
#' (\code{a_dirichlet}, \code{b_dirichlet}) and an expected sparsity level 
#' \code{rho_dirichlet}.
#'
#' @references
#' Jacobs, T., van Wieringen, W. N., & van der Pas, S. L. (2025).  
#' *Horseshoe Forests for High-Dimensional Causal Survival Analysis.*  
#' arXiv:2507.22004. https://doi.org/10.48550/arXiv.2507.22004
#' 
#' Chipman, H. A., George, E. I., & McCulloch, R. E. (2010). 
#' *BART: Bayesian additive regression trees.* Annals of Applied Statistics.
#'
#' Linero, A. R. (2018). *Bayesian regression trees for high-dimensional 
#' prediction and variable selection.* Journal of the American Statistical 
#' Association.
#' 
#' @examples
#' # Example: Continuous outcome, homogeneous treatment effect, two priors
#' n <- 50
#' p <- 3
#' X <- matrix(runif(n * p), ncol = p)
#' X_treat <- X_control <- X
#' treat <- rbinom(n, 1, X[,1])
#' tau <- 2
#' y <- X[, 1] + (0.5 - treat) * tau + rnorm(n)
#' 
#' # Fit a standard Causal Horseshoe Forest
#' fit_horseshoe <- CausalShrinkageForest(y = y,
#'                                        X_train_control = X_control,
#'                                        X_train_treat = X_treat,
#'                                        treatment_indicator_train = treat,
#'                                        outcome_type = "continuous",
#'                                        number_of_trees_treat = 5,
#'                                        number_of_trees_control = 5,
#'                                        prior_type_control = "horseshoe",
#'                                        prior_type_treat = "horseshoe",
#'                                        local_hp_control = 0.1/sqrt(5),
#'                                        local_hp_treat = 0.1/sqrt(5),
#'                                        global_hp_control = 0.1/sqrt(5),
#'                                        global_hp_treat = 0.1/sqrt(5),
#'                                        N_post = 10,
#'                                        N_burn = 5,
#'                                        store_posterior_sample = TRUE,
#'                                        verbose = FALSE
#' )
#' 
#' # Fit a Causal Shrinkage Forest with half-cauchy prior
#' fit_halfcauchy <- CausalShrinkageForest(y = y,
#'                                         X_train_control = X_control,
#'                                         X_train_treat = X_treat,
#'                                         treatment_indicator_train = treat,
#'                                         outcome_type = "continuous",
#'                                         number_of_trees_treat = 5,
#'                                         number_of_trees_control = 5,
#'                                         prior_type_control = "half-cauchy",
#'                                         prior_type_treat = "half-cauchy",
#'                                         local_hp_control = 1/sqrt(5),
#'                                         local_hp_treat = 1/sqrt(5),
#'                                         N_post = 10,
#'                                         N_burn = 5,
#'                                         store_posterior_sample = TRUE,
#'                                         verbose = FALSE
#' )
#' 
#' # Posterior mean CATEs
#' CATE_horseshoe <- colMeans(fit_horseshoe$train_predictions_sample_treat)
#' CATE_halfcauchy <- colMeans(fit_halfcauchy$train_predictions_sample_treat)
#' 
#' # Posteriors of the ATE
#' post_ATE_horseshoe <- rowMeans(fit_horseshoe$train_predictions_sample_treat)
#' post_ATE_halfcauchy <- rowMeans(fit_halfcauchy$train_predictions_sample_treat)
#' 
#' # Posterior mean ATE
#' ATE_horseshoe <- mean(post_ATE_horseshoe)
#' ATE_halfcauchy <- mean(post_ATE_halfcauchy)
#'
#' # Example: Interval-censored causal survival outcome
#' n <- 50; p <- 3
#' X_ic <- matrix(rnorm(n * p), ncol = p)
#' treat_ic <- rbinom(n, 1, 0.5)
#' true_t <- rexp(n, rate = exp(-X_ic[, 1] - 0.5 * treat_ic))
#' left_t  <- true_t * runif(n, 0.5, 1)
#' right_t <- true_t * runif(n, 1, 1.5)
#' exact <- sample(n, 15)
#' left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
#' rc <- sample(setdiff(seq_len(n), exact), 10); right_t[rc] <- Inf
#'
#' fit_ic <- CausalShrinkageForest(
#'   left_time = left_t, right_time = right_t,
#'   X_train_control = X_ic, X_train_treat = X_ic,
#'   treatment_indicator_train = treat_ic,
#'   outcome_type = "interval-censored",
#'   number_of_trees_control = 5, number_of_trees_treat = 5,
#'   prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
#'   local_hp_control = 0.1/sqrt(5), local_hp_treat = 0.1/sqrt(5),
#'   global_hp_control = 0.1/sqrt(5), global_hp_treat = 0.1/sqrt(5),
#'   N_post = 10, N_burn = 5,
#'   store_posterior_sample = TRUE, verbose = FALSE)
#'
#' @seealso
#' Model family: \code{\link{CausalHorseForest}} (causal, horseshoe prior),
#' \code{\link{ShrinkageTrees}} (non-causal, flexible prior),
#' \code{\link{HorseTrees}} (non-causal, horseshoe prior).
#'
#' Survival wrappers: \code{\link{SurvivalBCF}}, \code{\link{SurvivalShrinkageBCF}}.
#'
#' S3 methods: \code{\link{print.CausalShrinkageForest}},
#' \code{\link{summary.CausalShrinkageForest}},
#' \code{\link{predict.CausalShrinkageForest}},
#' \code{\link{plot.CausalShrinkageForest}}.
#'
#' @importFrom Rcpp evalCpp
#' @useDynLib ShrinkageTrees, .registration = TRUE
#' @importFrom stats sd qchisq qnorm runif
#' @importFrom parallel mclapply detectCores
#' @export
CausalShrinkageForest <- function(y = NULL,
                                  status = NULL,
                                  X_train_control,
                                  X_train_treat,
                                  treatment_indicator_train,
                                  X_test_control = NULL,
                                  X_test_treat = NULL,
                                  treatment_indicator_test = NULL,
                                  left_time = NULL,
                                  right_time = NULL,
                                  outcome_type = "continuous",
                                  timescale = "time",
                                  number_of_trees_control = 200,
                                  number_of_trees_treat = 200,
                                  prior_type_control = "horseshoe",
                                  prior_type_treat = "horseshoe",
                                  local_hp_control = NULL,
                                  local_hp_treat = NULL,
                                  global_hp_control = NULL,
                                  global_hp_treat = NULL,
                                  a_dirichlet_control = 0.5,
                                  a_dirichlet_treat = 0.5,
                                  b_dirichlet_control = 1.0,
                                  b_dirichlet_treat = 1.0,
                                  rho_dirichlet_control = NULL,
                                  rho_dirichlet_treat = NULL,
                                  power_control = 2.0,
                                  power_treat = 2.0,
                                  base_control = 0.95,
                                  base_treat = 0.95,
                                  p_grow = 0.5,
                                  p_prune = 0.5,
                                  nu = 3,
                                  q = 0.90,
                                  sigma = NULL,
                                  N_post = 5000,
                                  N_burn = 5000,
                                  delayed_proposal = 5,
                                  store_posterior_sample = FALSE,
                                  treatment_coding = "centered",
                                  propensity = NULL,
                                  propensity_test = NULL,
                                  n_chains = 1,
                                  verbose = TRUE) {

  # ── Multi-chain dispatch ──────────────────────────────────────────────────
  if (n_chains > 1) {
    mc <- match.call()

    chain_args <- list(
      y                         = y,
      status                    = status,
      X_train_control           = X_train_control,
      X_train_treat             = X_train_treat,
      treatment_indicator_train = treatment_indicator_train,
      X_test_control            = X_test_control,
      X_test_treat              = X_test_treat,
      treatment_indicator_test  = treatment_indicator_test,
      left_time                 = left_time,
      right_time                = right_time,
      outcome_type              = outcome_type,
      timescale                 = timescale,
      number_of_trees_control   = number_of_trees_control,
      number_of_trees_treat     = number_of_trees_treat,
      prior_type_control        = prior_type_control,
      prior_type_treat          = prior_type_treat,
      local_hp_control          = local_hp_control,
      local_hp_treat            = local_hp_treat,
      global_hp_control         = global_hp_control,
      global_hp_treat           = global_hp_treat,
      a_dirichlet_control       = a_dirichlet_control,
      a_dirichlet_treat         = a_dirichlet_treat,
      b_dirichlet_control       = b_dirichlet_control,
      b_dirichlet_treat         = b_dirichlet_treat,
      rho_dirichlet_control     = rho_dirichlet_control,
      rho_dirichlet_treat       = rho_dirichlet_treat,
      power_control             = power_control,
      power_treat               = power_treat,
      base_control              = base_control,
      base_treat                = base_treat,
      p_grow                    = p_grow,
      p_prune                   = p_prune,
      nu                        = nu,
      q                         = q,
      sigma                     = sigma,
      N_post                    = N_post,
      N_burn                    = N_burn,
      delayed_proposal          = delayed_proposal,
      store_posterior_sample    = store_posterior_sample,
      treatment_coding          = treatment_coding,
      propensity                = propensity,
      propensity_test           = propensity_test,
      n_chains                  = 1,
      verbose                   = FALSE
    )

    n_cores <- .resolve_cores(n_chains)
    if (verbose)
      message("Running ", n_chains, " chains (", n_cores, " cores) ...")

    chains <- parallel::mclapply(
      seq_len(n_chains),
      function(i) do.call(CausalShrinkageForest, chain_args),
      mc.cores = n_cores
    )

    combined      <- .combine_causal_chains(chains)
    combined$call <- mc
    return(combined)
  }

  # Check prior_type value
  allowed_prior <- c("horseshoe", "horseshoe_fw", "half-cauchy",
                     "standard", "dirichlet", "standard-halfcauchy", "dirichlet-halfcauchy")
  if (!prior_type_control %in% allowed_prior) {
    stop("Invalid prior_type_control Choose 'horseshoe', 'horseshoe_fw',
         'half-cauchy', 'standard', 'standard-halfnormal', 'standard-halfcauchy', 'dirichlet', or 'dirichlet-halfcauchy'.")
  }
  
  if (!prior_type_treat %in% allowed_prior) {
    stop("Invalid prior_type_treat. Choose 'horseshoe', 'horseshoe_fw',
         'half-cauchy', 'standard', 'standard-halfnormal', 'standard-halfcauchy', 'dirichlet', or 'dirichlet-halfcauchy'.")
  }
  
  # Prior-specific checks
  if (prior_type_control %in% c("horseshoe", "horseshoe_fw")) {
    if (is.null(local_hp_control) || is.null(global_hp_control)) {
      stop("For prior_type_control = 'horseshoe' or 'horseshoe_fw', you must provide both local_hp and global_hp.")
    }
  }
  
  if (prior_type_treat %in% c("horseshoe", "horseshoe_fw")) {
    if (is.null(local_hp_treat) || is.null(global_hp_treat)) {
      stop("For prior_type_treat = 'horseshoe' or 'horseshoe_fw', you must provide both local_hp and global_hp.")
    }
  }

  prior_type_control_user <- prior_type_control
  prior_type_treat_user <- prior_type_treat
  
  if (prior_type_control == "half-cauchy") {
    if (is.null(local_hp_control)) {
      stop("For prior_type = 'half-cauchy', you must provide local_hp_control.")
    }
    if (!is.null(global_hp_control)) {
      warning("global_hp_control is ignored for 'half-cauchy'.")
    }
    global_hp_control <- 1
    prior_type_control <- "halfcauchy"
  }
  
  if (prior_type_treat == "half-cauchy") {
    if (is.null(local_hp_treat)) {
      stop("For prior_type_treat = 'half-cauchy', you must provide 
           local_hp_treat.")
    }
    if (!is.null(global_hp_treat)) {
      warning("global_hp_treat is ignored for 'half-cauchy'.")
    }
    global_hp_treat <- 1
    prior_type_treat <- "halfcauchy"
  }
  
  reversible_flag_control <- TRUE
  reversible_flag_treat <- TRUE

  if (prior_type_control %in% c("standard", "dirichlet")) {
    if (is.null(local_hp_control)) {
      stop("For prior_type_control = 'standard' or 'dirichlet', you must provide local_hp.")
    }
    if (!is.null(global_hp_control)) {
      warning("global_hp_control is ignored for 'standard' or 'dirichlet' prior.")
    }

    global_hp_control <- 1          # placeholder (ignored by C++)
    reversible_flag_control <- FALSE

    if (delayed_proposal > 0) {
      delayed_proposal <- 0
    }
  }

  if (prior_type_control %in% c("dirichlet", "dirichlet-halfcauchy")) {
    dirichlet_bool_control <- TRUE
    if (prior_type_control == "dirichlet-halfcauchy") prior_type_control <- "standard-halfcauchy"
  } else {
    dirichlet_bool_control <- FALSE
  }

  if (prior_type_control %in% c("standard-halfcauchy", "dirichlet-halfcauchy")) {

    if (!is.null(global_hp_control)) {
      warning("global_hp_control is ignored for 'standard' or 'dirichlet' prior.")
    }

    global_hp_control <- 1          # placeholder (ignored by C++)
    reversible_flag_control <- FALSE

    if (delayed_proposal > 0) {
      delayed_proposal <- 0
    }
  }

  if (prior_type_treat %in% c("standard", "dirichlet", "standard-halfcauchy")) {
    if (is.null(local_hp_treat)) {
      stop("For prior_type_treat = 'standard' or 'dirichlet', you must provide local_hp_treat.")
    }
    if (!is.null(global_hp_treat)) {
      warning("global_hp_treat is ignored for 'standard' or 'dirichlet' prior.")
    }

    global_hp_treat <- 1          # placeholder (ignored by C++)
    reversible_flag_treat <- FALSE

    if (delayed_proposal > 0) {
      delayed_proposal <- 0
    }
  } 

  if (prior_type_treat %in% c("dirichlet", "dirichlet-halfcauchy")) {
    dirichlet_bool_treat <- TRUE
    if (prior_type_treat == "dirichlet-halfcauchy") prior_type_treat <- "standard-halfcauchy"
  } else {
    dirichlet_bool_treat <- FALSE
  }


  if (prior_type_treat %in% c("standard-halfcauchy", "dirichlet-halfcauchy")) {
    if (!is.null(global_hp_treat)) {
      warning("global_hp_treat is ignored for 'standard' or 'dirichlet' prior.")
    }

    global_hp_treat <- 1          # placeholder (ignored by C++)
    reversible_flag_treat <- FALSE

    if (delayed_proposal > 0) {
      delayed_proposal <- 0
    }
  } 

  # Check outcome_type value
  allowed_types <- c("continuous", "right-censored", "interval-censored")
  if (!outcome_type %in% allowed_types) {
    stop("Invalid outcome_type. Please choose 'continuous', 'right-censored',
         or 'interval-censored'.")
  }

  # Check that y is provided for non-interval-censored types
  if (is.null(y) && outcome_type != "interval-censored") {
    stop("'y' is required when outcome_type is not 'interval-censored'.")
  }

  # Check interval-censored arguments
  if (outcome_type == "interval-censored") {
    if (is.null(left_time) || is.null(right_time))
      stop("outcome_type = 'interval-censored' requires 'left_time' and 'right_time'.")
    if (length(left_time) != length(right_time))
      stop("'left_time' and 'right_time' must have the same length.")
    if (any(left_time > right_time))
      stop("All 'left_time' values must be <= corresponding 'right_time' values.")
    if (timescale == "time" && any(left_time <= 0))
      stop("left_time contains non-positive values, but timescale = 'time' requires strictly positive times.")
  }

  # Check consistency with status argument
  if (outcome_type == "right-censored" && is.null(status)) {
    stop("You specified outcome_type = 'right-censored', but did not provide a
         'status' vector.")
  }
  if (!is.null(status) && !is.null(y) && length(status) != length(y)) {
    stop("The length of 'status' must match the length of 'y'.")
  }

  if (!outcome_type %in% c("right-censored", "interval-censored") && !is.null(status)) {
    warning("You provided a 'status' vector, but outcome_type is not
            'right-censored' or 'interval-censored'. The 'status' vector will be ignored.")
  }

  # Check survival data and timescale
  if (outcome_type == "right-censored" && timescale == "time" && any(y < 0)) {
    stop("Outcome contains negative values, but timescale = 'time' for survival
         data requires non-negative times.")
  }
  
  # Coerce to matrix so nrow/ncol are always defined
  if (!is.matrix(X_train_control)) X_train_control <- as.matrix(X_train_control)
  if (!is.matrix(X_train_treat))   X_train_treat   <- as.matrix(X_train_treat)

  # Retrieve dimensions of training data
  n_train   <- nrow(X_train_control)
  p_control <- ncol(X_train_control)
  p_treat   <- ncol(X_train_treat)

  # Check matching row numbers
  n_obs <- if (outcome_type == "interval-censored") length(left_time) else length(y)
  if (nrow(X_train_control) != n_obs) {
    stop("X_train_control rows must match length of outcome data.")
  }
  if (nrow(X_train_treat) != n_obs) {
    stop("X_train_treat rows must match length of outcome data.")
  }
  if (length(treatment_indicator_train) != n_obs) {
    stop("treatment_indicator_train must match length of outcome data.")
  }
  if (!all(treatment_indicator_train %in% c(0, 1))) {
    stop("treatment_indicator_train must contain only 0 and 1.")
  }
  if (p_control < 1L) stop("X_train_control must have at least one column.")
  if (p_treat   < 1L) stop("X_train_treat must have at least one column.")

  # Validate treatment_coding
  treatment_coding <- match.arg(treatment_coding, c("centered", "binary", "adaptive", "invariant"))
  if (treatment_coding == "adaptive" && is.null(propensity)) {
    stop("propensity scores are required when treatment_coding = 'adaptive'.")
  }
  if (!is.null(propensity) && length(propensity) != n_obs) {
    stop("Length of 'propensity' must match the number of training observations.")
  }

  # Prepare propensity vectors for C++
  propensity_train_cpp <- if (!is.null(propensity)) as.numeric(propensity) else numeric(n_obs)

  # If test provided, check
  test_provided_flag <- !is.null(X_test_control) && !is.null(X_test_treat)
  if (!is.null(X_test_control) && !is.null(X_test_treat)) {
    n_test <- nrow(X_test_control)
    if (ncol(X_test_control) != p_control || ncol(X_test_treat) != p_treat) {
      stop("Number of columns in X_test_control or X_test_treat does not match 
           training data.")
    }
    if (!is.null(treatment_indicator_test) && 
        length(treatment_indicator_test) != n_test) {
      stop("treatment_indicator_test length must match number of test rows.")
    }
    X_test_control <- as.numeric(t(X_test_control))
    X_test_treat <- as.numeric(t(X_test_treat))
  } else {
    n_test <- 1
    X_test_control <- as.numeric(colMeans(X_train_control))
    X_test_treat <- as.numeric(colMeans(X_train_treat))
    treatment_indicator_test <- as.integer(rep(1, n_test))
  }
  
  # Treatment indicators as integer
  treatment_indicator_train <- as.integer(treatment_indicator_train)
  if (is.null(treatment_indicator_test)) {
    treatment_indicator_test <- as.integer(rep(1, n_test))
  } else {
    treatment_indicator_test <- as.integer(treatment_indicator_test)
  }

  # Prepare propensity for test data
  propensity_test_cpp <- if (!is.null(propensity_test)) {
    as.numeric(propensity_test)
  } else {
    rep(0.5, n_test)
  }

  # Force data types to be numeric and plain arrays
  N_post <- as.integer(N_post)[1]
  N_burn <- as.integer(N_burn)[1]
  power_control <- as.numeric(power_control)[1]
  power_treat <- as.numeric(power_treat)[1]
  base_control <- as.numeric(base_control)[1]
  base_treat <- as.numeric(base_treat)[1]
  p_grow <- as.numeric(p_grow)[1]
  p_prune <- as.numeric(p_prune)[1]
  X_control_mat <- X_train_control        # preserve matrix for predict() / vi plots
  X_treat_mat   <- X_train_treat
  X_train_treat <- as.numeric(t(X_train_treat))
  X_train_control <- as.numeric(t(X_train_control))
  
  # Default rho to number of features
  if (is.null(rho_dirichlet_control)) {
    rho_dirichlet_control <- p_control
  }
  if (is.null(rho_dirichlet_treat)) {
    rho_dirichlet_treat <- p_treat
  }
  
  # For interval-censored: derive y/status/ic_indicator from left_time/right_time
  if (outcome_type == "interval-censored") {
    left_time <- as.numeric(left_time)
    right_time <- as.numeric(right_time)

    status <- as.integer(left_time == right_time)
    ic_indicator <- as.integer(left_time < right_time & is.finite(right_time))

    y <- ifelse(status == 1, left_time,
                ifelse(ic_indicator == 1, (left_time + right_time) / 2,
                       left_time))

    # C++ expects finite boundary for right-censored obs
    right_time[!is.finite(right_time)] <- left_time[!is.finite(right_time)]
  }

  # Save originals for data slot (used by predict())
  y_causal_raw      <- y
  status_causal_raw <- status
  left_time_raw     <- if (outcome_type == "interval-censored") left_time else NULL
  right_time_raw    <- if (outcome_type == "interval-censored") right_time else NULL

  if (outcome_type == "right-censored") {

    # Convert y to numeric for C++ compatibility
    y <- as.numeric(y)
    
    # Log-transform survival times if timescale = "time"
    if (timescale == "time") {
      y <- log(y)
    }
    
    # Obtain estimated mean and standard deviation using censored_info()
    cens_inf <- censored_info(y, status)
    
    # Center the data using estimated mean
    y_mean <- cens_inf$mu
    y <- y - y_mean
    
    # Determine sigma (timescale parameter) and whether it is known
    if (is.null(sigma)) {
      sigma_hat <- cens_inf$sd
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma
      sigma_known <- TRUE
    }

    # REFACTOR THIS CODE BELOW; make shorter
    if (prior_type_control %in% c("standard-halfcauchy")) {
      if (is.null(local_hp_control)) {
        local_hp_control <- 2.0
      } else {
        local_hp_control <- local_hp_control / sigma_hat
      }
    }

    if (prior_type_treat %in% c("standard-halfcauchy")) {
      if (is.null(local_hp_treat)) {
        local_hp_treat <- 2.0
      } else {
        local_hp_treat <- local_hp_treat / sigma_hat
      }
    }
    
    # Standardize the centered data
    y <- y / sigma_hat
    
    # Set survival flag
    survival <- TRUE
    
    # Compute lambda parameter for error distribution prior
    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu
    
    # Fit a Causal Horseshoe Forest
    fit <- CausalHorseForest_cpp(
      nSEXP = n_train,
      p_treatSEXP = p_treat,
      p_controlSEXP = p_control,
      X_train_treatSEXP = X_train_treat,
      X_train_controlSEXP = X_train_control,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = numeric(n_train),
      observed_right_timeSEXP = y + 0,
      interval_censoring_indicatorSEXP = numeric(n_train),
      treatment_indicatorSEXP = treatment_indicator_train,
      n_testSEXP = n_test,
      X_test_controlSEXP = X_test_control,
      X_test_treatSEXP = X_test_treat,
      treatment_indicator_testSEXP = treatment_indicator_test,
      no_trees_treatSEXP = number_of_trees_treat,
      power_treatSEXP = power_treat,
      base_treatSEXP = base_treat,
      p_grow_treatSEXP = p_grow,
      p_prune_treatSEXP = p_prune,
      omega_treatSEXP = 1/2,
      prior_type_treatSEXP = prior_type_treat,
      param1_treatSEXP = local_hp_treat,
      param2_treatSEXP = global_hp_treat,
      reversible_treatSEXP = reversible_flag_treat, 
      dirichlet_bool_treatSEXP = dirichlet_bool_treat,
      a_dirichlet_treatSEXP = a_dirichlet_treat,
      b_dirichlet_treatSEXP = b_dirichlet_treat, 
      rho_dirichlet_treatSEXP = rho_dirichlet_treat,  
      no_trees_controlSEXP = number_of_trees_control,
      power_controlSEXP = power_control,
      base_controlSEXP = base_control,
      p_grow_controlSEXP = p_grow,
      p_prune_controlSEXP = p_prune,
      omega_controlSEXP = 1/2,
      prior_type_controlSEXP = prior_type_control,
      param1_controlSEXP = local_hp_control,
      param2_controlSEXP = global_hp_control,
      reversible_controlSEXP = reversible_flag_control,
      dirichlet_bool_controlSEXP = dirichlet_bool_control,
      a_dirichlet_controlSEXP = a_dirichlet_control,
      b_dirichlet_controlSEXP = b_dirichlet_control, 
      rho_dirichlet_controlSEXP = rho_dirichlet_control,  
      sigma_knownSEXP = sigma_known,
      sigmaSEXP = sigma_hat,
      lambdaSEXP = lambda,
      nuSEXP = nu,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      store_posterior_sample_controlSEXP = store_posterior_sample,
      store_posterior_sample_treatSEXP = store_posterior_sample,
      verboseSEXP = verbose,
      treatment_codingSEXP = treatment_coding,
      propensity_trainSEXP = propensity_train_cpp,
      propensity_testSEXP = propensity_test_cpp
    )
    
    if (timescale == "time") {
      
      # Total
      fit$train_predictions <- exp(fit$train_predictions * sigma_hat + y_mean)
      fit$test_predictions <- exp(fit$test_predictions * sigma_hat + y_mean)
      
      # Control
      fit$train_predictions_control <- exp(fit$train_predictions_control * 
                                             sigma_hat + y_mean)
      fit$test_predictions_control <- exp(fit$test_predictions_control * 
                                            sigma_hat + y_mean)
      
      # Treatment
      fit$train_predictions_treat <- exp(fit$train_predictions_treat * 
                                           sigma_hat)
      fit$test_predictions_treat <- exp(fit$test_predictions_treat * sigma_hat)
      
      # Posterior samples
      if (store_posterior_sample) { #
        fit$train_predictions_sample_control <- 
          exp(fit$train_predictions_sample_control * sigma_hat + y_mean)
        fit$test_predictions_sample_control <- 
          exp(fit$test_predictions_sample_control * sigma_hat + y_mean)
        fit$train_predictions_sample_treat <- 
          exp(fit$train_predictions_sample_treat * sigma_hat)
        fit$test_predictions_sample_treat <- 
          exp(fit$test_predictions_sample_treat * sigma_hat)
      }
    } else {
      # Total
      fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
      fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
      
      # Control
      fit$train_predictions_control <- 
        fit$train_predictions_control * sigma_hat + y_mean
      fit$test_predictions_control <- 
        fit$test_predictions_control * sigma_hat + y_mean
      
      # Treatment
      fit$train_predictions_treat <- fit$train_predictions_treat * sigma_hat
      fit$test_predictions_treat <- fit$test_predictions_treat * sigma_hat
      
      # Posterior samples
      if (store_posterior_sample) {
        fit$train_predictions_sample_control <- 
          fit$train_predictions_sample_control * sigma_hat + y_mean
        fit$test_predictions_sample_control <- 
          fit$test_predictions_sample_control * sigma_hat + y_mean
        fit$train_predictions_sample_treat <- 
          fit$train_predictions_sample_treat * sigma_hat
        fit$test_predictions_sample_treat <- 
          fit$test_predictions_sample_treat * sigma_hat
      }
    }
    
  # If interval-censored
  } else if (outcome_type == "interval-censored") {

    y <- as.numeric(y)

    # Log-transform if timescale = "time"
    if (timescale == "time") {
      y <- log(y)
      left_time <- log(left_time)
      right_time <- log(right_time)
    }

    # Estimate mu and sd
    cens_inf <- censored_info(y, status, left_time = left_time,
                              right_time = right_time, ic_indicator = ic_indicator)

    y_mean <- cens_inf$mu
    y <- y - y_mean
    left_time <- left_time - y_mean
    right_time <- right_time - y_mean

    if (is.null(sigma)) {
      sigma_hat <- cens_inf$sd
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma
      sigma_known <- TRUE
    }

    if (prior_type_control %in% c("standard-halfcauchy")) {
      if (is.null(local_hp_control)) {
        local_hp_control <- 2.0
      } else {
        local_hp_control <- local_hp_control / sigma_hat
      }
    }

    if (prior_type_treat %in% c("standard-halfcauchy")) {
      if (is.null(local_hp_treat)) {
        local_hp_treat <- 2.0
      } else {
        local_hp_treat <- local_hp_treat / sigma_hat
      }
    }

    y <- y / sigma_hat
    left_time <- left_time / sigma_hat
    right_time <- right_time / sigma_hat

    survival <- TRUE

    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu

    fit <- CausalHorseForest_cpp(
      nSEXP = n_train,
      p_treatSEXP = p_treat,
      p_controlSEXP = p_control,
      X_train_treatSEXP = X_train_treat,
      X_train_controlSEXP = X_train_control,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = left_time,
      observed_right_timeSEXP = right_time,
      interval_censoring_indicatorSEXP = ic_indicator,
      treatment_indicatorSEXP = treatment_indicator_train,
      n_testSEXP = n_test,
      X_test_controlSEXP = X_test_control,
      X_test_treatSEXP = X_test_treat,
      treatment_indicator_testSEXP = treatment_indicator_test,
      no_trees_treatSEXP = number_of_trees_treat,
      power_treatSEXP = power_treat,
      base_treatSEXP = base_treat,
      p_grow_treatSEXP = p_grow,
      p_prune_treatSEXP = p_prune,
      omega_treatSEXP = 1/2,
      prior_type_treatSEXP = prior_type_treat,
      param1_treatSEXP = local_hp_treat,
      param2_treatSEXP = global_hp_treat,
      reversible_treatSEXP = reversible_flag_treat,
      dirichlet_bool_treatSEXP = dirichlet_bool_treat,
      a_dirichlet_treatSEXP = a_dirichlet_treat,
      b_dirichlet_treatSEXP = b_dirichlet_treat,
      rho_dirichlet_treatSEXP = rho_dirichlet_treat,
      no_trees_controlSEXP = number_of_trees_control,
      power_controlSEXP = power_control,
      base_controlSEXP = base_control,
      p_grow_controlSEXP = p_grow,
      p_prune_controlSEXP = p_prune,
      omega_controlSEXP = 1/2,
      prior_type_controlSEXP = prior_type_control,
      param1_controlSEXP = local_hp_control,
      param2_controlSEXP = global_hp_control,
      reversible_controlSEXP = reversible_flag_control,
      dirichlet_bool_controlSEXP = dirichlet_bool_control,
      a_dirichlet_controlSEXP = a_dirichlet_control,
      b_dirichlet_controlSEXP = b_dirichlet_control,
      rho_dirichlet_controlSEXP = rho_dirichlet_control,
      sigma_knownSEXP = sigma_known,
      sigmaSEXP = sigma_hat,
      lambdaSEXP = lambda,
      nuSEXP = nu,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      store_posterior_sample_controlSEXP = store_posterior_sample,
      store_posterior_sample_treatSEXP = store_posterior_sample,
      verboseSEXP = verbose,
      treatment_codingSEXP = treatment_coding,
      propensity_trainSEXP = propensity_train_cpp,
      propensity_testSEXP = propensity_test_cpp
    )

    # Back-transform (identical to right-censored)
    if (timescale == "time") {
      fit$train_predictions <- exp(fit$train_predictions * sigma_hat + y_mean)
      fit$test_predictions <- exp(fit$test_predictions * sigma_hat + y_mean)
      fit$train_predictions_control <- exp(fit$train_predictions_control *
                                             sigma_hat + y_mean)
      fit$test_predictions_control <- exp(fit$test_predictions_control *
                                            sigma_hat + y_mean)
      fit$train_predictions_treat <- exp(fit$train_predictions_treat * sigma_hat)
      fit$test_predictions_treat <- exp(fit$test_predictions_treat * sigma_hat)
      if (store_posterior_sample) {
        fit$train_predictions_sample_control <-
          exp(fit$train_predictions_sample_control * sigma_hat + y_mean)
        fit$test_predictions_sample_control <-
          exp(fit$test_predictions_sample_control * sigma_hat + y_mean)
        fit$train_predictions_sample_treat <-
          exp(fit$train_predictions_sample_treat * sigma_hat)
        fit$test_predictions_sample_treat <-
          exp(fit$test_predictions_sample_treat * sigma_hat)
      }
    } else {
      fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
      fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
      fit$train_predictions_control <-
        fit$train_predictions_control * sigma_hat + y_mean
      fit$test_predictions_control <-
        fit$test_predictions_control * sigma_hat + y_mean
      fit$train_predictions_treat <- fit$train_predictions_treat * sigma_hat
      fit$test_predictions_treat <- fit$test_predictions_treat * sigma_hat
      if (store_posterior_sample) {
        fit$train_predictions_sample_control <-
          fit$train_predictions_sample_control * sigma_hat + y_mean
        fit$test_predictions_sample_control <-
          fit$test_predictions_sample_control * sigma_hat + y_mean
        fit$train_predictions_sample_treat <-
          fit$train_predictions_sample_treat * sigma_hat
        fit$test_predictions_sample_treat <-
          fit$test_predictions_sample_treat * sigma_hat
      }
    }

    # Otherwise, continuous
  } else {

    # Force outcome to plain numeric vector
    y <- as.numeric(y)

    # Create dummy status vector (not used for continuous)
    status <- rep(1, n_train)
    survival <- FALSE

    # Determine prior guess of sigma
    if (is.null(sigma)) {
      sigma_hat <- sd(y)      # Estimate sigma from data
      sigma_known <- FALSE
    } else {
      sigma_hat <- sigma      # Use provided sigma
      sigma_known <- TRUE
    }

    # REFACTOR THIS CODE BELOW; make shorter
    if (prior_type_control %in% c("standard-halfcauchy")) {
      if (is.null(local_hp_control)) {
        local_hp_control <- 2.0
      } else {
        local_hp_control <- local_hp_control / sigma_hat
      }
    }

    if (prior_type_treat %in% c("standard-halfcauchy")) {
      if (is.null(local_hp_treat)) {
        local_hp_treat <- 2.0
      } else {
        local_hp_treat <- local_hp_treat / sigma_hat
      }
    }
    
    # Compute hyperparameters of error variance prior
    qchi <- qchisq(1.0 - q, nu)
    lambda <- (sigma_hat^2 * qchi) / nu
    
    # Compute mean and standardize  the outcome
    y_mean <- mean(y)
    y <- y - y_mean
    y <- y / sigma_hat  # Standardize y
    
    # Fit a Causal Horseshoe Forest
    fit <- CausalHorseForest_cpp(
      nSEXP = n_train,
      p_treatSEXP = p_treat,
      p_controlSEXP = p_control,
      X_train_treatSEXP = X_train_treat,
      X_train_controlSEXP = X_train_control,
      ySEXP = y,
      status_indicatorSEXP = status,
      is_survivalSEXP = survival,
      observed_left_timeSEXP = numeric(n_train),
      observed_right_timeSEXP = y + 0,
      interval_censoring_indicatorSEXP = numeric(n_train),
      treatment_indicatorSEXP = treatment_indicator_train,
      n_testSEXP = n_test,
      X_test_controlSEXP = X_test_control,
      X_test_treatSEXP = X_test_treat,
      treatment_indicator_testSEXP = treatment_indicator_test,
      no_trees_treatSEXP = number_of_trees_treat,
      power_treatSEXP = power_treat,
      base_treatSEXP = base_treat,
      p_grow_treatSEXP = p_grow,
      p_prune_treatSEXP = p_prune,
      omega_treatSEXP = 1/2,
      prior_type_treatSEXP = prior_type_treat,
      param1_treatSEXP = local_hp_treat,
      param2_treatSEXP = global_hp_treat,
      reversible_treatSEXP = reversible_flag_treat, 
      dirichlet_bool_treatSEXP = dirichlet_bool_treat,
      a_dirichlet_treatSEXP = a_dirichlet_treat,
      b_dirichlet_treatSEXP = b_dirichlet_treat, 
      rho_dirichlet_treatSEXP = rho_dirichlet_treat,  
      no_trees_controlSEXP = number_of_trees_control,
      power_controlSEXP = power_control,
      base_controlSEXP = base_control,
      p_grow_controlSEXP = p_grow,
      p_prune_controlSEXP = p_prune,
      omega_controlSEXP = 1/2,
      prior_type_controlSEXP = prior_type_control,
      param1_controlSEXP = local_hp_control,
      param2_controlSEXP = global_hp_control,
      reversible_controlSEXP = reversible_flag_control,
      dirichlet_bool_controlSEXP = dirichlet_bool_control,
      a_dirichlet_controlSEXP = a_dirichlet_control,
      b_dirichlet_controlSEXP = b_dirichlet_control, 
      rho_dirichlet_controlSEXP = rho_dirichlet_control,
      sigma_knownSEXP = sigma_known,
      sigmaSEXP = sigma_hat,
      lambdaSEXP = lambda,
      nuSEXP = nu,
      N_postSEXP = N_post,
      N_burnSEXP = N_burn,
      delayed_proposalSEXP = delayed_proposal,
      store_posterior_sample_controlSEXP = store_posterior_sample,
      store_posterior_sample_treatSEXP = store_posterior_sample,
      verboseSEXP = verbose,
      treatment_codingSEXP = treatment_coding,
      propensity_trainSEXP = propensity_train_cpp,
      propensity_testSEXP = propensity_test_cpp
    )
    
    # Total
    fit$train_predictions <- fit$train_predictions * sigma_hat + y_mean
    fit$test_predictions <- fit$test_predictions * sigma_hat + y_mean
    
    # Control
    fit$train_predictions_control <- fit$train_predictions_control * 
      sigma_hat + y_mean
    fit$test_predictions_control <- fit$test_predictions_control * 
      sigma_hat + y_mean
    
    # Treatment
    fit$train_predictions_treat <- fit$train_predictions_treat * sigma_hat 
    fit$test_predictions_treat <- fit$test_predictions_treat * sigma_hat 
    
    # Posterior samples
    if (store_posterior_sample) {
      fit$train_predictions_sample <- 
        fit$train_predictions_sample *  sigma_hat + y_mean
      fit$test_predictions_sample <- 
        fit$test_predictions_sample * sigma_hat +  y_mean
      fit$train_predictions_sample_control <- 
        fit$train_predictions_sample_control * sigma_hat + y_mean
      fit$test_predictions_sample_control <- 
        fit$test_predictions_sample_control * sigma_hat + y_mean
      fit$train_predictions_sample_treat <- 
        fit$train_predictions_sample_treat * sigma_hat 
      fit$test_predictions_sample_treat <- 
        fit$test_predictions_sample_treat * sigma_hat 
    }
    
  }
  
  # remove burn-in of sigma
  if (!sigma_known) {
    fit$sigma <- fit$sigma[-seq_len(N_burn)]
  } 

  prior_type_control_cpp <- prior_type_control
  prior_type_treat_cpp <- prior_type_treat
  dirichlet_bool_control <- FALSE
  dirichlet_bool_treat <- FALSE

  obj <- NewCausalShrinkageForest(
    fit = fit,
    call = match.call(),
    outcome_type = outcome_type,
    timescale = timescale,
    n_train = n_train,
    p_control = p_control,
    p_treat = p_treat,
    n_test = n_test,
    test_provided = test_provided_flag,
    number_of_trees_control = number_of_trees_control,
    number_of_trees_treat = number_of_trees_treat,
    N_post = N_post,
    N_burn = N_burn,
    store_posterior_sample = store_posterior_sample,
    sigma_hat = sigma_hat,
    sigma_known = sigma_known,
    y_mean = y_mean,
    prior_type_control_user = prior_type_control_user,
    prior_type_control_cpp = prior_type_control_cpp,
    prior_type_treat_user = prior_type_treat_user,
    prior_type_treat_cpp = prior_type_treat_cpp,
    dirichlet_bool_control = dirichlet_bool_control,
    dirichlet_bool_treat = dirichlet_bool_treat,
    # Data stored for predict()
    y_train                   = y_causal_raw,
    X_train_control           = X_control_mat,
    X_train_treat             = X_treat_mat,
    treatment_indicator_train = treatment_indicator_train,
    status_train              = if (outcome_type %in% c("right-censored", "interval-censored")) status_causal_raw else NULL,
    left_time_train           = left_time_raw,
    right_time_train          = right_time_raw,
    ic_indicator_train        = if (outcome_type == "interval-censored") ic_indicator else NULL,
    # Hyperparameters stored for predict()
    p_grow           = p_grow,
    p_prune          = p_prune,
    delayed_proposal = delayed_proposal,
    nu               = nu,
    lambda           = lambda,
    power_control    = power_control,
    base_control     = base_control,
    param1_control   = local_hp_control,
    param2_control   = global_hp_control,
    omega_control    = 1/2,
    reversible_control   = reversible_flag_control,
    a_dirichlet_control  = a_dirichlet_control,
    b_dirichlet_control  = b_dirichlet_control,
    rho_dirichlet_control = rho_dirichlet_control,
    power_treat      = power_treat,
    base_treat       = base_treat,
    param1_treat     = local_hp_treat,
    param2_treat     = global_hp_treat,
    omega_treat      = 1/2,
    reversible_treat     = reversible_flag_treat,
    a_dirichlet_treat    = a_dirichlet_treat,
    b_dirichlet_treat    = b_dirichlet_treat,
    rho_dirichlet_treat  = rho_dirichlet_treat,
    treatment_coding     = treatment_coding,
    propensity_train     = propensity
  )

  return(obj)
}

---
title: "ShrinkageTrees: Bayesian Tree Ensembles for Survival Analysis and Causal Inference"
author: "Tijn Jacobs"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{ShrinkageTrees: Introduction and Usage}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  message   = FALSE,
  warning   = FALSE,
  fig.width = 6,
  fig.height = 3.5,
  out.width = "100%"
)
library(ShrinkageTrees)
set.seed(42)
```

## Introduction

**ShrinkageTrees** is an R package that brings Bayesian Additive
Regression Trees (BART; Chipman, George & McCulloch, 2010) to
**survival analysis** and **causal inference**, with a particular focus
on high-dimensional data.

The package implements BART-based models for **right-censored** and
**interval-censored** survival outcomes using an accelerated failure time
(AFT) formulation. Censored event times are handled through Bayesian
data augmentation in the Gibbs sampler, enabling full posterior inference
without proportional-hazards assumptions. For causal inference, the
package provides Bayesian Causal Forests (BCF; Hahn, Murray & Carvalho,
2020), which decompose the outcome into a prognostic function
$\mu(\mathbf{x})$ and a treatment-effect function $\tau(\mathbf{x})$,
each estimated by a separate tree ensemble. This two-forest structure
supports estimation of heterogeneous treatment effects (CATEs) and the
average treatment effect (ATE).

A key feature is the availability of multiple **regularisation strategies**
that can be freely combined within a single model:

- **Classical BART priors** on the tree structure and leaf parameters.
- **Dirichlet splitting priors** (DART; Linero, 2018) for structural
  variable selection.
- **Horseshoe shrinkage on the leaf step heights** (Jacobs, van Wieringen &
  van der Pas, 2025) — a global–local prior that aggressively shrinks
  uninformative leaves toward zero while preserving strong signals.
  This is the main methodological novelty implemented in the package.
- **Half-Cauchy shrinkage** — a lighter-weight alternative that provides
  local shrinkage without a global scale parameter.

### Package map

| Function | Task | Prior |
|---|---|---|
| `HorseTrees()` | Prediction (continuous / binary / survival*) | Horseshoe |
| `ShrinkageTrees()` | Prediction — flexible prior choice* | Horseshoe, DART, BART, … |
| `SurvivalBART()` | Survival prediction* | Classical BART |
| `SurvivalDART()` | Sparse survival prediction* | DART (Dirichlet) |
| `SurvivalBCF()` | Causal survival inference* | BCF (classical) |
| `SurvivalShrinkageBCF()` | Sparse causal survival inference* | BCF + DART |
| `CausalHorseForest()` | Causal inference (all outcomes*) | Horseshoe |
| `CausalShrinkageForest()` | Causal inference — flexible prior* | Horseshoe, DART, BART, … |

\* All survival functions support both right-censored and interval-censored outcomes.

All model-fitting functions return an S3 object with consistent
`print()`, `summary()`, `predict()`, and `plot()` methods.


## Key Concepts

Before diving into examples, we clarify a few concepts that appear throughout
the package interface.

### Outcome types and the `timescale` parameter

Every model-fitting function accepts an `outcome_type` argument:

- `"continuous"` — standard regression (default for most functions).
- `"binary"` — probit BART for binary outcomes (0/1).
- `"right-censored"` — accelerated failure time model for survival data.
  The outcome `y` contains (possibly censored) follow-up times, and the
  `status` vector indicates events (1) vs. censored observations (0).
- `"interval-censored"` — AFT model for interval-censored survival data.
  Instead of `y` and `status`, provide `left_time` and `right_time` vectors
  specifying the lower and upper bounds of the observation window for each
  individual. Three cases are distinguished:
    - **Exact events**: `left_time == right_time` (event observed exactly).
    - **Interval-censored**: `left_time < right_time` with finite
      `right_time` (event occurred somewhere in the interval).
    - **Right-censored**: `right_time = Inf` (event not yet observed).

  This convention follows `survival::Surv(type = "interval2")`.

For survival outcomes, the `timescale` argument controls how the package
treats the times:

- `timescale = "time"` (default): the supplied values are on the
  **original time scale** (positive numbers).
  The package internally applies a log-transform, i.e. models
  $\log(T) = f(\mathbf{x}) + \varepsilon$.
  Predictions from `summary()` and `predict()` are back-transformed
  to the time scale automatically.
- `timescale = "log"`: the supplied values are **already
  log-transformed**. No further transformation is applied.
  Predictions stay on the log scale.

In most applications you should use `timescale = "time"` and pass the
raw survival times directly.

### Shrinkage priors on the step heights

In a BART ensemble each tree contributes a step height (leaf parameter)
to the overall prediction. Classical BART assigns these step heights a
fixed-variance Gaussian prior, which regularises all leaves equally. In
high-dimensional settings, stronger and more adaptive regularisation is
desirable. **ShrinkageTrees** implements two shrinkage priors that are
placed directly on the step heights via a scale mixture of normals:

$$
h_\ell \mid \lambda_\ell, \tau, \omega \sim
\mathcal{N}(0,\; \omega\, \lambda_\ell^2\, \tau^2).
$$

Here $\tau$ is a **global** shrinkage parameter shared across all leaves,
$\lambda_\ell$ is a **local** scale specific to leaf $\ell$, and $\omega$
is a fixed scaling constant. The two currently implemented instantiations
are:

- **Horseshoe** (`prior_type = "horseshoe"`). Both $\lambda_\ell$ and
  $\tau$ receive independent half-Cauchy priors. The heavy tails of the
  half-Cauchy allow individual leaves to escape shrinkage when the data
  support a strong effect, while the global parameter $\tau$ pulls the
  bulk of the estimates toward zero. This is the default prior in
  `HorseTrees()` and `CausalHorseForest()`.
- **Half-Cauchy** (`prior_type = "half-cauchy"`). Only the local scales
  $\lambda_\ell$ receive a half-Cauchy prior; there is no global
  shrinkage parameter. This provides per-leaf adaptivity without the
  additional pooling across the ensemble.

A forest-wide variant of the horseshoe (`prior_type = "horseshoe_fw"`)
shares a single global $\tau$ across all trees in the forest rather than
one per tree. These priors can be selected in `ShrinkageTrees()` and
`CausalShrinkageForest()` via the `prior_type` argument, and can be
combined with DART's Dirichlet splitting prior for simultaneous
structural and parametric regularisation.

### Hyperparameter selection

The two most important hyperparameters are `local_hp` and `global_hp`.
These control the horseshoe prior on the step heights (leaf parameters):

$$
\mu_{jl} \mid \lambda_{jl}, \tau_j \sim \mathcal{N}(0, \lambda_{jl}^2 \tau_j^2),
\qquad \lambda_{jl} \sim \text{C}^+(0, \texttt{local\_hp}),
\qquad \tau_j \sim \text{C}^+(0, \texttt{global\_hp}),
$$

where $\text{C}^+$ denotes the half-Cauchy distribution.  Smaller values
produce stronger shrinkage toward zero; larger values allow more
variation.

**`HorseTrees()` and `CausalHorseForest()`** provide a convenience
parameter `k` that sets both scales automatically:
`local_hp = global_hp = k / sqrt(number_of_trees)`. The default
`k = 0.1` works well in many settings and is a good starting point.

**`ShrinkageTrees()` and `CausalShrinkageForest()`** expose `local_hp`
and `global_hp` directly (no `k`).  A common rule of thumb is:

- `local_hp = k / sqrt(number_of_trees)` with `k` in [0.05, 0.5].
- `global_hp = local_hp` (symmetric) or a larger value if you want
  less overall shrinkage.

The **survival functions** (`SurvivalBART`, `SurvivalDART`) use `k` to
calibrate the standard BART leaf prior:
`local_hp = range(log(y)) / (2 * k * sqrt(number_of_trees))`.
The default `k = 2` follows Chipman et al. (2010).

### The `store_posterior_sample` flag

When `store_posterior_sample = TRUE`, the fitted object stores the full
$N_\text{post} \times n$ matrix of posterior draws for predictions.
This is needed for:

- `predict()` on new data (it re-runs the sampler internally, so
  posterior samples are always produced);
- `plot(fit, type = "ate")` and `plot(fit, type = "cate")`, which
  require the full posterior distribution;
- `plot(fit, type = "survival")` — full posterior credible bands over
  both $\mu_i$ and $\sigma$ (without samples, only sigma-uncertainty
  bands are available);
- any custom posterior analysis (e.g. computing posterior credible
  intervals for individual predictions).

When `FALSE`, only posterior *means* and $\sigma$ draws are stored,
saving memory.  `print()` and `summary()` work in both cases, but
`predict()` will not be available.

### Treatment coding (`treatment_coding`)

All causal model functions — `CausalHorseForest()`,
`CausalShrinkageForest()`, `SurvivalBCF()`, and
`SurvivalShrinkageBCF()` — decompose the outcome as
$$
Y_i = \mu(\mathbf{x}_i) + b_i \cdot \tau(\mathbf{x}_i) + \varepsilon_i,
$$
where $b_i$ is a scalar that depends on the treatment assignment $Z_i$.
The `treatment_coding` argument controls how $b_i$ is defined.
Four options are available:

**`"centered"`** (default).
$b_i = Z_i - 1/2$, so that $b_i \in \{-1/2,\; 1/2\}$.
This is the original BCF parameterisation.

**`"binary"`**.
$b_i = Z_i$, so that $b_i \in \{0,\; 1\}$.
Standard binary coding; the treatment forest captures the full
effect of treatment on the treated.

**`"adaptive"`**.
$b_i = Z_i - \hat{e}(\mathbf{x}_i)$, where $\hat{e}(\mathbf{x}_i)$
is the estimated propensity score. This follows Hahn, Murray &
Carvalho (2020) and is the coding used in the `bcf` R package.
When using this option, a `propensity` vector must be supplied.

**`"invariant"`**.
Parameter-expanded (invariant) treatment coding. The coding
parameters $b_0$ and $b_1$ are assigned $N(0,\; 1/2)$ priors and
estimated within the Gibbs sampler via conjugate normal updates:
$$
Y_i = \mu(\mathbf{x}_i) + b_{Z_i} \cdot \tilde{\tau}(\mathbf{x}_i) + \varepsilon_i,
\qquad b_0,\; b_1 \sim N(0,\; 1/2).
$$
The treatment effect is $\tau(\mathbf{x}_i) = (b_1 - b_0) \cdot
\tilde{\tau}(\mathbf{x}_i)$, and the posterior draws of $b_0$ and $b_1$
are returned in the fitted object.  This parameterisation is invariant
to the coding of the treatment indicator (Hahn et al., 2020,
Section 5.2).

The examples below illustrate each option on a simple continuous-outcome
causal model.

```{r tc-data}
set.seed(50)
n_tc <- 60;  p_tc <- 5
X_tc <- matrix(rnorm(n_tc * p_tc), n_tc, p_tc)
W_tc <- rbinom(n_tc, 1, 0.5)
tau_tc <- 1.5 * (X_tc[, 1] > 0)
y_tc <- X_tc[, 1] + W_tc * tau_tc + rnorm(n_tc, sd = 0.5)
```

```{r tc-centered}
# Centered (default)
fit_tc_cen <- CausalHorseForest(
  y = y_tc,
  X_train_control = X_tc, X_train_treat = X_tc,
  treatment_indicator_train = W_tc,
  treatment_coding = "centered",
  number_of_trees = 5, N_post = 50, N_burn = 25,
  store_posterior_sample = TRUE, verbose = FALSE
)
cat("Centered — ATE:",
    round(mean(fit_tc_cen$train_predictions_treat), 3), "\n")
```

```{r tc-binary}
# Binary
fit_tc_bin <- CausalHorseForest(
  y = y_tc,
  X_train_control = X_tc, X_train_treat = X_tc,
  treatment_indicator_train = W_tc,
  treatment_coding = "binary",
  number_of_trees = 5, N_post = 50, N_burn = 25,
  store_posterior_sample = TRUE, verbose = FALSE
)
cat("Binary — ATE:",
    round(mean(fit_tc_bin$train_predictions_treat), 3), "\n")
```

```{r tc-adaptive}
# Adaptive (requires propensity scores)
ps_tc <- pnorm(0.3 * X_tc[, 1])   # simple propensity model for illustration

fit_tc_ada <- CausalHorseForest(
  y = y_tc,
  X_train_control = X_tc, X_train_treat = X_tc,
  treatment_indicator_train = W_tc,
  treatment_coding = "adaptive",
  propensity = ps_tc,
  number_of_trees = 5, N_post = 50, N_burn = 25,
  store_posterior_sample = TRUE, verbose = FALSE
)
cat("Adaptive — ATE:",
    round(mean(fit_tc_ada$train_predictions_treat), 3), "\n")
```

```{r tc-invariant}
# Invariant (parameter-expanded)
fit_tc_inv <- CausalHorseForest(
  y = y_tc,
  X_train_control = X_tc, X_train_treat = X_tc,
  treatment_indicator_train = W_tc,
  treatment_coding = "invariant",
  number_of_trees = 5, N_post = 50, N_burn = 25,
  store_posterior_sample = TRUE, verbose = FALSE
)
cat("Invariant — ATE:",
    round(mean(fit_tc_inv$train_predictions_treat), 3), "\n")

# Posterior draws of b0 and b1 are stored in the fitted object
cat("b0 posterior mean:", round(mean(fit_tc_inv$b0), 3), "\n")
cat("b1 posterior mean:", round(mean(fit_tc_inv$b1), 3), "\n")
```

The survival functions inherit `treatment_coding` support. For example,
`SurvivalBCF(..., treatment_coding = "invariant")` works out of the box.


## Included Datasets

The package ships with two TCGA datasets for high-dimensional survival
analysis and causal inference:

- **`pdac`** — TCGA pancreatic ductal adenocarcinoma (PAAD) cohort
  (n = 178). A data frame with overall survival times, a binary treatment
  indicator (radiation therapy vs. control), clinical covariates, and
  expression values of ~3,000 genes selected by median absolute deviation.
- **`ovarian`** — TCGA ovarian cancer (OV) cohort (n = 357). A list with
  `X` (357 x 2,000 gene expression matrix, log2-normalised TPM) and
  `clinical` (data frame with OS time/event, age, FIGO stage, tumor grade,
  and treatment: carboplatin vs cisplatin). See `?ovarian` for details.

### The PDAC Dataset

The `pdac` dataset contains overall survival
times, a binary treatment indicator (radiation therapy vs. control), clinical
covariates, and expression values of approximately 3,000 genes selected by
median absolute deviation.

```{r load-data}
library(ShrinkageTrees)
data("pdac")

# Dimensions and column overview
cat("Patients:", nrow(pdac), "\n")
cat("Columns :", ncol(pdac), "\n")
cat("Clinical columns:", paste(names(pdac)[1:13], collapse = ", "), "\n")
cat("Survival: time (months), censoring rate =",
    round(1 - mean(pdac$status), 2), "\n")
cat("Treatment: radiation =", sum(pdac$treatment),
    "/ control =", sum(1 - pdac$treatment), "\n")
```

We separate the outcome, treatment, and covariate matrix for the analyses
below.

```{r prepare-data}
time      <- pdac$time
status    <- pdac$status
treatment <- pdac$treatment
X         <- as.matrix(pdac[, !(names(pdac) %in% c("time", "status", "treatment"))])
```


## Prediction Models

This section demonstrates the single-forest models in **ShrinkageTrees**.
These models estimate a single function $f(\mathbf{x})$ of the covariates,
applicable to continuous, binary, and survival outcomes. We begin with a
binary-outcome example that will also serve as the propensity score model
for the causal analyses later.

### HorseTrees — binary outcome (propensity scores)

Before fitting causal models we estimate propensity scores
$\hat{e}(\mathbf{x}) = P(W=1 \mid \mathbf{x})$ using `HorseTrees()` with a
binary outcome. The probit link is used internally: predictions are on the
latent Gaussian scale and can be converted to probabilities with `pnorm()`.

The code block below uses reduced MCMC settings for illustration.
A real analysis would use `N_post = 5000, N_burn = 5000`.

```{r horsetrees-binary, eval=FALSE}
ps_fit <- HorseTrees(
  y            = treatment,
  X_train      = X,
  outcome_type = "binary",
  k            = 0.1,
  N_post       = 5000,
  N_burn       = 5000,
  verbose      = FALSE
)

propensity <- pnorm(ps_fit$train_predictions)
```

For the remainder of this vignette we use a short synthetic run to keep
build time low.

```{r horsetrees-binary-small}
set.seed(1)
n <- 80;  p <- 10
X_syn  <- matrix(rnorm(n * p), n, p)
W_syn  <- rbinom(n, 1, pnorm(0.8 * X_syn[, 1]))

ps_fit <- HorseTrees(
  y            = W_syn,
  X_train      = X_syn,
  outcome_type = "binary",
  number_of_trees = 5,
  k            = 0.5,
  N_post       = 50,
  N_burn       = 25,
  verbose      = FALSE
)

propensity_syn <- pnorm(ps_fit$train_predictions)
cat("Propensity scores — range: [",
    round(range(propensity_syn), 3), "]\n")
```

### HorseTrees — survival outcome

`HorseTrees()` handles right-censored data via an AFT model.
Pass `outcome_type = "right-censored"` and provide the `status` vector
(1 = event, 0 = censored). When `timescale = "time"` (the default),
the package log-transforms survival times internally and returns
predictions on the log scale (see *Key Concepts* above).

```{r horsetrees-survival}
set.seed(2)
log_T <- X_syn[, 1] + rnorm(n)
C     <- rexp(n, 0.5)
y_syn   <- pmin(exp(log_T), C)
d_syn   <- as.integer(exp(log_T) <= C)

ht_surv <- HorseTrees(
  y               = y_syn,
  status          = d_syn,
  X_train         = X_syn,
  outcome_type    = "right-censored",
  timescale       = "time",
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  store_posterior_sample = TRUE,
  verbose         = FALSE
)

cat("Posterior mean log-time (first 5 obs):",
    round(ht_surv$train_predictions[1:5], 3), "\n")
cat("Posterior sigma — mean:",
    round(mean(ht_surv$sigma), 3), "\n")
```

### HorseTrees — interval-censored outcome

When event times are not observed exactly but known to lie within an
interval, the package supports **interval censoring**. Instead of
providing `y` and `status`, pass `left_time` and `right_time` with
`outcome_type = "interval-censored"`.

The three censoring types are encoded as follows:

- **Exact event**: `left_time[i] == right_time[i]`
- **Interval-censored**: `left_time[i] < right_time[i]` (both finite)
- **Right-censored**: `right_time[i] = Inf`

This convention matches `survival::Surv(type = "interval2")`.

```{r horsetrees-ic}
set.seed(20)

# Generate true event times
true_T <- rexp(n, rate = exp(-0.5 * X_syn[, 1]))

# Create interval-censored observations
left_syn  <- true_T * runif(n, 0.5, 1.0)
right_syn <- true_T * runif(n, 1.0, 1.5)

# Mark some as exact observations and some as right-censored
exact_idx <- sample(n, 25)
left_syn[exact_idx]  <- true_T[exact_idx]
right_syn[exact_idx] <- true_T[exact_idx]

rc_idx <- sample(setdiff(seq_len(n), exact_idx), 15)
right_syn[rc_idx] <- Inf

cat("Exact events:", sum(left_syn == right_syn), "\n")
cat("Interval-censored:", sum(left_syn < right_syn & is.finite(right_syn)), "\n")
cat("Right-censored:", sum(!is.finite(right_syn)), "\n")

ht_ic <- HorseTrees(
  left_time       = left_syn,
  right_time      = right_syn,
  X_train         = X_syn,
  outcome_type    = "interval-censored",
  timescale       = "time",
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  store_posterior_sample = TRUE,
  verbose         = FALSE
)

cat("Posterior mean log-time (first 5 obs):",
    round(ht_ic$train_predictions[1:5], 3), "\n")
cat("Posterior sigma — mean:",
    round(mean(ht_ic$sigma), 3), "\n")
```

All survival functions (`SurvivalBART`, `SurvivalDART`, `SurvivalBCF`,
`SurvivalShrinkageBCF`) and the general-purpose functions (`ShrinkageTrees`,
`CausalShrinkageForest`, `CausalHorseForest`) accept `left_time` and
`right_time` in the same way. For example, using `SurvivalBART`:

```{r sbart-ic}
set.seed(21)
fit_sbart_ic <- SurvivalBART(
  left_time       = left_syn,
  right_time      = right_syn,
  X_train         = X_syn,
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

cat("SurvivalBART (IC) class:", class(fit_sbart_ic), "\n")
```

### ShrinkageTrees — flexible prior choice

While `HorseTrees()` fixes the prior to the horseshoe, the more general
`ShrinkageTrees()` function exposes the `prior_type` argument, allowing
the user to select among all implemented regularisation strategies.
Available options are `"horseshoe"`, `"horseshoe_fw"` (forest-wide),
`"half-cauchy"`, `"standard"`
(classical BART), and `"dirichlet"` (DART). Below we compare the
per-tree horseshoe and the forest-wide horseshoe on a continuous outcome.

```{r shrinkage-continuous}
set.seed(3)
y_cont <- X_syn[, 1] + 0.5 * X_syn[, 2] + rnorm(n)

# Horseshoe prior (default for HorseTrees)
fit_hs <- ShrinkageTrees(
  y               = y_cont,
  X_train         = X_syn,
  outcome_type    = "continuous",
  prior_type      = "horseshoe",
  local_hp        = 0.1 / sqrt(5),
  global_hp       = 0.1 / sqrt(5),
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

# Forest-wide horseshoe (horseshoe_fw)
fit_fw <- ShrinkageTrees(
  y               = y_cont,
  X_train         = X_syn,
  outcome_type    = "continuous",
  prior_type      = "horseshoe_fw",
  local_hp        = 0.1 / sqrt(5),
  global_hp       = 0.1 / sqrt(5),
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

cat("Horseshoe   — train RMSE:",
    round(sqrt(mean((fit_hs$train_predictions - y_cont)^2)), 3), "\n")
cat("Horseshoe FW— train RMSE:",
    round(sqrt(mean((fit_fw$train_predictions - y_cont)^2)), 3), "\n")
```

### SurvivalBART and SurvivalDART

`SurvivalBART()` and `SurvivalDART()` fit classical BART and DART models
for right-censored survival data under the AFT formulation. They
calibrate prior hyperparameters automatically from the data range,
providing a simple interface when horseshoe shrinkage is not needed.

```{r survival-bart-dart}
set.seed(4)

# SurvivalBART: classical BART prior, AFT likelihood
fit_sbart <- SurvivalBART(
  time            = y_syn,
  status          = d_syn,
  X_train         = X_syn,
  number_of_trees = 5,
  k               = 2.0,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

# SurvivalDART: Dirichlet (DART) splitting prior
fit_sdart <- SurvivalDART(
  time            = y_syn,
  status          = d_syn,
  X_train         = X_syn,
  number_of_trees = 5,
  k               = 2.0,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)

cat("SurvivalBART  class:", class(fit_sbart), "\n")
cat("SurvivalDART  class:", class(fit_sdart), "\n")
```

## High-Dimensional Survival Analysis

A key motivation for horseshoe shrinkage and the Dirichlet (DART) sparsity
prior is their behaviour in the **$p \gg n$ regime**: many covariates are
available but only a small subset drives the outcome. Classical BART may
struggle here because the standard Gaussian leaf prior is non-sparse and
does not concentrate on a small number of predictors.

We illustrate both priors on a sparse AFT simulation: $n = 60$ observations,
$p = 200$ predictors, and only three active predictors.

```{r hd-data}
set.seed(20)
n_hd <- 60;  p_hd <- 200
X_hd <- matrix(rnorm(n_hd * p_hd), n_hd, p_hd)

# True log-survival depends only on predictors 1, 2, and 3
log_T_hd <- 1.5 * X_hd[, 1] - 1.0 * X_hd[, 2] + 0.5 * X_hd[, 3] + rnorm(n_hd)
C_hd     <- rexp(n_hd, rate = 0.5)
y_hd     <- pmin(exp(log_T_hd), C_hd)
d_hd     <- as.integer(exp(log_T_hd) <= C_hd)

cat("n =", n_hd, "| p =", p_hd,
    "| active predictors = 3",
    "| censoring rate =", round(1 - mean(d_hd), 2), "\n")
```

**ShrinkageTrees (horseshoe)** places global–local shrinkage on the step
heights of every leaf, automatically regularising all 200 predictors
toward zero while preserving the signal in the three active ones.

```{r hd-horseshoe}
set.seed(21)
fit_hd_hs <- ShrinkageTrees(
  y               = y_hd,
  status          = d_hd,
  X_train         = X_hd,
  outcome_type    = "right-censored",
  prior_type      = "horseshoe",
  local_hp        = 0.1 / sqrt(10),
  global_hp       = 0.1 / sqrt(10),
  number_of_trees = 10,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)
```

**SurvivalDART** uses a Dirichlet prior on split probabilities to induce
structural sparsity: after burn-in, most splitting probability is
concentrated on truly predictive variables. Setting `rho_dirichlet = 3`
encodes the prior belief that approximately three predictors are active.

```{r hd-dart}
set.seed(22)
fit_hd_dart <- SurvivalDART(
  time            = y_hd,
  status          = d_hd,
  X_train         = X_hd,
  number_of_trees = 10,
  rho_dirichlet   = 3,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)
```

Both models run without error in the $p > n$ regime. We compare their
posterior mean predictions in log-time against the latent true values
used to generate the data.

```{r hd-compare}
rmse_hs   <- sqrt(mean((fit_hd_hs$train_predictions  - log_T_hd)^2))
rmse_dart <- sqrt(mean((fit_hd_dart$train_predictions - log_T_hd)^2))

cat(sprintf("%-18s  train RMSE (log-time): %.3f\n", "Horseshoe", rmse_hs))
cat(sprintf("%-18s  train RMSE (log-time): %.3f\n", "DART",      rmse_dart))
```

The DART model also produces **variable importance** plots that display
the posterior distribution of each predictor's splitting probability.
With only 50 posterior draws the top-10 plot below should already concentrate
most probability mass near the three truly active predictors.

```{r hd-vi, eval=requireNamespace("ggplot2", quietly=TRUE)}
plot(fit_hd_dart, type = "vi", n_vi = 10)
```

## Causal Forest Models

For causal inference, **ShrinkageTrees** provides Bayesian Causal Forest
(BCF) models that decompose the outcome into a prognostic component and
a treatment effect component:
$$
Y_i = \mu(\mathbf{x}_i) + W_i \cdot \tau(\mathbf{x}_i) + \varepsilon_i,
$$
where $\mu(\cdot)$ is the prognostic (control) function modelled by one
tree ensemble, and $\tau(\cdot)$ is the heterogeneous treatment effect
modelled by a second ensemble. This two-forest structure allows each
component to have its own regularisation, number of trees, and prior —
for instance, a standard BART prior for the prognostic forest and
horseshoe shrinkage for the treatment effect forest.

The package provides four causal model functions with increasing
generality: `SurvivalBCF()` (classical BCF for survival),
`SurvivalShrinkageBCF()` (BCF + DART for survival),
`CausalHorseForest()` (horseshoe BCF for all outcome types), and
`CausalShrinkageForest()` (fully configurable BCF). We illustrate each
below on synthetic data with a known treatment effect.

```{r causal-data}
set.seed(5)
tau_true <- 1.5 * (X_syn[, 1] > 0)    # heterogeneous treatment effect
y_causal <- X_syn[, 1] + W_syn * tau_true + rnorm(n, sd = 0.5)
```

### SurvivalBCF — classical BCF for survival

`SurvivalBCF()` fits a BCF model for right-censored survival outcomes using
classical BART priors.

```{r survbcf, eval=FALSE}
# Full analysis (eval=FALSE — use larger MCMC settings in practice)
fit_sbcf <- SurvivalBCF(
  time       = time,
  status     = status,
  X_train    = X,
  treatment  = treatment,
  propensity = propensity,   # from HorseTrees above
  N_post     = 5000,
  N_burn     = 5000,
  verbose    = FALSE
)
```

```{r survbcf-small}
set.seed(6)
fit_sbcf <- SurvivalBCF(
  time            = y_syn,
  status          = d_syn,
  X_train         = X_syn,
  treatment       = W_syn,
  number_of_trees_control = 5,
  number_of_trees_treat   = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)
cat("SurvivalBCF class:", class(fit_sbcf), "\n")
cat("ATE (posterior mean):",
    round(mean(fit_sbcf$train_predictions_treat), 3), "\n")
```

### SurvivalShrinkageBCF — sparse causal survival forest

`SurvivalShrinkageBCF()` extends BCF with a Dirichlet splitting prior on
both forests, inducing sparsity in high-dimensional settings.

```{r survsbcf-small}
set.seed(7)
fit_ssbcf <- SurvivalShrinkageBCF(
  time            = y_syn,
  status          = d_syn,
  X_train         = X_syn,
  treatment       = W_syn,
  number_of_trees_control = 5,
  number_of_trees_treat   = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)
cat("SurvivalShrinkageBCF class:", class(fit_ssbcf), "\n")
```

### CausalHorseForest — horseshoe causal forest

`CausalHorseForest()` is the primary novel contribution of this package.
It applies horseshoe shrinkage to the leaf parameters of both the prognostic
and treatment-effect forests. This enables effective regularisation when
many covariates are available but few are truly predictive of heterogeneous
treatment effects.

```{r causal-horse}
set.seed(8)
fit_chf <- CausalHorseForest(
  y                         = y_causal,
  X_train_control           = X_syn,
  X_train_treat             = X_syn,
  treatment_indicator_train = W_syn,
  outcome_type              = "continuous",
  number_of_trees           = 5,
  N_post                    = 50,
  N_burn                    = 25,
  store_posterior_sample    = TRUE,
  verbose                   = FALSE
)

cat("CausalHorseForest class:", class(fit_chf), "\n")

# Posterior mean CATE
cate_mean <- fit_chf$train_predictions_treat
cat("CATE — posterior mean (first 5):",
    round(cate_mean[1:5], 3), "\n")

# Posterior ATE
ate_samples <- rowMeans(fit_chf$train_predictions_sample_treat)
cat("ATE posterior mean:",
    round(mean(ate_samples), 3),
    "  95% CI: [",
    round(quantile(ate_samples, 0.025), 3), ",",
    round(quantile(ate_samples, 0.975), 3), "]\n")
```

The fitted object stores the posterior mean CATE for each training
observation in `train_predictions_treat`. When
`store_posterior_sample = TRUE`, the full posterior sample matrix is
available in `train_predictions_sample_treat`, from which the posterior
ATE distribution and credible intervals can be computed as shown above.

You can also supply separate test matrices to obtain out-of-sample
CATE predictions.

```{r causal-horse-test}
set.seed(9)
X_test <- matrix(rnorm(20 * p), 20, p)
W_test <- rbinom(20, 1, 0.5)

fit_chf_test <- CausalHorseForest(
  y                         = y_causal,
  X_train_control           = X_syn,
  X_train_treat             = X_syn,
  treatment_indicator_train = W_syn,
  X_test_control            = X_test,
  X_test_treat              = X_test,
  treatment_indicator_test  = W_test,
  outcome_type              = "continuous",
  number_of_trees           = 5,
  N_post                    = 50,
  N_burn                    = 25,
  store_posterior_sample    = TRUE,
  verbose                   = FALSE
)

cat("Test CATE (first 5):",
    round(fit_chf_test$test_predictions_treat[1:5], 3), "\n")
```

### CausalShrinkageForest — flexible causal priors

`CausalShrinkageForest()` is the most general causal model interface.
It allows independent prior choices for the prognostic and treatment
effect forests via `prior_type_control` and `prior_type_treat`. For
example, one could use a standard BART prior for the prognostic forest
(where variable selection is less critical) and horseshoe shrinkage for
the treatment forest (where most covariates are expected to be
irrelevant for the treatment effect).

```{r causal-shrinkage}
set.seed(10)
lh <- 0.1 / sqrt(5)

fit_csf <- CausalShrinkageForest(
  y                         = y_causal,
  X_train_control           = X_syn,
  X_train_treat             = X_syn,
  treatment_indicator_train = W_syn,
  outcome_type              = "continuous",
  prior_type_control        = "horseshoe",
  prior_type_treat          = "horseshoe",
  local_hp_control          = lh,
  global_hp_control         = lh,
  local_hp_treat            = lh,
  global_hp_treat           = lh,
  number_of_trees_control   = 5,
  number_of_trees_treat     = 5,
  N_post                    = 50,
  N_burn                    = 25,
  store_posterior_sample    = TRUE,
  verbose                   = FALSE
)

cat("CausalShrinkageForest class:", class(fit_csf), "\n")
cat("Acceptance ratio (control):",
    round(fit_csf$acceptance_ratio_control, 3), "\n")
cat("Acceptance ratio (treat)  :",
    round(fit_csf$acceptance_ratio_treat, 3), "\n")
```

The `horseshoe_fw` prior adds a forest-wide shrinkage parameter that is
tracked in the fitted object.

```{r causal-shrinkage-fw}
set.seed(11)
fit_fw2 <- CausalShrinkageForest(
  y                         = y_causal,
  X_train_control           = X_syn,
  X_train_treat             = X_syn,
  treatment_indicator_train = W_syn,
  outcome_type              = "continuous",
  prior_type_control        = "horseshoe_fw",
  prior_type_treat          = "horseshoe_fw",
  local_hp_control          = lh,
  global_hp_control         = lh,
  local_hp_treat            = lh,
  global_hp_treat           = lh,
  number_of_trees_control   = 5,
  number_of_trees_treat     = 5,
  N_post                    = 50,
  N_burn                    = 25,
  verbose                   = FALSE
)

cat("Forest-wide shrinkage (control, first 5 draws):\n")
print(round(fit_fw2$forestwide_shrinkage_control[1:5], 4))
```

## S3 Methods

All fitted objects — whether from prediction models or causal models —
support a consistent set of S3 methods: `print()`, `summary()`,
`predict()`, and `plot()`. This section illustrates each method using
the models fitted above.

### print()

Calling `print()` (or just typing the object name) displays a concise model
summary.

```{r print}
print(fit_chf)
```

For causal models the output additionally shows the number of trees in each
forest and prior details for both components.

```{r print-csf}
print(fit_csf)
```

### summary()

`summary()` returns a structured list and displays a richer description
including posterior statistics for $\sigma$, acceptance ratios, and
treatment effect estimates.

```{r summary-shrinkage}
smry <- summary(fit_hs)
print(smry)
```

For causal models the summary includes the posterior ATE with a 95% credible
interval (when `store_posterior_sample = TRUE`).

```{r summary-causal}
smry_c <- summary(fit_chf)
print(smry_c)

# Access the ATE directly
cat("ATE mean  :", round(smry_c$treatment_effect$ate, 3), "\n")
cat("ATE 95% CI: [",
    round(smry_c$treatment_effect$ate_lower, 3), ",",
    round(smry_c$treatment_effect$ate_upper, 3), "]\n")
```

##### Population vs. mixed ATE

By default the ATE credible interval is obtained by a **Bayesian
bootstrap**: at each MCMC iteration $s$ the observation-level CATEs
$\tau^{(s)}(x_i)$ are reweighted with Dirichlet(1, ..., 1) weights,

$$
\widehat{\mathrm{PATE}}^{(s)} \;=\; \sum_{i=1}^n w_i^{(s)}\, \tau^{(s)}(x_i),
\qquad (w_1^{(s)}, \dots, w_n^{(s)}) \sim \mathrm{Dir}(1, \dots, 1).
$$

The collection $\{\widehat{\mathrm{PATE}}^{(s)}\}$ approximates the
posterior of the **population ATE** and therefore propagates uncertainty
in both $\tau(\cdot)$ and the covariate distribution $F_X$. Setting
`bayesian_bootstrap = FALSE` reverts to equal $1/n$ weights, giving the
**mixed ATE** (MATE) that conditions on the observed covariates and has a
narrower credible interval.

```{r summary-pate-mate}
smry_pate <- summary(fit_chf, bayesian_bootstrap = TRUE)   # default
smry_mate <- summary(fit_chf, bayesian_bootstrap = FALSE)
```

The standalone helper `bayesian_bootstrap_ate()` returns both posteriors
and their draws in a single list, and also works on a
`CausalShrinkageForestPrediction` returned by `predict()` so that the
PATE integrates over a prespecified target population.

```{r bb-ate}
bb <- bayesian_bootstrap_ate(fit_chf)
cat("PATE:", round(bb$pate_mean, 3),
    " 95% CI: [", round(bb$pate_ci$lower, 3), ",",
                  round(bb$pate_ci$upper, 3), "]\n")
cat("MATE:", round(bb$mate_mean, 3),
    " 95% CI: [", round(bb$mate_ci$lower, 3), ",",
                  round(bb$mate_ci$upper, 3), "]\n")
```

### predict()

`predict()` computes the posterior predictive distribution on new data.
It returns a `ShrinkageTreesPrediction` object with posterior mean and
credible-interval vectors.

```{r predict}
X_new  <- matrix(rnorm(10 * p), 10, p)

pred <- predict(fit_hs, newdata = X_new)
print(pred)
```

```{r predict-ci}
# Point estimates and 95% credible intervals
head(data.frame(
  mean  = round(pred$mean,  3),
  lower = round(pred$lower, 3),
  upper = round(pred$upper, 3)
))
```

#### Causal predictions

For causal models (`CausalShrinkageForest` and `CausalHorseForest`),
`predict()` returns three sets of posterior summaries:

- **prognostic**: the control-forest prediction $\mu(\mathbf{x})$ — the
  expected outcome under control.
- **cate**: the Conditional Average Treatment Effect $\tau(\mathbf{x})$ —
  the additional effect of treatment for each individual.
- **total**: the combined prediction $\mu(\mathbf{x}) + \tau(\mathbf{x})$
  — the expected outcome under treatment.

For survival models with `timescale = "time"`, the prognostic and total
components are back-transformed to the original time scale (via
$\exp(\cdot)$), and the CATE becomes a **multiplicative time ratio**:
$\exp(\tau) > 1$ means treatment prolongs survival.

The `predict()` method requires two covariate matrices — one for each
forest — matching the columns used at fit time.

```{r predict-causal}
X_new_ctrl  <- matrix(rnorm(10 * p), 10, p)
X_new_treat <- matrix(rnorm(10 * p), 10, p)

pred_c <- predict(fit_chf, newdata_control = X_new_ctrl,
                  newdata_treat = X_new_treat)
print(pred_c)
```

```{r predict-causal-detail}
# Extract individual components
head(data.frame(
  prognostic = round(pred_c$prognostic$mean, 3),
  cate       = round(pred_c$cate$mean, 3),
  total      = round(pred_c$total$mean, 3)
))
```

### plot()

The `plot()` method produces diagnostic and inferential graphics using the
**ggplot2** package (a suggested dependency).

#### Sigma traceplot

```{r plot-trace, eval=requireNamespace("ggplot2", quietly=TRUE)}
plot(fit_hs, type = "trace")
```

#### Posterior ATE distribution (causal models)

The ATE density uses the Bayesian-bootstrap PATE posterior by default;
pass `bayesian_bootstrap = FALSE` to plot the (narrower) mixed ATE
density instead. See the summary section above for the definitions.

```{r plot-ate, eval=requireNamespace("ggplot2", quietly=TRUE)}
plot(fit_chf, type = "ate")                         # PATE (default)
plot(fit_chf, type = "ate", bayesian_bootstrap = FALSE)  # MATE
```

#### CATE caterpillar plot

```{r plot-cate, eval=requireNamespace("ggplot2", quietly=TRUE)}
plot(fit_chf, type = "cate")
```

#### Variable importance (Dirichlet prior)

Variable importance plots are available when `prior_type = "dirichlet"`.
For causal models with `prior_type_control = "dirichlet"` or
`prior_type_treat = "dirichlet"`, use `forest = "control"`, `"treat"`,
or `"both"`.

```{r vi-fit}
set.seed(12)
fit_dart <- ShrinkageTrees(
  y               = y_cont,
  X_train         = X_syn,
  outcome_type    = "continuous",
  prior_type      = "dirichlet",
  local_hp        = 0.1 / sqrt(5),
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  verbose         = FALSE
)
```

```{r plot-vi, eval=requireNamespace("ggplot2", quietly=TRUE)}
plot(fit_dart, type = "vi", n_vi = 10)
```

#### Survival curves

For survival models (`outcome_type = "right-censored"` or
`"interval-censored"`), the `plot()` method can draw posterior survival
curves derived from the fitted AFT log-normal model:
$$
S(t \mid \mathbf{x}_i) = 1 - \Phi\!\left(\frac{\log t - \mu_i}{\sigma}\right),
$$
where $\mu_i = f(\mathbf{x}_i)$ is the BART ensemble prediction and $\sigma$
is the residual standard deviation on the log-time scale.

The `type = "survival"` option supports two modes controlled by the `obs`
argument:

- **Population-averaged curve** (`obs = NULL`, the default): computes
  $\bar{S}(t) = n^{-1}\sum_i S(t \mid \mathbf{x}_i)$ at each MCMC
  iteration, giving credible bands that reflect full posterior uncertainty.
- **Individual curves** (`obs = c(1, 5, ...)`): one curve per selected
  training observation with its own credible band.

Additional options:

| Argument | Description |
|---|---|
| `level` | Width of the pointwise credible band (default `0.95`). |
| `t_grid` | Custom time grid (original scale). Auto-generated if `NULL`. |
| `km` | If `TRUE`, overlay the Kaplan–Meier estimate (population-average only). |

We use the survival fit from the earlier section:

```{r surv-curve-pop, eval=requireNamespace("ggplot2", quietly=TRUE)}
# Population-averaged survival curve with 95% credible band
plot(ht_surv, type = "survival")
```

```{r surv-curve-km, eval=requireNamespace("ggplot2", quietly=TRUE) && requireNamespace("survival", quietly=TRUE)}
# Same curve with the Kaplan-Meier estimate overlaid for comparison
plot(ht_surv, type = "survival", km = TRUE)
```

```{r surv-curve-ind, eval=requireNamespace("ggplot2", quietly=TRUE)}
# Individual survival curves for observations 1, 20, 40, 60, and 80
plot(ht_surv, type = "survival", obs = c(1, 20, 40, 60, 80))
```

```{r surv-curve-single, eval=requireNamespace("ggplot2", quietly=TRUE)}
# Single individual with a narrower 90% credible band
plot(ht_surv, type = "survival", obs = 1, level = 0.90)
```

When `store_posterior_sample = FALSE`, the credible bands only reflect
uncertainty in $\sigma$ (using plug-in posterior mean $\hat{\mu}_i$).
The survival functions (`SurvivalBART`, `SurvivalDART`, etc.) store
posterior samples by default, so full posterior bands are available
out of the box.

#### Posterior predictive survival curves

The survival curves above are based on the **training** data — they show
$S(t \mid \mathbf{x}_i)$ for the observations used to fit the model.
For **new** (out-of-sample) data, call `predict()` first and then
`plot()` on the prediction object.  This produces *posterior
predictive* survival curves that propagate full parameter uncertainty
through to the new covariate values:

```{r surv-pred-setup}
# New observations for prediction
set.seed(99)
X_new <- matrix(rnorm(20 * p), ncol = p)
pred_surv <- predict(ht_surv, newdata = X_new)
```

```{r surv-pred-pop, eval=requireNamespace("ggplot2", quietly=TRUE)}
# Population-averaged posterior predictive survival curve
plot(pred_surv, type = "survival")
```

```{r surv-pred-ind, eval=requireNamespace("ggplot2", quietly=TRUE)}
# Individual posterior predictive curves for selected new observations
plot(pred_surv, type = "survival", obs = c(1, 5, 10))
```

The same `level` and `t_grid` arguments are available as for the
training-data survival curves.  The Kaplan–Meier overlay (`km = TRUE`)
is not available for prediction objects, since observed event times are
only known for the training set.

## Multi-Chain MCMC

Running multiple independent chains improves mixing diagnostics and reduces
sensitivity to starting values. Pass `n_chains > 1` to any model-fitting
function; chains are run in parallel via `parallel::mclapply` on Unix-like
systems.

```{r multi-chain}
set.seed(13)
fit_2chain <- ShrinkageTrees(
  y               = y_cont,
  X_train         = X_syn,
  outcome_type    = "continuous",
  prior_type      = "horseshoe",
  local_hp        = 0.1 / sqrt(5),
  global_hp       = 0.1 / sqrt(5),
  number_of_trees = 5,
  N_post          = 50,
  N_burn          = 25,
  n_chains        = 2,
  verbose         = FALSE
)

cat("n_chains stored  :", fit_2chain$mcmc$n_chains, "\n")
cat("Total sigma draws:", length(fit_2chain$sigma),
    " (2 chains x 50 draws)\n")
cat("Per-chain acceptance ratios:\n")
print(round(fit_2chain$chains$acceptance_ratios, 3))
```

The same interface works for causal models.

```{r multi-chain-causal}
set.seed(14)
fit_causal_2chain <- CausalShrinkageForest(
  y                         = y_causal,
  X_train_control           = X_syn,
  X_train_treat             = X_syn,
  treatment_indicator_train = W_syn,
  outcome_type              = "continuous",
  prior_type_control        = "horseshoe",
  prior_type_treat          = "horseshoe",
  local_hp_control          = lh,
  global_hp_control         = lh,
  local_hp_treat            = lh,
  global_hp_treat           = lh,
  number_of_trees_control   = 5,
  number_of_trees_treat     = 5,
  N_post                    = 50,
  N_burn                    = 25,
  n_chains                  = 2,
  verbose                   = FALSE
)

cat("Pooled sigma draws:", length(fit_causal_2chain$sigma), "\n")
cat("Per-chain acceptance ratios (control):\n")
print(round(fit_causal_2chain$chains$acceptance_ratios_control, 3))
```

With multiple chains the traceplot shows one line per chain, and the
overlaid density plot compares the marginal posterior of $\sigma$ across
chains — both require `n_chains > 1`.

```{r plot-trace-2chain, eval=requireNamespace("ggplot2", quietly=TRUE)}
plot(fit_2chain, type = "trace")
```

```{r plot-density-2chain, eval=requireNamespace("ggplot2", quietly=TRUE)}
plot(fit_2chain, type = "density")
```

## Convergence Diagnostics

MCMC methods require careful assessment of convergence before trusting
the posterior summaries. Here are practical guidelines for
**ShrinkageTrees** models.

### Sigma traceplot

The traceplot of the error standard deviation $\sigma$ (via
`plot(fit, type = "trace")`) is the primary diagnostic. A well-mixing
chain should show:

- **No trend**: the trace should fluctuate around a stable level after
  burn-in. A persistent upward or downward drift indicates the sampler
  has not yet converged — increase `N_burn`.
- **Good mixing**: the chain should move freely across its stationary
  range. If the trace is "sticky" (stays in the same region for long
  stretches), this indicates poor mixing.
- **Chain agreement** (when `n_chains > 1`): separate chains should
  overlap substantially. The density overlay
  (`plot(fit, type = "density")`) makes this easy to check visually.

### Acceptance ratio

The `summary()` output reports the average Metropolis–Hastings acceptance
ratio for the tree structure proposals (grow/prune moves). As a rough
guide:

- **0.15–0.50** is typical and healthy for tree-based MCMC.
- **Very low** (< 0.05) means most proposals are rejected. The sampler
  is barely exploring tree space. Consider increasing `N_burn` and
  `N_post`, or relaxing the tree structure prior (lower `power`, higher
  `base`).
- **Very high** (> 0.70) means almost all proposals are accepted, which
  typically indicates the trees are staying very small (e.g. stumps).
  This is less concerning but may limit the model's expressiveness.

### Formal diagnostics with coda

When the suggested package **coda** is installed, `summary()` automatically
reports **effective sample size** (ESS) and — for multi-chain fits — the
**Gelman–Rubin $\hat{R}$**.

```{r coda-summary, eval=requireNamespace("coda", quietly=TRUE)}
# summary() includes convergence diagnostics when coda is available
summary(fit_2chain)
```

For more detailed diagnostics, convert the fitted object to a
`coda::mcmc.list` with `as.mcmc.list()`:

```{r coda-diagnostics, eval=requireNamespace("coda", quietly=TRUE)}
library(coda)
mcmc_obj <- as.mcmc.list(fit_2chain)

# Gelman-Rubin R-hat (values near 1 indicate convergence)
coda::gelman.diag(mcmc_obj)

# Effective sample size
coda::effectiveSize(mcmc_obj)

# Geweke diagnostic (per chain)
coda::geweke.diag(mcmc_obj[[1]])
```

The returned `mcmc.list` object is compatible with all **coda** functions,
including `coda::autocorr.plot()`, `coda::gelman.plot()`,
`coda::heidel.diag()`, and `coda::raftery.diag()`.

### Recommended MCMC settings

The examples in this vignette use very small `N_post` and `N_burn` to
keep build time low. For a real analysis:

- **Minimum**: `N_post = 2000, N_burn = 2000`.
- **Recommended**: `N_post = 5000, N_burn = 5000`.
- **High-dimensional or survival**: `N_post = 5000, N_burn = 10000`
  (the AFT data augmentation step can slow mixing, so a longer burn-in
  helps).
- **Multiple chains**: `n_chains = 2` or `4` to verify convergence and
  produce pooled posterior samples.

## Case Study: TCGA PAAD (Full Analysis)

The full analysis of the `pdac` dataset replicates the case study from
Jacobs, van Wieringen & van der Pas (2025). Due to the high-dimensional
covariate space (~3,000 genes) and the large MCMC settings needed for
reliable inference, the code below is provided for reference but is not
evaluated during vignette building. Pre-computed results can be reproduced
by running the `pdac_analysis` demo: `demo("pdac_analysis", package = "ShrinkageTrees")`.

### Step 1: Propensity score estimation

```{r pdac-ps, eval=FALSE}
data("pdac")

time      <- pdac$time
status    <- pdac$status
treatment <- pdac$treatment
X         <- as.matrix(pdac[, !(names(pdac) %in% c("time","status","treatment"))])

set.seed(2025)
ps_fit <- HorseTrees(
  y            = treatment,
  X_train      = X,
  outcome_type = "binary",
  k            = 0.1,
  N_post       = 5000,
  N_burn       = 5000,
  verbose      = FALSE
)

propensity <- pnorm(ps_fit$train_predictions)
```

```{r pdac-ps-overlap, eval=FALSE}
# Overlap plot
p0 <- propensity[treatment == 0]
p1 <- propensity[treatment == 1]

hist(p0, breaks = 15, col = rgb(1, 0.5, 0, 0.5), xlim = range(propensity),
     xlab = "Propensity score", main = "Propensity score overlap")
hist(p1, breaks = 15, col = rgb(0, 0.5, 0, 0.5), add = TRUE)
legend("topright", legend = c("Control", "Treated"),
       fill = c(rgb(1,0.5,0,0.5), rgb(0,0.5,0,0.5)))
```

### Step 2: Causal survival forest

```{r pdac-causal, eval=FALSE}
# Augment control matrix with propensity scores (BCF-style)
X_control <- cbind(propensity, X)

# Log-transform and centre survival times
log_time <- log(time) - mean(log(time))

set.seed(2025)
fit_pdac <- CausalHorseForest(
  y                         = log_time,
  status                    = status,
  X_train_control           = X_control,
  X_train_treat             = X,
  treatment_indicator_train = treatment,
  outcome_type              = "right-censored",
  timescale                 = "log",
  number_of_trees           = 200,
  N_post                    = 5000,
  N_burn                    = 5000,
  store_posterior_sample    = TRUE,
  verbose                   = FALSE
)
```

### Step 3: ATE and CATE estimation

```{r pdac-ate, eval=FALSE}
# Print model summary
print(fit_pdac)
smry_pdac <- summary(fit_pdac)
print(smry_pdac)

# ATE
cat("ATE posterior mean:",
    round(smry_pdac$treatment_effect$ate, 3), "\n")
cat("95% CI: [",
    round(smry_pdac$treatment_effect$ate_lower, 3), ",",
    round(smry_pdac$treatment_effect$ate_upper, 3), "]\n")
```

### Step 4: Diagnostics

```{r pdac-diag, eval=FALSE}
# Sigma convergence
plot(fit_pdac, type = "trace")

# Posterior ATE density
plot(fit_pdac, type = "ate")

# CATE caterpillar plot
plot(fit_pdac, type = "cate")
```



## References

Chipman, H. A., George, E. I., & McCulloch, R. E. (2010). Bayesian Additive
Regression Trees. *Annals of Applied Statistics*, 4(1), 266–298.

Hahn, P. R., Murray, J. S., & Carvalho, C. M. (2020). Bayesian Regression
Tree Models for Causal Inference: Regularization, Confounding, and
Heterogeneous Treatment Effects. *Bayesian Analysis*, 15(3), 965–1056.

Jacobs, T., van Wieringen, W. N., & van der Pas, S. L. (2025). Horseshoe
Forests for High-Dimensional Causal Survival Analysis. *arXiv preprint*
arXiv:2507.22004.

Linero, A. R. (2018). Bayesian Regression Trees for High-Dimensional
Prediction and Variable Selection. *Journal of the American Statistical
Association*, 113(522), 626–636.

Sparapani, R., Spanbauer, C., & McCulloch, R. (2021). Nonparametric Machine
Learning and Efficient Computation with Bayesian Additive Regression Trees:
The BART R Package. *Journal of Statistical Software*, 97(1), 1–66.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CausalHorseForest.R
\name{CausalHorseForest}
\alias{CausalHorseForest}
\title{Causal Horseshoe Forests}
\usage{
CausalHorseForest(
  y = NULL,
  status = NULL,
  X_train_control,
  X_train_treat,
  treatment_indicator_train,
  X_test_control = NULL,
  X_test_treat = NULL,
  treatment_indicator_test = NULL,
  left_time = NULL,
  right_time = NULL,
  outcome_type = "continuous",
  timescale = "time",
  number_of_trees = 200,
  k = 0.1,
  power = 2,
  base = 0.95,
  p_grow = 0.4,
  p_prune = 0.4,
  nu = 3,
  q = 0.9,
  sigma = NULL,
  N_post = 5000,
  N_burn = 5000,
  delayed_proposal = 5,
  store_posterior_sample = FALSE,
  treatment_coding = "centered",
  propensity = NULL,
  propensity_test = NULL,
  n_chains = 1,
  verbose = TRUE
)
}
\arguments{
\item{y}{Outcome vector. For survival, represents follow-up times (can be on
original or log scale depending on \code{timescale}). Set to \code{NULL}
when using \code{outcome_type = "interval-censored"}, as values are derived
from \code{left_time} and \code{right_time}.}

\item{status}{Optional event indicator vector (1 = event occurred,
0 = censored). Required when \code{outcome_type = "right-censored"}.
For interval-censored outcomes, this is derived automatically from
\code{left_time} and \code{right_time}.}

\item{X_train_control}{Covariate matrix for the control forest. Rows
correspond to samples, columns to covariates.}

\item{X_train_treat}{Covariate matrix for the treatment forest. Rows
correspond to samples, columns to covariates.}

\item{treatment_indicator_train}{Vector indicating treatment assignment for
training samples (1 = treated, 0 = control).}

\item{X_test_control}{Optional test covariate matrix for control forest. If
\code{NULL}, defaults to column means of \code{X_train_control}.}

\item{X_test_treat}{Optional test covariate matrix for treatment forest. If
\code{NULL}, defaults to column means of \code{X_train_treat}.}

\item{treatment_indicator_test}{Optional vector indicating treatment
assignment for test samples.}

\item{left_time}{Optional numeric vector of left (lower) time boundaries.
Required when \code{outcome_type = "interval-censored"}. Exact events
have \code{left_time == right_time}; right-censored observations have
\code{right_time = Inf}; interval-censored observations have finite
\code{left_time < right_time}.}

\item{right_time}{Optional numeric vector of right (upper) time boundaries.
Required when \code{outcome_type = "interval-censored"}. Use \code{Inf}
for right-censored observations.}

\item{outcome_type}{Type of outcome: one of \code{"continuous"},
\code{"right-censored"}, or \code{"interval-censored"}.
Default is \code{"continuous"}.}

\item{timescale}{For survival outcomes: either \code{"time"} (original time
scale, log-transformed internally) or \code{"log"} (already log-transformed).
Used when \code{outcome_type} is \code{"right-censored"} or
\code{"interval-censored"}.}

\item{number_of_trees}{Number of trees in each forest. Default is 200.}

\item{k}{Horseshoe prior scale hyperparameter. Default is 0.1. Controls
global-local shrinkage on step heights.}

\item{power}{Power parameter for tree structure prior. Default is 2.0.}

\item{base}{Base parameter for tree structure prior. Default is 0.95.}

\item{p_grow}{Probability of proposing a grow move. Default is 0.4.}

\item{p_prune}{Probability of proposing a prune move. Default is 0.4.}

\item{nu}{Degrees of freedom for the error variance prior. Default is 3.}

\item{q}{Quantile parameter for error variance prior. Default is 0.90.}

\item{sigma}{Optional known standard deviation of the outcome. If
\code{NULL}, estimated from data.}

\item{N_post}{Number of posterior samples to store. Default is 5000.}

\item{N_burn}{Number of burn-in iterations. Default is 5000.}

\item{delayed_proposal}{Number of delayed iterations before proposal updates.
Default is 5.}

\item{store_posterior_sample}{Logical; whether to store posterior samples of
predictions. Default is \code{FALSE}.}

\item{treatment_coding}{Treatment coding scheme for the two-forest model.
One of \code{"centered"} (default), \code{"binary"}, \code{"adaptive"},
or \code{"invariant"}.
\code{"centered"} uses \eqn{b_i \in \{-1/2, 1/2\}};
\code{"binary"} uses \eqn{b_i \in \{0, 1\}};
\code{"adaptive"} uses \eqn{b_i = A_i - \hat{e}(x_i)} where
\eqn{\hat{e}(x_i)} is the estimated propensity score;
\code{"invariant"} treats \eqn{b_0, b_1} as parameters estimated within
the Gibbs sampler with \eqn{b_j \sim N(0, 1/2)} priors, yielding a
parameterisation-invariant model (Hahn et al., 2020, Section 5.2).}

\item{propensity}{Optional numeric vector of propensity scores
\eqn{\hat{e}(x_i)} for training observations. Required when
\code{treatment_coding = "adaptive"}.}

\item{propensity_test}{Optional numeric vector of propensity scores for
test observations. Only used when \code{treatment_coding = "adaptive"}.
Defaults to \code{0.5} for all test observations if not provided.}

\item{n_chains}{Number of independent MCMC chains to run. Default is
\code{1} (standard single-chain behaviour). When \code{n_chains > 1} the
chains are run in parallel via \code{parallel::mclapply} and their
posterior samples are pooled into a single \code{CausalShrinkageForest}
object, so all existing \code{print} and \code{summary} methods work
without modification. On Windows, \code{mclapply} falls back to
sequential execution.}

\item{verbose}{Logical; whether to print verbose output during sampling.
Default is \code{TRUE}.}
}
\value{
An S3 object of class \code{"CausalShrinkageForest"} containing:
\describe{
\item{train_predictions}{Posterior mean predictions on training data
(combined forest).}
\item{test_predictions}{Posterior mean predictions on test data
(combined forest).}
\item{train_predictions_control}{Estimated control outcomes on training
data.}
\item{test_predictions_control}{Estimated control outcomes on test data.}
\item{train_predictions_treat}{Estimated treatment effects on training
data.}
\item{test_predictions_treat}{Estimated treatment effects on test data.}
\item{sigma}{Vector of posterior samples for the error standard deviation.}
\item{acceptance_ratio_control}{Average acceptance ratio in control
forest.}
\item{acceptance_ratio_treat}{Average acceptance ratio in treatment
forest.}
\item{train_predictions_sample_control}{Matrix of posterior samples for
control predictions (if \code{store_posterior_sample = TRUE}).}
\item{test_predictions_sample_control}{Matrix of posterior samples for
control predictions (if \code{store_posterior_sample = TRUE}).}
\item{train_predictions_sample_treat}{Matrix of posterior samples for
treatment effects (if \code{store_posterior_sample = TRUE}).}
\item{test_predictions_sample_treat}{Matrix of posterior samples for
treatment effects (if \code{store_posterior_sample = TRUE}).}
}
}
\description{
This function fits a (Bayesian) Causal Horseshoe Forest.
It can be used for estimation of conditional average treatments effects of
survival data given high-dimensional covariates. The outcome is decomposed in
a prognostic part (control) and a treatment effect part. For both of these,
we specify a Horseshoe Trees regression function.
Supports continuous, right-censored, and interval-censored outcomes.
}
\details{
The model separately regularizes the control and treatment trees using
Horseshoe priors with global-local shrinkage on the step heights.
This approach is designed for robust estimation of heterogeneous treatment
effects in high-dimensional settings.
It supports continuous, right-censored, and interval-censored survival
outcomes. For interval-censored data, provide \code{left_time} and
\code{right_time} instead of \code{y} and \code{status}; the event
indicators are derived internally following the
\code{survival::Surv(type = "interval2")} convention.
}
\examples{
# Example: Continuous outcome and homogeneous treatment effect
n <- 50
p <- 3
X_control <- matrix(runif(n * p), ncol = p)
X_treat <- matrix(runif(n * p), ncol = p)
treatment <- rbinom(n, 1, 0.5)
tau <- 2
y <- X_control[, 1] + (0.5 - treatment) * tau + rnorm(n)

fit <- CausalHorseForest(
  y = y,
  X_train_control = X_control,
  X_train_treat = X_treat,
  treatment_indicator_train = treatment,
  outcome_type = "continuous",
  number_of_trees = 5,
  N_post = 10,
  N_burn = 5,
  store_posterior_sample = TRUE,
  verbose = FALSE
)

\donttest{
## Example: Right-censored survival outcome
# Set data dimensions
n <- 100
p <- 1000

# Generate covariates
X <- matrix(runif(n * p), ncol = p)
X_treat <- X
treatment <- rbinom(n, 1, pnorm(X[, 1] - 1/2))

# Generate true survival times depending on X and treatment
linpred <- X[, 1] - X[, 2] + (treatment - 0.5) * (1 + X[, 2] / 2 + X[, 3] / 3 
                                                  + X[, 4] / 4)
true_time <- linpred + rnorm(n, 0, 0.5)

# Generate censoring times
censor_time <- log(rexp(n, rate = 1 / 5))

# Observed times and event indicator
time_obs <- pmin(true_time, censor_time)
status <- as.numeric(true_time == time_obs)

# Estimate propensity score using HorseTrees
fit_prop <- HorseTrees(
  y = treatment,
  X_train = X,
  outcome_type = "binary",
  number_of_trees = 200,
  N_post = 1000,
  N_burn = 1000
)

# Retrieve estimated probability of treatment (propensity score)
propensity <- fit_prop$train_probabilities

# Combine propensity score with covariates for control forest
X_control <- cbind(propensity, X)

# Fit the Causal Horseshoe Forest for survival outcome
fit_surv <- CausalHorseForest(
  y = time_obs,
  status = status,
  X_train_control = X_control,
  X_train_treat = X_treat,
  treatment_indicator_train = treatment,
  outcome_type = "right-censored",
  timescale = "log",
  number_of_trees = 200,
  k = 0.1,
  N_post = 1000,
  N_burn = 1000,
  store_posterior_sample = TRUE
)

## Evaluate and summarize results

# Evaluate C-index if survival package is available
if (requireNamespace("survival", quietly = TRUE)) {
  predicted_survtime <- fit_surv$train_predictions
  cindex_result <- survival::concordance(survival::Surv(time_obs, status) ~ predicted_survtime)
  c_index <- cindex_result$concordance
  cat("C-index:", round(c_index, 3), "\n")
} else {
  cat("Package 'survival' not available. Skipping C-index computation.\n")
}

# Compute posterior ATE samples
ate_samples <- rowMeans(fit_surv$train_predictions_sample_treat)
mean_ate <- mean(ate_samples)
ci_95 <- quantile(ate_samples, probs = c(0.025, 0.975))

cat("Posterior mean ATE:", round(mean_ate, 3), "\n")
cat("95\% credible interval: [", round(ci_95[1], 3), ", ", round(ci_95[2], 3), "]\n", sep = "")

# Plot histogram of ATE samples
hist(
  ate_samples,
  breaks = 30,
  col = "steelblue",
  freq = FALSE,
  border = "white",
  xlab = "Average Treatment Effect (ATE)",
  main = "Posterior distribution of ATE"
)
abline(v = mean_ate, col = "orange3", lwd = 2)
abline(v = ci_95, col = "orange3", lty = 2, lwd = 2)
abline(v = 1.541667, col = "darkred", lwd = 2)
legend(
  "topright",
  legend = c("Mean", "95\% CI", "Truth"),
  col = c("orange3", "orange3", "red"),
  lty = c(1, 2, 1),
  lwd = 2
)

## Plot individual CATE estimates

# Summarize posterior distribution per patient
posterior_matrix <- fit_surv$train_predictions_sample_treat
posterior_mean <- colMeans(posterior_matrix)
posterior_ci <- apply(posterior_matrix, 2, quantile, probs = c(0.025, 0.975))

df_cate <- data.frame(
  mean = posterior_mean,
  lower = posterior_ci[1, ],
  upper = posterior_ci[2, ]
)

# Sort patients by posterior mean CATE
df_cate_sorted <- df_cate[order(df_cate$mean), ]
n_patients <- nrow(df_cate_sorted)

# Create the plot
plot(
  x = df_cate_sorted$mean,
  y = 1:n_patients,
  type = "n",
  xlab = "CATE per patient (95\% credible interval)",
  ylab = "Patient index (sorted)",
  main = "Posterior CATE estimates",
  xlim = range(df_cate_sorted$lower, df_cate_sorted$upper)
)

# Add CATE intervals
segments(
  x0 = df_cate_sorted$lower,
  x1 = df_cate_sorted$upper,
  y0 = 1:n_patients,
  y1 = 1:n_patients,
  col = "steelblue"
)

# Add mean points
points(df_cate_sorted$mean, 1:n_patients, pch = 16, col = "orange3", lwd = 0.1)

# Add reference line at 0
abline(v = 0, col = "black", lwd = 2)

}

}
\seealso{
Model family: \code{\link{HorseTrees}} (non-causal, horseshoe prior),
\code{\link{ShrinkageTrees}} (non-causal, flexible prior),
\code{\link{CausalShrinkageForest}} (causal, flexible prior).

Survival wrappers: \code{\link{SurvivalBCF}}, \code{\link{SurvivalShrinkageBCF}}.

S3 methods: \code{\link{print.CausalShrinkageForest}},
\code{\link{summary.CausalShrinkageForest}},
\code{\link{predict.CausalShrinkageForest}},
\code{\link{plot.CausalShrinkageForest}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShrinkageTrees.R
\name{ShrinkageTrees}
\alias{ShrinkageTrees}
\title{General Shrinkage Regression Trees (ShrinkageTrees)}
\usage{
ShrinkageTrees(
  y = NULL,
  status = NULL,
  X_train,
  X_test = NULL,
  left_time = NULL,
  right_time = NULL,
  outcome_type = "continuous",
  timescale = "time",
  number_of_trees = 200,
  prior_type = "horseshoe",
  local_hp = NULL,
  global_hp = NULL,
  a_dirichlet = 0.5,
  b_dirichlet = 1,
  rho_dirichlet = NULL,
  power = 2,
  base = 0.95,
  p_grow = 0.4,
  p_prune = 0.4,
  nu = 3,
  q = 0.9,
  sigma = NULL,
  N_post = 1000,
  N_burn = 1000,
  delayed_proposal = 5,
  store_posterior_sample = TRUE,
  n_chains = 1,
  verbose = TRUE
)
}
\arguments{
\item{y}{Outcome vector. Numeric. Can represent continuous outcomes, binary
outcomes (0/1), or follow-up times for survival data. Set to \code{NULL}
when using \code{outcome_type = "interval-censored"}, as values are derived
from \code{left_time} and \code{right_time}.}

\item{status}{Optional censoring indicator vector (1 = event occurred,
0 = censored). Required if \code{outcome_type = "right-censored"}.
For interval-censored outcomes, this is derived automatically from
\code{left_time} and \code{right_time}.}

\item{X_train}{Covariate matrix for training. Each row corresponds to an
observation, and each column to a covariate.}

\item{X_test}{Optional covariate matrix for test data. If NULL, defaults to
the mean of the training covariates.}

\item{left_time}{Optional numeric vector of left (lower) time boundaries.
Required when \code{outcome_type = "interval-censored"}. Exact events
have \code{left_time == right_time}; right-censored observations have
\code{right_time = Inf}; interval-censored observations have finite
\code{left_time < right_time}.}

\item{right_time}{Optional numeric vector of right (upper) time boundaries.
Required when \code{outcome_type = "interval-censored"}. Use \code{Inf}
for right-censored observations.}

\item{outcome_type}{Type of outcome. One of \code{"continuous"},
\code{"binary"}, \code{"right-censored"}, or \code{"interval-censored"}.}

\item{timescale}{Indicates the scale of follow-up times. Options are
\code{"time"} (nonnegative follow-up times, will be log-transformed
internally) or \code{"log"} (already log-transformed). Used when
\code{outcome_type} is \code{"right-censored"} or
\code{"interval-censored"}.}

\item{number_of_trees}{Number of trees in the ensemble. Default is 200.}

\item{prior_type}{Type of prior on the step heights. Options include
\code{"horseshoe"}, \code{"horseshoe_fw"}, \code{"half-cauchy"},
\code{"standard"} and \code{"dirichlet"}.}

\item{local_hp}{Local hyperparameter controlling shrinkage on individual
step heights. Should typically be set smaller than 1 / sqrt(number_of_trees).
Required for \code{prior_type = "standard"}.}

\item{global_hp}{Global hyperparameter controlling overall shrinkage.
Must be specified for Horseshoe-type priors; ignored for
\code{prior_type = "half-cauchy"} or \code{"standard"}.}

\item{a_dirichlet}{First shape parameter of the Beta prior used in the
Dirichlet-Sparse splitting rule. Together with \code{b_dirichlet}, it
controls the expected sparsity level. Only when \code{prior_type = "dirichlet"}.}

\item{b_dirichlet}{Second shape parameter of the Beta prior for the
sparsity level. Larger values shrink splitting probabilities more strongly
toward uniform sparsity. Only when \code{prior_type = "dirichlet"}.}

\item{rho_dirichlet}{Sparsity hyperparameter. If left NULL, it defaults to
the number of covariates. Only when \code{prior_type = "dirichlet"}.}

\item{power}{Power parameter for the tree structure prior. Default is 2.0.}

\item{base}{Base parameter for the tree structure prior. Default is 0.95.}

\item{p_grow}{Probability of proposing a grow move. Default is 0.4.}

\item{p_prune}{Probability of proposing a prune move. Default is 0.4.}

\item{nu}{Degrees of freedom for the error distribution prior. Default is 3.}

\item{q}{Quantile hyperparameter for the error variance prior. Default is 0.90.}

\item{sigma}{Optional known value for error standard deviation. If NULL,
estimated from data.}

\item{N_post}{Number of posterior samples to store. Default is 1000.}

\item{N_burn}{Number of burn-in iterations. Default is 1000.}

\item{delayed_proposal}{Number of delayed iterations before proposal. Only
for reversible updates. Default is 5.}

\item{store_posterior_sample}{Logical; whether to store posterior samples for
each iteration. Default is TRUE.}

\item{n_chains}{Number of independent MCMC chains to run. Default is
\code{1} (standard single-chain behaviour). When \code{n_chains > 1} the
chains are run in parallel via \code{parallel::mclapply} and their
posterior samples are pooled into a single \code{ShrinkageTrees} object,
so all existing \code{print}, \code{summary}, and \code{predict} methods
work without modification. On Windows, \code{mclapply} falls back to
sequential execution.}

\item{verbose}{Logical; whether to print verbose output. Default is TRUE.}
}
\value{
An S3 object of class "ShrinkageTrees" containing the following elements:
\describe{
\item{train_predictions}{Vector of posterior mean predictions on the
training data.}
\item{test_predictions}{Vector of posterior mean predictions on the test
data (or on mean covariate vector if \code{X_test} not provided).}
\item{sigma}{Vector of posterior samples of the error variance.}
\item{acceptance_ratio}{Average acceptance ratio across trees during
sampling.}
\item{train_predictions_sample}{Matrix of posterior samples of training
predictions (iterations in rows, observations in columns). Present only if
\code{store_posterior_sample = TRUE}.}
\item{test_predictions_sample}{Matrix of posterior samples of test
predictions. Present only if \code{store_posterior_sample = TRUE}.}
\item{train_probabilities}{Vector of posterior mean probabilities on the
training data (only for \code{outcome_type = "binary"}).}
\item{test_probabilities}{Vector of posterior mean probabilities on the
test data (only for \code{outcome_type = "binary"}).}
\item{train_probabilities_sample}{Matrix of posterior samples of training
probabilities (only for \code{outcome_type = "binary"} and if
\code{store_posterior_sample = TRUE}).}
\item{test_probabilities_sample}{Matrix of posterior samples of test
probabilities (only for \code{outcome_type = "binary"} and if
\code{store_posterior_sample = TRUE}).}
}
}
\description{
Fits a Bayesian Shrinkage Tree model with flexible global-local priors on the
step heights. This function generalizes \code{\link{HorseTrees}} by allowing
different global-local shrinkage priors on the step heights.
Supports continuous, binary, right-censored, and interval-censored outcomes.
}
\details{
This function is a flexible generalization of \code{HorseTrees}.
Instead of using a single Horseshoe prior, it allows specifying different
global–local shrinkage configurations for the tree step heights. Further
methodological details on the Horseshoe Forest framework can be found in
Jacobs, van Wieringen & van der Pas (2025).

The \code{horseshoe} prior is the fully Bayesian global-local shrinkage
prior, where both the global and local shrinkage parameters are assigned
half-Cauchy distributions with scale hyperparameters \code{global_hp} and
\code{local_hp}, respectively. The global shrinkage parameter is defined
separately for each tree, allowing adaptive regularization per tree.

The \code{horseshoe_fw} prior (forest-wide horseshoe) is similar to
\code{horseshoe}, except that the global shrinkage parameter is shared
across all trees in the forest simultaneously.

The \code{half-cauchy} prior considers only local shrinkage and does not
include a global shrinkage component. It places a half-Cauchy prior on each
local shrinkage parameter with scale hyperparameter \code{local_hp}.

The \code{standard} prior (Chipman, George & McCulloch, 2010) corresponds to
the classical BART specification, where step heights are given a normal
prior with variance scaled by the number of trees. This prior does not
introduce a global shrinkage parameter and does not use global–local
structure.

The \code{dirichlet} prior implements the Dirichlet–Sparse splitting rule of
Linero (2018), in which splitting probabilities follow a Dirichlet prior
whose concentration is controlled by a Beta sparsity parameter
(\code{a_dirichlet}, \code{b_dirichlet}) and an expected sparsity level
\code{rho_dirichlet}.
}
\examples{
# Example: Continuous outcome with ShrinkageTrees, two priors
n <- 50
p <- 3
X <- matrix(runif(n * p), ncol = p)
X_test <- matrix(runif(n * p), ncol = p)
y <- X[, 1] + rnorm(n)

# Fit ShrinkageTrees with standard horseshoe prior
fit_horseshoe <- ShrinkageTrees(y = y,
                                X_train = X,
                                X_test = X_test,
                                outcome_type = "continuous",
                                number_of_trees = 5,
                                prior_type = "horseshoe",
                                local_hp = 0.1 / sqrt(5),
                                global_hp = 0.1 / sqrt(5),
                                N_post = 10,
                                N_burn = 5,
                                store_posterior_sample = TRUE,
                                verbose = FALSE)

# Fit ShrinkageTrees with half-Cauchy prior
fit_halfcauchy <- ShrinkageTrees(y = y,
                                 X_train = X,
                                 X_test = X_test,
                                 outcome_type = "continuous",
                                 number_of_trees = 5,
                                 prior_type = "half-cauchy",
                                 local_hp = 1 / sqrt(5),
                                 N_post = 10,
                                 N_burn = 5,
                                 store_posterior_sample = TRUE,
                                 verbose = FALSE)

# Posterior mean predictions
pred_horseshoe <- colMeans(fit_horseshoe$train_predictions_sample)
pred_halfcauchy <- colMeans(fit_halfcauchy$train_predictions_sample)

# Posteriors of the mean (global average prediction)
post_mean_horseshoe <- rowMeans(fit_horseshoe$train_predictions_sample)
post_mean_halfcauchy <- rowMeans(fit_halfcauchy$train_predictions_sample)

# Posterior mean prediction averages
mean_pred_horseshoe <- mean(post_mean_horseshoe)
mean_pred_halfcauchy <- mean(post_mean_halfcauchy)

# Example: Interval-censored survival outcome
n <- 50; p <- 3
X_ic <- matrix(rnorm(n * p), ncol = p)
true_t <- rexp(n, rate = exp(-X_ic[, 1]))
left_t  <- true_t * runif(n, 0.5, 1)
right_t <- true_t * runif(n, 1, 1.5)
exact <- sample(n, 15)
left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
rc <- sample(setdiff(seq_len(n), exact), 10); right_t[rc] <- Inf

fit_ic <- ShrinkageTrees(left_time = left_t, right_time = right_t,
                         X_train = X_ic,
                         outcome_type = "interval-censored",
                         prior_type = "horseshoe",
                         local_hp = 0.1 / sqrt(5),
                         global_hp = 0.1 / sqrt(5),
                         number_of_trees = 5,
                         N_post = 10, N_burn = 5,
                         verbose = FALSE)

}
\references{
Jacobs, T., van Wieringen, W. N., & van der Pas, S. L. (2025).
\emph{Horseshoe Forests for High-Dimensional Causal Survival Analysis.}
arXiv:2507.22004. https://doi.org/10.48550/arXiv.2507.22004
Chipman, H. A., George, E. I., & McCulloch, R. E. (2010).
\emph{BART: Bayesian additive regression trees.} Annals of Applied Statistics.

Linero, A. R. (2018). \emph{Bayesian regression trees for high-dimensional
prediction and variable selection.} Journal of the American Statistical
Association.
}
\seealso{
Model family: \code{\link{HorseTrees}} (horseshoe prior),
\code{\link{CausalHorseForest}} (causal inference),
\code{\link{CausalShrinkageForest}} (causal, flexible prior).

Survival wrappers: \code{\link{SurvivalBART}}, \code{\link{SurvivalDART}}.

S3 methods: \code{\link{print.ShrinkageTrees}},
\code{\link{summary.ShrinkageTrees}},
\code{\link{predict.ShrinkageTrees}},
\code{\link{plot.ShrinkageTrees}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.CausalShrinkageForestPrediction}
\alias{print.CausalShrinkageForestPrediction}
\title{Print a CausalShrinkageForestPrediction object}
\usage{
\method{print}{CausalShrinkageForestPrediction}(x, n_head = 6, digits = 3, ...)
}
\arguments{
\item{x}{A \code{CausalShrinkageForestPrediction} object.}

\item{n_head}{Number of observations to display per section. Default \code{6}.}

\item{digits}{Number of decimal places. Default \code{3}.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a formatted table of posterior mean predictions and credible
interval bounds for the first \code{n_head} observations, with separate
sections for the prognostic function \eqn{\mu(X)}, the CATE \eqn{\tau(X)},
and the total outcome \eqn{\mu(X) + \tau(X)}.
}
\seealso{
\code{\link{predict.CausalShrinkageForest}},
\code{\link{summary.CausalShrinkageForestPrediction}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{ovarian_truth}
\alias{ovarian_truth}
\title{Ground-truth quantities for the semi-synthesised ovarian dataset}
\format{
A data frame with one row per patient in \code{\link{ovarian}}
and the following columns:
\describe{
\item{true_log_T}{Numeric. True (uncensored) survival time on the
log scale.}
\item{true_T}{Numeric. True (uncensored) survival time on the
original scale.}
\item{true_mu}{Numeric. True prognostic function \eqn{\mu(x)}
(expected log survival time at the reference treatment).}
\item{true_tau}{Numeric. True conditional average treatment effect
\eqn{\tau(x)} on the log-survival scale.}
\item{true_propensity}{Numeric. True propensity for the treated
group (carboplatin) used to simulate the observed assignment.}
}
}
\usage{
ovarian_truth
}
\description{
The simulated quantities that correspond to the
\code{\link{ovarian}} dataset. Because the ovarian outcomes and
treatment assignment are generated from a known data-generating
process, the underlying potential outcomes, prognostic function,
conditional treatment effect, and propensity score are available for
validating estimators of treatment effects under right- and
interval-censored survival.
}
\examples{
data(ovarian)
data(ovarian_truth)
stopifnot(nrow(ovarian) == nrow(ovarian_truth))
# True (population) average treatment effect on the log-survival scale:
mean(ovarian_truth$true_tau)
}
\seealso{
\code{\link{ovarian}} for the observed semi-synthesised data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.ShrinkageTrees}
\alias{print.ShrinkageTrees}
\title{Print a ShrinkageTrees model}
\usage{
\method{print}{ShrinkageTrees}(x, ...)
}
\arguments{
\item{x}{A fitted \code{ShrinkageTrees} model object.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a concise summary of a fitted \code{ShrinkageTrees} model,
including outcome type, prior, MCMC settings, acceptance ratio, and
posterior mean sigma.
}
\seealso{
\code{\link{summary.ShrinkageTrees}}, \code{\link{HorseTrees}},
\code{\link{ShrinkageTrees}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.CausalShrinkageForest}
\alias{print.CausalShrinkageForest}
\title{Print a CausalShrinkageForest model}
\usage{
\method{print}{CausalShrinkageForest}(x, ...)
}
\arguments{
\item{x}{A fitted \code{CausalShrinkageForest} model object.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a concise summary of a fitted \code{CausalShrinkageForest} model
with per-forest columns for priors, tree counts, feature counts, and MCMC
acceptance ratios.
}
\seealso{
\code{\link{summary.CausalShrinkageForest}},
\code{\link{CausalHorseForest}}, \code{\link{CausalShrinkageForest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayesian_bootstrap.R
\name{bayesian_bootstrap_ate}
\alias{bayesian_bootstrap_ate}
\title{Bayesian bootstrap average treatment effect}
\usage{
bayesian_bootstrap_ate(object, alpha = 0.05)
}
\arguments{
\item{object}{Either a fitted \code{CausalShrinkageForest} (from
\code{\link{CausalShrinkageForest}} or \code{\link{CausalHorseForest}}
with \code{store_posterior_sample = TRUE}) or a
\code{CausalShrinkageForestPrediction} (from
\code{\link{predict.CausalShrinkageForest}}).}

\item{alpha}{One minus the credible level. Default \code{0.05}
(a 95 percent credible interval).}
}
\value{
A list with
\describe{
\item{pate_mean, pate_ci, pate_samples}{Posterior mean, credible
interval (named \code{lower} and \code{upper}), and full vector of
draws of the Bayesian-bootstrap PATE.}
\item{mate_mean, mate_ci, mate_samples}{Same quantities for the
equal-weight mixed ATE.}
\item{n, S}{Number of observations and posterior draws used.}
}
}
\description{
Post-hoc reweights the stored posterior CATE draws of a fitted causal
model to produce credible intervals for the \emph{population} ATE (PATE)
that incorporate uncertainty in the covariate distribution
\eqn{F_X}.
}
\details{
At each MCMC iteration \eqn{s} the conditional treatment effects
\eqn{\tau^{(s)}(x_i)} are reweighted with
\eqn{(w_1^{(s)}, \dots, w_n^{(s)}) \sim \mathrm{Dir}(1, \dots, 1)}
to give a draw
\deqn{\widehat{\mathrm{PATE}}^{(s)} = \sum_{i=1}^n w_i^{(s)}\, \tau^{(s)}(x_i).}
The collection \eqn{\{\widehat{\mathrm{PATE}}^{(s)}\}} approximates the
posterior of the PATE, integrating over \eqn{\tau(\cdot)} \emph{and}
\eqn{F_X}. The equal-weight mixed ATE (MATE),
\eqn{\widehat{\mathrm{MATE}}^{(s)} = n^{-1}\sum_i \tau^{(s)}(x_i)},
is returned alongside for comparison.

For reproducibility, call \code{set.seed()} before invoking the function
to fix the Dirichlet draws.
}
\examples{
# Small toy causal model (binary outcome, for speed)
set.seed(1)
n <- 40; p <- 3
X <- matrix(runif(n * p), ncol = p)
trt <- rbinom(n, 1, 0.5)
y <- X[, 1] + trt * (0.5 + X[, 2]) + rnorm(n)

fit <- CausalShrinkageForest(
  y = y,
  X_train_control = X, X_train_treat = X,
  treatment_indicator_train = trt,
  outcome_type = "continuous",
  number_of_trees_control = 5, number_of_trees_treat = 5,
  prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
  local_hp_control = 0.1, global_hp_control = 0.1,
  local_hp_treat = 0.1, global_hp_treat = 0.1,
  N_post = 20, N_burn = 10,
  store_posterior_sample = TRUE,
  verbose = FALSE
)

bb <- bayesian_bootstrap_ate(fit, alpha = 0.05)
bb$pate_mean
bb$pate_ci

}
\seealso{
\code{\link{summary.CausalShrinkageForest}},
\code{\link{plot.CausalShrinkageForest}},
\code{\link{predict.CausalShrinkageForest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SurvivalWrappers.R
\name{SurvivalShrinkageBCF}
\alias{SurvivalShrinkageBCF}
\title{SurvivalShrinkageBCF (Shrinkage Bayesian Causal Forest for survival data)}
\usage{
SurvivalShrinkageBCF(
  time = NULL,
  status = NULL,
  X_train,
  treatment,
  timescale = "time",
  propensity = NULL,
  treatment_coding = "centered",
  a_dir = 0.5,
  b_dir = 1,
  number_of_trees_control = 200,
  number_of_trees_treat = 50,
  power_control = 2,
  base_control = 0.95,
  power_treat = 3,
  base_treat = 0.25,
  N_post = 1000,
  N_burn = 1000,
  store_posterior_sample = TRUE,
  verbose = TRUE,
  left_time = NULL,
  right_time = NULL,
  ...
)
}
\arguments{
\item{time}{Outcome vector of (non-negative) survival times. Required for
right-censored outcomes; set to \code{NULL} when using interval censoring.}

\item{status}{Event indicator (1 = event, 0 = censored). Required for
right-censored outcomes; derived automatically for interval censoring.}

\item{X_train}{Design matrix for training data.}

\item{treatment}{Treatment indicator (0/1) for training data.}

\item{timescale}{Either \code{"time"} (log-transform internally) or
\code{"log"} (already log-transformed).}

\item{propensity}{Optional vector of propensity scores. If provided,
it is appended to the control forest design matrix. Required when
\code{treatment_coding = "adaptive"}.}

\item{treatment_coding}{Character string specifying how the treatment
indicator enters the model. One of \code{"centered"} (default, maps to
\eqn{-1/2} and \eqn{1/2}), \code{"binary"} (maps to \eqn{0} and \eqn{1}),
\code{"adaptive"} (maps to \eqn{z_i - \hat{e}(x_i)}, where
\eqn{\hat{e}(x_i)} is the propensity score), or \code{"invariant"}
(parameter-expanded coding with \eqn{b_0, b_1 \sim N(0, 1/2)} estimated
within the Gibbs sampler; Hahn et al., 2020, Section 5.2).}

\item{a_dir}{First shape parameter of the Beta prior controlling the
sparsity level in the Dirichlet splitting rule.}

\item{b_dir}{Second shape parameter of the Beta prior controlling the
sparsity level in the Dirichlet splitting rule.}

\item{number_of_trees_control}{Number of trees in the control forest.
Default is 200.}

\item{number_of_trees_treat}{Number of trees in the treatment forest.
Default is 50.}

\item{power_control, base_control}{Tree-structure prior parameters for the
control forest.}

\item{power_treat, base_treat}{Tree-structure prior parameters for the
treatment forest.}

\item{N_post}{Number of posterior samples to store.}

\item{N_burn}{Number of burn-in iterations.}

\item{store_posterior_sample}{Logical; if \code{TRUE} (default), store
the full \eqn{N_\text{post} \times n} matrix of posterior predictions.
Required for \code{predict()}, \code{plot(type = "survival")} with full
posterior credible bands, and custom posterior analyses. Set to
\code{FALSE} to save memory when only posterior means are needed.}

\item{verbose}{Logical; print sampling progress.}

\item{left_time}{Optional numeric vector of left (lower) time boundaries
for interval-censored data. Exact events have
\code{left_time == right_time}; right-censored observations have
\code{right_time = Inf}; interval-censored observations have finite
\code{left_time < right_time}. When provided together with
\code{right_time}, the model is fitted with
\code{outcome_type = "interval-censored"} and \code{time}/\code{status}
are ignored.}

\item{right_time}{Optional numeric vector of right (upper) time boundaries.
Use \code{Inf} for right-censored observations.}

\item{...}{Additional arguments passed to \code{\link{CausalShrinkageForest}}
to override default hyperparameters.}
}
\value{
An object of class \code{"CausalShrinkageForest"} fitted with
Dirichlet splitting priors and additional shrinkage.
}
\description{
Fits a survival version of a Bayesian Causal Forest (BCF) under an
accelerated failure time (AFT) model, combining Dirichlet splitting
priors with global-local shrinkage. Supports both right-censored and
interval-censored survival outcomes.
}
\details{
This wrapper extends \code{\link{SurvivalBCF}} by incorporating
Dirichlet sparsity in both the prognostic (control) and treatment
forests, while applying additional shrinkage to the control forest
via a half-Cauchy prior.

The SurvivalShrinkageBCF model decomposes the outcome as
\deqn{
  \log T = \mu(x) + a \cdot \tau(x) + \varepsilon,
}
where \eqn{\mu(x)} represents the prognostic (control) component and
\eqn{\tau(x)} the heterogeneous treatment effect.

In contrast to \code{SurvivalBCF}, this function:
\itemize{
\item Applies a Dirichlet splitting prior to both forests, inducing
structural sparsity in variable selection.
\item Combines Dirichlet sparsity with additional half-Cauchy shrinkage
in the control forest.
}

The Dirichlet prior follows the sparse splitting framework of Linero (2018),
where splitting probabilities are governed by a Beta-Dirichlet hierarchy.
The sparsity level is controlled by \code{a_dir} and \code{b_dir}.

Survival outcomes are modeled using an AFT formulation with censoring
handled via data augmentation. Both right-censored and interval-censored
data are supported. For interval-censored data, supply \code{left_time}
and \code{right_time} instead of \code{time} and \code{status}.
}
\examples{
set.seed(4)
n <- 30; p <- 5
X <- matrix(rnorm(n * p), ncol = p)
treatment <- rbinom(n, 1, 0.5)
log_T <- X[, 1] + treatment * (-0.5) + rnorm(n)
time <- exp(log_T)
status <- rbinom(n, 1, 0.7)

fit <- SurvivalShrinkageBCF(time = time, status = status, X_train = X,
                             treatment = treatment,
                             number_of_trees_control = 5,
                             number_of_trees_treat = 5,
                             N_post = 50, N_burn = 25,
                             verbose = FALSE)

# S3 methods
print(fit)
smry <- summary(fit)

# Posterior ATE with 95\% credible interval
cat("ATE:", round(smry$treatment_effect$ate, 3), "\n")

# Diagnostic and treatment-effect plots (requires ggplot2)
if (requireNamespace("ggplot2", quietly = TRUE)) {
  plot(fit, type = "trace")
  plot(fit, type = "cate")
}

# Interval-censored causal example
set.seed(14)
n <- 30; p <- 5
X <- matrix(rnorm(n * p), ncol = p)
treatment <- rbinom(n, 1, 0.5)
true_t <- exp(X[, 1] + treatment * (-0.5) + rnorm(n))
left_t  <- true_t * runif(n, 0.5, 1)
right_t <- true_t * runif(n, 1, 1.5)
exact <- sample(n, 10); left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
rc <- sample(setdiff(seq_len(n), exact), 5); right_t[rc] <- Inf

fit_ic <- SurvivalShrinkageBCF(left_time = left_t, right_time = right_t,
                                X_train = X, treatment = treatment,
                                number_of_trees_control = 5,
                                number_of_trees_treat = 5,
                                N_post = 50, N_burn = 25, verbose = FALSE)

}
\references{
Caron, A., Baio, G., & Manolopoulou, I. (2022).
Shrinkage Bayesian Causal Forests for Heterogeneous Treatment Effects Estimation.
Journal of Computational and Graphical Statistics, 31(4), 1202--1214.
https://doi.org/10.1080/10618600.2022.2067549
}
\seealso{
Related models: \code{\link{SurvivalBCF}} (standard BART priors),
\code{\link{CausalShrinkageForest}} (general shrinkage priors),
\code{\link{CausalHorseForest}} (horseshoe prior).

S3 methods: \code{\link{print.CausalShrinkageForest}},
\code{\link{summary.CausalShrinkageForest}},
\code{\link{predict.CausalShrinkageForest}},
\code{\link{plot.CausalShrinkageForest}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{as.mcmc.list.ShrinkageTrees}
\alias{as.mcmc.list.ShrinkageTrees}
\title{Convert MCMC output to a coda mcmc.list}
\usage{
\method{as.mcmc.list}{ShrinkageTrees}(x, ...)
}
\arguments{
\item{x}{A fitted \code{ShrinkageTrees} object.}

\item{...}{Currently unused.}
}
\value{
A \code{\link[coda]{mcmc.list}} object.
Each chain is an \code{\link[coda]{mcmc}} object whose columns include:
\describe{
\item{sigma}{Posterior draws of the residual standard deviation
(continuous and survival outcomes only).}
}
}
\description{
Converts the posterior draws stored in a \code{ShrinkageTrees} object into
a \code{\link[coda]{mcmc.list}} for use with the \pkg{coda} package's
convergence diagnostics (Gelman--Rubin \eqn{\hat{R}}, effective sample
size, Geweke test, etc.).
}
\details{
Requires the suggested package \pkg{coda}.
For single-chain fits the returned object contains one chain.
}
\examples{
\donttest{
fit <- HorseTrees(y = rnorm(50), X_train = matrix(rnorm(250), 50, 5),
                  N_post = 200, N_burn = 100, n_chains = 2)
if (requireNamespace("coda", quietly = TRUE)) {
  mcmc_obj <- coda::as.mcmc.list(fit)
  coda::gelman.diag(mcmc_obj)
  coda::effectiveSize(mcmc_obj)
}
}
}
\seealso{
\code{\link{summary.ShrinkageTrees}} which reports R-hat and ESS
automatically when coda is available.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{predict.CausalShrinkageForest}
\alias{predict.CausalShrinkageForest}
\title{Posterior predictive inference for a CausalShrinkageForest model}
\usage{
\method{predict}{CausalShrinkageForest}(
  object,
  newdata_control,
  newdata_treat,
  level = 0.95,
  bayesian_bootstrap = TRUE,
  ...
)
}
\arguments{
\item{object}{A fitted \code{CausalShrinkageForest} model object.}

\item{newdata_control}{A matrix of new covariates for the control forest,
with the same number of columns as \code{X_train_control} at fit time.}

\item{newdata_treat}{A matrix of new covariates for the treatment forest,
with the same number of columns as \code{X_train_treat} at fit time.
Must have the same number of rows as \code{newdata_control}.}

\item{level}{Credible interval width. Default \code{0.95}.}

\item{bayesian_bootstrap}{Logical; if \code{TRUE} (default), the ATE over
\code{newdata} is computed by reweighting each iteration's CATE vector
with Dirichlet(1, ..., 1) weights (population ATE, PATE). If \code{FALSE},
equal \eqn{1/n} weights are used (mixed ATE, MATE).}

\item{...}{Currently unused.}
}
\value{
A \code{CausalShrinkageForestPrediction} object with elements:
\describe{
\item{prognostic}{List with \code{mean}, \code{lower}, \code{upper}:
posterior summaries of the prognostic function
\eqn{\mu(X_{\text{new}})}.}
\item{cate}{List with \code{mean}, \code{lower}, \code{upper}:
posterior summaries of the CATE \eqn{\tau(X_{\text{new}})}.}
\item{total}{List with \code{mean}, \code{lower}, \code{upper}:
posterior summaries of the total outcome
\eqn{\mu(X_{\text{new}}) + \tau(X_{\text{new}})}.}
\item{ate}{List with \code{mean}, \code{lower}, \code{upper}: posterior
summary of the average treatment effect over \code{newdata}. For
survival with \code{timescale = "time"}, reported as a multiplicative
time ratio on the original scale.}
\item{cate_samples}{\eqn{S \times n_{\text{new}}} matrix of posterior
CATE draws on the scale reported in \code{cate}.}
\item{bayesian_bootstrap}{Flag indicating whether the reported ATE CI
used Dirichlet reweighting.}
\item{n}{Number of test observations.}
\item{level}{Credible level used.}
\item{outcome_type}{Outcome type inherited from the fitted model.}
\item{timescale}{Timescale inherited from the fitted model.}
}
}
\description{
Re-runs the MCMC sampler on new covariate data using the stored training
data and hyperparameters, returning posterior mean predictions and credible
interval bounds for three quantities: the \strong{prognostic function}
(control-forest prediction \eqn{\mu(X)}), the \strong{Conditional Average
Treatment Effect} (CATE, \eqn{\tau(X)}), and the \strong{total predicted
outcome} (\eqn{\mu(X) + \tau(X)}).
}
\details{
The causal forest decomposes the expected outcome as
\deqn{E[Y \mid X] = \mu(X) + \tau(X) \cdot W,}
where \eqn{\mu(X)} is the prognostic function (control forest),
\eqn{\tau(X)} is the CATE (treatment forest), and \eqn{W} is the
treatment indicator.

For \strong{continuous} outcomes and survival with
\code{timescale = "log"}, all three components are on the response scale:
\code{prognostic} and \code{total} include the intercept shift
(\eqn{+ \bar{y}}), while \code{cate} is the pure additive treatment
effect with no intercept.

For \strong{survival with \code{timescale = "time"}}, predictions are
back-transformed to the original time scale:
\itemize{
\item \code{prognostic}: posterior expected baseline survival time
\eqn{E[\exp(\mu(X))]}.
\item \code{cate}: multiplicative effect on survival time
\eqn{\exp(\tau(X))}; a value greater than 1 means treatment prolongs
survival.
\item \code{total}: posterior expected survival time under the observed
treatment \eqn{E[\exp(\mu(X) + \tau(X))]}.
}
}
\seealso{
\code{\link{CausalHorseForest}}, \code{\link{CausalShrinkageForest}},
\code{\link{print.CausalShrinkageForestPrediction}},
\code{\link{summary.CausalShrinkageForestPrediction}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.summary.CausalShrinkageForestPrediction}
\alias{print.summary.CausalShrinkageForestPrediction}
\title{Print a CausalShrinkageForestPrediction summary}
\usage{
\method{print}{summary.CausalShrinkageForestPrediction}(x, digits = 3, ...)
}
\arguments{
\item{x}{A \code{summary.CausalShrinkageForestPrediction} object.}

\item{digits}{Number of decimal places. Default \code{3}.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays distributional summaries (min, Q1, median, max) of the posterior
mean predictions and credible interval bounds, separately for the prognostic
function, CATE, and total outcome.
}
\seealso{
\code{\link{summary.CausalShrinkageForestPrediction}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShrinkageTrees-package.R
\docType{package}
\name{ShrinkageTrees-package}
\alias{ShrinkageTrees-package}
\title{ShrinkageTrees: Bayesian Tree Ensembles for Survival Analysis and Causal Inference}
\description{
Bayesian regression tree ensembles for survival analysis and causal inference. Implements BART, DART, Bayesian Causal Forests (BCF), and Horseshoe Forest models. Supports right-censored and interval-censored survival outcomes via accelerated failure time (AFT) formulations. Designed for high-dimensional prediction and heterogeneous treatment effect estimation.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/tijn-jacobs/ShrinkageTrees}
  \item Report bugs at \url{https://github.com/tijn-jacobs/ShrinkageTrees/issues}
}

}
\author{
\strong{Maintainer}: Tijn Jacobs \email{t.jacobs@vu.nl} (\href{https://orcid.org/0009-0003-6188-9296}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{pdac}
\alias{pdac}
\title{Processed TCGA PAAD dataset (pdac)}
\format{
A data frame with rows corresponding to patients and columns as described above.
}
\source{
\doi{10.1016/j.ccell.2017.07.007}
}
\usage{
pdac
}
\description{
A reduced and cleaned subset of the TCGA pancreatic ductal adenocarcinoma (PAAD)
dataset, derived from The Cancer Genome Atlas (TCGA) PAAD cohort. This version,
\code{pdac}, is smaller and simplified for practical analyses and package examples.
}
\details{
This dataset was originally compiled and curated in the open-source \code{pdacR}
package by Torre-Healy et al. (2023), which harmonized and integrated the TCGA
PAAD gene expression and clinical data. The current version further reduces and
simplifies the data for efficient modeling demonstrations and survival analyses.

The data frame includes:
\itemize{
\item \strong{time}: Overall survival time in months.
\item \strong{status}: Event indicator; 1 = event occurred, 0 = censored.
\item \strong{treatment}: Binary treatment indicator; 1 = radiation therapy, 0 = control.
\item \strong{age}: Age at initial pathologic diagnosis (numeric).
\item \strong{sex}: Binary sex indicator; 1 = male, 0 = female.
\item \strong{grade}: Tumor differentiation grade (ordinal; 1 = well, 2 = moderate, 3 = poor, 4 = undifferentiated).
\item \strong{tumor.cellularity}: Tumor cellularity estimate (numeric).
\item \strong{tumor.purity}: Tumor purity class (binary; 1 = high, 0 = low).
\item \strong{absolute.purity}: Absolute purity estimate (numeric).
\item \strong{moffitt.cluster}: Moffitt transcriptional subtype (binary; 1 = basal-like, 0 = classical).
\item \strong{meth.leukocyte.percent}: DNA methylation leukocyte estimate (numeric).
\item \strong{meth.purity.mode}: DNA methylation purity mode (numeric).
\item \strong{stage}: Nodal stage indicator (binary; 1 = n1, 0 = n0).
\item \strong{lymph.nodes}: Number of lymph nodes examined (numeric).
\item \strong{Driver gene columns}: Expression values of key driver genes (e.g., KRAS, TP53, CDKN2A, SMAD4, BRCA1, BRCA2).
\item \strong{Other gene columns}: Expression values of ~3,000 most variable non-driver genes (based on median absolute deviation).
}
}
\references{
\itemize{
\item Raphael BJ, et al. "Integrated genomic characterization of pancreatic ductal adenocarcinoma." Cancer Cell. 2017 Aug 14;32(2):185–203.e13. PMID: 28810144.
\item Torre-Healy LA, Kawalerski RR, Oh K, et al. "Open-source curation of a pancreatic ductal adenocarcinoma gene expression analysis platform (pdacR) supports a two-subtype model." Communications Biology. 2023; https://doi.org/10.1038/s42003-023-04461-6.
\item The Cancer Genome Atlas (TCGA), PAAD project, DbGaP: phs000178.
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{ovarian}
\alias{ovarian}
\title{Semi-synthesised TCGA Ovarian Cancer Dataset}
\format{
A data frame with 357 rows (patients) and 1007 columns:
\itemize{
\item \strong{OS_time}: Numeric. Observed survival time in days (simulated).
\item \strong{OS_event}: Integer. Event indicator (simulated).
1 = event observed, 0 = right-censored.
\item \strong{treatment}: Integer. Simulated treatment assignment.
1 = carboplatin, 0 = cisplatin. Driven primarily by
\code{year_of_diagnosis} as an instrumental variable
(cisplatin era pre-~2000, carboplatin after).
\item \strong{age}: Integer. Age at initial pathologic diagnosis in years.
\item \strong{figo_stage}: Integer. FIGO stage coded as 2 = Stage II,
3 = Stage III, 4 = Stage IV.
\item \strong{tumor_grade}: Integer. Histologic tumor grade coded as
2 = G2, 3 = G3, 4 = G4. Rows with GX (unknown grade) were
excluded.
\item \strong{year_of_diagnosis}: Integer. Year of initial pathologic diagnosis
(approx. 1992--2013). Used as an instrumental variable for
treatment assignment in the DGP.
\item \strong{right_time, left_time}: Numeric. Interval-censoring bounds
derived from the simulated survival times, suitable for passing
to the package's interval-censored survival interface
(\code{right_time = Inf} for right-censored observations,
\code{left_time == right_time} for exact events).
\item \strong{year_of_diagnosis.1}: Integer. Duplicate of
\code{year_of_diagnosis} left in place from the data-assembly
join; retained for reproducibility and may be ignored.
\item \strong{ENSG...}: Numeric. Log2(TPM + 1) normalised gene expression
levels for 997 Ensembl genes (columns named by versioned Ensembl
gene IDs, e.g. \code{ENSG00000270372.1}). Genes were selected as
the most variable transcripts across TCGA-OV samples, ranked by
median absolute deviation (MAD).
}
}
\source{
\url{https://portal.gdc.cancer.gov/projects/TCGA-OV}
}
\usage{
ovarian
}
\description{
Gene expression and clinical covariates for ovarian cancer patients from
The Cancer Genome Atlas (TCGA-OV), combined with semi-synthetic
survival outcomes and treatment assignment. Real covariates (age,
FIGO stage, tumor grade, gene expression) are retained; survival
times, event indicator, and treatment assignment are simulated from a
known data-generating process so that the true treatment effect is
available for validation (see \code{\link{ovarian_truth}}).
}
\details{
RNA-seq data were downloaded from the GDC portal using the
\code{TCGAbiolinks} package (STAR - Counts workflow). Expression values
were normalised to TPM and log2-transformed as log2(TPM + 1). Genes with
median TPM <= 1 across all samples were removed prior to MAD filtering.
Clinical data were obtained from the BCR Biotab clinical supplement.
Treatment assignment was derived from the drug table
(\code{clinical_drug_ov}), restricted to adjuvant (first-line) treatment
records. Samples were matched between expression and clinical data using
the 12-character TCGA patient barcode.
}
\examples{
data(ovarian)

# Dimensions: patients x (6 clinical + 2000 gene columns)
dim(ovarian)

# Survival outcome
head(ovarian[, c("OS_time", "OS_event", "treatment")])

# KM plot by treatment
if (requireNamespace("survival", quietly = TRUE)) {
  library(survival)
  fit <- survfit(Surv(OS_time, OS_event) ~ treatment, data = ovarian)
  plot(fit, col = c("blue", "red"), xlab = "Time (days)", ylab = "Survival")
  legend("topright", c("Carboplatin", "Cisplatin"), col = c("blue", "red"), lty = 1)
}
}
\references{
Cancer Genome Atlas Research Network (2011). Integrated genomic analyses
of ovarian carcinoma. \emph{Nature}, 474, 609--615.
\doi{10.1038/nature10166}

Colaprico, A. et al. (2016). TCGAbiolinks: an R/Bioconductor package for
integrative analysis with GDC data. \emph{Nucleic Acids Research}, 44(8).
\doi{10.1093/nar/gkv1507}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.CausalShrinkageForestPrediction}
\alias{summary.CausalShrinkageForestPrediction}
\title{Summarise a CausalShrinkageForestPrediction object}
\usage{
\method{summary}{CausalShrinkageForestPrediction}(object, ...)
}
\arguments{
\item{object}{A \code{CausalShrinkageForestPrediction} object.}

\item{...}{Currently unused.}
}
\value{
A \code{summary.CausalShrinkageForestPrediction} object.
}
\description{
Returns distributional summaries (min, Q1, median, max) of the posterior
mean predictions and credible interval bounds across all test observations,
separately for the prognostic function, CATE, and total outcome.
}
\seealso{
\code{\link{predict.CausalShrinkageForest}},
\code{\link{print.summary.CausalShrinkageForestPrediction}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SurvivalWrappers.R
\name{SurvivalDART}
\alias{SurvivalDART}
\title{SurvivalDART}
\usage{
SurvivalDART(
  time = NULL,
  status = NULL,
  X_train,
  X_test = NULL,
  timescale = "time",
  number_of_trees = 200,
  a_dirichlet = 0.5,
  b_dirichlet = 1,
  rho_dirichlet = NULL,
  k = 2,
  N_post = 1000,
  N_burn = 1000,
  store_posterior_sample = TRUE,
  verbose = TRUE,
  left_time = NULL,
  right_time = NULL,
  ...
)
}
\arguments{
\item{time}{Outcome vector of (non-negative) survival times. Required for
right-censored outcomes; set to \code{NULL} when using interval censoring.}

\item{status}{Event indicator (1 = event, 0 = censored). Required for
right-censored outcomes; derived automatically for interval censoring.}

\item{X_train}{Design matrix for training data.}

\item{X_test}{Optional test matrix. If NULL, predictions are computed at
the column means of \code{X_train}.}

\item{timescale}{Either \code{"time"} (log-transform internally) or
\code{"log"} (already log-transformed).}

\item{number_of_trees}{Number of trees in the ensemble. Default is 200.}

\item{a_dirichlet, b_dirichlet}{Beta hyperparameters controlling sparsity
in the Dirichlet splitting rule.}

\item{rho_dirichlet}{Expected number of active predictors. If NULL,
defaults to the number of covariates in \code{X_train}.}

\item{k}{Scaling constant used to calibrate the prior variance of the
step heights.}

\item{N_post}{Number of posterior samples to store.}

\item{N_burn}{Number of burn-in iterations.}

\item{store_posterior_sample}{Logical; if \code{TRUE} (default), store
the full \eqn{N_\text{post} \times n} matrix of posterior predictions.
Required for \code{predict()}, \code{plot(type = "survival")} with full
posterior credible bands, and custom posterior analyses. Set to
\code{FALSE} to save memory when only posterior means are needed.}

\item{verbose}{Logical; print sampling progress.}

\item{left_time}{Optional numeric vector of left (lower) time boundaries
for interval-censored data. Exact events have
\code{left_time == right_time}; right-censored observations have
\code{right_time = Inf}; interval-censored observations have finite
\code{left_time < right_time}. When provided together with
\code{right_time}, the model is fitted with
\code{outcome_type = "interval-censored"} and \code{time}/\code{status}
are ignored.}

\item{right_time}{Optional numeric vector of right (upper) time boundaries.
Use \code{Inf} for right-censored observations.}

\item{...}{Additional arguments passed to \code{\link{ShrinkageTrees}}
to override default hyperparameters.}
}
\value{
An object of class \code{"ShrinkageTrees"} fitted under a Dirichlet
splitting prior (DART) within an AFT formulation.

See \code{\link{ShrinkageTrees}} for a full description of returned components.
}
\description{
Fits an Accelerated Failure Time (AFT) model using the Dirichlet splitting
prior (DART), which induces structural sparsity through a Beta-Dirichlet
hierarchy on splitting probabilities.
Supports both right-censored and interval-censored survival outcomes.
}
\details{
This function provides a survival-specific wrapper for DART under an
AFT formulation for right-censored or interval-censored outcomes.

For right-censored data, supply \code{time} and \code{status}.
For interval-censored data, supply \code{left_time} and \code{right_time}
instead; event indicators are derived internally following the
\code{survival::Surv(type = "interval2")} convention.

Structural regularisation is induced through a Dirichlet prior on
splitting probabilities, encouraging sparse feature usage in
high-dimensional settings.

Users requiring alternative shrinkage priors on the leaf parameters
(e.g., Horseshoe or half-Cauchy priors) should use
\code{\link{ShrinkageTrees}} directly.
}
\examples{
set.seed(2)
n <- 30; p <- 5
X <- matrix(rnorm(n * p), ncol = p)
time <- rexp(n, rate = exp(0.5 * X[, 1]))
status <- rbinom(n, 1, 0.7)

fit <- SurvivalDART(time = time, status = status, X_train = X,
                    number_of_trees = 5, N_post = 50, N_burn = 25,
                    verbose = FALSE)

# S3 methods
print(fit)
smry <- summary(fit)

# Posterior predictions on new data
X_new <- matrix(rnorm(10 * p), ncol = p)
pred <- predict(fit, newdata = X_new)
print(pred)

# Variable importance and survival plots (requires ggplot2)
if (requireNamespace("ggplot2", quietly = TRUE)) {
  plot(fit, type = "vi", n_vi = 5)

  # Posterior survival curves for training data
  plot(fit, type = "survival")

  # Posterior predictive survival curves for new data
  plot(pred, type = "survival")
  plot(pred, type = "survival", obs = c(1, 5))
}

# Interval-censored example
set.seed(12)
n <- 30; p <- 5
X <- matrix(rnorm(n * p), ncol = p)
true_t <- rexp(n, rate = exp(0.5 * X[, 1]))
left_t  <- true_t * runif(n, 0.5, 1)
right_t <- true_t * runif(n, 1, 1.5)
exact <- sample(n, 10); left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
rc <- sample(setdiff(seq_len(n), exact), 5); right_t[rc] <- Inf

fit_ic <- SurvivalDART(left_time = left_t, right_time = right_t,
                        X_train = X, number_of_trees = 5,
                        N_post = 50, N_burn = 25, verbose = FALSE)

}
\seealso{
Related models: \code{\link{SurvivalBART}} (standard BART prior),
\code{\link{ShrinkageTrees}} (general shrinkage priors).

S3 methods: \code{\link{print.ShrinkageTrees}},
\code{\link{summary.ShrinkageTrees}},
\code{\link{predict.ShrinkageTrees}},
\code{\link{plot.ShrinkageTrees}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.ShrinkageTreesPrediction}
\alias{summary.ShrinkageTreesPrediction}
\title{Summarise a ShrinkageTreesPrediction object}
\usage{
\method{summary}{ShrinkageTreesPrediction}(object, ...)
}
\arguments{
\item{object}{A \code{ShrinkageTreesPrediction} object.}

\item{...}{Currently unused.}
}
\value{
A \code{summary.ShrinkageTreesPrediction} object.
}
\description{
Returns distributional summaries (min, Q1, median, max) of the posterior
mean predictions and credible interval bounds across all observations.
}
\seealso{
\code{\link{predict.ShrinkageTrees}},
\code{\link{print.summary.ShrinkageTreesPrediction}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot.CausalShrinkageForest}
\alias{plot.CausalShrinkageForest}
\title{Plot diagnostics for a CausalShrinkageForest model}
\usage{
\method{plot}{CausalShrinkageForest}(
  x,
  type = c("trace", "density", "ate", "cate", "vi"),
  forest = c("both", "control", "treat"),
  n_vi = 10,
  bayesian_bootstrap = TRUE,
  ...
)
}
\arguments{
\item{x}{A \code{CausalShrinkageForest} object.}

\item{type}{Character; one of:
\describe{
\item{\code{"trace"}}{Sigma traceplot (chain mixing).}
\item{\code{"density"}}{Overlaid posterior density of sigma per chain.}
\item{\code{"ate"}}{Posterior density of the average treatment effect
(ATE) with 95 percent credible region. Requires
\code{store_posterior_sample = TRUE}.}
\item{\code{"cate"}}{Point estimates and 95 percent credible intervals
for the CATE of each training observation, sorted by posterior mean.
Requires \code{store_posterior_sample = TRUE}.}
\item{\code{"vi"}}{Posterior credible intervals for variable inclusion
probabilities (Dirichlet prior only). Controlled by \code{forest}.}
}}

\item{forest}{For \code{type = "vi"}: which forest to display.
One of \code{"both"} (default), \code{"control"}, or \code{"treat"}.
When \code{"both"}, a named list of two \pkg{ggplot2} objects is returned.}

\item{n_vi}{Integer; number of top variables for \code{type = "vi"}.
Default \code{10}.}

\item{bayesian_bootstrap}{Logical; only used when \code{type = "ate"}. If
\code{TRUE} (default), the ATE posterior is computed by reweighting each
iteration's CATE vector with Dirichlet(1, ..., 1) weights, giving the
population ATE (PATE). If \code{FALSE}, equal \eqn{1/n} weights are used,
giving the mixed ATE (MATE).}

\item{...}{Additional arguments (currently unused).}
}
\value{
A \pkg{ggplot2} object, or (for \code{type = "vi"} with
\code{forest = "both"}) a named list with elements \code{control} and
\code{treat}.
}
\description{
Visualises posterior draws using \pkg{ggplot2}.
Requires the suggested package \pkg{ggplot2}.
}
\examples{
\donttest{
if (requireNamespace("ggplot2", quietly = TRUE)) {
  set.seed(1)
  n <- 60; p <- 5
  X <- matrix(rnorm(n * p), ncol = p)
  w <- rbinom(n, 1, 0.5)
  y <- X[, 1] + w * 1.5 * (X[, 2] > 0) + rnorm(n, sd = 0.5)

  fit <- CausalShrinkageForest(
    y = y,
    X_train_control = X, X_train_treat = X,
    treatment_indicator_train = w,
    prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
    local_hp_control = 0.1, global_hp_control = 0.1,
    local_hp_treat  = 0.1, global_hp_treat  = 0.1,
    number_of_trees_control = 5, number_of_trees_treat = 5,
    N_post = 50, N_burn = 25,
    store_posterior_sample = TRUE,
    verbose = FALSE
  )

  plot(fit, type = "trace")
  plot(fit, type = "ate")
  plot(fit, type = "cate")
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.summary.CausalShrinkageForest}
\alias{print.summary.CausalShrinkageForest}
\title{Print a CausalShrinkageForest model summary}
\usage{
\method{print}{summary.CausalShrinkageForest}(x, n_vi = 10, ...)
}
\arguments{
\item{x}{A \code{summary.CausalShrinkageForest} object.}

\item{n_vi}{Maximum number of variables to display per variable importance
table. Default \code{10}.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a detailed summary of a \code{CausalShrinkageForest} model,
including model specification, treatment effect estimates, prognostic
function, posterior sigma, variable importance for each forest, and MCMC
diagnostics.
}
\seealso{
\code{\link{summary.CausalShrinkageForest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HorseTrees.R
\name{HorseTrees}
\alias{HorseTrees}
\title{Horseshoe Regression Trees (HorseTrees)}
\usage{
HorseTrees(
  y = NULL,
  status = NULL,
  X_train,
  X_test = NULL,
  left_time = NULL,
  right_time = NULL,
  outcome_type = "continuous",
  timescale = "time",
  number_of_trees = 200,
  k = 0.1,
  power = 2,
  base = 0.95,
  p_grow = 0.4,
  p_prune = 0.4,
  nu = 3,
  q = 0.9,
  sigma = NULL,
  N_post = 1000,
  N_burn = 1000,
  delayed_proposal = 5,
  store_posterior_sample = TRUE,
  n_chains = 1,
  verbose = TRUE
)
}
\arguments{
\item{y}{Outcome vector. Numeric. Can represent continuous outcomes, binary
outcomes (0/1), or follow-up times for survival data. Set to \code{NULL}
(default) when using \code{outcome_type = "interval-censored"}, as values
are derived from \code{left_time} and \code{right_time}.}

\item{status}{Optional censoring indicator vector (1 = event occurred,
0 = censored). Required if \code{outcome_type = "right-censored"}.
For interval-censored outcomes, this is derived automatically from
\code{left_time} and \code{right_time}.}

\item{X_train}{Covariate matrix for training. Each row corresponds to an
observation, and each column to a covariate.}

\item{X_test}{Optional covariate matrix for test data. If NULL, defaults to
the mean of the training covariates.}

\item{left_time}{Optional numeric vector of left (lower) time boundaries.
Required when \code{outcome_type = "interval-censored"}. Exact events
have \code{left_time == right_time}; right-censored observations have
\code{right_time = Inf}; interval-censored observations have finite
\code{left_time < right_time}.}

\item{right_time}{Optional numeric vector of right (upper) time boundaries.
Required when \code{outcome_type = "interval-censored"}. Use \code{Inf}
for right-censored observations.}

\item{outcome_type}{Type of outcome. One of \code{"continuous"},
\code{"binary"}, \code{"right-censored"}, or \code{"interval-censored"}.}

\item{timescale}{Indicates the scale of follow-up times. Options are
\code{"time"} (nonnegative follow-up times, will be log-transformed internally)
or \code{"log"} (already log-transformed). Only used when
\code{outcome_type = "right-censored"} or \code{"interval-censored"}.}

\item{number_of_trees}{Number of trees in the ensemble. Default is 200.}

\item{k}{Horseshoe scale hyperparameter (default 0.1). This parameter
controls the overall level of shrinkage by setting the scale for both
global and local shrinkage components. The local and global hyperparameters
are parameterized as
\eqn{\alpha = \frac{k}{\sqrt{\mathrm{number\_of\_trees}}}}
to ensure adaptive regularization across trees.}

\item{power}{Power parameter for tree structure prior. Default is 2.0.}

\item{base}{Base parameter for tree structure prior. Default is 0.95.}

\item{p_grow}{Probability of proposing a grow move. Default is 0.4.}

\item{p_prune}{Probability of proposing a prune move. Default is 0.4.}

\item{nu}{Degrees of freedom for the error distribution prior. Default is 3.}

\item{q}{Quantile hyperparameter for the error variance prior.
Default is 0.90.}

\item{sigma}{Optional known value for error standard deviation. If NULL,
estimated from data.}

\item{N_post}{Number of posterior samples to store. Default is 1000.}

\item{N_burn}{Number of burn-in iterations. Default is 1000.}

\item{delayed_proposal}{Number of delayed iterations before proposal. Only
for reversible updates. Default is 5.}

\item{store_posterior_sample}{Logical; whether to store posterior samples for
each iteration. Default is TRUE.}

\item{n_chains}{Number of independent MCMC chains to run. Default is
\code{1} (standard single-chain behaviour). When \code{n_chains > 1} the
chains are run in parallel via \code{parallel::mclapply} and their
posterior samples are pooled into a single \code{ShrinkageTrees} object,
so all existing \code{print}, \code{summary}, and \code{predict} methods
work without modification. On Windows, \code{mclapply} falls back to
sequential execution.}

\item{verbose}{Logical; whether to print verbose output. Default is TRUE.}
}
\value{
An S3 object of class \code{"ShrinkageTrees"} with the following elements:
\describe{
\item{train_predictions}{Vector of posterior mean predictions on the
training data.}
\item{test_predictions}{Vector of posterior mean predictions on the test
data (or on mean covariate vector if \code{X_test} not provided).}
\item{sigma}{Vector of posterior samples of the error variance.}
\item{acceptance_ratio}{Average acceptance ratio across trees during
sampling.}
\item{train_predictions_sample}{Matrix of posterior samples of training
predictions (iterations in rows, observations in columns). Present only
if \code{store_posterior_sample = TRUE}.}
\item{test_predictions_sample}{Matrix of posterior samples of test
predictions. Present only if \code{store_posterior_sample = TRUE}.}
\item{train_probabilities}{Vector of posterior mean probabilities on the
training data (only for \code{outcome_type = "binary"}).}
\item{test_probabilities}{Vector of posterior mean probabilities on the
test data (only for \code{outcome_type = "binary"}).}
\item{train_probabilities_sample}{Matrix of posterior samples of training
probabilities (only for \code{outcome_type = "binary"} and if
\code{store_posterior_sample = TRUE}).}
\item{test_probabilities_sample}{Matrix of posterior samples of test
probabilities (only for \code{outcome_type = "binary"} and if
\code{store_posterior_sample = TRUE}).}
}
}
\description{
Fits a Bayesian Horseshoe Trees model with a single learner.
Implements regularization on the step heights using a global-local Horseshoe
prior, controlled via the parameter \code{k}. Supports continuous, binary,
right-censored, and interval-censored (survival) outcomes.
}
\details{
For continuous outcomes, the model centers and optionally standardizes the
outcome using a prior guess of the standard deviation.
For binary outcomes, the function uses a probit link formulation.
For right-censored outcomes (survival data), the function can handle
follow-up times either on the original time scale or log-transformed.
For interval-censored outcomes, provide \code{left_time} and
\code{right_time} instead of \code{y} and \code{status}; the event
indicators are derived internally following the
\code{survival::Surv(type = "interval2")} convention.
Generalized implementation with multiple prior possibilities is given by
\code{\link{ShrinkageTrees}}.
}
\examples{
# Minimal example: continuous outcome
n <- 25
p <- 5
X <- matrix(rnorm(n * p), ncol = p)
y <- X[, 1] + rnorm(n)
fit1 <- HorseTrees(y = y, X_train = X, outcome_type = "continuous", 
                   number_of_trees = 5, N_post = 75, N_burn = 25, 
                   verbose = FALSE)

# Minimal example: binary outcome
X <- matrix(rnorm(n * p), ncol = p)
y <- ifelse(X[, 1] + rnorm(n) > 0, 1, 0)
fit2 <- HorseTrees(y = y, X_train = X, outcome_type = "binary", 
                   number_of_trees = 5, N_post = 75, N_burn = 25, 
                   verbose = FALSE)

# Minimal example: right-censored outcome
X <- matrix(rnorm(n * p), ncol = p)
time <- rexp(n, rate = 0.1)
status <- rbinom(n, 1, 0.7)
fit3 <- HorseTrees(y = time, status = status, X_train = X,
                   outcome_type = "right-censored", number_of_trees = 5,
                   N_post = 75, N_burn = 25, verbose = FALSE)

# Minimal example: interval-censored outcome
X <- matrix(rnorm(n * p), ncol = p)
true_t <- rexp(n, rate = 0.1)
left_t  <- true_t * runif(n, 0.5, 1)
right_t <- true_t * runif(n, 1, 1.5)
# Mark some as exact, some as right-censored
exact <- sample(n, 8); left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
rc <- sample(setdiff(seq_len(n), exact), 5); right_t[rc] <- Inf
fit4 <- HorseTrees(left_time = left_t, right_time = right_t, X_train = X,
                   outcome_type = "interval-censored", number_of_trees = 5,
                   N_post = 75, N_burn = 25, verbose = FALSE)

# Larger continuous example (not run automatically)
\donttest{
n <- 100
p <- 100
X <- matrix(rnorm(100 * p), ncol = p)
X_test <- matrix(rnorm(50 * p), ncol = p)
y <- X[, 1] + X[, 2] - X[, 3] + rnorm(100, sd = 0.5)

fit5 <- HorseTrees(y = y,
                   X_train = X,
                   X_test = X_test,
                   outcome_type = "continuous",
                   number_of_trees = 200,
                   N_post = 2500,
                   N_burn = 2500,
                   store_posterior_sample = TRUE,
                   verbose = TRUE)

plot(fit4$sigma, type = "l", ylab = expression(sigma),
     xlab = "Iteration", main = "Sigma traceplot")

hist(fit4$train_predictions_sample[, 1],
     main = "Posterior distribution of prediction outcome individual 1",
     xlab = "Prediction", breaks = 20)
}
                       
}
\seealso{
Model family: \code{\link{ShrinkageTrees}} (flexible prior choice),
\code{\link{CausalHorseForest}} (causal inference),
\code{\link{CausalShrinkageForest}} (causal, flexible prior).

Survival wrappers: \code{\link{SurvivalBART}}, \code{\link{SurvivalDART}}.

S3 methods: \code{\link{print.ShrinkageTrees}},
\code{\link{summary.ShrinkageTrees}},
\code{\link{predict.ShrinkageTrees}},
\code{\link{plot.ShrinkageTrees}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{predict.ShrinkageTrees}
\alias{predict.ShrinkageTrees}
\title{Posterior predictive inference for a ShrinkageTrees model}
\usage{
\method{predict}{ShrinkageTrees}(object, newdata, level = 0.95, ...)
}
\arguments{
\item{object}{A fitted \code{ShrinkageTrees} model object.}

\item{newdata}{A matrix (or object coercible to one) of new covariates with
the same number of columns as the training data.}

\item{level}{Credible interval width. Default \code{0.95}.}

\item{...}{Currently unused.}
}
\value{
A \code{ShrinkageTreesPrediction} object with elements:
\describe{
\item{mean}{Posterior mean predictions (length \code{nrow(newdata)}).}
\item{lower}{Lower credible interval bound.}
\item{upper}{Upper credible interval bound.}
\item{n}{Number of test observations.}
\item{level}{Credible level used.}
\item{outcome_type}{Outcome type inherited from the fitted model.}
\item{timescale}{Timescale inherited from the fitted model (survival only).}
\item{predictions_sample}{(Survival only) \code{N_post} x \code{n}
matrix of posterior predictive draws on the original scale.}
\item{sigma}{(Survival only) Posterior draws of sigma on the log-time
scale (length \code{N_post}).}
}
}
\description{
Re-runs the MCMC sampler on new covariate data using the stored training
data and hyperparameters, returning posterior mean predictions and credible
interval bounds.
}
\seealso{
\code{\link{HorseTrees}}, \code{\link{ShrinkageTrees}},
\code{\link{print.ShrinkageTreesPrediction}},
\code{\link{summary.ShrinkageTreesPrediction}},
\code{\link{plot.ShrinkageTreesPrediction}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot.ShrinkageTrees}
\alias{plot.ShrinkageTrees}
\title{Plot diagnostics for a ShrinkageTrees model}
\usage{
\method{plot}{ShrinkageTrees}(
  x,
  type = c("trace", "density", "vi", "survival"),
  n_vi = 10,
  obs = NULL,
  t_grid = NULL,
  level = 0.95,
  km = FALSE,
  ...
)
}
\arguments{
\item{x}{A \code{ShrinkageTrees} object.}

\item{type}{Character; one of:
\describe{
\item{\code{"trace"}}{Sigma traceplot across MCMC iterations (one line
per chain). Useful for assessing chain mixing.}
\item{\code{"density"}}{Overlaid posterior density of sigma, one curve
per chain.}
\item{\code{"vi"}}{Posterior credible intervals for variable inclusion
probabilities (top \code{n_vi} predictors). Only available for
Dirichlet prior models.}
\item{\code{"survival"}}{Posterior survival curves
\eqn{S(t | x_i) = 1 - \Phi((\log t - \mu_i) / \sigma)} with pointwise
credible bands, derived from the AFT log-normal model. Only available
for survival outcome types (\code{"right-censored"} or
\code{"interval-censored"}).
\strong{Population-averaged curve} (default, \code{obs = NULL}):
computes \eqn{\bar{S}(t) = n^{-1} \sum_i S(t | x_i)} at each MCMC
iteration. The credible band reflects posterior uncertainty in both
\eqn{\mu_i} and \eqn{\sigma} when \code{store_posterior_sample = TRUE},
or sigma-only uncertainty otherwise.
\strong{Individual curves} (\code{obs = c(1, 5, ...)}):
one curve per selected training observation with its own credible band.
Set \code{km = TRUE} to overlay the Kaplan--Meier estimate as a
non-parametric reference (population-averaged plot only).}
}}

\item{n_vi}{Integer; number of top variables to display when
\code{type = "vi"}. Default \code{10}.}

\item{obs}{Integer vector of training-set observation indices for
individual survival curves, or \code{NULL} (default) for the
population-averaged curve. Indices must be between 1 and the number
of training observations. Used only when \code{type = "survival"}.}

\item{t_grid}{Optional numeric vector of time points (on the original
time scale) at which to evaluate the survival function. If \code{NULL}
(default), a grid of 200 equally spaced points is generated from the
range of observed training times. Used only when
\code{type = "survival"}.}

\item{level}{Width of the pointwise credible band for
\code{type = "survival"}. Default \code{0.95} (a 95 percent credible
interval at each time point).}

\item{km}{Logical; if \code{TRUE} and \code{type = "survival"}
with \code{obs = NULL}, overlay the Kaplan--Meier curve (dashed black
step function) as a non-parametric reference. Default \code{FALSE}.
Requires the \pkg{survival} package. Ignored with a message when
\code{obs} is not \code{NULL}.}

\item{...}{Additional arguments (currently unused).}
}
\value{
A \pkg{ggplot2} object.
}
\description{
Visualises posterior draws using \pkg{ggplot2}.
Requires the suggested package \pkg{ggplot2}.
}
\examples{
\donttest{
if (requireNamespace("ggplot2", quietly = TRUE)) {
  set.seed(1)
  n <- 50; p <- 5
  X <- matrix(rnorm(n * p), ncol = p)
  y <- X[, 1] + rnorm(n)

  # Fit a small continuous model
  fit <- ShrinkageTrees(
    y = y, X_train = X,
    prior_type = "horseshoe",
    local_hp = 0.1, global_hp = 0.1,
    number_of_trees = 5,
    N_post = 50, N_burn = 25,
    verbose = FALSE
  )

  # Sigma traceplot -- check chain mixing
  plot(fit, type = "trace")

  # Overlaid posterior densities of sigma per chain
  plot(fit, type = "density")
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.CausalShrinkageForest}
\alias{summary.CausalShrinkageForest}
\title{Summarise a CausalShrinkageForest model}
\usage{
\method{summary}{CausalShrinkageForest}(object, bayesian_bootstrap = TRUE, ...)
}
\arguments{
\item{object}{A fitted \code{CausalShrinkageForest} model object.}

\item{bayesian_bootstrap}{Logical; if \code{TRUE} (default), the ATE
posterior is computed by reweighting the per-iteration CATE vector with
Dirichlet(1, ..., 1) weights (Bayesian bootstrap). This gives a draw from
the posterior of the \emph{population} ATE (PATE), with a credible
interval that accounts for uncertainty in the covariate distribution. If
\code{FALSE}, equal \eqn{1/n} weights are used, giving the mixed ATE
(MATE), which conditions on the observed covariates. Ignored when
posterior samples are not stored.}

\item{...}{Currently unused.}
}
\value{
A \code{summary.CausalShrinkageForest} object with elements:
\describe{
\item{call}{The original model call.}
\item{outcome_type}{Outcome type.}
\item{timescale}{Timescale for survival outcomes.}
\item{prior}{Prior specification for control and treatment forests.}
\item{mcmc}{MCMC settings.}
\item{data_info}{Training and test data dimensions.}
\item{treatment_effect}{List with \code{ate} (posterior mean ATE),
\code{cate_sd} (SD of individual CATEs), and optionally
\code{ate_lower}, \code{ate_upper} (95 percent credible interval;
requires \code{store_posterior_sample = TRUE}) and
\code{bayesian_bootstrap} (the flag used to produce the CI).}
\item{prognostic}{Summary of the prognostic function (mean, SD, range).}
\item{sigma}{Named vector with posterior mean, SD, and 95 percent
credible interval of sigma (if estimated).}
\item{variable_importance_control}{Variable importance for the control
forest (if available).}
\item{variable_importance_treat}{Variable importance for the treatment
forest (if available).}
\item{acceptance_ratios}{List with acceptance ratios for each forest.}
}
}
\description{
Returns an inspectable list with treatment effect estimates, prognostic
function summaries, posterior sigma, variable importance for each forest,
and MCMC diagnostics.
}
\seealso{
\code{\link{print.summary.CausalShrinkageForest}},
\code{\link{CausalHorseForest}}, \code{\link{CausalShrinkageForest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CausalShrinkageForest.R
\name{CausalShrinkageForest}
\alias{CausalShrinkageForest}
\title{General Causal Shrinkage Forests}
\usage{
CausalShrinkageForest(
  y = NULL,
  status = NULL,
  X_train_control,
  X_train_treat,
  treatment_indicator_train,
  X_test_control = NULL,
  X_test_treat = NULL,
  treatment_indicator_test = NULL,
  left_time = NULL,
  right_time = NULL,
  outcome_type = "continuous",
  timescale = "time",
  number_of_trees_control = 200,
  number_of_trees_treat = 200,
  prior_type_control = "horseshoe",
  prior_type_treat = "horseshoe",
  local_hp_control = NULL,
  local_hp_treat = NULL,
  global_hp_control = NULL,
  global_hp_treat = NULL,
  a_dirichlet_control = 0.5,
  a_dirichlet_treat = 0.5,
  b_dirichlet_control = 1,
  b_dirichlet_treat = 1,
  rho_dirichlet_control = NULL,
  rho_dirichlet_treat = NULL,
  power_control = 2,
  power_treat = 2,
  base_control = 0.95,
  base_treat = 0.95,
  p_grow = 0.5,
  p_prune = 0.5,
  nu = 3,
  q = 0.9,
  sigma = NULL,
  N_post = 5000,
  N_burn = 5000,
  delayed_proposal = 5,
  store_posterior_sample = FALSE,
  treatment_coding = "centered",
  propensity = NULL,
  propensity_test = NULL,
  n_chains = 1,
  verbose = TRUE
)
}
\arguments{
\item{y}{Outcome vector. Numeric. Represents continuous outcomes or follow-up times.
Set to \code{NULL} when using \code{outcome_type = "interval-censored"}, as
values are derived from \code{left_time} and \code{right_time}.}

\item{status}{Optional event indicator vector (1 = event occurred, 0 = censored).
Required when \code{outcome_type = "right-censored"}. For interval-censored
outcomes, this is derived automatically from \code{left_time} and
\code{right_time}.}

\item{X_train_control}{Covariate matrix for the control forest. Rows correspond to samples,
columns to covariates.}

\item{X_train_treat}{Covariate matrix for the treatment forest.}

\item{treatment_indicator_train}{Vector indicating treatment assignment for training samples
(1 = treated, 0 = control).}

\item{X_test_control}{Optional covariate matrix for control forest test data. Defaults to
column means of \code{X_train_control} if NULL.}

\item{X_test_treat}{Optional covariate matrix for treatment forest test data. Defaults to
column means of \code{X_train_treat} if NULL.}

\item{treatment_indicator_test}{Optional vector indicating treatment assignment for test data.}

\item{left_time}{Optional numeric vector of left (lower) time boundaries.
Required when \code{outcome_type = "interval-censored"}. Exact events
have \code{left_time == right_time}; right-censored observations have
\code{right_time = Inf}; interval-censored observations have finite
\code{left_time < right_time}.}

\item{right_time}{Optional numeric vector of right (upper) time boundaries.
Required when \code{outcome_type = "interval-censored"}. Use \code{Inf}
for right-censored observations.}

\item{outcome_type}{Type of outcome: one of \code{"continuous"},
\code{"right-censored"}, or \code{"interval-censored"}.
Default is \code{"continuous"}.}

\item{timescale}{For survival outcomes: either \code{"time"} (original scale, log-transformed
internally) or \code{"log"} (already log-transformed). Default is \code{"time"}.
Used when \code{outcome_type} is \code{"right-censored"} or
\code{"interval-censored"}.}

\item{number_of_trees_control}{Number of trees in the control forest. Default is 200.}

\item{number_of_trees_treat}{Number of trees in the treatment forest. Default is 200.}

\item{prior_type_control}{Type of prior on control forest step heights. One of
\code{"horseshoe"}, \code{"horseshoe_fw"}, or \code{"half-cauchy"}.
Default is \code{"horseshoe"}.}

\item{prior_type_treat}{Type of prior on treatment forest step heights. Same options as
\code{prior_type_control}.}

\item{local_hp_control}{Local hyperparameter controlling shrinkage on individual steps
(control forest). Required for all prior types.}

\item{local_hp_treat}{Local hyperparameter for treatment forest.}

\item{global_hp_control}{Global hyperparameter for control forest. Required for horseshoe-type
priors; ignored for \code{"half-cauchy"}.}

\item{global_hp_treat}{Global hyperparameter for treatment forest.}

\item{a_dirichlet_control}{First shape parameter of the Beta prior used in the
Dirichlet–Sparse splitting rule for the control forest. Together with
\code{b_dirichlet_control}, it controls the expected sparsity level.}

\item{a_dirichlet_treat}{First shape parameter of the Beta prior used in the
Dirichlet–Sparse splitting rule for the treatment forest.}

\item{b_dirichlet_control}{Second shape parameter of the Beta prior for the
sparsity level in the control forest. Larger values shrink splitting
probabilities more strongly toward uniform sparsity.}

\item{b_dirichlet_treat}{Second shape parameter of the Beta prior governing
sparsity in the treatment forest.}

\item{rho_dirichlet_control}{Sparsity hyperparameter for the control forest.
Represents the \emph{expected number of active predictors}. If left NULL, it
defaults to the number of covariates in the control forest.}

\item{rho_dirichlet_treat}{Sparsity hyperparameter for the treatment forest,
interpreted as the expected number of active predictors. Defaults to the
number of covariates in the treatment forest if not specified.}

\item{power_control}{Power parameter for the control forest tree structure prior splitting probability.}

\item{power_treat}{Power parameter for the treatment forest tree structure prior splitting probability.}

\item{base_control}{Base parameter for the control forest tree structure prior splitting probability.}

\item{base_treat}{Base parameter for the treatment forest tree structure prior splitting probability.}

\item{p_grow}{Probability of proposing a grow move. Default is 0.5. These are fixed at 0.5 for prior_type
\code{"standard"} and \code{"dirichlet"}.}

\item{p_prune}{Probability of proposing a prune move. Default is 0.5. These are fixed at 0.5 for prior_type
\code{"standard"} and \code{"dirichlet"}.}

\item{nu}{Degrees of freedom for the error variance prior. Default is 3.}

\item{q}{Quantile parameter for error variance prior. Default is 0.90.}

\item{sigma}{Optional known standard deviation of the outcome. If NULL, estimated from data.}

\item{N_post}{Number of posterior samples to store. Default is 5000.}

\item{N_burn}{Number of burn-in iterations. Default is 5000.}

\item{delayed_proposal}{Number of delayed iterations before proposal updates. Default is 5.}

\item{store_posterior_sample}{Logical; whether to store posterior samples of predictions.
Default is \code{FALSE}.}

\item{treatment_coding}{Treatment coding scheme for the two-forest model.
One of \code{"centered"} (default), \code{"binary"}, \code{"adaptive"},
or \code{"invariant"}.
\code{"centered"} uses \eqn{b_i \in \{-1/2, 1/2\}};
\code{"binary"} uses \eqn{b_i \in \{0, 1\}};
\code{"adaptive"} uses \eqn{b_i = A_i - \hat{e}(x_i)} where
\eqn{\hat{e}(x_i)} is the estimated propensity score;
\code{"invariant"} treats \eqn{b_0, b_1} as parameters estimated within
the Gibbs sampler with \eqn{b_j \sim N(0, 1/2)} priors, yielding a
parameterisation-invariant model (Hahn et al., 2020, Section 5.2).}

\item{propensity}{Optional numeric vector of propensity scores
\eqn{\hat{e}(x_i)} for training observations. Required when
\code{treatment_coding = "adaptive"}.}

\item{propensity_test}{Optional numeric vector of propensity scores for
test observations. Only used when \code{treatment_coding = "adaptive"}.
Defaults to \code{0.5} for all test observations if not provided.}

\item{n_chains}{Number of independent MCMC chains to run. Default is
\code{1} (standard single-chain behaviour). When \code{n_chains > 1} the
chains are run in parallel via \code{parallel::mclapply} and their
posterior samples are pooled into a single \code{CausalShrinkageForest}
object, so all existing \code{print} and \code{summary} methods work
without modification. On Windows, \code{mclapply} falls back to
sequential execution.}

\item{verbose}{Logical; whether to print verbose output. Default is \code{TRUE}.}
}
\value{
An S3 object of class \code{"CausalShrinkageForest"} containing:
\describe{
\item{train_predictions}{Posterior mean predictions on training data (combined forest).}
\item{test_predictions}{Posterior mean predictions on test data (combined forest).}
\item{train_predictions_control}{Estimated control outcomes on training data.}
\item{test_predictions_control}{Estimated control outcomes on test data.}
\item{train_predictions_treat}{Estimated treatment effects on training data.}
\item{test_predictions_treat}{Estimated treatment effects on test data.}
\item{sigma}{Vector of posterior samples for the error standard deviation.}
\item{acceptance_ratio_control}{Average acceptance ratio in control forest.}
\item{acceptance_ratio_treat}{Average acceptance ratio in treatment forest.}
\item{train_predictions_sample_control}{Matrix of posterior samples for control predictions
(if \code{store_posterior_sample = TRUE}).}
\item{test_predictions_sample_control}{Matrix of posterior samples for control predictions
(if \code{store_posterior_sample = TRUE}).}
\item{train_predictions_sample_treat}{Matrix of posterior samples for treatment effects
(if \code{store_posterior_sample = TRUE}).}
\item{test_predictions_sample_treat}{Matrix of posterior samples for treatment effects
(if \code{store_posterior_sample = TRUE}).}
}
}
\description{
Fits a (Bayesian) Causal Shrinkage Forest model for estimating heterogeneous treatment effects.
This function generalizes \code{\link{CausalHorseForest}} by allowing flexible
global-local shrinkage priors on the step heights in both the control and treatment forests.
It supports continuous, right-censored, and interval-censored survival outcomes.
}
\details{
This function is a flexible generalization of \code{CausalHorseForest}.
The Causal Shrinkage Forest model decomposes the outcome into a prognostic
(control) and a treatment effect part. Each part is modeled by its own
shrinkage tree ensemble, with separate flexible global-local shrinkage
priors. It is particularly useful for estimating heterogeneous treatment
effects in high-dimensional settings. Further
methodological details on the Horseshoe Forest framework can be found in
Jacobs, van Wieringen & van der Pas (2025).

The \code{horseshoe} prior is the fully Bayesian global-local shrinkage
prior, where both the global and local shrinkage parameters are assigned
half-Cauchy distributions with scale hyperparameters \code{global_hp} and
\code{local_hp}, respectively. The global shrinkage parameter is defined
separately for each tree, allowing adaptive regularization per tree.

The \code{horseshoe_fw} prior (forest-wide horseshoe) is similar to
\code{horseshoe}, except that the global shrinkage parameter is shared
across all trees in the forest simultaneously.

The \code{half-cauchy} prior considers only local shrinkage and does not
include a global shrinkage component. It places a half-Cauchy prior on each
local shrinkage parameter with scale hyperparameter \code{local_hp}.

The \code{dirichlet} prior implements the Dirichlet–Sparse splitting rule of
Linero (2018), in which splitting probabilities follow a Dirichlet prior
whose concentration is controlled by a Beta sparsity parameter
(\code{a_dirichlet}, \code{b_dirichlet}) and an expected sparsity level
\code{rho_dirichlet}.
}
\examples{
# Example: Continuous outcome, homogeneous treatment effect, two priors
n <- 50
p <- 3
X <- matrix(runif(n * p), ncol = p)
X_treat <- X_control <- X
treat <- rbinom(n, 1, X[,1])
tau <- 2
y <- X[, 1] + (0.5 - treat) * tau + rnorm(n)

# Fit a standard Causal Horseshoe Forest
fit_horseshoe <- CausalShrinkageForest(y = y,
                                       X_train_control = X_control,
                                       X_train_treat = X_treat,
                                       treatment_indicator_train = treat,
                                       outcome_type = "continuous",
                                       number_of_trees_treat = 5,
                                       number_of_trees_control = 5,
                                       prior_type_control = "horseshoe",
                                       prior_type_treat = "horseshoe",
                                       local_hp_control = 0.1/sqrt(5),
                                       local_hp_treat = 0.1/sqrt(5),
                                       global_hp_control = 0.1/sqrt(5),
                                       global_hp_treat = 0.1/sqrt(5),
                                       N_post = 10,
                                       N_burn = 5,
                                       store_posterior_sample = TRUE,
                                       verbose = FALSE
)

# Fit a Causal Shrinkage Forest with half-cauchy prior
fit_halfcauchy <- CausalShrinkageForest(y = y,
                                        X_train_control = X_control,
                                        X_train_treat = X_treat,
                                        treatment_indicator_train = treat,
                                        outcome_type = "continuous",
                                        number_of_trees_treat = 5,
                                        number_of_trees_control = 5,
                                        prior_type_control = "half-cauchy",
                                        prior_type_treat = "half-cauchy",
                                        local_hp_control = 1/sqrt(5),
                                        local_hp_treat = 1/sqrt(5),
                                        N_post = 10,
                                        N_burn = 5,
                                        store_posterior_sample = TRUE,
                                        verbose = FALSE
)

# Posterior mean CATEs
CATE_horseshoe <- colMeans(fit_horseshoe$train_predictions_sample_treat)
CATE_halfcauchy <- colMeans(fit_halfcauchy$train_predictions_sample_treat)

# Posteriors of the ATE
post_ATE_horseshoe <- rowMeans(fit_horseshoe$train_predictions_sample_treat)
post_ATE_halfcauchy <- rowMeans(fit_halfcauchy$train_predictions_sample_treat)

# Posterior mean ATE
ATE_horseshoe <- mean(post_ATE_horseshoe)
ATE_halfcauchy <- mean(post_ATE_halfcauchy)

# Example: Interval-censored causal survival outcome
n <- 50; p <- 3
X_ic <- matrix(rnorm(n * p), ncol = p)
treat_ic <- rbinom(n, 1, 0.5)
true_t <- rexp(n, rate = exp(-X_ic[, 1] - 0.5 * treat_ic))
left_t  <- true_t * runif(n, 0.5, 1)
right_t <- true_t * runif(n, 1, 1.5)
exact <- sample(n, 15)
left_t[exact] <- true_t[exact]; right_t[exact] <- true_t[exact]
rc <- sample(setdiff(seq_len(n), exact), 10); right_t[rc] <- Inf

fit_ic <- CausalShrinkageForest(
  left_time = left_t, right_time = right_t,
  X_train_control = X_ic, X_train_treat = X_ic,
  treatment_indicator_train = treat_ic,
  outcome_type = "interval-censored",
  number_of_trees_control = 5, number_of_trees_treat = 5,
  prior_type_control = "horseshoe", prior_type_treat = "horseshoe",
  local_hp_control = 0.1/sqrt(5), local_hp_treat = 0.1/sqrt(5),
  global_hp_control = 0.1/sqrt(5), global_hp_treat = 0.1/sqrt(5),
  N_post = 10, N_burn = 5,
  store_posterior_sample = TRUE, verbose = FALSE)

}
\references{
Jacobs, T., van Wieringen, W. N., & van der Pas, S. L. (2025).
\emph{Horseshoe Forests for High-Dimensional Causal Survival Analysis.}
arXiv:2507.22004. https://doi.org/10.48550/arXiv.2507.22004

Chipman, H. A., George, E. I., & McCulloch, R. E. (2010).
\emph{BART: Bayesian additive regression trees.} Annals of Applied Statistics.

Linero, A. R. (2018). \emph{Bayesian regression trees for high-dimensional
prediction and variable selection.} Journal of the American Statistical
Association.
}
\seealso{
Model family: \code{\link{CausalHorseForest}} (causal, horseshoe prior),
\code{\link{ShrinkageTrees}} (non-causal, flexible prior),
\code{\link{HorseTrees}} (non-causal, horseshoe prior).

Survival wrappers: \code{\link{SurvivalBCF}}, \code{\link{SurvivalShrinkageBCF}}.

S3 methods: \code{\link{print.CausalShrinkageForest}},
\code{\link{summary.CausalShrinkageForest}},
\code{\link{predict.CausalShrinkageForest}},
\code{\link{plot.CausalShrinkageForest}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.ShrinkageTreesPrediction}
\alias{print.ShrinkageTreesPrediction}
\title{Print a ShrinkageTreesPrediction object}
\usage{
\method{print}{ShrinkageTreesPrediction}(x, n_head = 6, digits = 3, ...)
}
\arguments{
\item{x}{A \code{ShrinkageTreesPrediction} object.}

\item{n_head}{Number of observations to display. Default \code{6}.}

\item{digits}{Number of decimal places. Default \code{3}.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a formatted table of posterior mean predictions and credible
interval bounds for the first \code{n_head} observations.
}
\seealso{
\code{\link{predict.ShrinkageTrees}},
\code{\link{summary.ShrinkageTreesPrediction}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.ShrinkageTrees}
\alias{summary.ShrinkageTrees}
\title{Summarise a ShrinkageTrees model}
\usage{
\method{summary}{ShrinkageTrees}(object, ...)
}
\arguments{
\item{object}{A fitted \code{ShrinkageTrees} model object.}

\item{...}{Currently unused.}
}
\value{
A \code{summary.ShrinkageTrees} object with elements:
\describe{
\item{call}{The original model call.}
\item{outcome_type}{Outcome type (\code{"continuous"}, \code{"binary"},
\code{"right-censored"}, or \code{"interval-censored"}).}
\item{timescale}{Timescale for survival outcomes (\code{"time"} or
\code{"log"}).}
\item{prior}{Prior specification.}
\item{mcmc}{MCMC settings (trees, draws, burn-in).}
\item{data_info}{Training and test data dimensions.}
\item{sigma}{Named vector with posterior mean, SD, and 95 percent
credible interval of sigma (continuous and survival outcomes only).}
\item{predictions}{List with \code{train} (and optionally \code{test})
prediction summaries (mean, SD, range).}
\item{variable_importance}{Named vector of posterior inclusion
probabilities, sorted decreasingly (if available).}
\item{acceptance_ratio}{MCMC acceptance ratio vector.}
\item{diagnostics}{(When \pkg{coda} is installed) A list with
\code{ess} (effective sample size) and, for multi-chain fits,
\code{rhat} (Gelman--Rubin \eqn{\hat{R}}).}
}
}
\description{
Returns an inspectable list with posterior sigma summaries, prediction
summaries, variable importance (posterior inclusion probabilities), and
MCMC diagnostics.
}
\seealso{
\code{\link{print.summary.ShrinkageTrees}},
\code{\link{as.mcmc.list.ShrinkageTrees}},
\code{\link{HorseTrees}}, \code{\link{ShrinkageTrees}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot.ShrinkageTreesPrediction}
\alias{plot.ShrinkageTreesPrediction}
\title{Plot posterior predictive survival curves}
\usage{
\method{plot}{ShrinkageTreesPrediction}(x, type = "survival", obs = NULL, t_grid = NULL, level = 0.95, ...)
}
\arguments{
\item{x}{A \code{ShrinkageTreesPrediction} object returned by
\code{\link{predict.ShrinkageTrees}} for a survival model.}

\item{type}{Character; currently only \code{"survival"} is supported.}

\item{obs}{Integer vector of predicted-observation indices for individual
survival curves, or \code{NULL} (default) for the population-averaged
curve across all predicted observations.}

\item{t_grid}{Optional numeric vector of time points (on the original
time scale) at which to evaluate the survival function. If \code{NULL}
(default), a grid of 200 equally spaced points is generated
automatically.}

\item{level}{Width of the pointwise credible band. Default \code{0.95}.}

\item{...}{Additional arguments (currently unused).}
}
\value{
A \pkg{ggplot2} object.
}
\description{
Plots posterior predictive survival curves for new observations from
a \code{ShrinkageTreesPrediction} object. Only available for survival
outcome types (\code{"right-censored"} or \code{"interval-censored"}).
}
\examples{
\donttest{
if (requireNamespace("ggplot2", quietly = TRUE)) {
  set.seed(1)
  n <- 40; p <- 3
  X <- matrix(rnorm(n * p), ncol = p)
  X_test <- matrix(rnorm(10 * p), ncol = p)
  time <- rexp(n, rate = exp(0.5 * X[, 1]))
  status <- rbinom(n, 1, 0.7)

  fit_surv <- SurvivalBART(
    time = time, status = status, X_train = X,
    number_of_trees = 5, N_post = 50, N_burn = 25,
    store_posterior_sample = TRUE, verbose = FALSE
  )

  pred <- predict(fit_surv, newdata = X_test)
  plot(pred, type = "survival")
}
}
}
\seealso{
\code{\link{predict.ShrinkageTrees}},
\code{\link{plot.ShrinkageTrees}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.summary.ShrinkageTreesPrediction}
\alias{print.summary.ShrinkageTreesPrediction}
\title{Print a ShrinkageTreesPrediction summary}
\usage{
\method{print}{summary.ShrinkageTreesPrediction}(x, digits = 3, ...)
}
\arguments{
\item{x}{A \code{summary.ShrinkageTreesPrediction} object.}

\item{digits}{Number of decimal places. Default \code{3}.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays distributional summaries (min, Q1, median, max) of the posterior
mean predictions and credible interval bounds.
}
\seealso{
\code{\link{summary.ShrinkageTreesPrediction}}
}
